@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2015-2016 Cirilo Bernardo <cirilo.bernardo@gmail.com>
- * Copyright (C) 2020 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2020-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,13 +29,13 @@
 
 #include <wx/dir.h>
 #include <wx/dynlib.h>
-#include <wx/filename.h>
 #include <wx/log.h>
 #include <wx/stdpaths.h>
 #include <wx/string.h>
 
 #include <common.h>
 #include <paths.h>
+#include <wx_filename.h>
 #include "3d_plugin_manager.h"
 #include "plugins/3d/3d_plugin.h"
 #include "3d_cache/sg/scenegraph.h"
@@ -55,7 +55,7 @@
 S3D_PLUGIN_MANAGER::S3D_PLUGIN_MANAGER()
 {
     // create the initial file filter list entry
-    m_FileFilters.push_back( _( "All Files" ) + wxT( " (*.*)|*.*" ) );
+    m_FileFilters.emplace_back( _( "All Files" ) + wxT( " (*.*)|*.*" ) );
 
     // discover and load plugins
     loadPlugins();
@@ -65,11 +65,11 @@ S3D_PLUGIN_MANAGER::S3D_PLUGIN_MANAGER()
     {
         std::multimap< const wxString, KICAD_PLUGIN_LDR_3D* >::const_iterator sM = m_ExtMap.begin();
         std::multimap< const wxString, KICAD_PLUGIN_LDR_3D* >::const_iterator eM = m_ExtMap.end();
-        wxLogTrace( MASK_3D_PLUGINMGR, " * Extension [plugin name]:\n" );
+        wxLogTrace( MASK_3D_PLUGINMGR, wxT( " * Extension [plugin name]:\n" ) );
 
         while( sM != eM )
         {
-            wxLogTrace( MASK_3D_PLUGINMGR, "   + '%s' [%s]\n", sM->first.GetData(),
+            wxLogTrace( MASK_3D_PLUGINMGR, wxT( "   + '%s' [%s]\n" ), sM->first.GetData(),
                         sM->second->GetKicadPluginName() );
             ++sM;
         }
@@ -77,7 +77,7 @@ S3D_PLUGIN_MANAGER::S3D_PLUGIN_MANAGER()
     }
     else
     {
-        wxLogTrace( MASK_3D_PLUGINMGR, " * No plugins available\n" );
+        wxLogTrace( MASK_3D_PLUGINMGR, wxT( " * No plugins available\n" ) );
     }
 
 
@@ -86,17 +86,17 @@ S3D_PLUGIN_MANAGER::S3D_PLUGIN_MANAGER()
         /// list of file filters
         std::list< wxString >::const_iterator sFF = m_FileFilters.begin();
         std::list< wxString >::const_iterator eFF = m_FileFilters.end();
-        wxLogTrace( MASK_3D_PLUGINMGR, " * File filters:\n" );
+        wxLogTrace( MASK_3D_PLUGINMGR, wxT( " * File filters:\n" ) );
 
         while( sFF != eFF )
         {
-            wxLogTrace( MASK_3D_PLUGINMGR, " + '%s'\n", (*sFF).GetData() );
+            wxLogTrace( MASK_3D_PLUGINMGR, wxT( " + '%s'\n" ), (*sFF).GetData() );
             ++sFF;
         }
     }
     else
     {
-        wxLogTrace( MASK_3D_PLUGINMGR, " * No file filters available\n" );
+        wxLogTrace( MASK_3D_PLUGINMGR, wxT( " * No file filters available\n" ) );
     }
 #endif  // DEBUG
 }
@@ -120,57 +120,48 @@ S3D_PLUGIN_MANAGER::~S3D_PLUGIN_MANAGER()
 
 void S3D_PLUGIN_MANAGER::loadPlugins( void )
 {
-    std::list< wxString > searchpaths;
-    std::list< wxString > pluginlist;
-    wxFileName fn;
+    std::list<wxString> searchpaths;
+    std::list<wxString> pluginlist;
+    wxFileName          fn;
 
 #ifndef __WXMAC__
-
-#ifdef DEBUG
-    // set up to work from the build directory
-    fn.Assign( wxStandardPaths::Get().GetExecutablePath() );
-    fn.AppendDir( wxT( ".." ) );
-    fn.AppendDir( wxT( "plugins" ) );
-    fn.AppendDir( wxT( "3d" ) );
-
-    std::string testpath = std::string( fn.GetPathWithSep().ToUTF8() );
-    checkPluginPath( testpath, searchpaths );
-
-    // add subdirectories too
-    wxDir debugPluginDir;
-    wxString subdir;
-
-    debugPluginDir.Open( testpath );
-
-    if( debugPluginDir.IsOpened() && debugPluginDir.GetFirst( &subdir, wxEmptyString, wxDIR_DIRS ) )
+    if( wxGetEnv( wxT( "KICAD_RUN_FROM_BUILD_DIR" ), nullptr ) )
     {
-        checkPluginPath( testpath + subdir, searchpaths );
+        // set up to work from the build directory
+        fn.Assign( wxStandardPaths::Get().GetExecutablePath() );
+        fn.AppendDir( wxT( ".." ) );
+        fn.AppendDir( wxT( "plugins" ) );
+        fn.AppendDir( wxT( "3d" ) );
 
-        while( debugPluginDir.GetNext( &subdir ) )
+        std::string testpath = std::string( fn.GetPathWithSep().ToUTF8() );
+        checkPluginPath( testpath, searchpaths );
+
+        // add subdirectories too
+        wxDir    debugPluginDir;
+        wxString subdir;
+
+        debugPluginDir.Open( testpath );
+
+        if( debugPluginDir.IsOpened()
+            && debugPluginDir.GetFirst( &subdir, wxEmptyString, wxDIR_DIRS ) )
+        {
             checkPluginPath( testpath + subdir, searchpaths );
+
+            while( debugPluginDir.GetNext( &subdir ) )
+                checkPluginPath( testpath + subdir, searchpaths );
+        }
     }
-#endif
 
     fn.AssignDir( PATHS::GetStockPlugins3DPath() );
     checkPluginPath( std::string( fn.GetPathWithSep().ToUTF8() ), searchpaths );
-
-    // check for per-user third party plugins
-    // note: GetUserDataDir() gives '.pcbnew' rather than '.kicad' since it uses the exe name;
-    fn.AssignDir( PATHS::GetUserPlugins3DPath() );
-    checkPluginPath( fn.GetPathWithSep(), searchpaths );
 #else
-
    // Search path on OS X is
-   // (1) User     ~/Library/Application Support/kicad/PlugIns/3d
-   checkPluginPath( PATHS::GetOSXKicadUserDataDir() + wxT( "/PlugIns/3d" ), searchpaths );
-
-   // (2) Machine  /Library/Application Support/kicad/PlugIns/3d
+   // (1) Machine  /Library/Application Support/kicad/PlugIns/3d
    checkPluginPath( PATHS::GetOSXKicadMachineDataDir() + wxT( "/PlugIns/3d" ), searchpaths );
 
-   // (3) Bundle   kicad.app/Contents/PlugIns/3d
+   // (2) Bundle   kicad.app/Contents/PlugIns/3d
    fn.AssignDir( PATHS::GetStockPlugins3DPath() );
    checkPluginPath( fn.GetPathWithSep(), searchpaths );
-
 #endif
 
     std::list< wxString >::iterator sPL = searchpaths.begin();
@@ -178,7 +169,7 @@ void S3D_PLUGIN_MANAGER::loadPlugins( void )
 
     while( sPL != ePL )
     {
-        wxLogTrace( MASK_3D_PLUGINMGR, "%s:%s:%d  * [DEBUG] searching path: '%s'",
+        wxLogTrace( MASK_3D_PLUGINMGR, wxT( "%s:%s:%d  * [DEBUG] searching path: '%s'" ),
                     __FILE__, __FUNCTION__, __LINE__, (*sPL).ToUTF8() );
 
         listPlugins( *sPL, pluginlist );
@@ -197,13 +188,13 @@ void S3D_PLUGIN_MANAGER::loadPlugins( void )
 
         if( pp->Open( sPL->ToUTF8() ) )
         {
-            wxLogTrace( MASK_3D_PLUGINMGR, "%s:%s:%d * [DEBUG] adding plugin",
+            wxLogTrace( MASK_3D_PLUGINMGR, wxT( "%s:%s:%d * [DEBUG] adding plugin" ),
                         __FILE__, __FUNCTION__, __LINE__ );
 
             m_Plugins.push_back( pp );
             int nf = pp->GetNFilters();
 
-            wxLogTrace( MASK_3D_PLUGINMGR, "%s:%s:%d * [DEBUG] adding %d filters",
+            wxLogTrace( MASK_3D_PLUGINMGR, wxT( "%s:%s:%d * [DEBUG] adding %d filters" ),
                         __FILE__, __FUNCTION__, __LINE__, nf );
 
             for( int i = 0; i < nf; ++i )
@@ -221,7 +212,7 @@ void S3D_PLUGIN_MANAGER::loadPlugins( void )
         }
         else
         {
-            wxLogTrace( MASK_3D_PLUGINMGR, "%s:%s:%d * [DEBUG] deleting plugin",
+            wxLogTrace( MASK_3D_PLUGINMGR, wxT( "%s:%s:%d * [DEBUG] deleting plugin" ),
                         __FILE__, __FUNCTION__, __LINE__ );
 
             delete pp;
@@ -230,7 +221,7 @@ void S3D_PLUGIN_MANAGER::loadPlugins( void )
         ++sPL;
     }
 
-    wxLogTrace( MASK_3D_PLUGINMGR, "%s:%s:%d * [DEBUG] plugins loaded",
+    wxLogTrace( MASK_3D_PLUGINMGR, wxT( "%s:%s:%d * [DEBUG] plugins loaded" ),
                 __FILE__, __FUNCTION__, __LINE__ );
 }
 
@@ -257,7 +248,7 @@ void S3D_PLUGIN_MANAGER::listPlugins( const wxString& aPath, std::list< wxString
     // Per definition a loadable "xxx.bundle" is similar to an "xxx.app" app
     // bundle being a folder with some special content in it. We obviously don't
     // want to have that here for our loadable module, so just use ".so".
-    nameFilter.Append( ".so" );
+    nameFilter.Append( wxS( ".so" ) );
 #endif
 
     wxString lp = wd.GetNameWithSep();
@@ -288,7 +279,7 @@ void S3D_PLUGIN_MANAGER::checkPluginName( const wxString& aPath,
 
     wxFileName path( ExpandEnvVarSubstitutions( aPath, nullptr ) );
 
-    path.Normalize();
+    path.Normalize( FN_NORMALIZE_FLAGS );
 
     // determine if the path is already in the list
     wxString wxpath = path.GetFullPath();
@@ -305,27 +296,22 @@ void S3D_PLUGIN_MANAGER::checkPluginName( const wxString& aPath,
 
     aPluginList.push_back( wxpath );
 
-    wxLogTrace( MASK_3D_PLUGINMGR, " * [INFO] found 3D plugin '%s'\n", wxpath.GetData() );
+    wxLogTrace( MASK_3D_PLUGINMGR, wxT( " * [INFO] found 3D plugin '%s'\n" ), wxpath.GetData() );
 }
 
 
 void S3D_PLUGIN_MANAGER::checkPluginPath( const wxString& aPath,
                                           std::list< wxString >& aSearchList )
 {
-    // check the existence of a path and add it to the path search list
     if( aPath.empty() )
         return;
 
-    wxLogTrace( MASK_3D_PLUGINMGR, " * [INFO] checking for 3D plugins in '%s'\n", aPath.GetData() );
+    wxLogTrace( MASK_3D_PLUGINMGR, wxT( " * [INFO] checking if valid plugin directory '%s'\n" ),
+                aPath.GetData() );
 
     wxFileName path;
-
-    if( aPath.StartsWith( "${" ) || aPath.StartsWith( "$(" ) )
-        path.Assign( ExpandEnvVarSubstitutions( aPath, nullptr ), "" );
-    else
-        path.Assign( aPath, "" );
-
-    path.Normalize();
+    path.AssignDir( aPath );
+    path.Normalize( FN_NORMALIZE_FLAGS );
 
     if( !wxFileName::DirExists( path.GetFullPath() ) )
         return;
@@ -377,7 +363,7 @@ void S3D_PLUGIN_MANAGER::addExtensionMap( KICAD_PLUGIN_LDR_3D* aPlugin )
 
     int nExt = aPlugin->GetNExtensions();
 
-    wxLogTrace( MASK_3D_PLUGINMGR, "%s:%s:%d * [INFO] adding %d extensions",
+    wxLogTrace( MASK_3D_PLUGINMGR, wxT( "%s:%s:%d * [INFO] adding %d extensions" ),
                 __FILE__, __FUNCTION__, __LINE__, nExt );
 
     for( int i = 0; i < nExt; ++i )
@@ -390,7 +376,7 @@ void S3D_PLUGIN_MANAGER::addExtensionMap( KICAD_PLUGIN_LDR_3D* aPlugin )
 
         if( !ws.empty() )
         {
-            m_ExtMap.insert( std::pair< const wxString, KICAD_PLUGIN_LDR_3D* >( ws, aPlugin ) );
+            m_ExtMap.emplace( ws, aPlugin );
         }
 
     }
@@ -415,10 +401,10 @@ SCENEGRAPH* S3D_PLUGIN_MANAGER::Load3DModel( const wxString& aFileName, std::str
 #endif
 
     // .gz files are compressed versions that may have additional information in the previous extension
-    if( ext_to_find == "gz" )
+    if( ext_to_find == wxT( "gz" ) )
     {
         wxFileName second( raw.GetName() );
-        ext_to_find = second.GetExt() + ".gz";
+        ext_to_find = second.GetExt() + wxT( ".gz" );
     }
 
     std::pair < std::multimap< const wxString, KICAD_PLUGIN_LDR_3D* >::iterator,
@@ -452,7 +438,7 @@ void S3D_PLUGIN_MANAGER::ClosePlugins( void )
     std::list< KICAD_PLUGIN_LDR_3D* >::iterator sP = m_Plugins.begin();
     std::list< KICAD_PLUGIN_LDR_3D* >::iterator eP = m_Plugins.end();
 
-    wxLogTrace( MASK_3D_PLUGINMGR, "%s:%s:%d * [INFO] closing %d extensions",
+    wxLogTrace( MASK_3D_PLUGINMGR, wxT( "%s:%s:%d * [INFO] closing %d extensions" ),
                 __FILE__, __FUNCTION__, __LINE__, static_cast<int>( m_Plugins.size() ) );
 
     while( sP != eP )
