@@ -44,6 +44,7 @@ using namespace KIFONT;
 
 
 FT_Library OUTLINE_FONT::m_freeType = nullptr;
+std::mutex OUTLINE_FONT::m_freeTypeMutex;
 
 OUTLINE_FONT::OUTLINE_FONT() :
         m_face(NULL),
@@ -51,6 +52,8 @@ OUTLINE_FONT::OUTLINE_FONT() :
         m_fakeBold( false ),
         m_fakeItal( false )
 {
+    std::lock_guard<std::mutex> guard( m_freeTypeMutex );
+
     if( !m_freeType )
         FT_Init_FreeType( &m_freeType );
 }
@@ -58,6 +61,8 @@ OUTLINE_FONT::OUTLINE_FONT() :
 
 wxString OUTLINE_FONT::FreeTypeVersion()
 {
+    std::lock_guard<std::mutex> guard( m_freeTypeMutex );
+
     if( !m_freeType )
         FT_Init_FreeType( &m_freeType );
 
@@ -90,31 +95,36 @@ wxString OUTLINE_FONT::FontLibraryVersion()
 
 OUTLINE_FONT* OUTLINE_FONT::LoadFont( const wxString& aFontName, bool aBold, bool aItalic )
 {
-    OUTLINE_FONT* font = new OUTLINE_FONT();
+    std::unique_ptr<OUTLINE_FONT> font = std::make_unique<OUTLINE_FONT>();
 
     wxString fontFile;
     using fc = fontconfig::FONTCONFIG;
 
     fc::FF_RESULT retval = Fontconfig()->FindFont( aFontName, fontFile, aBold, aItalic );
 
+    if( retval == fc::FF_RESULT::FF_ERROR )
+        return nullptr;
+
     if( retval == fc::FF_RESULT::FF_MISSING_BOLD || retval == fc::FF_RESULT::FF_MISSING_BOLD_ITAL )
         font->SetFakeBold();
 
     if( retval == fc::FF_RESULT::FF_MISSING_ITAL || retval == fc::FF_RESULT::FF_MISSING_BOLD_ITAL )
         font->SetFakeItal();
 
-    if( retval != fc::FF_RESULT::FF_ERROR )
-        (void) font->loadFace( fontFile );
+    if( font->loadFace( fontFile ) != 0 )
+        return nullptr;
 
     font->m_fontName = aFontName;       // Keep asked-for name, even if we substituted.
     font->m_fontFileName = fontFile;
 
-    return font;
+    return font.release();
 }
 
 
 FT_Error OUTLINE_FONT::loadFace( const wxString& aFontFileName )
 {
+    std::lock_guard<std::mutex> guard( m_freeTypeMutex );
+
     // TODO: check that going from wxString to char* with UTF-8
     // conversion for filename makes sense on any/all platforms
     FT_Error e = FT_New_Face( m_freeType, aFontFileName.mb_str( wxConvUTF8 ), 0, &m_face );
@@ -323,6 +333,18 @@ VECTOR2I OUTLINE_FONT::getTextAsGlyphs( BOX2I* aBBox, std::vector<std::unique_pt
                                         const VECTOR2I& aPosition, const EDA_ANGLE& aAngle,
                                         bool aMirror, const VECTOR2I& aOrigin,
                                         TEXT_STYLE_FLAGS aTextStyle ) const
+{
+    std::lock_guard<std::mutex> guard( m_freeTypeMutex );
+
+    return getTextAsGlyphsUnlocked( aBBox, aGlyphs, aText, aSize, aPosition, aAngle, aMirror,
+            aOrigin, aTextStyle );
+}
+
+VECTOR2I OUTLINE_FONT::getTextAsGlyphsUnlocked( BOX2I* aBBox, std::vector<std::unique_ptr<GLYPH>>* aGlyphs,
+        const wxString& aText, const VECTOR2I& aSize,
+        const VECTOR2I& aPosition, const EDA_ANGLE& aAngle,
+        bool aMirror, const VECTOR2I& aOrigin,
+        TEXT_STYLE_FLAGS aTextStyle ) const
 {
     VECTOR2D glyphSize = aSize;
     FT_Face  face = m_face;
@@ -366,7 +388,7 @@ VECTOR2I OUTLINE_FONT::getTextAsGlyphs( BOX2I* aBBox, std::vector<std::unique_pt
             {
                 FT_Matrix matrix;
                 // Create a 12 degree slant
-                const float angle = ( -M_PI * 12 ) / 180.0f;
+                const float angle = (float)( -M_PI * 12.0f ) / 180.0f;
                 matrix.xx = (FT_Fixed) ( cos( angle ) * 0x10000L );
                 matrix.xy = (FT_Fixed) ( -sin( angle ) * 0x10000L );
                 matrix.yx = (FT_Fixed) ( 0 * 0x10000L );  // Don't rotate in the y direction
@@ -466,7 +488,7 @@ VECTOR2I OUTLINE_FONT::getTextAsGlyphs( BOX2I* aBBox, std::vector<std::unique_pt
     {
         std::vector<std::unique_ptr<GLYPH>> underscoreGlyphs;
 
-        getTextAsGlyphs( nullptr, &underscoreGlyphs, wxT( "_" ), aSize, { 0, 0 }, ANGLE_0, false,
+        getTextAsGlyphsUnlocked( nullptr, &underscoreGlyphs, wxT( "_" ), aSize, { 0, 0 }, ANGLE_0, false,
                          { 0, 0 }, aTextStyle & ~TEXT_STYLE::OVERBAR );
 
         OUTLINE_GLYPH* underscoreGlyph = static_cast<OUTLINE_GLYPH*>( underscoreGlyphs[0].get() );
@@ -552,6 +574,9 @@ void OUTLINE_FONT::RenderToOpenGLCanvas( KIGFX::OPENGL_GAL& aGal, const wxString
     unsigned int         glyphCount;
     hb_glyph_info_t*     glyphInfo = hb_buffer_get_glyph_infos( buf, &glyphCount );
     hb_glyph_position_t* glyphPos = hb_buffer_get_glyph_positions( buf, &glyphCount );
+    
+    std::lock_guard<std::mutex> guard( m_freeTypeMutex );
+
     hb_font_t*           referencedFont = hb_ft_font_create_referenced( m_face );
 
     hb_ft_font_set_funcs( referencedFont );
