@@ -36,6 +36,7 @@
 #include <kiplatform/app.h>
 #include <kiplatform/environment.h>
 #include <settings/settings_manager.h>
+#include <tool/action_manager.h>
 #include <logging.h>
 
 #include <wx/dynlib.h>
@@ -416,7 +417,7 @@ KIWAY_PLAYER* KIWAY::Player( FRAME_T aFrameType, bool doCreate, wxTopLevelWindow
             FACE_T  face_type = KifaceType( aFrameType );
             KIFACE* kiface = KiFACE( face_type );
 
-            frame = (KIWAY_PLAYER*) kiface->CreateWindow(
+            frame = (KIWAY_PLAYER*) kiface->CreateKiWindow(
                                             aParent,    // Parent window of frame in modal mode,
                                                         // NULL in non modal mode
                                             aFrameType,
@@ -465,7 +466,10 @@ bool KIWAY::PlayerClose( FRAME_T aFrameType, bool doForce )
         return true;
 
     if( frame->NonUserClose( doForce ) )
+    {
+        m_playerFrameId[aFrameType] = wxID_NONE;
         return true;
+    }
 
     return false;
 }
@@ -493,6 +497,13 @@ void KIWAY::ExpressMail( FRAME_T aDestination, MAIL_T aCommand, std::string& aPa
 }
 
 
+void KIWAY::GetActions( std::vector<TOOL_ACTION*>& aActions ) const
+{
+    for( TOOL_ACTION* action : ACTION_MANAGER::GetActionList() )
+        aActions.push_back( action );
+}
+
+
 void KIWAY::SetLanguage( int aLanguage )
 {
     wxString errMsg;
@@ -646,6 +657,14 @@ bool KIWAY::ProcessEvent( wxEvent& aEvent )
 }
 
 
+int KIWAY::ProcessJob( KIWAY::FACE_T aFace, JOB* job )
+{
+    KIFACE* kiface = KiFACE( aFace );
+
+    return kiface->HandleJob( job );
+}
+
+
 void KIWAY::OnKiCadExit()
 {
     if( m_ctl & KFCTL_CPP_PROJECT_SUITE )
