@@ -110,7 +110,7 @@ LANGUAGE_DESCR LanguagesList[] =
     { wxLANGUAGE_SWEDISH,    ID_LANGUAGE_SWEDISH,    wxT( "Svenska" ),  true },
 //    { wxLANGUAGE_VIETNAMESE, ID_LANGUAGE_VIETNAMESE, wxT( "Tiếng Việt" ), true },
 //    { wxLANGUAGE_TURKISH,    ID_LANGUAGE_TURKISH,    wxT( "Türkçe" ),   true },
-    { wxLANGUAGE_UKRAINIAN,  ID_LANGUAGE_UKRANIAN,   wxT( "Українець" ),   true },
+    { wxLANGUAGE_UKRAINIAN,  ID_LANGUAGE_UKRANIAN,   wxT( "Українська" ),   true },
     { wxLANGUAGE_CHINESE_SIMPLIFIED, ID_LANGUAGE_CHINESE_SIMPLIFIED,
             wxT( "简体中文" ), true },
     { wxLANGUAGE_CHINESE_TRADITIONAL, ID_LANGUAGE_CHINESE_TRADITIONAL,
@@ -775,6 +775,26 @@ void PGM_BASE::SetLanguageIdentifier( int menu_id )
 }
 
 
+wxString PGM_BASE::GetLanguageTag()
+{
+    const wxLanguageInfo* langInfo = wxLocale::GetLanguageInfo( m_language_id );
+
+    if( !langInfo )
+        return "";
+    else
+    {
+#if wxCHECK_VERSION( 3, 1, 6 )
+        wxString str = langInfo->GetCanonicalWithRegion();
+#else
+        wxString str = langInfo->CanonicalName;
+#endif
+        str.Replace( "_", "-" );
+
+        return str;
+    }
+}
+
+
 void PGM_BASE::SetLanguagePath()
 {
     wxLocale::AddCatalogLookupPathPrefix( PATHS::GetLocaleDataPath() );
