@@ -154,7 +154,6 @@ void GERBER_PLOTTER::SetGerberCoordinatesFormat( int aResolution, bool aUseInche
 
 void GERBER_PLOTTER::emitDcode( const VECTOR2D& pt, int dcode )
 {
-
     fprintf( m_outputFile, "X%dY%dD%02d*\n", KiROUND( pt.x ), KiROUND( pt.y ), dcode );
 }
 
@@ -902,18 +901,16 @@ void GERBER_PLOTTER::plotArc( const VECTOR2I& aCenter, const EDA_ANGLE& aStartAn
 }
 
 
-void GERBER_PLOTTER::PlotGerberRegion( const SHAPE_LINE_CHAIN& aPoly, void* aData )
+void GERBER_PLOTTER::PlotGerberRegion( const SHAPE_LINE_CHAIN& aPoly, GBR_METADATA* aGbrMetadata )
 {
     if( aPoly.PointCount() <= 2 )
         return;
 
-    GBR_METADATA* gbr_metadata = static_cast<GBR_METADATA*>( aData );
-
     bool clearTA_AperFunction = false;     // true if a TA.AperFunction is used
 
-    if( gbr_metadata )
+    if( aGbrMetadata )
     {
-        std::string attrib = gbr_metadata->m_ApertureMetadata.FormatAttribute( !m_useX2format );
+        std::string attrib = aGbrMetadata->m_ApertureMetadata.FormatAttribute( !m_useX2format );
 
         if( !attrib.empty() )
         {
@@ -922,7 +919,7 @@ void GERBER_PLOTTER::PlotGerberRegion( const SHAPE_LINE_CHAIN& aPoly, void* aDat
         }
     }
 
-    PlotPoly( aPoly, FILL_T::FILLED_SHAPE, 0 , gbr_metadata );
+    PlotPoly( aPoly, FILL_T::FILLED_SHAPE, 0 , aGbrMetadata );
 
     // Clear the TA attribute, to avoid the next item to inherit it:
     if( clearTA_AperFunction )
@@ -939,18 +936,17 @@ void GERBER_PLOTTER::PlotGerberRegion( const SHAPE_LINE_CHAIN& aPoly, void* aDat
 }
 
 
-void GERBER_PLOTTER::PlotGerberRegion( const std::vector<VECTOR2I>& aCornerList, void* aData )
+void GERBER_PLOTTER::PlotGerberRegion( const std::vector<VECTOR2I>& aCornerList,
+                                       GBR_METADATA* aGbrMetadata )
 {
     if( aCornerList.size() <= 2 )
         return;
 
-    GBR_METADATA* gbr_metadata = static_cast<GBR_METADATA*>( aData );
-
     bool clearTA_AperFunction = false;     // true if a TA.AperFunction is used
 
-    if( gbr_metadata )
+    if( aGbrMetadata )
     {
-        std::string attrib = gbr_metadata->m_ApertureMetadata.FormatAttribute( !m_useX2format );
+        std::string attrib = aGbrMetadata->m_ApertureMetadata.FormatAttribute( !m_useX2format );
 
         if( !attrib.empty() )
         {
@@ -959,7 +955,7 @@ void GERBER_PLOTTER::PlotGerberRegion( const std::vector<VECTOR2I>& aCornerList,
         }
     }
 
-    PlotPoly( aCornerList, FILL_T::FILLED_SHAPE, 0, gbr_metadata );
+    PlotPoly( aCornerList, FILL_T::FILLED_SHAPE, 0, aGbrMetadata );
 
     // Clear the TA attribute, to avoid the next item to inherit it:
     if( clearTA_AperFunction )
@@ -976,6 +972,19 @@ void GERBER_PLOTTER::PlotGerberRegion( const std::vector<VECTOR2I>& aCornerList,
 }
 
 
+void GERBER_PLOTTER::PlotPolyAsRegion( const SHAPE_LINE_CHAIN& aPoly, FILL_T aFill,
+                                       int aWidth, GBR_METADATA* aGbrMetadata )
+{
+    // plot a filled polygon using Gerber region, therefore adding X2 attributes
+    // to the solid polygon
+    if( aWidth )
+        PlotPoly( aPoly, FILL_T::NO_FILL, aWidth, aGbrMetadata );
+
+    if( aFill != FILL_T::NO_FILL )
+        PlotGerberRegion( aPoly, aGbrMetadata );
+}
+
+
 void GERBER_PLOTTER::PlotPoly( const SHAPE_LINE_CHAIN& aPoly, FILL_T aFill, int aWidth,
                                void* aData )
 {
@@ -1322,7 +1331,7 @@ void GERBER_PLOTTER::FlashPadOval( const VECTOR2I& aPos, const VECTOR2I& aSize,
         if( orient.IsCardinal90() )
             std::swap( size.x, size.y );
 
-        VECTOR2I pos_device = userToDeviceCoordinates( aPos );
+        VECTOR2D pos_device = userToDeviceCoordinates( aPos );
         int      aperture_attrib = gbr_metadata ? gbr_metadata->GetApertureAttrib() : 0;
         selectAperture( size, 0, ANGLE_0, APERTURE::AT_OVAL, aperture_attrib );
 
@@ -1353,7 +1362,7 @@ void GERBER_PLOTTER::FlashPadOval( const VECTOR2I& aPos, const VECTOR2I& aSize,
                         orient -= ANGLE_180;
                 }
 
-                VECTOR2I pos_device = userToDeviceCoordinates( aPos );
+                VECTOR2D pos_device = userToDeviceCoordinates( aPos );
                 int      aperture_attrib = gbr_metadata ? gbr_metadata->GetApertureAttrib() : 0;
                 selectAperture( size, 0, orient, APERTURE::AM_ROTATED_OVAL, aperture_attrib );
 
@@ -1416,7 +1425,7 @@ void GERBER_PLOTTER::FlashPadRect( const VECTOR2I& pos, const VECTOR2I& aSize,
         }
         else
         {
-            VECTOR2I pos_device = userToDeviceCoordinates( pos );
+            VECTOR2D pos_device = userToDeviceCoordinates( pos );
             int      aperture_attrib = gbr_metadata ? gbr_metadata->GetApertureAttrib() : 0;
             selectAperture( size, 0, ANGLE_0, APERTURE::AT_RECT, aperture_attrib );
 
@@ -1433,7 +1442,7 @@ void GERBER_PLOTTER::FlashPadRect( const VECTOR2I& pos, const VECTOR2I& aSize,
         {
             m_hasApertureRotRect = true;
 
-            VECTOR2I pos_device = userToDeviceCoordinates( pos );
+            VECTOR2D pos_device = userToDeviceCoordinates( pos );
             int      aperture_attrib = gbr_metadata ? gbr_metadata->GetApertureAttrib() : 0;
             selectAperture( size, 0, aOrient, APERTURE::AM_ROT_RECT, aperture_attrib );
 
@@ -1755,7 +1764,7 @@ void GERBER_PLOTTER::FlashPadChamferRoundRect( const VECTOR2I& aShapePos, const
     if( aData )
         gbr_metadata = *static_cast<GBR_METADATA*>( aData );
 
-    VECTOR2I              pos_device = userToDeviceCoordinates( aShapePos );
+    VECTOR2D              pos_device = userToDeviceCoordinates( aShapePos );
     SHAPE_POLY_SET        outline;
     std::vector<VECTOR2I> cornerList;
 
@@ -1994,6 +2003,20 @@ void GERBER_PLOTTER::Text( const VECTOR2I&             aPos,
 }
 
 
+void GERBER_PLOTTER::PlotText( const VECTOR2I& aPos, const COLOR4D& aColor,
+                    const wxString& aText,
+                    const TEXT_ATTRIBUTES& aAttributes,
+                    KIFONT::FONT* aFont,
+                    void* aData )
+{
+    GBR_METADATA* gbr_metadata = static_cast<GBR_METADATA*>( aData );
+
+    if( gbr_metadata )
+        formatNetAttribute( &gbr_metadata->m_NetlistMetadata );
+
+    PLOTTER::PlotText( aPos, aColor, aText, aAttributes, aFont, aData );
+}
+
 void GERBER_PLOTTER::SetLayerPolarity( bool aPositive )
 {
     if( aPositive )
