@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2020 CERN
- * Copyright (C) 2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2021-2022 KiCad Developers, see AUTHORS.txt for contributors.
  * @author Jon Evans <jon@craftyjon.com>
  *
  * This program is free software: you can redistribute it and/or modify it
@@ -109,6 +109,10 @@ PROJECT_FILE::PROJECT_FILE( const wxString& aFullPath ) :
     m_NetSettings = std::make_shared<NET_SETTINGS>( this, "net_settings" );
 
     m_params.emplace_back( new PARAM_LAYER_PRESET( "board.layer_presets", &m_LayerPresets ) );
+
+    m_params.emplace_back( new PARAM_VIEWPORT( "board.viewports", &m_Viewports ) );
+
+    m_params.emplace_back( new PARAM_VIEWPORT3D( "board.3dviewports", &m_Viewports3D ) );
 }
 
 
@@ -460,7 +464,7 @@ bool PROJECT_FILE::MigrateFromLegacy( wxConfigBase* aCfg )
                 wxString       entry;
                 nlohmann::json arr   = nlohmann::json::array();
 
-                wxLogTrace( traceSettings, "Migrating sheet names" );
+                wxLogTrace( traceSettings, wxT( "Migrating sheet names" ) );
 
                 aCfg->SetPath( wxT( "/sheetnames" ) );
 
@@ -470,7 +474,8 @@ bool PROJECT_FILE::MigrateFromLegacy( wxConfigBase* aCfg )
 
                     if( tokens.size() == 2 )
                     {
-                        wxLogTrace( traceSettings, "%d: %s = %s", sheet, tokens[0], tokens[1] );
+                        wxLogTrace( traceSettings, wxT( "%d: %s = %s" ), sheet, tokens[0],
+                                    tokens[1] );
                         arr.push_back( nlohmann::json::array( { tokens[0], tokens[1] } ) );
                     }
                 }
@@ -485,12 +490,12 @@ bool PROJECT_FILE::MigrateFromLegacy( wxConfigBase* aCfg )
 
     std::vector<wxString> groups;
 
-    groups.emplace_back( "" );
+    groups.emplace_back( wxEmptyString );
 
     auto loadLegacyPairs =
             [&]( const std::string& aGroup ) -> bool
             {
-                wxLogTrace( traceSettings, "Migrating group %s", aGroup );
+                wxLogTrace( traceSettings, wxT( "Migrating group %s" ), aGroup );
                 bool     success = true;
                 wxString keyStr;
                 wxString val;
@@ -504,7 +509,7 @@ bool PROJECT_FILE::MigrateFromLegacy( wxConfigBase* aCfg )
 
                     std::string key( keyStr.ToUTF8() );
 
-                    wxLogTrace( traceSettings, "    %s = %s", key, val );
+                    wxLogTrace( traceSettings, wxT( "    %s = %s" ), key, val );
 
                     try
                     {
