@@ -66,6 +66,10 @@ SETTINGS_MANAGER::SETTINGS_MANAGER( bool aHeadless ) :
 
     // create the common settings shared by all applications.  Not loaded immediately
     m_common_settings = RegisterSettings( new COMMON_SETTINGS, false );
+
+    // Create the built-in color settings
+    // Here to allow the Python API to access the built-in colors
+    registerBuiltinColorSettings();
 }
 
 SETTINGS_MANAGER::~SETTINGS_MANAGER()
@@ -203,7 +207,7 @@ COLOR_SETTINGS* SETTINGS_MANAGER::GetColorSettings( const wxString& aName )
         if( !ret )
         {
             ret = registerColorSettings( aName );
-            *ret = *m_color_settings.at( "_builtin_default" );
+            *ret = *m_color_settings.at( COLOR_SETTINGS::COLOR_BUILTIN_DEFAULT );
             ret->SetFilename( wxT( "user" ) );
             ret->SetReadOnly( false );
         }
@@ -212,7 +216,7 @@ COLOR_SETTINGS* SETTINGS_MANAGER::GetColorSettings( const wxString& aName )
     }
 
     // This had better work
-    return m_color_settings.at( "_builtin_default" );
+    return m_color_settings.at( COLOR_SETTINGS::COLOR_BUILTIN_DEFAULT );
 }
 
 
@@ -305,11 +309,17 @@ COLOR_SETTINGS* SETTINGS_MANAGER::GetMigratedColorSettings()
 }
 
 
+void SETTINGS_MANAGER::registerBuiltinColorSettings()
+{
+    for( COLOR_SETTINGS* settings : COLOR_SETTINGS::CreateBuiltinColorSettings() )
+        m_color_settings[settings->GetFilename()] = RegisterSettings( settings, false );
+}
+
+
 void SETTINGS_MANAGER::loadAllColorSettings()
 {
     // Create the built-in color settings
-    for( COLOR_SETTINGS* settings : COLOR_SETTINGS::CreateBuiltinColorSettings() )
-        m_color_settings[settings->GetFilename()] = RegisterSettings( settings, false );
+    registerBuiltinColorSettings();
 
     wxFileName third_party_path;
     const ENV_VAR_MAP& env = Pgm().GetLocalEnvVariables();
