@@ -112,9 +112,9 @@ public:
     std::vector<SCH_ITEM*> GetBusLabels() const;
 
     /// Returns the candidate net name for a driver
-    const wxString& GetNameForDriver( SCH_ITEM* aItem );
+    const wxString& GetNameForDriver( SCH_ITEM* aItem ) const;
 
-    const wxString GetNameForDriver( SCH_ITEM* aItem ) const;
+    const wxString GetNetclassForDriver( SCH_ITEM* aItem ) const;
 
     /// Combines another subgraph on the same sheet into this one.
     void Absorb( CONNECTION_SUBGRAPH* aOther );
@@ -224,7 +224,7 @@ public:
     CONNECTION_SUBGRAPH* m_hier_parent;
 
     /// A cache of escaped netnames from schematic items
-    std::unordered_map<SCH_ITEM*, wxString> m_driver_name_cache;
+    mutable std::unordered_map<SCH_ITEM*, wxString> m_driver_name_cache;
 
     /**
      * Stores the primary driver for the multiple drivers ERC check.  This is the chosen driver
@@ -236,11 +236,33 @@ public:
     SCH_ITEM* m_second_driver;
 };
 
-/// Associates a net code with the final name of a net
-typedef std::pair<wxString, int> NET_NAME_CODE;
+struct NET_NAME_CODE_CACHE_KEY
+{
+    wxString  Name;
+    int       Netcode;
+
+    bool operator==(const NET_NAME_CODE_CACHE_KEY& other) const
+    {
+        return Name == other.Name && Netcode == other.Netcode;
+    }
+};
+
+namespace std
+{
+    template <>
+    struct hash<NET_NAME_CODE_CACHE_KEY>
+    {
+        std::size_t operator()( const NET_NAME_CODE_CACHE_KEY& k ) const
+        {
+            const std::size_t prime = 19937;
+
+            return hash<wxString>()( k.Name ) ^ ( hash<int>()( k.Netcode ) * prime );
+        }
+    };
+}
 
 /// Associates a NET_CODE_NAME with all the subgraphs in that net
-typedef std::map<NET_NAME_CODE, std::vector<CONNECTION_SUBGRAPH*>> NET_MAP;
+typedef std::unordered_map<NET_NAME_CODE_CACHE_KEY, std::vector<CONNECTION_SUBGRAPH*>> NET_MAP;
 
 /**
  * Calculates the connectivity of a schematic and generates netlists
@@ -375,7 +397,7 @@ private:
      * and then the connection for the chosen driver is propagated to all the
      * other items in the subgraph.
      */
-    void buildConnectionGraph();
+    void buildConnectionGraph( std::function<void( SCH_ITEM* )>* aChangedItemHandler );
 
     /**
      * Generates individual item subgraphs on a per-sheet basis
@@ -422,8 +444,11 @@ private:
      *
      * If this subgraph contains hierarchical links, this method will descent the
      * hierarchy and propagate the connectivity across all linked sheets.
+     *
+     * @param aSubgraph is the subgraph being processed
+     * @param aForce prevents this routine from skipping subgraphs
      */
-    void propagateToNeighbors( CONNECTION_SUBGRAPH* aSubgraph );
+    void propagateToNeighbors( CONNECTION_SUBGRAPH* aSubgraph, bool aForce );
 
     /**
      * Search for a matching bus member inside a bus connection
@@ -460,6 +485,8 @@ private:
      */
     bool ercCheckMultipleDrivers( const CONNECTION_SUBGRAPH* aSubgraph );
 
+    bool ercCheckNetclassConflicts( const std::vector<CONNECTION_SUBGRAPH*>& subgraphs );
+
     /**
      * Checks one subgraph for conflicting connections between net and bus labels
      *
@@ -537,9 +564,13 @@ private:
      */
     int ercCheckHierSheets();
 
-public:
-    // TODO(JE) Remove this when pressure valve is removed
-    static bool m_allowRealTime;
+    /**
+     * Get the number of pins in a given subgraph
+     * @param aLocSubgraph Subgraph to search
+     * @return total number of pins in the subgraph
+     */
+    size_t hasPins( const CONNECTION_SUBGRAPH* aLocSubgraph );
+
 
 private:
     // All the sheets in the schematic (as long as we don't have partial updates)
@@ -559,20 +590,20 @@ private:
 
     std::vector<std::pair<SCH_SHEET_PATH, SCH_PIN*>> m_invisible_power_pins;
 
-    std::unordered_map< wxString, std::shared_ptr<BUS_ALIAS> > m_bus_alias_cache;
+    std::unordered_map<wxString, std::shared_ptr<BUS_ALIAS>> m_bus_alias_cache;
 
-    std::map<wxString, int> m_net_name_to_code_map;
+    std::unordered_map<wxString, int> m_net_name_to_code_map;
 
-    std::map<wxString, int> m_bus_name_to_code_map;
+    std::unordered_map<wxString, int> m_bus_name_to_code_map;
 
-    std::map<wxString, std::vector<const CONNECTION_SUBGRAPH*>> m_global_label_cache;
+    std::unordered_map<wxString, std::vector<const CONNECTION_SUBGRAPH*>> m_global_label_cache;
 
     std::map< std::pair<SCH_SHEET_PATH, wxString>,
               std::vector<const CONNECTION_SUBGRAPH*> > m_local_label_cache;
 
     std::unordered_map<wxString, std::vector<CONNECTION_SUBGRAPH*>> m_net_name_to_subgraphs_map;
 
-    std::map<SCH_ITEM*, CONNECTION_SUBGRAPH*> m_item_to_subgraph_map;
+    std::unordered_map<SCH_ITEM*, CONNECTION_SUBGRAPH*> m_item_to_subgraph_map;
 
     NET_MAP m_net_code_to_subgraphs_map;
 
