@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2015 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2012 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,7 +29,6 @@
 #include <schematic.h>
 #include <project.h>
 #include <kiface_base.h>
-#include <bitmaps.h>
 #include <reporter.h>
 #include <wildcards_and_files_ext.h>
 #include <sch_marker.h>
@@ -40,7 +39,7 @@
 #include <erc.h>
 #include <id.h>
 #include <confirm.h>
-#include <dialogs/wx_html_report_box.h>
+#include <widgets/wx_html_report_box.h>
 #include <wx/ffile.h>
 #include <wx/filedlg.h>
 #include <wx/hyperlink.h>
@@ -49,14 +48,29 @@
 #include <string_utils.h>
 #include <kiplatform/ui.h>
 
+// wxWidgets spends *far* too long calcuating column widths (most of it, believe it or
+// not, in repeatedly creating/destroying a wxDC to do the measurement in).
+// Use default column widths instead.
+static int DEFAULT_SINGLE_COL_WIDTH = 660;
+
+
+static SCHEMATIC*            g_lastERCSchematic = nullptr;
+static bool                  g_lastERCRun = false;
+static std::vector<wxString> g_lastERCIgnored;
+
+
 DIALOG_ERC::DIALOG_ERC( SCH_EDIT_FRAME* parent ) :
         DIALOG_ERC_BASE( parent ),
         PROGRESS_REPORTER_BASE( 1 ),
         m_parent( parent ),
+        m_markerTreeModel( nullptr ),
         m_running( false ),
         m_ercRun( false ),
-        m_severities( RPT_SEVERITY_ERROR | RPT_SEVERITY_WARNING )
+        m_centerMarkerOnIdle( nullptr ),
+        m_severities( 0 )
 {
+    m_currentSchematic = &parent->Schematic();
+
     SetName( DIALOG_ERC_WINDOW_NAME ); // Set a window name to be able to find it
 
     EESCHEMA_SETTINGS* settings = dynamic_cast<EESCHEMA_SETTINGS*>( Kiface().KifaceSettings() );
@@ -64,22 +78,29 @@ DIALOG_ERC::DIALOG_ERC( SCH_EDIT_FRAME* parent ) :
 
     m_messages->SetImmediateMode();
 
-    m_markerProvider = new SHEETLIST_ERC_ITEMS_PROVIDER( &m_parent->Schematic() );
-    m_markerTreeModel = new RC_TREE_MODEL( parent, m_markerDataView );
+    m_markerProvider = std::make_shared<SHEETLIST_ERC_ITEMS_PROVIDER>( &m_parent->Schematic() );
+
+    m_markerTreeModel = new ERC_TREE_MODEL( parent, m_markerDataView );
     m_markerDataView->AssociateModel( m_markerTreeModel );
+    m_markerTreeModel->Update( m_markerProvider, m_severities );
 
-    m_markerTreeModel->SetSeverities( m_severities );
-    m_markerTreeModel->SetProvider( m_markerProvider );
-    syncCheckboxes();
-    updateDisplayedCounts();
+    m_ignoredList->InsertColumn( 0, wxEmptyString, wxLIST_FORMAT_LEFT, DEFAULT_SINGLE_COL_WIDTH );
 
-    // We use a sdbSizer to get platform-dependent ordering of the action buttons, but
-    // that requires us to correct the button labels here.
-    m_sdbSizer1OK->SetLabel( _( "Run ERC" ) );
-    m_sdbSizer1Cancel->SetLabel( _( "Close" ) );
-    m_sdbSizer1->Layout();
+    if( m_currentSchematic == g_lastERCSchematic )
+    {
+        m_ercRun = g_lastERCRun;
 
-    m_sdbSizer1OK->SetDefault();
+        for( const wxString& str : g_lastERCIgnored )
+            m_ignoredList->InsertItem( m_ignoredList->GetItemCount(), str );
+    }
+
+    m_notebook->SetSelection( 0 );
+
+    SetupStandardButtons( { { wxID_OK,     _( "Run ERC" ) },
+                            { wxID_CANCEL, _( "Close" )   } } );
+
+    m_violationsTitleTemplate = m_notebook->GetPageText( 0 );
+    m_ignoredTitleTemplate    = m_notebook->GetPageText( 1 );
 
     m_errorsBadge->SetMaximumNumber( 999 );
     m_warningsBadge->SetMaximumNumber( 999 );
@@ -87,6 +108,13 @@ DIALOG_ERC::DIALOG_ERC( SCH_EDIT_FRAME* parent ) :
 
     UpdateAnnotationWarning();
 
+    Layout();
+
+    SetFocus();
+
+    syncCheckboxes();
+    updateDisplayedCounts();
+
     // Now all widgets have the size fixed, call FinishDialogSettings
     finishDialogSettings();
 }
@@ -94,6 +122,14 @@ DIALOG_ERC::DIALOG_ERC( SCH_EDIT_FRAME* parent ) :
 
 DIALOG_ERC::~DIALOG_ERC()
 {
+    g_lastERCSchematic = m_currentSchematic;
+    g_lastERCRun = m_ercRun;
+
+    g_lastERCIgnored.clear();
+
+    for( int ii = 0; ii < m_ignoredList->GetItemCount(); ++ii )
+        g_lastERCIgnored.push_back( m_ignoredList->GetItemText( ii ) );
+
     EESCHEMA_SETTINGS* settings = dynamic_cast<EESCHEMA_SETTINGS*>( Kiface().KifaceSettings() );
     wxASSERT( settings );
 
@@ -112,7 +148,7 @@ void DIALOG_ERC::UpdateAnnotationWarning()
         if( !m_infoBar->IsShown() )
         {
             wxHyperlinkCtrl* button = new wxHyperlinkCtrl( m_infoBar, wxID_ANY,
-                                                           _("Show Annotation dialog"),
+                                                           _( "Show Annotation dialog" ),
                                                            wxEmptyString );
 
             button->Bind( wxEVT_COMMAND_HYPERLINK, std::function<void( wxHyperlinkEvent& aEvent )>(
@@ -176,18 +212,54 @@ void DIALOG_ERC::updateDisplayedCounts()
     int numWarnings = 0;
     int numExcluded = 0;
 
+    int numMarkers = 0;
+
     if( m_markerProvider )
     {
+        numMarkers += m_markerProvider->GetCount();
         numErrors += m_markerProvider->GetCount( RPT_SEVERITY_ERROR );
         numWarnings += m_markerProvider->GetCount( RPT_SEVERITY_WARNING );
         numExcluded += m_markerProvider->GetCount( RPT_SEVERITY_EXCLUSION );
     }
 
-    if( !m_ercRun )
+    bool markersOverflowed = false;
+
+    // We don't currently have a limit on ERC violations, so the above is always false.
+
+    wxString num;
+    wxString msg;
+
+    if( m_ercRun )
     {
-        numErrors = -1;
-        numWarnings = -1;
+        num.Printf( markersOverflowed ? wxT( "%d+" ) : wxT( "%d" ), numMarkers );
+        msg.Printf( m_violationsTitleTemplate, num );
     }
+    else
+    {
+        msg = m_violationsTitleTemplate;
+        msg.Replace( wxT( "(%s)" ), wxEmptyString );
+    }
+
+    m_notebook->SetPageText( 0, msg );
+
+    if( m_ercRun )
+    {
+        num.Printf( wxT( "%d" ), m_ignoredList->GetItemCount() );
+        msg.sprintf( m_ignoredTitleTemplate, num );
+    }
+    else
+    {
+        msg = m_ignoredTitleTemplate;
+        msg.Replace( wxT( "(%s)" ), wxEmptyString );
+    }
+
+    m_notebook->SetPageText( 1, msg );
+
+    if( !m_ercRun && numErrors == 0 )
+        numErrors = -1;
+
+    if( !m_ercRun && numWarnings == 0 )
+        numWarnings = -1;
 
     m_errorsBadge->UpdateNumber( numErrors, RPT_SEVERITY_ERROR );
     m_warningsBadge->UpdateNumber( numWarnings, RPT_SEVERITY_WARNING );
@@ -195,9 +267,26 @@ void DIALOG_ERC::updateDisplayedCounts()
 }
 
 
+void DIALOG_ERC::OnDeleteOneClick( wxCommandEvent& aEvent )
+{
+    if( m_notebook->GetSelection() == 0 )
+    {
+        // Clear the selection.  It may be the selected ERC marker.
+        m_parent->GetToolManager()->RunAction( EE_ACTIONS::clearSelection, true );
+
+        m_markerTreeModel->DeleteCurrentItem( true );
+
+        // redraw the schematic
+        redrawDrawPanel();
+    }
+
+    updateDisplayedCounts();
+}
+
+
 /* Delete the old ERC markers, over the whole hierarchy
  */
-void DIALOG_ERC::OnEraseDrcMarkersClick( wxCommandEvent& event )
+void DIALOG_ERC::OnDeleteAllClick( wxCommandEvent& event )
 {
     bool includeExclusions = false;
     int  numExcluded = 0;
@@ -209,7 +298,8 @@ void DIALOG_ERC::OnEraseDrcMarkersClick( wxCommandEvent& event )
     {
         wxMessageDialog dlg( this, _( "Delete exclusions too?" ), _( "Delete All Markers" ),
                              wxYES_NO | wxCANCEL | wxCENTER | wxICON_QUESTION );
-        dlg.SetYesNoLabels( _( "Errors and Warnings Only" ) , _( "Errors, Warnings and Exclusions" ) );
+        dlg.SetYesNoLabels( _( "Errors and Warnings Only" ),
+                            _( "Errors, Warnings and Exclusions" ) );
 
         int ret = dlg.ShowModal();
 
@@ -221,9 +311,9 @@ void DIALOG_ERC::OnEraseDrcMarkersClick( wxCommandEvent& event )
 
     deleteAllMarkers( includeExclusions );
 
-    m_ercRun = false;
+    // redraw the schematic
+    redrawDrawPanel();
     updateDisplayedCounts();
-    m_parent->GetCanvas()->Refresh();
 }
 
 
@@ -280,14 +370,29 @@ void DIALOG_ERC::OnRunERCClick( wxCommandEvent& event )
     m_parent->RecordERCExclusions();
     deleteAllMarkers( true );
 
-    m_notebook->ChangeSelection( 0 );   // Display the "Tests Running..." tab
+    std::vector<std::reference_wrapper<RC_ITEM>> violations = ERC_ITEM::GetItemsWithSeverities();
+    m_ignoredList->DeleteAllItems();
+
+    for( std::reference_wrapper<RC_ITEM>& item : violations )
+    {
+        if( sch->ErcSettings().GetSeverity( item.get().GetErrorCode() ) == RPT_SEVERITY_IGNORE )
+        {
+            m_ignoredList->InsertItem( m_ignoredList->GetItemCount(),
+                                       wxT( " • " ) + item.get().GetErrorText() );
+        }
+    }
+
+    Raise();
+
+    m_runningResultsBook->ChangeSelection( 0 );   // Display the "Tests Running..." tab
     m_messages->Clear();
-    wxYield();                          // Allow time slice to refresh Messages
+    wxYield();                                    // Allow time slice to refresh Messages
 
     m_running = true;
     m_sdbSizer1Cancel->SetLabel( _( "Cancel" ) );
     m_sdbSizer1OK->Enable( false );
-    m_buttondelmarkers->Enable( false );
+    m_deleteOneMarker->Enable( false );
+    m_deleteAllMarkers->Enable( false );
     m_saveReport->Enable( false );
 
     sch->GetSheets().AnnotatePowerSymbols();
@@ -316,7 +421,7 @@ void DIALOG_ERC::OnRunERCClick( wxCommandEvent& event )
     if( m_cancelled )
         m_messages->Report( _( "-------- ERC cancelled by user.<br><br>" ), RPT_SEVERITY_INFO );
     else
-        m_messages->Report( _( "ERC completed.<br><br>" ), RPT_SEVERITY_INFO );
+        m_messages->Report( _( "Done.<br><br>" ), RPT_SEVERITY_INFO );
 
     Raise();
     wxYield();                                    // Allow time slice to refresh Messages
@@ -324,17 +429,19 @@ void DIALOG_ERC::OnRunERCClick( wxCommandEvent& event )
     m_running = false;
     m_sdbSizer1Cancel->SetLabel( _( "Close" ) );
     m_sdbSizer1OK->Enable( true );
-    m_buttondelmarkers->Enable( true );
+    m_deleteOneMarker->Enable( true );
+    m_deleteAllMarkers->Enable( true );
     m_saveReport->Enable( true );
 
     if( !m_cancelled )
     {
         wxMilliSleep( 500 );
-        m_notebook->ChangeSelection( 1 );
+        m_runningResultsBook->ChangeSelection( 1 );
         KIPLATFORM::UI::ForceFocus( m_markerDataView );
     }
 
     m_ercRun = true;
+    redrawDrawPanel();
     updateDisplayedCounts();
 }
 
@@ -379,6 +486,8 @@ void DIALOG_ERC::testErc()
     m_parent->RecalculateConnections( NO_CLEANUP );
     sch->ConnectionGraph()->RunERC();
 
+    AdvancePhase( _( "Checking units..." ) );
+
     // Test is all units of each multiunit symbol have the same footprint assigned.
     if( settings.IsTestEnabled( ERCE_DIFFERENT_UNIT_FP ) )
     {
@@ -386,6 +495,14 @@ void DIALOG_ERC::testErc()
         tester.TestMultiunitFootprints();
     }
 
+    if( settings.IsTestEnabled( ERCE_MISSING_UNIT )
+            || settings.IsTestEnabled( ERCE_MISSING_INPUT_PIN )
+            || settings.IsTestEnabled( ERCE_MISSING_POWER_INPUT_PIN )
+            || settings.IsTestEnabled( ERCE_MISSING_BIDI_PIN ) )
+    {
+        tester.TestMissingUnits();
+    }
+
     AdvancePhase( _( "Checking pins..." ) );
 
     if( settings.IsTestEnabled( ERCE_DIFFERENT_UNIT_NET ) )
@@ -396,7 +513,7 @@ void DIALOG_ERC::testErc()
             || settings.IsTestEnabled( ERCE_POWERPIN_NOT_DRIVEN )
             || settings.IsTestEnabled( ERCE_PIN_NOT_DRIVEN ) )
     {
-        tester.TestPinToPin();
+         tester.TestPinToPin();
     }
 
     // Test similar labels (i;e. labels which are identical when
@@ -413,6 +530,12 @@ void DIALOG_ERC::testErc()
         tester.TestTextVars( m_parent->GetCanvas()->GetView()->GetDrawingSheet() );
     }
 
+    if( settings.IsTestEnabled( ERCE_SIMULATION_MODEL ) )
+    {
+        AdvancePhase( _( "Checking SPICE models..." ) );
+        tester.TestSimModelIssues();
+    }
+
     if( settings.IsTestEnabled( ERCE_NOCONNECT_CONNECTED ) )
     {
         AdvancePhase( _( "Checking no connect pins for connections..." ) );
@@ -425,10 +548,16 @@ void DIALOG_ERC::testErc()
         tester.TestLibSymbolIssues();
     }
 
+    if( settings.IsTestEnabled( ERCE_ENDPOINT_OFF_GRID ) )
+    {
+        AdvancePhase( _( "Checking for off grid pins and wires..." ) );
+        tester.TestOffGridEndpoints( m_parent->GetCanvas()->GetView()->GetGAL()->GetGridSize().x );
+    }
+
     m_parent->ResolveERCExclusions();
 
-    // Display diags:
-    m_markerTreeModel->SetProvider( m_markerProvider );
+    // Update marker list:
+    m_markerTreeModel->Update( m_markerProvider, m_severities );
 
     // Display new markers from the current screen:
     for( SCH_ITEM* marker : m_parent->GetScreen()->Items().OfType( SCH_MARKER_T ) )
@@ -447,8 +576,39 @@ void DIALOG_ERC::OnERCItemSelected( wxDataViewEvent& aEvent )
     SCH_SHEET_PATH  sheet;
     SCH_ITEM*       item = m_parent->Schematic().GetSheets().GetItem( itemID, &sheet );
 
-    if( item && item->GetClass() != wxT( "DELETED_SHEET_ITEM" ) )
+    if( m_centerMarkerOnIdle )
     {
+        // we already came from a cross-probe of the marker in the document; don't go
+        // around in circles
+    }
+    else if( item && item->GetClass() != wxT( "DELETED_SHEET_ITEM" ) )
+    {
+        const RC_TREE_NODE* node = RC_TREE_MODEL::ToNode( aEvent.GetItem() );
+
+        if( node )
+        {
+            // Determine the owning sheet for sheet-specific items
+            std::shared_ptr<ERC_ITEM> ercItem =
+                    std::static_pointer_cast<ERC_ITEM>( node->m_RcItem );
+            switch( node->m_Type )
+            {
+            case RC_TREE_NODE::MARKER:
+                if( ercItem->IsSheetSpecific() )
+                    sheet = ercItem->GetSpecificSheetPath();
+                break;
+            case RC_TREE_NODE::MAIN_ITEM:
+                if( ercItem->MainItemHasSheetPath() )
+                    sheet = ercItem->GetMainItemSheetPath();
+                break;
+            case RC_TREE_NODE::AUX_ITEM:
+                if( ercItem->AuxItemHasSheetPath() )
+                    sheet = ercItem->GetAuxItemSheetPath();
+                break;
+            default:
+                break;
+            }
+        }
+
         WINDOW_THAWER thawer( m_parent );
 
         if( !sheet.empty() && sheet != m_parent->GetCurrentSheet() )
@@ -495,7 +655,6 @@ void DIALOG_ERC::OnERCItemRClick( wxDataViewEvent& aEvent )
     std::shared_ptr<RC_ITEM>  rcItem = node->m_RcItem;
     wxString  listName;
     wxMenu    menu;
-    wxString  msg;
 
     switch( settings.GetSeverity( rcItem->GetErrorCode() ) )
     {
@@ -603,7 +762,7 @@ void DIALOG_ERC::OnERCItemRClick( wxDataViewEvent& aEvent )
         }
 
         // Rebuild model and view
-        static_cast<RC_TREE_MODEL*>( aEvent.GetModel() )->SetProvider( m_markerProvider );
+        static_cast<RC_TREE_MODEL*>( aEvent.GetModel() )->Update( m_markerProvider, m_severities );
         modified = true;
         break;
 
@@ -619,7 +778,7 @@ void DIALOG_ERC::OnERCItemRClick( wxDataViewEvent& aEvent )
         }
 
         // Rebuild model and view
-        static_cast<RC_TREE_MODEL*>( aEvent.GetModel() )->SetProvider( m_markerProvider );
+        static_cast<RC_TREE_MODEL*>( aEvent.GetModel() )->Update( m_markerProvider, m_severities );
         modified = true;
         break;
 
@@ -634,7 +793,7 @@ void DIALOG_ERC::OnERCItemRClick( wxDataViewEvent& aEvent )
         ScreenList.DeleteMarkers( MARKER_BASE::MARKER_ERC, rcItem->GetErrorCode() );
 
         // Rebuild model and view
-        static_cast<RC_TREE_MODEL*>( aEvent.GetModel() )->SetProvider( m_markerProvider );
+        static_cast<RC_TREE_MODEL*>( aEvent.GetModel() )->Update( m_markerProvider, m_severities );
         modified = true;
     }
         break;
@@ -659,19 +818,48 @@ void DIALOG_ERC::OnERCItemRClick( wxDataViewEvent& aEvent )
 
 void DIALOG_ERC::PrevMarker()
 {
-    if( m_notebook->GetSelection() != 1 )
-        m_notebook->SetSelection( 1 );
+    if( m_notebook->IsShown() )
+    {
+        if( m_notebook->GetSelection() != 0 )
+            m_notebook->SetSelection( 0 );
 
-    m_markerTreeModel->PrevMarker();
+        m_markerTreeModel->PrevMarker();
+    }
 }
 
 
 void DIALOG_ERC::NextMarker()
 {
-    if( m_notebook->GetSelection() != 1 )
-        m_notebook->SetSelection( 1 );
+    if( m_notebook->IsShown() )
+    {
+        if( m_notebook->GetSelection() != 0 )
+            m_notebook->SetSelection( 0 );
 
-    m_markerTreeModel->NextMarker();
+        m_markerTreeModel->NextMarker();
+    }
+}
+
+
+void DIALOG_ERC::SelectMarker( const SCH_MARKER* aMarker )
+{
+    if( m_notebook->IsShown() )
+    {
+        m_notebook->SetSelection( 0 );
+        m_markerTreeModel->SelectMarker( aMarker );
+
+        // wxWidgets on some platforms fails to correctly ensure that a selected item is
+        // visible, so we have to do it in a separate idle event.
+        m_centerMarkerOnIdle = aMarker;
+        Bind( wxEVT_IDLE, &DIALOG_ERC::centerMarkerIdleHandler, this );
+    }
+}
+
+
+void DIALOG_ERC::centerMarkerIdleHandler( wxIdleEvent& aEvent )
+{
+    m_markerTreeModel->CenterMarker( m_centerMarkerOnIdle );
+    m_centerMarkerOnIdle = nullptr;
+    Unbind( wxEVT_IDLE, &DIALOG_ERC::centerMarkerIdleHandler, this );
 }
 
 
@@ -682,7 +870,7 @@ void DIALOG_ERC::ExcludeMarker( SCH_MARKER* aMarker )
     if( marker != nullptr )
         m_markerTreeModel->SelectMarker( marker );
 
-    if( m_notebook->GetSelection() != 1 )
+    if( m_notebook->GetSelection() != 0 )
         return;
 
     RC_TREE_NODE* node = RC_TREE_MODEL::ToNode( m_markerDataView->GetCurrentItem() );
@@ -708,6 +896,21 @@ void DIALOG_ERC::ExcludeMarker( SCH_MARKER* aMarker )
 }
 
 
+void DIALOG_ERC::OnIgnoreItemRClick( wxListEvent& event )
+{
+    wxMenu menu;
+
+    menu.Append( 1, _( "Edit ignored violations..." ), _( "Open the Schematic Setup... dialog" ) );
+
+    switch( GetPopupMenuSelectionFromUser( menu ) )
+    {
+    case 1:
+        m_parent->ShowSchematicSetupDialog( _( "Violation Severity" ) );
+        break;
+    }
+}
+
+
 void DIALOG_ERC::OnSeverity( wxCommandEvent& aEvent )
 {
     int flag = 0;
@@ -730,14 +933,7 @@ void DIALOG_ERC::OnSeverity( wxCommandEvent& aEvent )
 
     syncCheckboxes();
 
-    // Set the provider's severity levels through the TreeModel so that the old tree
-    // can be torn down before the severity changes.
-    //
-    // It's not clear this is required, but we've had a lot of issues with wxDataView
-    // being cranky on various platforms.
-
-    m_markerTreeModel->SetSeverities( m_severities );
-
+    m_markerTreeModel->Update( m_markerProvider, m_severities );
     updateDisplayedCounts();
 }
 
@@ -747,13 +943,16 @@ void DIALOG_ERC::deleteAllMarkers( bool aIncludeExclusions )
     // Clear current selection list to avoid selection of deleted items
     m_parent->GetToolManager()->RunAction( EE_ACTIONS::clearSelection, true );
 
-    m_markerTreeModel->DeleteItems( false, aIncludeExclusions, true );
+    m_markerTreeModel->DeleteItems( false, aIncludeExclusions, false );
+
+    SCH_SCREENS screens( m_parent->Schematic().Root() );
+    screens.DeleteAllMarkers( MARKER_BASE::MARKER_ERC, aIncludeExclusions );
 }
 
 
 void DIALOG_ERC::OnSaveReport( wxCommandEvent& aEvent )
 {
-    wxFileName fn( "ERC." + ReportFileExtension );
+    wxFileName fn( wxS( "ERC." ) + ReportFileExtension );
 
     wxFileDialog dlg( this, _( "Save Report to File" ), Prj().GetProjectPath(), fn.GetFullName(),
                       ReportFileWildcard(), wxFD_SAVE | wxFD_OVERWRITE_PROMPT );
@@ -827,7 +1026,7 @@ bool DIALOG_ERC::writeReport( const wxString& aFullFileName )
             default:                                 break;
             }
 
-            msg << marker->GetRCItem()->ShowReport( GetUserUnits(), severity, itemMap );
+            msg << marker->GetRCItem()->ShowReport( m_parent, severity, itemMap );
         }
     }
 
