@@ -74,26 +74,19 @@ DIALOG_SIM_MODEL<T_symbol, T_field>::DIALOG_SIM_MODEL( wxWindow* aParent, T_symb
                    return StrNumCmp( lhs->GetNumber(), rhs->GetNumber(), true ) < 0;
                } );
 
-
     m_typeChoice->Clear();
 
     m_scintillaTricks = new SCINTILLA_TRICKS( m_codePreview, wxT( "{}" ), false );
 
     m_paramGridMgr->Bind( wxEVT_PG_SELECTED, &DIALOG_SIM_MODEL::onParamGridSelectionChange, this );
 
-    m_paramGrid->SetValidationFailureBehavior( wxPG_VFB_STAY_IN_PROPERTY
-                                               | wxPG_VFB_BEEP
-                                               | wxPG_VFB_MARK_CELL );
-
     wxPropertyGrid* grid = m_paramGrid->GetGrid();
 
-    //grid->SetCellBackgroundColour( grid->GetPropertyDefaultCell().GetBgCol() );
-    //grid->SetCellTextColour( grid->GetPropertyDefaultCell().GetFgCol();
-
     // In wx 3.0 the color will be wrong sometimes.
     grid->SetCellDisabledTextColour( wxSystemSettings::GetColour( wxSYS_COLOUR_GRAYTEXT ) );
 
     grid->Bind( wxEVT_SET_FOCUS, &DIALOG_SIM_MODEL::onParamGridSetFocus, this );
+    grid->Bind( wxEVT_UPDATE_UI, &DIALOG_SIM_MODEL::onUpdateUI, this );
 
     grid->AddActionTrigger( wxPG_ACTION_EDIT, WXK_RETURN );
     grid->DedicateKey( WXK_RETURN );
@@ -169,6 +162,14 @@ bool DIALOG_SIM_MODEL<T_symbol, T_field>::TransferDataToWindow()
         m_fields[ VALUE_FIELD ].SetText( wxT( "${SIM.PARAMS}" ) );
     }
 
+    std::vector<LIB_PIN*> sourcePins = m_symbol.GetAllLibPins();
+
+    std::sort( sourcePins.begin(), sourcePins.end(),
+               []( const LIB_PIN* lhs, const LIB_PIN* rhs )
+               {
+                   return StrNumCmp( lhs->GetNumber(), rhs->GetNumber(), true ) < 0;
+               } );
+
     std::string libraryFilename = SIM_MODEL::GetFieldValue( &m_fields, SIM_LIBRARY::LIBRARY_FIELD );
 
     if( libraryFilename != "" )
@@ -179,8 +180,11 @@ bool DIALOG_SIM_MODEL<T_symbol, T_field>::TransferDataToWindow()
         if( !loadLibrary( libraryFilename ) )
         {
             m_libraryPathText->ChangeValue( libraryFilename );
-            m_modelNameChoice->SetSelection( -1 );
             m_curModelType = SIM_MODEL::ReadTypeFromFields( m_fields );
+            m_libraryModelsMgr.CreateModel( nullptr, sourcePins, m_fields );
+
+            m_modelNameChoice->Append( _( "<unknown>" ) );
+            m_modelNameChoice->SetSelection( 0 );
         }
         else
         {
@@ -255,14 +259,6 @@ bool DIALOG_SIM_MODEL<T_symbol, T_field>::TransferDataToWindow()
         m_curModelType = SIM_MODEL::ReadTypeFromFields( m_fields );
     }
 
-    std::vector<LIB_PIN*> sourcePins = m_symbol.GetAllLibPins();
-
-    std::sort( sourcePins.begin(), sourcePins.end(),
-               []( const LIB_PIN* lhs, const LIB_PIN* rhs )
-               {
-                   return StrNumCmp( lhs->GetNumber(), rhs->GetNumber(), true ) < 0;
-               } );
-
     for( SIM_MODEL::TYPE type : SIM_MODEL::TYPE_ITERATOR() )
     {
         wxString           msg;
@@ -302,33 +298,28 @@ bool DIALOG_SIM_MODEL<T_symbol, T_field>::TransferDataFromWindow()
 {
     m_pinAssignmentsGrid->CommitPendingChanges();
 
-    // This should have been done in wxPGTextCtrlEditor::OnTextCtrlEvent(), but something must
-    // be clearing it before we get here, resulting in CommitChangesFromEditor() doing nothing
-    m_paramGrid->GetGrid()->EditorsValueWasModified();
-    m_paramGrid->GetGrid()->CommitChangesFromEditor();
-
     if( !DIALOG_SIM_MODEL_BASE::TransferDataFromWindow() )
         return false;
 
-    if( !m_modelNameChoice->IsEmpty() )
-    {
-        std::string modelName = m_modelNameChoice->GetStringSelection().ToStdString();
-
-        SIM_MODEL::SetFieldValue( m_fields, SIM_LIBRARY::NAME_FIELD, modelName );
-    }
-
     std::string path;
+    std::string name;
 
-    if( m_useLibraryModelRadioButton->GetValue() || isIbisLoaded() )
+    if( m_useLibraryModelRadioButton->GetValue() )
     {
         path = m_libraryPathText->GetValue();
         wxFileName fn( path );
 
         if( fn.MakeRelativeTo( Prj().GetProjectPath() ) && !fn.GetFullPath().StartsWith( ".." ) )
             path = fn.GetFullPath();
+
+        if( !m_modelNameChoice->IsEmpty() )
+            name = m_modelNameChoice->GetStringSelection().ToStdString();
+        else if( dynamic_cast<SIM_MODEL_SPICE_FALLBACK*>( &curModel() ) )
+            name = SIM_MODEL::GetFieldValue( &m_fields, SIM_LIBRARY::NAME_FIELD, false );
     }
 
     SIM_MODEL::SetFieldValue( m_fields, SIM_LIBRARY::LIBRARY_FIELD, path );
+    SIM_MODEL::SetFieldValue( m_fields, SIM_LIBRARY::NAME_FIELD, name );
 
     if( isIbisLoaded() )
     {
@@ -337,17 +328,19 @@ bool DIALOG_SIM_MODEL<T_symbol, T_field>::TransferDataFromWindow()
 
         if( ibismodel )
         {
-            SIM_MODEL::SetFieldValue(
-                    m_fields, SIM_LIBRARY_KIBIS::PIN_FIELD,
-                    ibismodel->GetIbisPins().at( m_ibisPinCombobox->GetSelection() ).first );
+            std::string pins;
+            std::string modelName = std::string( m_ibisModelCombobox->GetValue().c_str() );
+            std::string differential;
 
-            SIM_MODEL::SetFieldValue(
-                    m_fields, SIM_LIBRARY_KIBIS::MODEL_FIELD,
-                    std::string( m_ibisModelCombobox->GetValue().c_str() ) );
+            if( m_ibisPinCombobox->GetSelection() >= 0 )
+                pins = ibismodel->GetIbisPins().at( m_ibisPinCombobox->GetSelection() ).first;
 
-            SIM_MODEL::SetFieldValue(
-                    m_fields, SIM_LIBRARY_KIBIS::DIFF_FIELD,
-                    ibismodel->CanDifferential() && m_differentialCheckbox->GetValue() ? "1" : "" );
+            if( ibismodel->CanDifferential() && m_differentialCheckbox->GetValue() )
+                differential = "1";
+
+            SIM_MODEL::SetFieldValue( m_fields, SIM_LIBRARY_KIBIS::PIN_FIELD, pins );
+            SIM_MODEL::SetFieldValue( m_fields, SIM_LIBRARY_KIBIS::MODEL_FIELD, modelName );
+            SIM_MODEL::SetFieldValue( m_fields, SIM_LIBRARY_KIBIS::DIFF_FIELD, differential );
         }
     }
 
@@ -368,11 +361,13 @@ bool DIALOG_SIM_MODEL<T_symbol, T_field>::TransferDataFromWindow()
 template <typename T_symbol, typename T_field>
 void DIALOG_SIM_MODEL<T_symbol, T_field>::updateWidgets()
 {
-    updateIbisWidgets();
-    updateInstanceWidgets();
-    updateModelParamsTab();
-    updateModelCodeTab();
-    updatePinAssignments();
+    SIM_MODEL* model = &curModel();
+
+    updateIbisWidgets( model );
+    updateInstanceWidgets( model );
+    updateModelParamsTab( model );
+    updateModelCodeTab( model );
+    updatePinAssignments( model );
 
     std::string ref = SIM_MODEL::GetFieldValue( &m_fields, SIM_REFERENCE_FIELD );
 
@@ -388,9 +383,9 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::updateWidgets()
 
 
 template <typename T_symbol, typename T_field>
-void DIALOG_SIM_MODEL<T_symbol, T_field>::updateIbisWidgets()
+void DIALOG_SIM_MODEL<T_symbol, T_field>::updateIbisWidgets( SIM_MODEL* aModel )
 {
-    SIM_MODEL_KIBIS* modelkibis = isIbisLoaded() ? dynamic_cast<SIM_MODEL_KIBIS*>( &curModel() )
+    SIM_MODEL_KIBIS* modelkibis = isIbisLoaded() ? dynamic_cast<SIM_MODEL_KIBIS*>( aModel )
                                                  : nullptr;
 
     m_ibisModelCombobox->Show( isIbisLoaded() );
@@ -404,16 +399,16 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::updateIbisWidgets()
 
 
 template <typename T_symbol, typename T_field>
-void DIALOG_SIM_MODEL<T_symbol, T_field>::updateInstanceWidgets()
+void DIALOG_SIM_MODEL<T_symbol, T_field>::updateInstanceWidgets( SIM_MODEL* aModel )
 {
     // Change the Type choice to match the current device type.
-    if( &curModel() != m_prevModel )
+    if( aModel != m_prevModel )
     {
         m_deviceTypeChoice->Clear();
 
         if( m_useLibraryModelRadioButton->GetValue() )
         {
-            m_deviceTypeChoice->Append( curModel().GetDeviceInfo().description );
+            m_deviceTypeChoice->Append( aModel->GetDeviceInfo().description );
             m_deviceTypeChoice->SetSelection( 0 );
         }
         else
@@ -425,7 +420,7 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::updateInstanceWidgets()
 
                 m_deviceTypeChoice->Append( SIM_MODEL::DeviceInfo( deviceType ).description );
 
-                if( deviceType == curModel().GetDeviceType() )
+                if( deviceType == aModel->GetDeviceType() )
                     m_deviceTypeChoice->SetSelection( m_deviceTypeChoice->GetCount() - 1 );
             }
         }
@@ -434,11 +429,11 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::updateInstanceWidgets()
 
         for( SIM_MODEL::TYPE type : SIM_MODEL::TYPE_ITERATOR() )
         {
-            if( SIM_MODEL::TypeInfo( type ).deviceType == curModel().GetDeviceType() )
+            if( SIM_MODEL::TypeInfo( type ).deviceType == aModel->GetDeviceType() )
             {
                 m_typeChoice->Append( SIM_MODEL::TypeInfo( type ).description );
 
-                if( type == curModel().GetType() )
+                if( type == aModel->GetType() )
                     m_typeChoice->SetSelection( m_typeChoice->GetCount() - 1 );
             }
         }
@@ -446,19 +441,14 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::updateInstanceWidgets()
 
     m_typeChoice->Enable( !m_useLibraryModelRadioButton->GetValue() || isIbisLoaded() );
 
-    if( dynamic_cast<SIM_MODEL_RAW_SPICE*>( &curModel() )
-            || dynamic_cast<SIM_MODEL_SPICE_FALLBACK*>( &curModel() ) )
-    {
+    if( dynamic_cast<SIM_MODEL_RAW_SPICE*>( aModel ) )
         m_modelNotebook->SetSelection( 1 );
-    }
     else
-    {
         m_modelNotebook->SetSelection( 0 );
-    }
 
-    if( curModel().HasPrimaryValue() )
+    if( aModel->HasPrimaryValue() )
     {
-        const SIM_MODEL::PARAM& primary = curModel().GetParam( 0 );
+        const SIM_MODEL::PARAM& primary = aModel->GetParam( 0 );
 
         m_saveInValueCheckbox->SetLabel( wxString::Format( _( "Save parameter '%s (%s)' in Value "
                                                               "field" ),
@@ -476,9 +466,9 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::updateInstanceWidgets()
 
 
 template <typename T_symbol, typename T_field>
-void DIALOG_SIM_MODEL<T_symbol, T_field>::updateModelParamsTab()
+void DIALOG_SIM_MODEL<T_symbol, T_field>::updateModelParamsTab( SIM_MODEL* aModel )
 {
-    if( &curModel() != m_prevModel )
+    if( aModel != m_prevModel )
     {
         // This wxPropertyGridManager column and header stuff has to be here because it segfaults in
         // the constructor.
@@ -530,8 +520,8 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::updateModelParamsTab()
 
         m_paramGrid->CollapseAll();
 
-        for( int i = 0; i < curModel().GetParamCount(); ++i )
-            addParamPropertyIfRelevant( i );
+        for( int i = 0; i < aModel->GetParamCount(); ++i )
+            addParamPropertyIfRelevant( aModel, i );
 
         m_paramGrid->CollapseAll();
         m_paramGrid->Expand( "AC" );
@@ -563,15 +553,18 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::updateModelParamsTab()
         // Model values other than the currently edited value may have changed. Update them.
         // This feature is called "autofill" and present only in certain models. Don't do it for
         // models that don't have it for performance reasons.
-        if( curModel().HasAutofill() )
-            ( *it )->SetValueFromString( param.value->ToString() );
+        if( aModel->HasAutofill() )
+            ( *it )->SetValueFromString( param.value );
     }
 }
 
 
 template <typename T_symbol, typename T_field>
-void DIALOG_SIM_MODEL<T_symbol, T_field>::updateModelCodeTab()
+void DIALOG_SIM_MODEL<T_symbol, T_field>::updateModelCodeTab( SIM_MODEL* aModel )
 {
+    if( dynamic_cast<SIM_MODEL_SPICE_FALLBACK*>( aModel ) )
+        return;
+
     wxString   text;
     SPICE_ITEM item;
 
@@ -580,7 +573,7 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::updateModelCodeTab()
     if( m_useInstanceModelRadioButton->GetValue() || item.modelName == "" )
         item.modelName = m_fields.at( REFERENCE_FIELD ).GetText();
 
-    text << curModel().SpiceGenerator().Preview( item );
+    text << aModel->SpiceGenerator().Preview( item );
 
     m_codePreview->SetText( text );
     m_codePreview->SelectNone();
@@ -588,9 +581,9 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::updateModelCodeTab()
 
 
 template <typename T_symbol, typename T_field>
-void DIALOG_SIM_MODEL<T_symbol, T_field>::updatePinAssignments()
+void DIALOG_SIM_MODEL<T_symbol, T_field>::updatePinAssignments( SIM_MODEL* aModel )
 {
-    removeOrphanedPinAssignments();
+    removeOrphanedPinAssignments( aModel );
 
     // Reset the grid.
 
@@ -601,9 +594,9 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::updatePinAssignments()
         m_pinAssignmentsGrid->SetCellValue( row, PIN_COLUMN::MODEL, _( "Not Connected" ) );
 
     // Now set up the grid values in the Model column.
-    for( int modelPinIndex = 0; modelPinIndex < curModel().GetPinCount(); ++modelPinIndex )
+    for( int modelPinIndex = 0; modelPinIndex < aModel->GetPinCount(); ++modelPinIndex )
     {
-        wxString symbolPinNumber = curModel().GetPin( modelPinIndex ).symbolPinNumber;
+        wxString symbolPinNumber = aModel->GetPin( modelPinIndex ).symbolPinNumber;
 
         if( symbolPinNumber == "" )
             continue;
@@ -613,7 +606,7 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::updatePinAssignments()
         if( symbolPinRow == -1 )
             continue;
 
-        wxString modelPinString = getModelPinString( modelPinIndex );
+        wxString modelPinString = getModelPinString( aModel, modelPinIndex );
         m_pinAssignmentsGrid->SetCellValue( symbolPinRow, PIN_COLUMN::MODEL, modelPinString );
     }
 
@@ -630,14 +623,14 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::updatePinAssignments()
         std::vector<BITMAPS> modelPinIcons;
         wxArrayString        modelPinChoices;
 
-        for( int jj = 0; jj < curModel().GetPinCount(); ++jj )
+        for( int jj = 0; jj < aModel->GetPinCount(); ++jj )
         {
-            if( curModel().GetPin( jj ).symbolPinNumber != "" )
+            if( aModel->GetPin( jj ).symbolPinNumber != "" )
                 modelPinIcons.push_back( PinShapeGetBitmap( GRAPHIC_PINSHAPE::LINE ) );
             else
                 modelPinIcons.push_back( BITMAPS::INVALID_BITMAP );
 
-            modelPinChoices.Add( getModelPinString( jj ) );
+            modelPinChoices.Add( getModelPinString( aModel, jj ) );
         }
 
         modelPinIcons.push_back( BITMAPS::INVALID_BITMAP );
@@ -652,11 +645,9 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::updatePinAssignments()
 
     // TODO: Show a preview of the symbol with the pin numbers shown.
 
-    SIM_MODEL* model = &curModel();
-
-    if( model->GetType() == SIM_MODEL::TYPE::SUBCKT )
+    if( aModel->GetType() == SIM_MODEL::TYPE::SUBCKT )
     {
-        SIM_MODEL_SUBCKT* subckt = static_cast<SIM_MODEL_SUBCKT*>( model );
+        SIM_MODEL_SUBCKT* subckt = static_cast<SIM_MODEL_SUBCKT*>( aModel );
         m_subckt->SetText( subckt->GetSpiceCode() );
     }
     else
@@ -668,12 +659,12 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::updatePinAssignments()
 
 
 template <typename T_symbol, typename T_field>
-void DIALOG_SIM_MODEL<T_symbol, T_field>::removeOrphanedPinAssignments()
+void DIALOG_SIM_MODEL<T_symbol, T_field>::removeOrphanedPinAssignments( SIM_MODEL* aModel )
 {
-    for( int i = 0; i < curModel().GetPinCount(); ++i )
+    for( int i = 0; i < aModel->GetPinCount(); ++i )
     {
-        if( !m_symbol.GetPin( curModel().GetPin( i ).symbolPinNumber ) )
-            curModel().SetPinSymbolPinNumber( i, "" );
+        if( !m_symbol.GetPin( aModel->GetPin( i ).symbolPinNumber ) )
+            aModel->SetPinSymbolPinNumber( i, "" );
     }
 }
 
@@ -682,6 +673,9 @@ template <typename T_symbol, typename T_field>
 bool DIALOG_SIM_MODEL<T_symbol, T_field>::loadLibrary( const wxString& aLibraryPath,
                                                        bool aForceReload )
 {
+    if( m_prevLibrary == aLibraryPath && !aForceReload )
+        return true;
+
     wxString           msg;
     WX_STRING_REPORTER reporter( &msg );
 
@@ -735,75 +729,77 @@ bool DIALOG_SIM_MODEL<T_symbol, T_field>::loadLibrary( const wxString& aLibraryP
         m_ibisPinCombobox->SetSelection( -1 );
     }
 
+    m_prevLibrary = aLibraryPath;
     return true;
 }
 
 
 template <typename T_symbol, typename T_field>
-void DIALOG_SIM_MODEL<T_symbol, T_field>::addParamPropertyIfRelevant( int aParamIndex )
+void DIALOG_SIM_MODEL<T_symbol, T_field>::addParamPropertyIfRelevant( SIM_MODEL* aModel,
+                                                                      int aParamIndex )
 {
-    if( curModel().GetParam( aParamIndex ).info.dir == SIM_MODEL::PARAM::DIR_OUT )
+    if( aModel->GetParam( aParamIndex ).info.dir == SIM_MODEL::PARAM::DIR_OUT )
         return;
 
-    switch( curModel().GetParam( aParamIndex ).info.category )
+    switch( aModel->GetParam( aParamIndex ).info.category )
     {
     case CATEGORY::AC:
         m_paramGrid->HideProperty( "AC", false );
-        m_paramGrid->AppendIn( "AC", newParamProperty( aParamIndex ) );
+        m_paramGrid->AppendIn( "AC", newParamProperty( aModel, aParamIndex ) );
         break;
 
     case CATEGORY::DC:
         m_paramGrid->HideProperty( "DC", false );
-        m_paramGrid->AppendIn( "DC", newParamProperty( aParamIndex ) );
+        m_paramGrid->AppendIn( "DC", newParamProperty( aModel, aParamIndex ) );
         break;
 
     case CATEGORY::CAPACITANCE:
         m_paramGrid->HideProperty( "Capacitance", false );
-        m_paramGrid->AppendIn( "Capacitance", newParamProperty( aParamIndex ) );
+        m_paramGrid->AppendIn( "Capacitance", newParamProperty( aModel, aParamIndex ) );
         break;
 
     case CATEGORY::TEMPERATURE:
         m_paramGrid->HideProperty( "Temperature", false );
-        m_paramGrid->AppendIn( "Temperature", newParamProperty( aParamIndex ) );
+        m_paramGrid->AppendIn( "Temperature", newParamProperty( aModel, aParamIndex ) );
         break;
 
     case CATEGORY::NOISE:
         m_paramGrid->HideProperty( "Noise", false );
-        m_paramGrid->AppendIn( "Noise", newParamProperty( aParamIndex ) );
+        m_paramGrid->AppendIn( "Noise", newParamProperty( aModel, aParamIndex ) );
         break;
 
     case CATEGORY::DISTRIBUTED_QUANTITIES:
         m_paramGrid->HideProperty( "Distributed Quantities", false );
-        m_paramGrid->AppendIn( "Distributed Quantities", newParamProperty( aParamIndex ) );
+        m_paramGrid->AppendIn( "Distributed Quantities", newParamProperty( aModel, aParamIndex ) );
         break;
 
     case CATEGORY::WAVEFORM:
         m_paramGrid->HideProperty( "Waveform", false );
-        m_paramGrid->AppendIn( "Waveform", newParamProperty( aParamIndex ) );
+        m_paramGrid->AppendIn( "Waveform", newParamProperty( aModel, aParamIndex ) );
         break;
 
     case CATEGORY::GEOMETRY:
         m_paramGrid->HideProperty( "Geometry", false );
-        m_paramGrid->AppendIn( "Geometry", newParamProperty( aParamIndex ) );
+        m_paramGrid->AppendIn( "Geometry", newParamProperty( aModel, aParamIndex ) );
         break;
 
     case CATEGORY::LIMITING_VALUES:
         m_paramGrid->HideProperty( "Limiting Values", false );
-        m_paramGrid->AppendIn( "Limiting Values", newParamProperty( aParamIndex ) );
+        m_paramGrid->AppendIn( "Limiting Values", newParamProperty( aModel, aParamIndex ) );
         break;
 
     case CATEGORY::ADVANCED:
         m_paramGrid->HideProperty( "Advanced", false );
-        m_paramGrid->AppendIn( "Advanced", newParamProperty( aParamIndex ) );
+        m_paramGrid->AppendIn( "Advanced", newParamProperty( aModel, aParamIndex ) );
         break;
 
     case CATEGORY::FLAGS:
         m_paramGrid->HideProperty( "Flags", false );
-        m_paramGrid->AppendIn( "Flags", newParamProperty( aParamIndex ) );
+        m_paramGrid->AppendIn( "Flags", newParamProperty( aModel, aParamIndex ) );
         break;
 
     default:
-        m_paramGrid->Insert( m_firstCategory, newParamProperty( aParamIndex ) );
+        m_paramGrid->Insert( m_firstCategory, newParamProperty( aModel, aParamIndex ) );
         break;
 
     case CATEGORY::INITIAL_CONDITIONS:
@@ -814,9 +810,10 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::addParamPropertyIfRelevant( int aParam
 
 
 template <typename T_symbol, typename T_field>
-wxPGProperty* DIALOG_SIM_MODEL<T_symbol, T_field>::newParamProperty( int aParamIndex ) const
+wxPGProperty* DIALOG_SIM_MODEL<T_symbol, T_field>::newParamProperty( SIM_MODEL* aModel ,
+                                                                     int aParamIndex ) const
 {
-    const SIM_MODEL::PARAM& param = curModel().GetParam( aParamIndex );
+    const SIM_MODEL::PARAM& param = aModel->GetParam( aParamIndex );
     wxString paramDescription;
 
     if( param.info.description == "" )
@@ -830,18 +827,18 @@ wxPGProperty* DIALOG_SIM_MODEL<T_symbol, T_field>::newParamProperty( int aParamI
     {
     case SIM_VALUE::TYPE_BOOL:
         // TODO.
-        prop = new SIM_BOOL_PROPERTY( paramDescription, param.info.name, curModel(), aParamIndex );
+        prop = new SIM_BOOL_PROPERTY( paramDescription, param.info.name, *aModel, aParamIndex );
         prop->SetAttribute( wxPG_BOOL_USE_CHECKBOX, true );
         break;
 
     case SIM_VALUE::TYPE_INT:
-        prop = new SIM_STRING_PROPERTY( paramDescription, param.info.name, curModel(),
-                                        aParamIndex, SIM_VALUE::TYPE_INT );
+        prop = new SIM_STRING_PROPERTY( paramDescription, param.info.name, *aModel, aParamIndex,
+                                        SIM_VALUE::TYPE_INT );
         break;
 
     case SIM_VALUE::TYPE_FLOAT:
-        prop = new SIM_STRING_PROPERTY( paramDescription, param.info.name, curModel(),
-                                        aParamIndex, SIM_VALUE::TYPE_FLOAT );
+        prop = new SIM_STRING_PROPERTY( paramDescription, param.info.name, *aModel, aParamIndex,
+                                        SIM_VALUE::TYPE_FLOAT );
         break;
 
     //case TYPE_COMPLEX:
@@ -850,13 +847,12 @@ wxPGProperty* DIALOG_SIM_MODEL<T_symbol, T_field>::newParamProperty( int aParamI
     case SIM_VALUE::TYPE_STRING:
         if( param.info.enumValues.empty() )
         {
-            prop = new SIM_STRING_PROPERTY( paramDescription, param.info.name, curModel(),
+            prop = new SIM_STRING_PROPERTY( paramDescription, param.info.name, *aModel,
                                             aParamIndex, SIM_VALUE::TYPE_STRING );
         }
         else
         {
-            prop = new SIM_ENUM_PROPERTY( paramDescription, param.info.name, curModel(),
-                                          aParamIndex );
+            prop = new SIM_ENUM_PROPERTY( paramDescription, param.info.name, *aModel, aParamIndex );
         }
         break;
 
@@ -869,15 +865,9 @@ wxPGProperty* DIALOG_SIM_MODEL<T_symbol, T_field>::newParamProperty( int aParamI
 
     // Legacy due to the way we extracted the parameters from Ngspice.
     #if wxCHECK_VERSION( 3, 1, 0 )
-        if( param.isOtherVariant )
-            prop->SetCell( 3, wxString::FromUTF8( param.info.defaultValueOfOtherVariant ) );
-        else
-            prop->SetCell( 3, wxString::FromUTF8( param.info.defaultValue ) );
+        prop->SetCell( 3, wxString::FromUTF8( param.info.defaultValue ) );
     #else
-        if( param.isOtherVariant )
-            prop->SetCell( 3, wxString::FromUTF8( param.info.defaultValueOfOtherVariant.c_str() ) );
-        else
-            prop->SetCell( 3, wxString::FromUTF8( param.info.defaultValue.c_str() ) );
+        prop->SetCell( 3, wxString::FromUTF8( param.info.defaultValue.c_str() ) );
     #endif
 
     wxString typeStr;
@@ -967,9 +957,10 @@ wxString DIALOG_SIM_MODEL<T_symbol, T_field>::getSymbolPinString( int symbolPinI
 
 
 template <typename T_symbol, typename T_field>
-wxString DIALOG_SIM_MODEL<T_symbol, T_field>::getModelPinString( int aModelPinIndex ) const
+wxString DIALOG_SIM_MODEL<T_symbol, T_field>::getModelPinString( SIM_MODEL* aModel,
+                                                                 int aModelPinIndex ) const
 {
-    const wxString& pinName = curModel().GetPin( aModelPinIndex ).name;
+    const wxString& pinName = aModel->GetPin( aModelPinIndex ).name;
 
     LOCALE_IO toggle;
 
@@ -1219,10 +1210,18 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::onTypeChoice( wxCommandEvent& aEvent )
             {
                 int idx = m_modelNameChoice->GetSelection();
 
-                auto& kibisModel = static_cast<SIM_MODEL_KIBIS&>( m_libraryModelsMgr.GetModels().at( idx ).get() );
+                auto& baseModel = static_cast<SIM_MODEL_KIBIS&>( m_libraryModelsMgr.GetModels().at( idx ).get() );
 
-                m_libraryModelsMgr.SetModel( idx, std::make_unique<SIM_MODEL_KIBIS>( type, kibisModel,
-                                                                                     m_fields, sourcePins ) );
+                m_libraryModelsMgr.SetModel( idx, std::make_unique<SIM_MODEL_KIBIS>( type, baseModel ) );
+
+                try
+                {
+                    m_libraryModelsMgr.GetModels().at( idx ).get().ReadDataFields( &m_fields, sourcePins );
+                }
+                catch( IO_ERROR& err )
+                {
+                    DisplayErrorMessage( this, err.What() );
+                }
             }
 
             m_curModelType = type;
@@ -1238,12 +1237,7 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::onTypeChoice( wxCommandEvent& aEvent )
 template <typename T_symbol, typename T_field>
 void DIALOG_SIM_MODEL<T_symbol, T_field>::onPageChanging( wxBookCtrlEvent& event )
 {
-    // This should have been done in wxPGTextCtrlEditor::OnTextCtrlEvent(), but something must
-    // be clearing it before we get here, resulting in CommitChangesFromEditor() doing nothing
-    m_paramGrid->GetGrid()->EditorsValueWasModified();
-    m_paramGrid->GetGrid()->CommitChangesFromEditor();
-
-    updateModelCodeTab();
+    updateModelCodeTab( &curModel() );
 }
 
 
@@ -1266,7 +1260,7 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::onPinAssignmentsGridCellChange( wxGrid
             std::string( m_sortedPartPins.at( symbolPinIndex )->GetShownNumber().ToUTF8() ) );
     }
 
-    updatePinAssignments();
+    updatePinAssignments( &curModel() );
 
     aEvent.Skip();
 }
@@ -1292,10 +1286,7 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::onParamGridSetFocus( wxFocusEvent& aEv
     // Tab key is pressed. This is inconvenient, so we fix that here.
 
     wxPropertyGrid* grid = m_paramGrid->GetGrid();
-
-    grid->CommitChangesFromEditor();
-
-    wxPGProperty* selected = grid->GetSelection();
+    wxPGProperty*   selected = grid->GetSelection();
 
     if( !selected )
         selected = grid->wxPropertyGridInterface::GetFirst();
@@ -1312,8 +1303,6 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::onParamGridSelectionChange( wxProperty
 {
     wxPropertyGrid* grid = m_paramGrid->GetGrid();
 
-    grid->CommitChangesFromEditor();
-
     // Jump over categories.
     if( grid->GetSelection() && grid->GetSelection()->IsCategory() )
     {
@@ -1359,6 +1348,7 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::onParamGridSelectionChange( wxProperty
     }
 
     wxWindow* editorControl = grid->GetEditorControl();
+
     if( !editorControl )
     {
         m_prevParamGridSelection = grid->GetSelection();
@@ -1371,6 +1361,32 @@ void DIALOG_SIM_MODEL<T_symbol, T_field>::onParamGridSelectionChange( wxProperty
 }
 
 
+template <typename T_symbol, typename T_field>
+void DIALOG_SIM_MODEL<T_symbol, T_field>::onUpdateUI( wxUpdateUIEvent& aEvent )
+{
+    // This is currently patched in wxPropertyGrid::ScrollWindow() in the Mac wxWidgets fork.
+    // However, we may need this version if it turns out to be an issue on other platforms and
+    // we can't get it upstreamed.
+#if 0
+    // It's a shame to do this on the UpdateUI event, but neither the wxPropertyGridManager,
+    // wxPropertyGridPage, wxPropertyGrid, nor the wxPropertyGrid's GetCanvas() window appear
+    // to get scroll events.
+
+    wxPropertyGrid* grid = m_paramGrid->GetGrid();
+    wxTextCtrl*     ctrl = grid->GetEditorTextCtrl();
+
+    if( ctrl )
+    {
+        wxRect ctrlRect = ctrl->GetScreenRect();
+        wxRect gridRect = grid->GetScreenRect();
+
+        if( ctrlRect.GetTop() < gridRect.GetTop() || ctrlRect.GetBottom() > gridRect.GetBottom() )
+            grid->ClearSelection();
+    }
+#endif
+}
+
+
 template <typename T_symbol, typename T_field>
 void DIALOG_SIM_MODEL<T_symbol, T_field>::adjustParamGridColumns( int aWidth, bool aForce )
 {
