@@ -891,7 +891,7 @@ bool PANEL_SYM_LIB_TABLE::TransferDataFromWindow()
         m_globalTable->Clear();
         m_globalTable->m_rows.transfer( m_globalTable->m_rows.end(), global_model()->m_rows.begin(),
                                       global_model()->m_rows.end(), global_model()->m_rows );
-        m_globalTable->reindex();
+        m_globalTable->reindex( true );
     }
 
     if( project_model() && *project_model() != *m_projectTable )
@@ -901,7 +901,7 @@ bool PANEL_SYM_LIB_TABLE::TransferDataFromWindow()
         m_projectTable->Clear();
         m_projectTable->m_rows.transfer( m_projectTable->m_rows.end(), project_model()->m_rows.begin(),
                                        project_model()->m_rows.end(), project_model()->m_rows );
-        m_projectTable->reindex();
+        m_projectTable->reindex( true );
     }
 
     return true;
@@ -1014,9 +1014,7 @@ size_t PANEL_SYM_LIB_TABLE::m_pageNdx = 0;
 
 void InvokeSchEditSymbolLibTable( KIWAY* aKiway, wxWindow *aParent )
 {
-    auto* schEditor = (SCH_EDIT_FRAME*) aKiway->Player( FRAME_SCH, false );
     auto* symbolEditor = (SYMBOL_EDIT_FRAME*) aKiway->Player( FRAME_SCH_SYMBOL_EDITOR, false );
-    auto* symbolViewer = (SYMBOL_VIEWER_FRAME*) aKiway->Player( FRAME_SCH_VIEWER, false );
 
     SYMBOL_LIB_TABLE* globalTable = &SYMBOL_LIB_TABLE::GetGlobalLibTable();
     wxString          globalTablePath = SYMBOL_LIB_TABLE::GetGlobalTableFileName();
@@ -1093,23 +1091,13 @@ void InvokeSchEditSymbolLibTable( KIWAY* aKiway, wxWindow *aParent )
         }
     }
 
-    if( schEditor )
-        schEditor->SyncView();
-
     if( symbolEditor )
     {
-        // Check if the currently selected symbol library been removed or disabled.
-        if( !currentLib.empty() && projectTable && !projectTable->HasLibrary( currentLib, true ) )
-        {
-            symbolEditor->SetCurLib( wxEmptyString );
-            symbolEditor->emptyScreen();
-        }
-
-        symbolEditor->SyncLibraries( true );
         symbolEditor->ThawLibraryTree();
-        symbolEditor->RefreshLibraryTree();
     }
 
-    if( symbolViewer )
-        symbolViewer->ReCreateLibList();
+    std::string payload = "";
+    aKiway->ExpressMail( FRAME_SCH, MAIL_RELOAD_LIB, payload );
+    aKiway->ExpressMail( FRAME_SCH_SYMBOL_EDITOR, MAIL_RELOAD_LIB, payload );
+    aKiway->ExpressMail( FRAME_SCH_VIEWER, MAIL_RELOAD_LIB, payload );
 }
