@@ -4,7 +4,7 @@
  * Copyright (C) 2013 Jean-Pierre Charras, jp.charras at wanadoo.fr
  * Copyright (C) 2013 Wayne Stambaugh <stambaughw@gmail.com>
  * Copyright (C) 2013 CERN (www.cern.ch)
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -37,7 +37,6 @@
 #include <profile.h>
 #include <project/project_file.h>
 #include <project_rescue.h>
-#include <wx_html_report_box.h>
 #include <dialog_HTML_reporter_base.h>
 #include <reporter.h>
 #include <richio.h>
@@ -54,8 +53,9 @@
 #include <tool/actions.h>
 #include <tool/tool_manager.h>
 #include <tools/sch_editor_control.h>
+#include <tools/sch_navigate_tool.h>
 #include <trace_helpers.h>
-#include <widgets/infobar.h>
+#include <widgets/wx_infobar.h>
 #include <wildcards_and_files_ext.h>
 #include <drawing_sheet/ds_data_model.h>
 #include <wx/app.h>
@@ -67,41 +67,13 @@
 #include <paths.h>
 #include <wx_filename.h>  // For ::ResolvePossibleSymlinks
 #include <widgets/wx_progress_reporters.h>
+#include <widgets/wx_html_report_box.h>
 
-
-///< Helper widget to select whether a new project should be created for a file when saving
-class CREATE_PROJECT_CHECKBOX : public wxPanel
-{
-public:
-    CREATE_PROJECT_CHECKBOX( wxWindow* aParent )
-            : wxPanel( aParent )
-    {
-        m_cbCreateProject = new wxCheckBox( this, wxID_ANY,
-                                            _( "Create a new project for this schematic" ) );
-        m_cbCreateProject->SetValue( true );
-        m_cbCreateProject->SetToolTip( _( "Creating a project will enable features such as "
-                                          "text variables, net classes, and ERC exclusions" ) );
-
-        wxBoxSizer* sizer = new wxBoxSizer( wxHORIZONTAL );
-        sizer->Add( m_cbCreateProject, 0, wxALL, 8 );
-
-        SetSizerAndFit( sizer );
-    }
-
-    bool GetValue() const
-    {
-        return m_cbCreateProject->GetValue();
-    }
-
-    static wxWindow* Create( wxWindow* aParent )
-    {
-        return new CREATE_PROJECT_CHECKBOX( aParent );
-    }
-
-protected:
-    wxCheckBox* m_cbCreateProject;
-};
-
+#if wxCHECK_VERSION( 3, 1, 7 )
+#include "widgets/filedlg_hook_save_project.h"
+#else
+#include "widgets/legacyfiledlg_save_project.h"
+#endif
 
 bool SCH_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, int aCtl )
 {
@@ -136,7 +108,7 @@ bool SCH_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
         return false;
 
 #ifdef PROFILE
-    PROF_COUNTER openFiles( "OpenProjectFile" );
+    PROF_TIMER openFiles( "OpenProjectFile" );
 #endif
 
     wxFileName pro = fullFileName;
@@ -260,7 +232,10 @@ bool SCH_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
 
         try
         {
-            Schematic().SetRoot( pi->Load( fullFileName, &Schematic() ) );
+            {
+                wxBusyCursor busy;
+                Schematic().SetRoot( pi->Load( fullFileName, &Schematic() ) );
+            }
 
             if( !pi->GetError().IsEmpty() )
             {
@@ -375,7 +350,7 @@ bool SCH_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
                 wxString paths;
                 wxArrayString libNames;
 
-                SYMBOL_LIBS::LibNamesAndPaths( &Prj(), false, &paths, &libNames );
+                SYMBOL_LIBS::GetLibNamesAndPaths( &Prj(), &paths, &libNames );
 
                 if( !libNames.IsEmpty() )
                 {
@@ -399,7 +374,7 @@ bool SCH_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
 
                     libNames.Clear();
                     paths.Clear();
-                    SYMBOL_LIBS::LibNamesAndPaths( &Prj(), true, &paths, &libNames );
+                    SYMBOL_LIBS::SetLibNamesAndPaths( &Prj(), paths, libNames );
                 }
 
                 if( !cfg || !cfg->m_RescueNeverShow )
@@ -475,8 +450,14 @@ bool SCH_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
                 screen->UpdateLocalLibSymbolLinks();
 
             // Restore all of the loaded symbol and sheet instances from the root sheet.
-            sheetList.UpdateSymbolInstances( Schematic().RootScreen()->GetSymbolInstances() );
-            sheetList.UpdateSheetInstances( Schematic().RootScreen()->GetSheetInstances() );
+            if( Schematic().RootScreen()->GetFileFormatVersionAtLoad() < 20221002 )
+                sheetList.UpdateSymbolInstanceData( Schematic().RootScreen()->GetSymbolInstances());
+
+            if( Schematic().RootScreen()->GetFileFormatVersionAtLoad() < 20221110 )
+                sheetList.UpdateSheetInstanceData( Schematic().RootScreen()->GetSheetInstances());
+
+            for( SCH_SCREEN* screen = schematic.GetFirst(); screen; screen = schematic.GetNext() )
+                screen->MigrateSimModels();
         }
 
         Schematic().ConnectionGraph()->Reset();
@@ -489,7 +470,6 @@ bool SCH_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
         {
             DIALOG_MIGRATE_BUSES dlg( this );
             dlg.ShowQuasiModal();
-            RecalculateConnections( NO_CLEANUP );
             OnModify();
         }
 
@@ -506,12 +486,16 @@ bool SCH_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
     RecomputeIntersheetRefs();
     GetCurrentSheet().UpdateAllScreenReferences();
 
-    // re-create junctions if needed. Eeschema optimizes wires by merging
+    // Re-create junctions if needed. Eeschema optimizes wires by merging
     // colinear segments. If a schematic is saved without a valid
     // cache library or missing installed libraries, this can cause connectivity errors
     // unless junctions are added.
+    //
+    // TODO: (RFB) This really needs to be put inside the Load() function of the SCH_LEGACY_PLUGIN
+    // I can't put it right now because of the extra code that is above to convert legacy bus-bus
+    // entries to bus wires
     if( schFileType == SCH_IO_MGR::SCH_LEGACY )
-        FixupJunctions();
+        Schematic().FixupJunctions();
 
     SyncView();
     GetScreen()->ClearDrawingState();
@@ -519,7 +503,8 @@ bool SCH_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
     TestDanglingEnds();
 
     UpdateHierarchyNavigator();
-    UpdateTitle();
+    updateTitle();
+    m_toolManager->GetTool<SCH_NAVIGATE_TOOL>()->ResetHistory();
 
     wxFileName fn = Prj().AbsolutePath( GetScreen()->GetFileName() );
 
@@ -527,7 +512,8 @@ bool SCH_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
     {
         m_infoBar->RemoveAllButtons();
         m_infoBar->AddCloseButton();
-        m_infoBar->ShowMessage( _( "Schematic is read only." ), wxICON_WARNING );
+        m_infoBar->ShowMessage( _( "Schematic is read only." ),
+                                wxICON_WARNING, WX_INFOBAR::MESSAGE_TYPE::OUTDATED_SAVE );
     }
 
 #ifdef PROFILE
@@ -540,7 +526,6 @@ bool SCH_EDIT_FRAME::OpenProjectFiles( const std::vector<wxString>& aFileSet, in
 
 bool SCH_EDIT_FRAME::AppendSchematic()
 {
-    wxString    fullFileName;
     SCH_SCREEN* screen = GetScreen();
 
     if( !screen )
@@ -558,9 +543,13 @@ bool SCH_EDIT_FRAME::AppendSchematic()
     if( dlg.ShowModal() == wxID_CANCEL )
         return false;
 
-    fullFileName = dlg.GetPath();
+    return AddSheetAndUpdateDisplay( dlg.GetPath() );
+}
 
-    if( !LoadSheetFromFile( GetCurrentSheet().Last(), &GetCurrentSheet(), fullFileName ) )
+
+bool SCH_EDIT_FRAME::AddSheetAndUpdateDisplay( const wxString aFullFileName )
+{
+    if( !LoadSheetFromFile( GetCurrentSheet().Last(), &GetCurrentSheet(), aFullFileName ) )
         return false;
 
     initScreenZoom();
@@ -623,10 +612,11 @@ void SCH_EDIT_FRAME::OnImportProject( wxCommandEvent& aEvent )
 
         SCH_PLUGIN::SCH_PLUGIN_RELEASER plugin( SCH_IO_MGR::FindPlugin( loader.second ) );
         wxCHECK( plugin, /*void*/ );
-        allWildcards += "*." + formatWildcardExt( plugin->GetFileExtension() ) + ";";
+        allWildcards += wxS( "*." ) + formatWildcardExt( plugin->GetFileExtension() ) + wxS( ";" );
     }
 
-    fileFilters = _( "All supported formats|" ) + allWildcards + "|" + fileFilters;
+    fileFilters = _( "All supported formats" ) + wxS( "|" ) + allWildcards + wxS( "|" ) +
+                  fileFilters;
 
     wxFileDialog dlg( this, _( "Import Schematic" ), path, wxEmptyString, fileFilters,
                       wxFD_OPEN | wxFD_FILE_MUST_EXIST ); // TODO
@@ -688,6 +678,9 @@ bool SCH_EDIT_FRAME::saveSchematicFile( SCH_SHEET* aSheet, const wxString& aSave
 
     wxCHECK( screen, false );
 
+    // Cannot save to nowhere
+    wxCHECK( !aSavePath.IsEmpty(), false );
+
     // Construct the name of the file to be saved
     schematicFileName = Prj().AbsolutePath( aSavePath );
     oldFileName = schematicFileName;
@@ -698,12 +691,13 @@ bool SCH_EDIT_FRAME::saveSchematicFile( SCH_SHEET* aSheet, const wxString& aSave
     if( !IsWritable( schematicFileName ) )
         return false;
 
-    wxStandardPaths& paths = wxStandardPaths::Get();
-    wxString tempFile = wxFileName::CreateTempFileName(
-            paths.GetTempDir() + wxFileName::GetPathSeparator() + wxT( "eeschema" ) );
+    wxString tempFile = wxFileName::CreateTempFileName( wxS( "eeschema" ) );
 
     // Save
-    wxLogTrace( traceAutoSave, "Saving file " + schematicFileName.GetFullPath() );
+    wxLogTrace( traceAutoSave, wxS( "Saving file " ) + schematicFileName.GetFullPath() );
+
+    if( m_infoBar->GetMessageType() == WX_INFOBAR::MESSAGE_TYPE::OUTDATED_SAVE )
+        m_infoBar->Dismiss();
 
     SCH_IO_MGR::SCH_FILE_T pluginType = SCH_IO_MGR::GuessPluginTypeFromSchPath(
             schematicFileName.GetFullPath() );
@@ -784,10 +778,10 @@ bool SCH_EDIT_FRAME::SaveProject( bool aSaveAs )
     wxString msg;
     SCH_SCREEN* screen;
     SCH_SCREENS screens( Schematic().Root() );
-    bool        saveCopyAs       = aSaveAs && !Kiface().IsSingle();
-    bool        success          = true;
-    bool        updateFileType   = false;
-    bool        createNewProject = false;
+    bool        saveCopy          = aSaveAs && !Kiface().IsSingle();
+    bool        success           = true;
+    bool        updateFileHistory = false;
+    bool        createNewProject  = false;
 
     // I want to see it in the debugger, show me the string!  Can't do that with wxFileName.
     wxString    fileName = Prj().AbsolutePath( Schematic().Root().GetFileName() );
@@ -823,10 +817,18 @@ bool SCH_EDIT_FRAME::SaveProject( bool aSaveAs )
                           savePath.GetFullName(), KiCadSchematicFileWildcard(),
                           wxFD_SAVE | wxFD_OVERWRITE_PROMPT );
 
+#if wxCHECK_VERSION( 3, 1, 7 )
+        FILEDLG_HOOK_SAVE_PROJECT newProjectHook;
+#endif
+
+        // Add a "Create a project" checkbox in standalone mode and one isn't loaded
         if( Kiface().IsSingle() || aSaveAs )
         {
-            // Add a "Create a project" checkbox in standalone mode and one isn't loaded
-            dlg.SetExtraControlCreator( &CREATE_PROJECT_CHECKBOX::Create );
+#if wxCHECK_VERSION( 3, 1, 7 )
+            dlg.SetCustomizeHook( newProjectHook );
+#else
+            dlg.SetExtraControlCreator( &LEGACYFILEDLG_SAVE_PROJECT::Create );
+#endif
         }
 
         if( dlg.ShowModal() == wxID_CANCEL )
@@ -849,13 +851,19 @@ bool SCH_EDIT_FRAME::SaveProject( bool aSaveAs )
             return false;
         }
 
+#if wxCHECK_VERSION( 3, 1, 7 )
+        if( newProjectHook.IsAttachedToDialog() )
+            createNewProject = newProjectHook.GetCreateNewProject();
+#else
         if( wxWindow* ec = dlg.GetExtraControl() )
-            createNewProject = static_cast<CREATE_PROJECT_CHECKBOX*>( ec )->GetValue();
+            createNewProject = static_cast<LEGACYFILEDLG_SAVE_PROJECT*>( ec )->GetValue();
+#endif
 
-        if( !saveCopyAs )
+        if( !saveCopy )
         {
             Schematic().Root().SetFileName( newFileName.GetFullName() );
             Schematic().RootScreen()->SetFileName( newFileName.GetFullPath() );
+            updateFileHistory = true;
         }
         else
         {
@@ -916,7 +924,7 @@ bool SCH_EDIT_FRAME::SaveProject( bool aSaveAs )
                 return false;
             }
 
-            if( saveCopyAs )
+            if( saveCopy )
                 filenameMap[screen] = tmp.GetFullPath();
             else
                 screen->SetFileName( tmp.GetFullPath() );
@@ -933,8 +941,13 @@ bool SCH_EDIT_FRAME::SaveProject( bool aSaveAs )
             sheet.MakeFilePathRelativeToParentSheet();
         }
     }
+    else if( !fn.FileExists() )
+    {
+        // File doesn't exist yet; true if we just imported something
+        updateFileHistory = true;
+    }
 
-    if( filenameMap.empty() || !saveCopyAs )
+    if( filenameMap.empty() || !saveCopy )
     {
         for( size_t i = 0; i < screens.GetCount(); i++ )
             filenameMap[screens.GetScreen( i )] = screens.GetScreen( i )->GetFileName();
@@ -999,10 +1012,10 @@ bool SCH_EDIT_FRAME::SaveProject( bool aSaveAs )
 
         if( tmpFn.IsOk() && tmpFn.GetExt() != KiCadSchematicFileExtension )
         {
-            updateFileType = true;
+            updateFileHistory = true;
             tmpFn.SetExt( KiCadSchematicFileExtension );
 
-            for( auto item : screen->Items().OfType( SCH_SHEET_T ) )
+            for( EDA_ITEM* item : screen->Items().OfType( SCH_SHEET_T ) )
             {
                 SCH_SHEET* sheet = static_cast<SCH_SHEET*>( item );
                 wxFileName sheetFileName = sheet->GetFileName();
@@ -1017,10 +1030,14 @@ bool SCH_EDIT_FRAME::SaveProject( bool aSaveAs )
 
             filenameMap[screen] = tmpFn.GetFullPath();
 
-            if( !saveCopyAs )
+            if( !saveCopy )
                 screen->SetFileName( tmpFn.GetFullPath() );
         }
 
+        // Do not save sheet symbols with no valid filename set
+        if( !tmpFn.IsOk() )
+            continue;
+
         std::vector<SCH_SHEET_PATH>& sheets = screen->GetClientSheetPaths();
 
         if( sheets.size() == 1 )
@@ -1029,12 +1046,15 @@ bool SCH_EDIT_FRAME::SaveProject( bool aSaveAs )
             screen->SetVirtualPageNumber( 0 );  // multiple uses; no way to store the real sheet #
 
         // This is a new schematic file so make sure it has a unique ID.
-        if( !saveCopyAs && tmpFn.GetFullPath() != screen->GetFileName() )
+        if( !saveCopy && tmpFn.GetFullPath() != screen->GetFileName() )
             screen->AssignNewUuid();
 
         success &= saveSchematicFile( screens.GetSheet( i ), tmpFn.GetFullPath() );
     }
 
+    if( success )
+        m_autoSaveRequired = false;
+
     // One or more of the modified sheets did not save correctly so update the auto save file.
     if( !aSaveAs && !success )
         success &= updateAutoSaveFile();
@@ -1042,7 +1062,7 @@ bool SCH_EDIT_FRAME::SaveProject( bool aSaveAs )
     if( aSaveAs && success )
         LockFile( Schematic().RootScreen()->GetFileName() );
 
-    if( updateFileType )
+    if( updateFileHistory )
         UpdateFileHistory( Schematic().RootScreen()->GetFileName() );
 
     // Save the sheet name map to the project file
@@ -1074,12 +1094,12 @@ bool SCH_EDIT_FRAME::SaveProject( bool aSaveAs )
     wxFileName projectPath( filenameMap.at( Schematic().RootScreen() ) );
     projectPath.SetExt( ProjectFileExtension );
 
-    if( Prj().IsNullProject() || ( aSaveAs && !saveCopyAs ) )
+    if( Prj().IsNullProject() || ( aSaveAs && !saveCopy ) )
     {
         Prj().SetReadOnly( !createNewProject );
         GetSettingsManager()->SaveProjectAs( projectPath.GetFullPath() );
     }
-    else if( saveCopyAs && createNewProject )
+    else if( saveCopy && createNewProject )
     {
         GetSettingsManager()->SaveProjectCopy( projectPath.GetFullPath() );
     }
@@ -1097,7 +1117,7 @@ bool SCH_EDIT_FRAME::SaveProject( bool aSaveAs )
             SetStatusText( msg, 0 );
     }
 
-    UpdateTitle();
+    updateTitle();
 
     if( m_infoBar->GetMessageType() == WX_INFOBAR::MESSAGE_TYPE::OUTDATED_SAVE )
         m_infoBar->Dismiss();
@@ -1144,17 +1164,23 @@ bool SCH_EDIT_FRAME::doAutoSave()
         fn.SetName( GetAutoSaveFilePrefix() + fn.GetName() );
 
         if( saveSchematicFile( screens.GetSheet( i ), fn.GetFullPath() ) )
+        {
+            // This was only an auto-save, not a real save.  Reset the modified flag.
             screens.GetScreen( i )->SetContentModified();
+        }
         else
+        {
             autoSaveOk = false;
+        }
     }
 
     if( autoSaveOk && updateAutoSaveFile() )
     {
-        m_autoSaveState = false;
+        m_autoSaveRequired = false;
+        m_autoSavePending = false;
 
-        if( !Kiface().IsSingle() &&
-            GetSettingsManager()->GetCommonSettings()->m_Backup.backup_on_autosave )
+        if( !Kiface().IsSingle()
+                && GetSettingsManager()->GetCommonSettings()->m_Backup.backup_on_autosave )
         {
             GetSettingsManager()->TriggerBackupIfNeeded( NULL_REPORTER::GetInstance() );
         }
@@ -1179,7 +1205,8 @@ bool SCH_EDIT_FRAME::importFile( const wxString& aFileName, int aFileType )
     case SCH_IO_MGR::SCH_CADSTAR_ARCHIVE:
     case SCH_IO_MGR::SCH_EAGLE:
         // We insist on caller sending us an absolute path, if it does not, we say it's a bug.
-        wxASSERT_MSG( filename.IsAbsolute(), wxT( "Import schematic: path is not absolute!" ) );
+        wxCHECK_MSG( filename.IsAbsolute(), false,
+                     wxT( "Import schematic: path is not absolute!" ) );
 
         if( !LockFile( aFileName ) )
         {
@@ -1210,16 +1237,7 @@ bool SCH_EDIT_FRAME::importFile( const wxString& aFileName, int aFileType )
             // to KiCad), so set it to an empty one
             DS_DATA_MODEL& drawingSheet = DS_DATA_MODEL::GetTheInstance();
             drawingSheet.SetEmptyLayout();
-
             BASE_SCREEN::m_DrawingSheetFileName = "empty.kicad_wks";
-            wxFileName layoutfn( Prj().GetProjectPath(), BASE_SCREEN::m_DrawingSheetFileName );
-            wxFFile layoutfile;
-
-            if( layoutfile.Open( layoutfn.GetFullPath(), "wb" ) )
-            {
-                layoutfile.Write( DS_DATA_MODEL::EmptyLayout() );
-                layoutfile.Close();
-            }
 
             newfilename.SetPath( Prj().GetProjectPath() );
             newfilename.SetName( Prj().GetProjectName() );
@@ -1227,17 +1245,10 @@ bool SCH_EDIT_FRAME::importFile( const wxString& aFileName, int aFileType )
 
             SetScreen( GetCurrentSheet().LastScreen() );
 
-            Schematic().Root().SetFileName( newfilename.GetFullPath() );
+            Schematic().Root().SetFileName( newfilename.GetFullName() );
             GetScreen()->SetFileName( newfilename.GetFullPath() );
             GetScreen()->SetContentModified();
 
-            // Only fix junctions for CADSTAR importer for now as it may cause issues with
-            // other importers
-            if( fileType == SCH_IO_MGR::SCH_CADSTAR_ARCHIVE )
-            {
-                FixupJunctions();
-            }
-
             RecalculateConnections( GLOBAL_CLEANUP );
 
             // Only perform the dangling end test on root sheet.
@@ -1250,7 +1261,7 @@ bool SCH_EDIT_FRAME::importFile( const wxString& aFileName, int aFileType )
             SyncView();
 
             UpdateHierarchyNavigator();
-            UpdateTitle();
+            updateTitle();
         }
         catch( const IO_ERROR& ioe )
         {
@@ -1267,6 +1278,20 @@ bool SCH_EDIT_FRAME::importFile( const wxString& aFileName, int aFileType )
 
             return false;
         }
+        catch( const std::exception& exc )
+        {
+            CreateScreens();
+            m_toolManager->RunAction( ACTIONS::zoomFitScreen, true );
+
+            wxString msg = wxString::Format( _( "Unhandled exception occurred loading schematic "
+                                                "'%s'." ), aFileName );
+            DisplayErrorMessage( this, msg, exc.what() );
+
+            msg.Printf( _( "Failed to load '%s'." ), aFileName );
+            SetMsgPanel( wxEmptyString, msg );
+
+            return false;
+        }
 
         return true;
 
@@ -1369,6 +1394,9 @@ bool SCH_EDIT_FRAME::updateAutoSaveFile()
 
 void SCH_EDIT_FRAME::CheckForAutoSaveFile( const wxFileName& aFileName )
 {
+    if( !IsGUI() )
+        return;
+
     wxCHECK_RET( aFileName.IsOk(), wxT( "Invalid file name!" ) );
 
     wxLogTrace( traceAutoSave,
@@ -1408,7 +1436,7 @@ void SCH_EDIT_FRAME::CheckForAutoSaveFile( const wxFileName& aFileName )
             wxString tmp = recoveredFn.GetName();
 
             // Strip "_autosave-" prefix from the auto save file name.
-            tmp.Replace( GetAutoSaveFilePrefix(), wxEmptyString, false );
+            tmp.Replace( GetAutoSaveFilePrefix(), wxT( "" ), false );
             recoveredFn.SetName( tmp );
 
             wxFileName backupFn = recoveredFn;
@@ -1503,4 +1531,3 @@ const wxString& SCH_EDIT_FRAME::getAutoSaveFileName() const
 
     return autoSaveFileName;
 }
-
