@@ -148,7 +148,7 @@ bool NETLIST_EXPORTER_SPICE::DoWriteNetlist( OUTPUTFORMATTER& aFormatter, unsign
 
 void NETLIST_EXPORTER_SPICE::WriteHead( OUTPUTFORMATTER& aFormatter, unsigned aNetlistOptions )
 {
-    aFormatter.Print( 0, "KiCad schematic\n" );
+    aFormatter.Print( 0, ".title KiCad schematic\n" );
 }
 
 
@@ -220,7 +220,7 @@ bool NETLIST_EXPORTER_SPICE::ReadSchematicAndLibraries( unsigned aNetlistOptions
             if( !symbol || symbol->GetFieldText( SIM_ENABLE_FIELD ) == wxT( "0" ) )
                 continue;
 
-            CreatePinList( symbol, &sheet, true );
+            std::vector<PIN_INFO> pins = CreatePinList( symbol, &sheet, true );
 
             SPICE_ITEM spiceItem;
 
@@ -267,8 +267,8 @@ bool NETLIST_EXPORTER_SPICE::ReadSchematicAndLibraries( unsigned aNetlistOptions
             {
                 readRefName( sheet, *symbol, spiceItem, refNames );
                 readModel( sheet, *symbol, spiceItem );
-                readPinNumbers( *symbol, spiceItem );
-                readPinNetNames( *symbol, spiceItem, ncCounter );
+                readPinNumbers( *symbol, spiceItem, pins );
+                readPinNetNames( *symbol, spiceItem, pins, ncCounter );
 
                 // TODO: transmission line handling?
 
@@ -387,49 +387,55 @@ void NETLIST_EXPORTER_SPICE::ReadDirectives( unsigned aNetlistOptions )
             wxStringTokenizer tokenizer( text, wxT( "\r\n" ), wxTOKEN_STRTOK );
             bool              foundDirective = false;
 
+            auto isDirective =
+                    []( const wxString& line, const wxString& dir )
+                    {
+                        return line == dir || line.StartsWith( dir + wxS( " " ) );
+                    };
+
             while( tokenizer.HasMoreTokens() )
             {
                 wxString line = tokenizer.GetNextToken().Upper();
 
                 if( line.StartsWith( wxT( "." ) ) )
                 {
-                    if(    line.StartsWith( wxT( ".AC" ) )
-                        || line.StartsWith( wxT( ".CONTROL" ) )
-                        || line.StartsWith( wxT( ".CSPARAM" ) )
-                        || line.StartsWith( wxT( ".DISTO" ) )
-                        || line.StartsWith( wxT( ".ELSE" ) )
-                        || line.StartsWith( wxT( ".ELSEIF" ) )
-                        || line.StartsWith( wxT( ".END" ) )
-                        || line.StartsWith( wxT( ".ENDC" ) )
-                        || line.StartsWith( wxT( ".ENDIF" ) )
-                        || line.StartsWith( wxT( ".ENDS" ) )
-                        || line.StartsWith( wxT( ".FOUR" ) )
-                        || line.StartsWith( wxT( ".FUNC" ) )
-                        || line.StartsWith( wxT( ".GLOBAL" ) )
-                        || line.StartsWith( wxT( ".IC" ) )
-                        || line.StartsWith( wxT( ".IF" ) )
-                        || line.StartsWith( wxT( ".INCLUDE" ) )
-                        || line.StartsWith( wxT( ".LIB" ) )
-                        || line.StartsWith( wxT( ".MEAS" ) )
-                        || line.StartsWith( wxT( ".MODEL" ) )
-                        || line.StartsWith( wxT( ".NODESET" ) )
-                        || line.StartsWith( wxT( ".NOISE" ) )
-                        || line.StartsWith( wxT( ".OP" ) )
-                        || line.StartsWith( wxT( ".OPTIONS" ) )
-                        || line.StartsWith( wxT( ".PARAM" ) )
-                        || line.StartsWith( wxT( ".PLOT" ) )
-                        || line.StartsWith( wxT( ".PRINT" ) )
-                        || line.StartsWith( wxT( ".PROBE" ) )
-                        || line.StartsWith( wxT( ".PZ" ) )
-                        || line.StartsWith( wxT( ".SAVE" ) )
-                        || line.StartsWith( wxT( ".SENS" ) )
-                        || line.StartsWith( wxT( ".SP" ) )
-                        || line.StartsWith( wxT( ".SUBCKT" ) )
-                        || line.StartsWith( wxT( ".TEMP" ) )
-                        || line.StartsWith( wxT( ".TF" ) )
-                        || line.StartsWith( wxT( ".TITLE" ) )
-                        || line.StartsWith( wxT( ".TRAN" ) )
-                        || line.StartsWith( wxT( ".WIDTH" ) ) )
+                    if(    isDirective( line, wxS( ".AC" ) )
+                        || isDirective( line, wxS( ".CONTROL" ) )
+                        || isDirective( line, wxS( ".CSPARAM" ) )
+                        || isDirective( line, wxS( ".DISTO" ) )
+                        || isDirective( line, wxS( ".ELSE" ) )
+                        || isDirective( line, wxS( ".ELSEIF" ) )
+                        || isDirective( line, wxS( ".END" ) )
+                        || isDirective( line, wxS( ".ENDC" ) )
+                        || isDirective( line, wxS( ".ENDIF" ) )
+                        || isDirective( line, wxS( ".ENDS" ) )
+                        || isDirective( line, wxS( ".FOUR" ) )
+                        || isDirective( line, wxS( ".FUNC" ) )
+                        || isDirective( line, wxS( ".GLOBAL" ) )
+                        || isDirective( line, wxS( ".IC" ) )
+                        || isDirective( line, wxS( ".IF" ) )
+                        || isDirective( line, wxS( ".INCLUDE" ) )
+                        || isDirective( line, wxS( ".LIB" ) )
+                        || isDirective( line, wxS( ".MEAS" ) )
+                        || isDirective( line, wxS( ".MODEL" ) )
+                        || isDirective( line, wxS( ".NODESET" ) )
+                        || isDirective( line, wxS( ".NOISE" ) )
+                        || isDirective( line, wxS( ".OP" ) )
+                        || isDirective( line, wxS( ".OPTIONS" ) )
+                        || isDirective( line, wxS( ".PARAM" ) )
+                        || isDirective( line, wxS( ".PLOT" ) )
+                        || isDirective( line, wxS( ".PRINT" ) )
+                        || isDirective( line, wxS( ".PROBE" ) )
+                        || isDirective( line, wxS( ".PZ" ) )
+                        || isDirective( line, wxS( ".SAVE" ) )
+                        || isDirective( line, wxS( ".SENS" ) )
+                        || isDirective( line, wxS( ".SP" ) )
+                        || isDirective( line, wxS( ".SUBCKT" ) )
+                        || isDirective( line, wxS( ".TEMP" ) )
+                        || isDirective( line, wxS( ".TF" ) )
+                        || isDirective( line, wxS( ".TITLE" ) )
+                        || isDirective( line, wxS( ".TRAN" ) )
+                        || isDirective( line, wxS( ".WIDTH" ) ) )
                     {
                         foundDirective = true;
                         break;
@@ -495,7 +501,7 @@ void NETLIST_EXPORTER_SPICE::readModel( SCH_SHEET_PATH& aSheet, SCH_SYMBOL& aSym
     if( auto rawSpiceModel = dynamic_cast<const SIM_MODEL_RAW_SPICE*>( aItem.model ) )
     {
         int      libParamIndex = static_cast<int>( SIM_MODEL_RAW_SPICE::SPICE_PARAM::LIB );
-        wxString path = rawSpiceModel->GetParam( libParamIndex ).value->ToString();
+        wxString path = rawSpiceModel->GetParam( libParamIndex ).value;
 
         if( !path.IsEmpty() )
             m_rawIncludes.insert( path );
@@ -527,20 +533,20 @@ void NETLIST_EXPORTER_SPICE::readModel( SCH_SHEET_PATH& aSheet, SCH_SYMBOL& aSym
 }
 
 
-void NETLIST_EXPORTER_SPICE::readPinNumbers( SCH_SYMBOL& aSymbol, SPICE_ITEM& aItem )
+void NETLIST_EXPORTER_SPICE::readPinNumbers( SCH_SYMBOL& aSymbol, SPICE_ITEM& aItem,
+                                             const std::vector<PIN_INFO>& aPins  )
 {
-    for( const PIN_INFO& pin : m_sortedSymbolPinList )
-        aItem.pinNumbers.emplace_back( std::string( pin.num.ToUTF8() ) );
+    for( const PIN_INFO& pin : aPins )
+        aItem.pinNumbers.emplace_back( pin.num.ToStdString() );
 }
 
 
 void NETLIST_EXPORTER_SPICE::readPinNetNames( SCH_SYMBOL& aSymbol, SPICE_ITEM& aItem,
-                                              int& aNcCounter )
+                                              const std::vector<PIN_INFO>& aPins, int& aNcCounter )
 {
-    for( const PIN_INFO& pinInfo : m_sortedSymbolPinList )
+    for( const PIN_INFO& pinInfo : aPins )
     {
-        std::string netName = GenerateItemPinNetName( std::string( pinInfo.netName.ToUTF8() ),
-                                                      aNcCounter );
+        std::string netName = GenerateItemPinNetName( pinInfo.netName.ToStdString(), aNcCounter );
 
         aItem.pinNetNames.push_back( netName );
         m_nets.insert( netName );
@@ -632,6 +638,12 @@ void NETLIST_EXPORTER_SPICE::WriteDirectives( OUTPUTFORMATTER& aFormatter,
     if( aNetlistOptions & OPTION_SAVE_ALL_CURRENTS )
         aFormatter.Print( 0, ".probe alli\n" );
 
+    auto isSimCommand =
+            []( const wxString& candidate, const wxString& dir )
+            {
+                return candidate == dir || candidate.StartsWith( dir + wxS( " " ) );
+            };
+
     for( const wxString& directive : m_directives )
     {
         bool simCommand = false;
@@ -640,15 +652,15 @@ void NETLIST_EXPORTER_SPICE::WriteDirectives( OUTPUTFORMATTER& aFormatter,
         {
             wxString candidate = directive.Upper();
 
-            simCommand = ( candidate.StartsWith( wxT( ".AC" ) )
-                        || candidate.StartsWith( wxT( ".DC" ) )
-                        || candidate.StartsWith( wxT( ".TRAN" ) )
-                        || candidate.StartsWith( wxT( ".OP" ) )
-                        || candidate.StartsWith( wxT( ".DISTO" ) )
-                        || candidate.StartsWith( wxT( ".NOISE" ) )
-                        || candidate.StartsWith( wxT( ".PZ" ) )
-                        || candidate.StartsWith( wxT( ".SENS" ) )
-                        || candidate.StartsWith( wxT( ".TF" ) ) );
+            simCommand = ( isSimCommand( candidate, wxS( ".AC" ) )
+                        || isSimCommand( candidate, wxS( ".DC" ) )
+                        || isSimCommand( candidate, wxS( ".TRAN" ) )
+                        || isSimCommand( candidate, wxS( ".OP" ) )
+                        || isSimCommand( candidate, wxS( ".DISTO" ) )
+                        || isSimCommand( candidate, wxS( ".NOISE" ) )
+                        || isSimCommand( candidate, wxS( ".PZ" ) )
+                        || isSimCommand( candidate, wxS( ".SENS" ) )
+                        || isSimCommand( candidate, wxS( ".TF" ) ) );
         }
 
         if( !simCommand || ( aNetlistOptions & OPTION_SIM_COMMAND ) )
