@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2015 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 2004-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2004-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -189,7 +189,8 @@ wxString SCH_FIELD::GetShownText( int aDepth, bool aAllowExtraText ) const
     std::function<bool( wxString* )> labelResolver =
             [&]( wxString* token ) -> bool
             {
-                return static_cast<SCH_LABEL_BASE*>( m_parent )->ResolveTextVar( token, aDepth + 1 );
+                return static_cast<SCH_LABEL_BASE*>( m_parent )->ResolveTextVar( token,
+                                                                                 aDepth + 1 );
             };
 
     wxString text = EDA_TEXT::GetShownText();
@@ -968,6 +969,9 @@ void SCH_FIELD::Plot( PLOTTER* aPlotter, bool aBackground ) const
 
     penWidth = std::max( penWidth, settings->GetMinPenWidth() );
 
+    // clamp the pen width to be sure the text is readable
+    penWidth = std::min( penWidth, std::min( GetTextSize().x, GetTextSize().y ) / 4 );
+
     if( !IsVisible() )
         return;
 
@@ -1021,8 +1025,14 @@ void SCH_FIELD::Plot( PLOTTER* aPlotter, bool aBackground ) const
     if( !font )
         font = KIFONT::FONT::GetFont( settings->GetDefaultFont(), IsBold(), IsItalic() );
 
-    aPlotter->Text( textpos, color, GetShownText(), orient, GetTextSize(),  hjustify, vjustify,
-                    penWidth, IsItalic(), IsBold(), false, font );
+    TEXT_ATTRIBUTES attrs = GetAttributes();
+    attrs.m_StrokeWidth = penWidth;
+    attrs.m_Halign = hjustify;
+    attrs.m_Valign = vjustify;
+    attrs.m_Angle = orient;
+    attrs.m_Multiline = false;
+
+    aPlotter->PlotText( textpos, color, GetShownText(), attrs, font );
 
     if( IsHypertext() )
     {
@@ -1031,6 +1041,8 @@ void SCH_FIELD::Plot( PLOTTER* aPlotter, bool aBackground ) const
         std::vector<wxString>                      pageHrefs;
         BOX2I                                      bbox = GetBoundingBox();
 
+        wxCHECK( label, /* void */ );
+
         label->GetIntersheetRefs( &pages );
 
         for( const std::pair<wxString, wxString>& page : pages )
