@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2020 Thomas Pointhuber <thomas.pointhuber@gmx.at>
- * Copyright (C) 2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,7 +25,7 @@
 #include <iostream>
 #include <unordered_map>
 
-#include <convert_to_biu.h>
+#include <base_units.h>
 #include <ki_exception.h>
 
 #include <wx/log.h>
@@ -43,7 +43,12 @@ ALTIUM_SCH_RECORD ReadRecord( const std::map<wxString, wxString>& aProps )
 
 constexpr int Altium2KiCadUnit( const int val, const int frac )
 {
-    return Mils2iu( val ) * 10 + Mils2iu( frac ) / 10000; // TODO: correct, rounding issues?
+    constexpr double int_limit = ( std::numeric_limits<int>::max() - 10 ) / 2.54;
+
+    double dbase = 10 * schIUScale.MilsToIU( val );
+    double dfrac = schIUScale.MilsToIU( frac ) / 10000.0;
+
+    return KiROUND( Clamp<double>( -int_limit, ( dbase + dfrac ) / 10.0, int_limit ) ) * 10;
 }
 
 
@@ -82,7 +87,7 @@ template <typename T>
 T ReadEnum( const std::map<wxString, wxString>& aProps, const wxString& aKey, int aLower,
             int aUpper, T aDefault )
 {
-    int value = ALTIUM_PARSER::ReadInt( aProps, aKey, static_cast<int>( aDefault ));
+    int value = ALTIUM_PARSER::ReadInt( aProps, aKey, static_cast<int>( aDefault ) );
 
     if( value < aLower || value > aUpper )
         return aDefault;
@@ -103,6 +108,18 @@ ASCH_STORAGE_FILE::ASCH_STORAGE_FILE( ALTIUM_PARSER& aReader )
 }
 
 
+ASCH_ADDITIONAL_FILE::ASCH_ADDITIONAL_FILE( ALTIUM_PARSER& aReader )
+{
+    aReader.Skip( 5 );
+    FileName = aReader.ReadWxString();
+    uint32_t dataSize = aReader.Read<uint32_t>();
+    Data = aReader.ReadVector( dataSize );
+
+    if( aReader.HasParsingError() )
+        THROW_IO_ERROR( "Additional stream was not parsed correctly" );
+}
+
+
 ASCH_SYMBOL::ASCH_SYMBOL( const std::map<wxString, wxString>& aProps )
 {
     wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::COMPONENT );
@@ -113,12 +130,13 @@ ASCH_SYMBOL::ASCH_SYMBOL( const std::map<wxString, wxString>& aProps )
     componentdescription = ALTIUM_PARSER::ReadString( aProps, "COMPONENTDESCRIPTION", "" );
 
     orientation = ALTIUM_PARSER::ReadInt( aProps, "ORIENTATION", 0 );
-    isMirrored  = ALTIUM_PARSER::ReadBool( aProps, "ISMIRRORED", false );
-    location = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                        -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    isMirrored = ALTIUM_PARSER::ReadBool( aProps, "ISMIRRORED", false );
+    location = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                         -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
 
     partcount        = ALTIUM_PARSER::ReadInt( aProps, "PARTCOUNT", 0 );
     displaymodecount = ALTIUM_PARSER::ReadInt( aProps, "DISPLAYMODECOUNT", 0 );
+    m_indexInSheet   = ALTIUM_PARSER::ReadInt( aProps, "INDEXINSHEET", -1 );
     displaymode      = ALTIUM_PARSER::ReadInt( aProps, "DISPLAYMODE", 0 );
 }
 
@@ -164,7 +182,7 @@ ASCH_PIN::ASCH_PIN( const std::map<wxString, wxString>& aProps )
     int xfrac = ALTIUM_PARSER::ReadInt( aProps, "LOCATION.X_FRAC", 0 );
     int y     = ALTIUM_PARSER::ReadInt( aProps, "LOCATION.Y", 0 );
     int yfrac = ALTIUM_PARSER::ReadInt( aProps, "LOCATION.Y_FRAC", 0 );
-    location  = wxPoint( Altium2KiCadUnit( x, xfrac ), -Altium2KiCadUnit( y, yfrac ) );
+    location  = VECTOR2I( Altium2KiCadUnit( x, xfrac ), -Altium2KiCadUnit( y, yfrac ) );
 
     int p     = ALTIUM_PARSER::ReadInt( aProps, "PINLENGTH", 0 );
     int pfrac = ALTIUM_PARSER::ReadInt( aProps, "PINLENGTH_FRAC", 0 );
@@ -182,25 +200,29 @@ ASCH_PIN::ASCH_PIN( const std::map<wxString, wxString>& aProps )
         kicadX += p;
         kicadXfrac += pfrac;
         break;
+
     case ASCH_RECORD_ORIENTATION::UPWARDS:
         kicadY += p;
         kicadYfrac += pfrac;
         break;
+
     case ASCH_RECORD_ORIENTATION::LEFTWARDS:
         kicadX -= p;
         kicadXfrac -= pfrac;
         break;
+
     case ASCH_RECORD_ORIENTATION::DOWNWARDS:
         kicadY -= p;
         kicadYfrac -= pfrac;
         break;
+
     default:
         wxLogWarning( "Pin has unexpected orientation" );
         break;
     }
 
-    kicadLocation = wxPoint( Altium2KiCadUnit( kicadX, kicadXfrac ),
-                             -Altium2KiCadUnit( kicadY, kicadYfrac ) );
+    kicadLocation = VECTOR2I( Altium2KiCadUnit( kicadX, kicadXfrac ),
+                              -Altium2KiCadUnit( kicadY, kicadYfrac ) );
 }
 
 
@@ -211,8 +233,8 @@ ASCH_LABEL::ASCH_LABEL( const std::map<wxString, wxString>& aProps )
     ownerindex = ReadOwnerIndex( aProps );
     ownerpartid = ReadOwnerPartId( aProps );
 
-    location = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                        -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    location = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                         -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
 
     text = ALTIUM_PARSER::ReadString( aProps, "TEXT", "" );
 
@@ -232,21 +254,29 @@ ASCH_TEXT_FRAME::ASCH_TEXT_FRAME( const std::map<wxString, wxString>& aProps )
     wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::NOTE
               || ReadRecord( aProps ) == ALTIUM_SCH_RECORD::TEXT_FRAME );
 
-    location = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                        -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
-    size = wxSize( ReadKiCadUnitFrac( aProps, "CORNER.X" ) - location.x,
-                   -ReadKiCadUnitFrac( aProps, "CORNER.Y" ) - location.y );
+    BottomLeft = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                           -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    TopRight = VECTOR2I( ReadKiCadUnitFrac( aProps, "CORNER.X" ),
+                         -ReadKiCadUnitFrac( aProps, "CORNER.Y" ) );
 
-    text = ALTIUM_PARSER::ReadString( aProps, "TEXT", "" );
-    text.Replace( "~1", "\n", true );
+    Location = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                         -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    Size = wxSize( ReadKiCadUnitFrac( aProps, "CORNER.X" ) - Location.x,
+                   -ReadKiCadUnitFrac( aProps, "CORNER.Y" ) - Location.y );
 
-    fontId = ALTIUM_PARSER::ReadInt( aProps, "FONTID", 0 );
-    isWordWrapped = ALTIUM_PARSER::ReadBool( aProps, "WORDWRAP", false );
-    border = ALTIUM_PARSER::ReadBool( aProps, "SHOWBORDER", false );
-    textMargin = ReadKiCadUnitFrac( aProps, "TEXTMARGIN" );
-    areaColor = ALTIUM_PARSER::ReadInt( aProps, "AREACOLOR", 0 );
+    Text = ALTIUM_PARSER::ReadString( aProps, "TEXT", "" );
+    Text.Replace( "~1", "\n", true );
 
-    alignment = ReadEnum<ASCH_TEXT_FRAME_ALIGNMENT>( aProps, "ALIGNMENT", 1, 3,
+    FontID = ALTIUM_PARSER::ReadInt( aProps, "FONTID", 0 );
+    IsWordWrapped = ALTIUM_PARSER::ReadBool( aProps, "WORDWRAP", false );
+    ShowBorder = ALTIUM_PARSER::ReadBool( aProps, "SHOWBORDER", false );
+    TextMargin = ReadKiCadUnitFrac( aProps, "TEXTMARGIN" );
+    AreaColor = ALTIUM_PARSER::ReadInt( aProps, "AREACOLOR", 0 );
+    BorderColor = ALTIUM_PARSER::ReadInt( aProps, "COLOR", 0 );
+
+    IsSolid = ALTIUM_PARSER::ReadBool( aProps, "WORDWRAP", true );
+
+    Alignment = ReadEnum<ASCH_TEXT_FRAME_ALIGNMENT>( aProps, "ALIGNMENT", 1, 3,
                                                      ASCH_TEXT_FRAME_ALIGNMENT::LEFT );
 }
 
@@ -285,24 +315,27 @@ ASCH_POLYLINE::ASCH_POLYLINE( const std::map<wxString, wxString>& aProps )
 {
     wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::POLYLINE );
 
-    ownerindex = ReadOwnerIndex( aProps );
-    ownerpartid = ReadOwnerPartId( aProps );
-    ownerpartdisplaymode = ALTIUM_PARSER::ReadInt( aProps, "OWNERPARTDISPLAYMODE", 0 );
+    OwnerIndex = ReadOwnerIndex( aProps );
+    OwnerPartID = ReadOwnerPartId( aProps );
+    OwnerPartDisplayMode = ALTIUM_PARSER::ReadInt( aProps, "OWNERPARTDISPLAYMODE", 0 );
 
     int locationCount = ALTIUM_PARSER::ReadInt( aProps, "LOCATIONCOUNT", 0 );
 
     for( int i = 1; i <= locationCount; i++ )
     {
         const wxString si = std::to_string( i );
-        points.emplace_back( ReadKiCadUnitFrac( aProps, "X" + si ),
+        Points.emplace_back( ReadKiCadUnitFrac( aProps, "X" + si ),
                              -ReadKiCadUnitFrac( aProps, "Y" + si ) );
     }
 
-    lineWidth = ReadKiCadUnitFrac( aProps, "LINEWIDTH" );
+    LineWidth = ReadKiCadUnitFrac( aProps, "LINEWIDTH" );
+    Color = ALTIUM_PARSER::ReadInt( aProps, "COLOR", 0 );
 
     int linestyleVar = ALTIUM_PARSER::ReadInt( aProps, "LINESTYLEEXT", 0 );
-    linestyleVar     = ALTIUM_PARSER::ReadInt( aProps, "LINESTYLE", linestyleVar ); // overwrite if present
-    linestyle        = linestyleVar >= 0 && linestyleVar <= 3 ?
+
+    // overwrite if present.
+    linestyleVar     = ALTIUM_PARSER::ReadInt( aProps, "LINESTYLE", linestyleVar );
+    LineStyle        = linestyleVar >= 0 && linestyleVar <= 3 ?
                                 static_cast<ASCH_POLYLINE_LINESTYLE>( linestyleVar ) :
                                 ASCH_POLYLINE_LINESTYLE::SOLID;
 }
@@ -312,9 +345,9 @@ ASCH_POLYGON::ASCH_POLYGON( const std::map<wxString, wxString>& aProps )
 {
     wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::POLYGON );
 
-    ownerindex = ReadOwnerIndex( aProps );
-    ownerpartid = ReadOwnerPartId( aProps );
-    ownerpartdisplaymode = ALTIUM_PARSER::ReadInt( aProps, "OWNERPARTDISPLAYMODE", 0 );
+    OwnerIndex = ReadOwnerIndex( aProps );
+    OwnerPartID = ReadOwnerPartId( aProps );
+    OwnerPartDisplayMode = ALTIUM_PARSER::ReadInt( aProps, "OWNERPARTDISPLAYMODE", 0 );
 
     int locationCount = ALTIUM_PARSER::ReadInt( aProps, "LOCATIONCOUNT", 0 );
 
@@ -325,11 +358,11 @@ ASCH_POLYGON::ASCH_POLYGON( const std::map<wxString, wxString>& aProps )
                              -ReadKiCadUnitFrac( aProps, "Y" + si ) );
     }
 
-    lineWidth = ReadKiCadUnitFrac( aProps, "LINEWIDTH" );
-    isSolid   = ALTIUM_PARSER::ReadBool( aProps, "ISSOLID", false );
+    LineWidth = ReadKiCadUnitFrac( aProps, "LINEWIDTH" );
+    IsSolid   = ALTIUM_PARSER::ReadBool( aProps, "ISSOLID", false );
 
-    color     = ALTIUM_PARSER::ReadInt( aProps, "COLOR", 0 );
-    areacolor = ALTIUM_PARSER::ReadInt( aProps, "AREACOLOR", 0 );
+    Color     = ALTIUM_PARSER::ReadInt( aProps, "COLOR", 0 );
+    AreaColor = ALTIUM_PARSER::ReadInt( aProps, "AREACOLOR", 0 );
 }
 
 
@@ -337,43 +370,67 @@ ASCH_ROUND_RECTANGLE::ASCH_ROUND_RECTANGLE( const std::map<wxString, wxString>&
 {
     wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::ROUND_RECTANGLE );
 
-    ownerindex = ReadOwnerIndex( aProps );
-    ownerpartid = ReadOwnerPartId( aProps );
-    ownerpartdisplaymode = ALTIUM_PARSER::ReadInt( aProps, "OWNERPARTDISPLAYMODE", 0 );
+    OwnerIndex = ReadOwnerIndex( aProps );
+    OwnerPartID = ReadOwnerPartId( aProps );
+    OwnerPartDisplayMode = ALTIUM_PARSER::ReadInt( aProps, "OWNERPARTDISPLAYMODE", 0 );
 
-    bottomLeft = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                          -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
-    topRight   = wxPoint( ReadKiCadUnitFrac( aProps, "CORNER.X" ),
-                          -ReadKiCadUnitFrac( aProps, "CORNER.Y" ) );
+    BottomLeft = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                           -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    TopRight = VECTOR2I( ReadKiCadUnitFrac( aProps, "CORNER.X" ),
+                           -ReadKiCadUnitFrac( aProps, "CORNER.Y" ) );
 
-    cornerradius = wxSize( ReadKiCadUnitFrac( aProps, "CORNERXRADIUS" ),
+    CornerRadius = wxSize( ReadKiCadUnitFrac( aProps, "CORNERXRADIUS" ),
                            -ReadKiCadUnitFrac( aProps, "CORNERYRADIUS" ) );
 
-    lineWidth     = ReadKiCadUnitFrac( aProps, "LINEWIDTH" );
-    isSolid       = ALTIUM_PARSER::ReadBool( aProps, "ISSOLID", false );
-    isTransparent = ALTIUM_PARSER::ReadBool( aProps, "TRANSPARENT", false );
+    LineWidth     = ReadKiCadUnitFrac( aProps, "LINEWIDTH" );
+    IsSolid       = ALTIUM_PARSER::ReadBool( aProps, "ISSOLID", false );
+    IsTransparent = ALTIUM_PARSER::ReadBool( aProps, "TRANSPARENT", false );
 
-    color     = ALTIUM_PARSER::ReadInt( aProps, "COLOR", 0 );
-    areacolor = ALTIUM_PARSER::ReadInt( aProps, "AREACOLOR", 0 );
+    Color     = ALTIUM_PARSER::ReadInt( aProps, "COLOR", 0 );
+    AreaColor = ALTIUM_PARSER::ReadInt( aProps, "AREACOLOR", 0 );
 }
 
 
 ASCH_ARC::ASCH_ARC( const std::map<wxString, wxString>& aProps )
 {
-    wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::ARC );
+    m_IsElliptical = ReadRecord( aProps ) == ALTIUM_SCH_RECORD::ELLIPTICAL_ARC;
+    wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::ARC || m_IsElliptical );
 
     ownerindex = ReadOwnerIndex( aProps );
     ownerpartid = ReadOwnerPartId( aProps );
     ownerpartdisplaymode = ALTIUM_PARSER::ReadInt( aProps, "OWNERPARTDISPLAYMODE", 0 );
 
-    center = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                      -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
-    radius = ReadKiCadUnitFrac( aProps, "RADIUS" );
+    m_Center = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                       -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    m_Radius = ReadKiCadUnitFrac( aProps, "RADIUS" );
+    m_SecondaryRadius = m_Radius;
 
-    startAngle = ALTIUM_PARSER::ReadDouble( aProps, "STARTANGLE", 0 );
-    endAngle   = ALTIUM_PARSER::ReadDouble( aProps, "ENDANGLE", 0 );
+    if( m_IsElliptical )
+        m_SecondaryRadius = ReadKiCadUnitFrac( aProps, "SECONDARYRADIUS" );
 
-    lineWidth = ReadKiCadUnitFrac( aProps, "LINEWIDTH" );
+    m_StartAngle = ALTIUM_PARSER::ReadDouble( aProps, "STARTANGLE", 0 );
+    m_EndAngle   = ALTIUM_PARSER::ReadDouble( aProps, "ENDANGLE", 0 );
+
+    m_LineWidth = ReadKiCadUnitFrac( aProps, "LINEWIDTH" );
+}
+
+
+ASCH_ELLIPSE::ASCH_ELLIPSE( const std::map<wxString, wxString>& aProps )
+{
+    wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::ELLIPSE );
+
+    OwnerIndex = ReadOwnerIndex( aProps );
+    OwnerPartID = ReadOwnerPartId( aProps );
+
+    Center = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                       -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+
+    Radius = ReadKiCadUnitFrac( aProps, "RADIUS" );
+    SecondaryRadius = ReadKiCadUnitFrac( aProps, "SECONDARYRADIUS" );
+
+    AreaColor = ALTIUM_PARSER::ReadInt( aProps, "AREACOLOR", 0 );
+    IsNotAccesible = ALTIUM_PARSER::ReadBool( aProps, "ISNOTACCESIBLE", false );
+    IsSolid = ALTIUM_PARSER::ReadBool( aProps, "ISSOLID", false );
 }
 
 
@@ -385,34 +442,124 @@ ASCH_LINE::ASCH_LINE( const std::map<wxString, wxString>& aProps )
     ownerpartid = ReadOwnerPartId( aProps );
     ownerpartdisplaymode = ALTIUM_PARSER::ReadInt( aProps, "OWNERPARTDISPLAYMODE", 0 );
 
-    point1 = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                      -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
-    point2 = wxPoint( ReadKiCadUnitFrac( aProps, "CORNER.X" ),
-                      -ReadKiCadUnitFrac( aProps, "CORNER.Y" ) );
+    point1 = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                       -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    point2 = VECTOR2I( ReadKiCadUnitFrac( aProps, "CORNER.X" ),
+                       -ReadKiCadUnitFrac( aProps, "CORNER.Y" ) );
 
     lineWidth = ReadKiCadUnitFrac( aProps, "LINEWIDTH" );
 }
 
 
+ASCH_SIGNAL_HARNESS::ASCH_SIGNAL_HARNESS( const std::map<wxString, wxString>& aProps )
+{
+    wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::SIGNAL_HARNESS );
+
+    OwnerPartID = ReadOwnerPartId( aProps );
+
+    int locationCount = ALTIUM_PARSER::ReadInt( aProps, "LOCATIONCOUNT", 0 );
+
+    for( int i = 1; i <= locationCount; i++ )
+    {
+        const wxString si = std::to_string( i );
+        Points.emplace_back( ReadKiCadUnitFrac( aProps, "X" + si ),
+                             -ReadKiCadUnitFrac( aProps, "Y" + si ) );
+    }
+
+    IndexInSheet = ALTIUM_PARSER::ReadInt( aProps, "INDEXINSHEET", 0 );
+
+    Color = ALTIUM_PARSER::ReadInt( aProps, "COLOR", 0 );
+    LineWidth = ReadKiCadUnitFrac( aProps, "LINEWIDTH" );
+}
+
+
+ASCH_HARNESS_CONNECTOR::ASCH_HARNESS_CONNECTOR( const std::map<wxString, wxString>& aProps )
+{
+    wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::HARNESS_CONNECTOR );
+
+    OwnerPartID = ReadOwnerPartId( aProps );
+
+    Location = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                         -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    Size = wxSize( ReadKiCadUnitFrac( aProps, "XSIZE" ), ReadKiCadUnitFrac( aProps, "YSIZE" ) );
+
+    Color = ALTIUM_PARSER::ReadInt( aProps, "COLOR", 0 );
+    AreaColor = ALTIUM_PARSER::ReadInt( aProps, "AREACOLOR", 0 );
+
+    IndexInSheet = 0;
+    LineWidth = 0;;
+    LocationPrimaryConnectionPosition = 0;
+}
+
+
+ASCH_HARNESS_ENTRY::ASCH_HARNESS_ENTRY( const std::map<wxString, wxString>& aProps )
+{
+    wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::HARNESS_ENTRY );
+
+    // use SCH_ALTIUM_PLUGIN::m_harnessEntryParent instead, because this property sometimes
+    // does not exist in altium file!
+    // ownerindex = ReadOwnerIndex( aProps );
+
+    OwnerPartID = ReadOwnerPartId( aProps );
+
+    IndexInSheet = ALTIUM_PARSER::ReadInt( aProps, "INDEXINSHEET", 0 );
+
+    DistanceFromTop = ReadKiCadUnitFrac1( aProps, "DISTANCEFROMTOP" );
+
+    Side = ReadEnum<ASCH_SHEET_ENTRY_SIDE>( aProps, "SIDE", 0, 3, ASCH_SHEET_ENTRY_SIDE::LEFT );
+
+    Name = ALTIUM_PARSER::ReadString( aProps, "NAME", "" );
+
+    OwnerIndexAdditionalList = ALTIUM_PARSER::ReadBool( aProps, "OWNERINDEXADDITIONALLIST", true );
+
+    Color = ALTIUM_PARSER::ReadInt( aProps, "COLOR", 0 );
+    AreaColor = ALTIUM_PARSER::ReadInt( aProps, "AREACOLOR", 0 );
+    TextColor = ALTIUM_PARSER::ReadInt( aProps, "TEXTCOLOR", 0 );
+    TextFontID = ALTIUM_PARSER::ReadInt( aProps, "TEXTFONTID", 0 );
+    TextStyle = 0;
+}
+
+
+ASCH_HARNESS_TYPE::ASCH_HARNESS_TYPE( const std::map<wxString, wxString>& aProps )
+{
+    wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::HARNESS_TYPE );
+
+    //ownerindex = ReadOwnerIndex( aProps ); // use SCH_ALTIUM_PLUGIN::m_harnessEntryParent instead!
+    OwnerPartID = ReadOwnerPartId( aProps );
+
+    Text = ALTIUM_PARSER::ReadString( aProps, "TEXT", "" );
+
+    Location = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                         -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+
+    IsHidden = ALTIUM_PARSER::ReadBool( aProps, "ISHIDDEN", false );
+    OwnerIndexAdditionalList = ALTIUM_PARSER::ReadBool( aProps, "OWNERINDEXADDITIONALLIST", true );
+
+    IndexInSheet = ALTIUM_PARSER::ReadInt( aProps, "INDEXINSHEET", 0 );
+    Color = ALTIUM_PARSER::ReadInt( aProps, "COLOR", 0 );
+    FontID = ALTIUM_PARSER::ReadInt( aProps, "TEXTFONTID", 0 );
+}
+
+
 ASCH_RECTANGLE::ASCH_RECTANGLE( const std::map<wxString, wxString>& aProps )
 {
     wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::RECTANGLE );
 
-    ownerindex = ReadOwnerIndex( aProps );
-    ownerpartid = ReadOwnerPartId( aProps );
-    ownerpartdisplaymode = ALTIUM_PARSER::ReadInt( aProps, "OWNERPARTDISPLAYMODE", 0 );
+    OwnerIndex = ReadOwnerIndex( aProps );
+    OwnerPartID = ReadOwnerPartId( aProps );
+    OwnerPartDisplayMode = ALTIUM_PARSER::ReadInt( aProps, "OWNERPARTDISPLAYMODE", 0 );
 
-    bottomLeft = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                          -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
-    topRight   = wxPoint( ReadKiCadUnitFrac( aProps, "CORNER.X" ),
-                          -ReadKiCadUnitFrac( aProps, "CORNER.Y" ) );
+    BottomLeft = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                           -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    TopRight = VECTOR2I( ReadKiCadUnitFrac( aProps, "CORNER.X" ),
+                         -ReadKiCadUnitFrac( aProps, "CORNER.Y" ) );
 
-    lineWidth     = ReadKiCadUnitFrac( aProps, "LINEWIDTH" );
-    isSolid       = ALTIUM_PARSER::ReadBool( aProps, "ISSOLID", false );
-    isTransparent = ALTIUM_PARSER::ReadBool( aProps, "TRANSPARENT", false );
+    LineWidth     = ReadKiCadUnitFrac( aProps, "LINEWIDTH" );
+    IsSolid       = ALTIUM_PARSER::ReadBool( aProps, "ISSOLID", false );
+    IsTransparent = ALTIUM_PARSER::ReadBool( aProps, "TRANSPARENT", false );
 
-    color     = ALTIUM_PARSER::ReadInt( aProps, "COLOR", 0 );
-    areacolor = ALTIUM_PARSER::ReadInt( aProps, "AREACOLOR", 0 );
+    Color     = ALTIUM_PARSER::ReadInt( aProps, "COLOR", 0 );
+    AreaColor = ALTIUM_PARSER::ReadInt( aProps, "AREACOLOR", 0 );
 }
 
 
@@ -420,8 +567,8 @@ ASCH_SHEET_SYMBOL::ASCH_SHEET_SYMBOL( const std::map<wxString, wxString>& aProps
 {
     wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::SHEET_SYMBOL );
 
-    location = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                        -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    location = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                         -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
     size     = wxSize( ReadKiCadUnitFrac( aProps, "XSIZE" ),
                        ReadKiCadUnitFrac( aProps, "YSIZE" ) );
 
@@ -457,8 +604,8 @@ ASCH_POWER_PORT::ASCH_POWER_PORT( const std::map<wxString, wxString>& aProps )
 
     ownerpartid = ReadOwnerPartId( aProps );
 
-    location    = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                           -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    location = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                         -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
 
     orientation = ReadEnum<ASCH_RECORD_ORIENTATION>( aProps, "ORIENTATION", 0, 3,
                                                      ASCH_RECORD_ORIENTATION::RIGHTWARDS );
@@ -470,23 +617,32 @@ ASCH_POWER_PORT::ASCH_POWER_PORT( const std::map<wxString, wxString>& aProps )
                                              ASCH_POWER_PORT_STYLE::CIRCLE );
 }
 
+
 ASCH_PORT::ASCH_PORT( const std::map<wxString, wxString>& aProps )
 {
     wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::PORT );
 
-    ownerpartid = ReadOwnerPartId( aProps );
+    OwnerPartID = ReadOwnerPartId( aProps );
 
-    location = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                        -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    Location = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                         -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
 
-    name = ALTIUM_PARSER::ReadString( aProps, "NAME", "" );
-    harnessType = ALTIUM_PARSER::ReadString( aProps, "HARNESSTYPE", "" );
+    Name = ALTIUM_PARSER::ReadString( aProps, "NAME", "" );
+    HarnessType = ALTIUM_PARSER::ReadString( aProps, "HARNESSTYPE", "" );
 
-    width  = ReadKiCadUnitFrac( aProps, "WIDTH" );
-    height = ReadKiCadUnitFrac( aProps, "HEIGHT" );
+    Width  = ReadKiCadUnitFrac( aProps, "WIDTH" );
+    Height = ReadKiCadUnitFrac( aProps, "HEIGHT" );
 
-    iotype = ReadEnum<ASCH_PORT_IOTYPE>( aProps, "IOTYPE", 0, 3, ASCH_PORT_IOTYPE::UNSPECIFIED );
-    style = ReadEnum<ASCH_PORT_STYLE>( aProps, "STYLE", 0, 7, ASCH_PORT_STYLE::NONE_HORIZONTAL );
+    IOtype = ReadEnum<ASCH_PORT_IOTYPE>( aProps, "IOTYPE", 0, 3, ASCH_PORT_IOTYPE::UNSPECIFIED );
+    Style = ReadEnum<ASCH_PORT_STYLE>( aProps, "STYLE", 0, 7, ASCH_PORT_STYLE::NONE_HORIZONTAL );
+
+    AreaColor = ALTIUM_PARSER::ReadInt( aProps, "AREACOLOR", 0 );
+    Color = ALTIUM_PARSER::ReadInt( aProps, "COLOR", 0 );
+    FontID = ALTIUM_PARSER::ReadInt( aProps, "TEXTFONTID", 0 );
+    TextColor = ALTIUM_PARSER::ReadInt( aProps, "TEXTCOLOR", 0 );
+
+    Alignment = ReadEnum<ASCH_TEXT_FRAME_ALIGNMENT>( aProps, "ALIGNMENT", 1, 3,
+                                                     ASCH_TEXT_FRAME_ALIGNMENT::LEFT );
 }
 
 
@@ -494,11 +650,11 @@ ASCH_NO_ERC::ASCH_NO_ERC( const std::map<wxString, wxString>& aProps )
 {
     wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::NO_ERC );
 
-    location = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                        -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    location = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                         -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
 
     isActive   = ALTIUM_PARSER::ReadBool( aProps, "ISACTIVE", true );
-    supressAll = ALTIUM_PARSER::ReadInt( aProps, "SUPPRESSALL", true );
+    suppressAll = ALTIUM_PARSER::ReadInt( aProps, "SUPPRESSALL", true );
 }
 
 
@@ -508,8 +664,8 @@ ASCH_NET_LABEL::ASCH_NET_LABEL( const std::map<wxString, wxString>& aProps )
 
     text = ALTIUM_PARSER::ReadString( aProps, "TEXT", "" );
 
-    location = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                        -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    location = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                         -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
 
     orientation = ReadEnum<ASCH_RECORD_ORIENTATION>( aProps, "ORIENTATION", 0, 3,
                                                      ASCH_RECORD_ORIENTATION::RIGHTWARDS );
@@ -560,8 +716,8 @@ ASCH_JUNCTION::ASCH_JUNCTION( const std::map<wxString, wxString>& aProps )
 
     ownerpartid = ReadOwnerPartId( aProps );
 
-    location = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                        -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    location = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                         -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
 }
 
 
@@ -570,14 +726,15 @@ ASCH_IMAGE::ASCH_IMAGE( const std::map<wxString, wxString>& aProps )
     wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::IMAGE );
 
     indexinsheet = ALTIUM_PARSER::ReadInt( aProps, "INDEXINSHEET", 0 );
+    ownerindex = ReadOwnerIndex( aProps );
     ownerpartid = ReadOwnerPartId( aProps );
 
     filename = ALTIUM_PARSER::ReadString( aProps, "FILENAME", "" );
 
-    location = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                        -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
-    corner = wxPoint( ReadKiCadUnitFrac( aProps, "CORNER.X" ),
-                      -ReadKiCadUnitFrac( aProps, "CORNER.Y" ) );
+    location = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                         -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    corner = VECTOR2I( ReadKiCadUnitFrac( aProps, "CORNER.X" ),
+                       -ReadKiCadUnitFrac( aProps, "CORNER.Y" ) );
 
     embedimage = ALTIUM_PARSER::ReadBool( aProps, "EMBEDIMAGE", false );
     keepaspect = ALTIUM_PARSER::ReadBool( aProps, "KEEPASPECT", false );
@@ -590,17 +747,20 @@ ASCH_SHEET_FONT::ASCH_SHEET_FONT( const std::map<wxString, wxString>& aProps, in
 
     const wxString sid = std::to_string( aId );
 
-    fontname = ALTIUM_PARSER::ReadString( aProps, "FONTNAME" + sid, "" );
+    FontName = ALTIUM_PARSER::ReadString( aProps, "FONTNAME" + sid, "" );
 
-    size     = ReadKiCadUnitFrac( aProps, "SIZE" + sid );
-    rotation = ALTIUM_PARSER::ReadInt( aProps, "ROTATION" + sid, 0 );
+    Size     = ReadKiCadUnitFrac( aProps, "SIZE" + sid );
+    Rotation = ALTIUM_PARSER::ReadInt( aProps, "ROTATION" + sid, 0 );
 
-    italic    = ALTIUM_PARSER::ReadBool( aProps, "ITALIC" + sid, false );
-    bold      = ALTIUM_PARSER::ReadBool( aProps, "BOLD" + sid, false );
-    underline = ALTIUM_PARSER::ReadBool( aProps, "UNDERLINE" + sid, false );
+    Italic    = ALTIUM_PARSER::ReadBool( aProps, "ITALIC" + sid, false );
+    Bold      = ALTIUM_PARSER::ReadBool( aProps, "BOLD" + sid, false );
+    Underline = ALTIUM_PARSER::ReadBool( aProps, "UNDERLINE" + sid, false );
+
+    AreaColor = ALTIUM_PARSER::ReadInt( aProps, "AREACOLOR" + sid, 0 );
 }
 
-wxPoint ASchSheetGetSize( ASCH_SHEET_SIZE aSheetSize )
+
+VECTOR2I ASchSheetGetSize( ASCH_SHEET_SIZE aSheetSize )
 {
     // From: https://github.com/vadmium/python-altium/blob/master/format.md#sheet
     switch( aSheetSize )
@@ -655,8 +815,8 @@ ASCH_SHEET_NAME::ASCH_SHEET_NAME( const std::map<wxString, wxString>& aProps )
     orientation = ReadEnum<ASCH_RECORD_ORIENTATION>( aProps, "ORIENTATION", 0, 3,
                                                      ASCH_RECORD_ORIENTATION::RIGHTWARDS );
 
-    location = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                        -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    location = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                         -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
 
     isHidden = ALTIUM_PARSER::ReadBool( aProps, "ISHIDDEN", false );
 }
@@ -674,8 +834,8 @@ ASCH_FILE_NAME::ASCH_FILE_NAME( const std::map<wxString, wxString>& aProps )
     orientation = ReadEnum<ASCH_RECORD_ORIENTATION>( aProps, "ORIENTATION", 0, 3,
                                                      ASCH_RECORD_ORIENTATION::RIGHTWARDS );
 
-    location = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                        -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    location = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                         -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
 
     isHidden = ALTIUM_PARSER::ReadBool( aProps, "ISHIDDEN", false );
 }
@@ -697,8 +857,8 @@ ASCH_DESIGNATOR::ASCH_DESIGNATOR( const std::map<wxString, wxString>& aProps )
     orientation = ReadEnum<ASCH_RECORD_ORIENTATION>( aProps, "ORIENTATION", 0, 3,
                                                      ASCH_RECORD_ORIENTATION::RIGHTWARDS );
 
-    location = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                        -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    location = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                         -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
 }
 
 
@@ -721,14 +881,15 @@ ASCH_IMPLEMENTATION_LIST::ASCH_IMPLEMENTATION_LIST( const std::map<wxString, wxS
     ownerindex = ReadOwnerIndex( aProps );
 }
 
+
 ASCH_BUS_ENTRY::ASCH_BUS_ENTRY( const std::map<wxString, wxString>& aProps )
 {
     wxASSERT( ReadRecord( aProps ) == ALTIUM_SCH_RECORD::BUS_ENTRY );
 
-    location = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                        -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
-    corner   = wxPoint( ReadKiCadUnitFrac( aProps, "CORNER.X" ),
-                        -ReadKiCadUnitFrac( aProps, "CORNER.Y" ) );
+    location = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                         -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    corner = VECTOR2I( ReadKiCadUnitFrac( aProps, "CORNER.X" ),
+                       -ReadKiCadUnitFrac( aProps, "CORNER.Y" ) );
 }
 
 
@@ -739,8 +900,8 @@ ASCH_PARAMETER::ASCH_PARAMETER( const std::map<wxString, wxString>& aProps )
     ownerindex = ReadOwnerIndex( aProps );
     ownerpartid = ReadOwnerPartId( aProps );
 
-    location = wxPoint( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
-                        -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
+    location = VECTOR2I( ReadKiCadUnitFrac( aProps, "LOCATION.X" ),
+                         -ReadKiCadUnitFrac( aProps, "LOCATION.Y" ) );
 
     justification = ReadEnum<ASCH_LABEL_JUSTIFICATION>( aProps, "JUSTIFICATION", 0, 8,
                                                         ASCH_LABEL_JUSTIFICATION::BOTTOM_LEFT );
