@@ -35,7 +35,7 @@ void CADSTAR_SCH_ARCHIVE_PARSER::Parse()
     if( m_progressReporter )
         m_progressReporter->BeginPhase( 0 ); // Read file
 
-    XNODE* fileRootNode = LoadArchiveFile( Filename, wxT( "CADSTARSCM" ), m_progressReporter );
+    m_rootNode = LoadArchiveFile( Filename, wxT( "CADSTARSCM" ), m_progressReporter );
 
     if( m_progressReporter )
     {
@@ -44,13 +44,13 @@ void CADSTAR_SCH_ARCHIVE_PARSER::Parse()
         std::vector<wxString> subNodeChildrenToCount = { wxT( "LIBRARY" ), wxT( "PARTS" ),
                                                          wxT( "SCHEMATIC" ) };
 
-        long numOfSteps = GetNumberOfStepsForReporting( fileRootNode, subNodeChildrenToCount );
+        long numOfSteps = GetNumberOfStepsForReporting( m_rootNode, subNodeChildrenToCount );
         m_progressReporter->SetMaxProgress( numOfSteps );
     }
 
     m_context.CheckPointCallback = [&](){ checkPoint(); };
 
-    XNODE* cNode = fileRootNode->GetChildren();
+    XNODE* cNode = m_rootNode->GetChildren();
 
     if( !cNode )
         THROW_MISSING_NODE_IO_ERROR( wxT( "HEADER" ), wxT( "CADSTARSCM" ) );
@@ -145,7 +145,8 @@ void CADSTAR_SCH_ARCHIVE_PARSER::Parse()
         checkPoint();
     }
 
-    delete fileRootNode;
+    delete m_rootNode;
+    m_rootNode = nullptr;
 }
 
 
