@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2016 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 1992-2021 Kicad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2023 Kicad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -27,6 +27,7 @@
 
 #include <bitmaps.h>
 #include <core/mirror.h>
+#include <core/kicad_algo.h>
 #include <sch_draw_panel.h>
 #include <trigo.h>
 #include <sch_edit_frame.h>
@@ -45,18 +46,14 @@
 #include <pgm_base.h>
 #include <wx/log.h>
 
-#define SHEET_NAME_CANONICAL "Sheet name"
-#define SHEET_FILE_CANONICAL "Sheet file"
+// N.B. Do not change these values without transitioning the file format
+#define SHEET_NAME_CANONICAL "Sheetname"
+#define SHEET_FILE_CANONICAL "Sheetfile"
 #define USER_FIELD_CANONICAL "Field%d"
 
 
 const wxString SCH_SHEET::GetDefaultFieldName( int aFieldNdx, bool aTranslated )
 {
-    static void* locale = nullptr;
-    static wxString sheetnameDefault;
-    static wxString sheetfilenameDefault;
-    static wxString userFieldDefault;
-
     if( !aTranslated )
     {
         switch( aFieldNdx )
@@ -67,27 +64,17 @@ const wxString SCH_SHEET::GetDefaultFieldName( int aFieldNdx, bool aTranslated )
         }
     }
 
-    // Fetching translations can take a surprising amount of time when loading libraries,
-    // so only do it when necessary.
-    if( Pgm().GetLocale() != locale )
-    {
-        sheetnameDefault     = _( SHEET_NAME_CANONICAL );
-        sheetfilenameDefault = _( SHEET_FILE_CANONICAL );
-        userFieldDefault     = _( USER_FIELD_CANONICAL );
-        locale = Pgm().GetLocale();
-    }
-
     // Fixed values for the mandatory fields
     switch( aFieldNdx )
     {
-    case  SHEETNAME:     return sheetnameDefault;
-    case  SHEETFILENAME: return sheetfilenameDefault;
-    default:             return wxString::Format( userFieldDefault, aFieldNdx );
+    case  SHEETNAME:     return _( SHEET_NAME_CANONICAL );
+    case  SHEETFILENAME: return _( SHEET_FILE_CANONICAL );
+    default:             return wxString::Format( _( USER_FIELD_CANONICAL ), aFieldNdx );
     }
 }
 
 
-SCH_SHEET::SCH_SHEET( EDA_ITEM* aParent, const wxPoint& aPos, wxSize aSize,
+SCH_SHEET::SCH_SHEET( EDA_ITEM* aParent, const VECTOR2I& aPos, wxSize aSize,
                       FIELDS_AUTOPLACED aAutoplaceFields ) :
         SCH_ITEM( aParent, SCH_SHEET_T )
 {
@@ -206,7 +193,7 @@ int SCH_SHEET::GetScreenCount() const
 
 bool SCH_SHEET::IsRootSheet() const
 {
-    wxCHECK_MSG( Schematic(), false, wxT( "Can't call IsRootSheet without setting a schematic" ) );
+    wxCHECK_MSG( Schematic(), false, "Can't call IsRootSheet without setting a schematic" );
 
     return &Schematic()->Root() == this;
 }
@@ -214,20 +201,50 @@ bool SCH_SHEET::IsRootSheet() const
 
 void SCH_SHEET::GetContextualTextVars( wxArrayString* aVars ) const
 {
+    auto add =
+            [&]( const wxString& aVar )
+            {
+                if( !alg::contains( *aVars, aVar ) )
+                    aVars->push_back( aVar );
+            };
+
     for( int i = 0; i < SHEET_MANDATORY_FIELDS; ++i )
-        aVars->push_back( m_fields[i].GetCanonicalName().Upper() );
+        add( m_fields[i].GetCanonicalName().Upper() );
 
     for( size_t i = SHEET_MANDATORY_FIELDS; i < m_fields.size(); ++i )
-        aVars->push_back( m_fields[i].GetName() );
+        add( m_fields[i].GetName() );
+
+    SCH_SHEET_PATH sheetPath = findSelf();
+
+    if( sheetPath.size() >= 2 )
+    {
+        sheetPath.pop_back();
+        sheetPath.Last()->GetContextualTextVars( aVars );
+    }
+    else if( Schematic() )
+    {
+        Schematic()->GetContextualTextVars( aVars );
+    }
+
+    add( wxT( "#" ) );
+    add( wxT( "##" ) );
+    add( wxT( "SHEETPATH" ) );
 
-    aVars->push_back( wxT( "#" ) );
-    aVars->push_back( wxT( "##" ) );
     m_screen->GetTitleBlock().GetContextualTextVars( aVars );
 }
 
 
 bool SCH_SHEET::ResolveTextVar( wxString* token, int aDepth ) const
 {
+    if( !Schematic() )
+        return false;
+
+    if( token->Contains( ':' ) )
+    {
+        if( Schematic()->ResolveCrossReference( token, aDepth + 1 ) )
+            return true;
+    }
+
     for( int i = 0; i < SHEET_MANDATORY_FIELDS; ++i )
     {
         if( token->IsSameAs( m_fields[i].GetCanonicalName().Upper() ) )
@@ -257,14 +274,8 @@ bool SCH_SHEET::ResolveTextVar( wxString* token, int aDepth ) const
 
     if( token->IsSameAs( wxT( "#" ) ) )
     {
-        for( const SCH_SHEET_PATH& sheet : Schematic()->GetSheets() )
-        {
-            if( sheet.Last() == this )   // Current sheet path found
-            {
-                *token = wxString::Format( wxT( "%s" ), sheet.GetPageNumber() );
-                return true;
-            }
-        }
+        *token = wxString::Format( "%s", findSelf().GetPageNumber() );
+        return true;
     }
     else if( token->IsSameAs( wxT( "##" ) ) )
     {
@@ -272,17 +283,33 @@ bool SCH_SHEET::ResolveTextVar( wxString* token, int aDepth ) const
         *token = wxString::Format( wxT( "%d" ), (int) sheetList.size() );
         return true;
     }
+    else if( token->IsSameAs( wxT( "SHEETPATH" ) ) )
+    {
+        *token = findSelf().PathHumanReadable();
+        return true;
+    }
+
+    // See if parent can resolve it (these will recurse to ancestors)
+
+    SCH_SHEET_PATH sheetPath = findSelf();
+
+    if( sheetPath.size() >= 2 )
+    {
+        sheetPath.pop_back();
+
+        if( sheetPath.Last()->ResolveTextVar( token, aDepth + 1 ) )
+            return true;
+    }
+    else
+    {
+        if( Schematic()->ResolveTextVar( token, aDepth + 1 ) )
+            return true;
+    }
 
     return false;
 }
 
 
-bool SCH_SHEET::UsesDefaultStroke() const
-{
-    return m_borderWidth == 0 && m_borderColor == COLOR4D::UNSPECIFIED;
-}
-
-
 void SCH_SHEET::SwapData( SCH_ITEM* aItem )
 {
     wxCHECK_RET( aItem->Type() == SCH_SHEET_T,
@@ -379,7 +406,7 @@ bool SCH_SHEET::HasPin( const wxString& aName ) const
 }
 
 
-bool SCH_SHEET::doIsConnected( const wxPoint& aPosition ) const
+bool SCH_SHEET::doIsConnected( const VECTOR2I& aPosition ) const
 {
     for( SCH_SHEET_PIN* sheetPin : m_pins )
     {
@@ -438,7 +465,7 @@ bool SCH_SHEET::HasUndefinedPins() const
 
 int bumpToNextGrid( const int aVal, const int aDirection )
 {
-    constexpr int gridSize = Mils2iu( 50 );
+    constexpr int gridSize = schIUScale.MilsToIU( 50 );
 
     int base = aVal / gridSize;
     int excess = abs( aVal % gridSize );
@@ -469,7 +496,7 @@ int SCH_SHEET::GetMinWidth( bool aFromLeft ) const
 
         if( edge == SHEET_SIDE::TOP || edge == SHEET_SIDE::BOTTOM )
         {
-            EDA_RECT pinRect = m_pins[i]->GetBoundingBox();
+            BOX2I pinRect = m_pins[i]->GetBoundingBox();
 
             pinsLeft = std::min( pinsLeft, pinRect.GetLeft() );
             pinsRight = std::max( pinsRight, pinRect.GetRight() );
@@ -488,7 +515,7 @@ int SCH_SHEET::GetMinWidth( bool aFromLeft ) const
     else
         pinMinWidth = m_pos.x + m_size.x - pinsLeft;
 
-    return std::max( pinMinWidth, Mils2iu( MIN_SHEET_WIDTH ) );
+    return std::max( pinMinWidth, schIUScale.MilsToIU( MIN_SHEET_WIDTH ) );
 }
 
 
@@ -503,7 +530,7 @@ int SCH_SHEET::GetMinHeight( bool aFromTop ) const
 
         if( edge == SHEET_SIDE::RIGHT || edge == SHEET_SIDE::LEFT )
         {
-            EDA_RECT pinRect = m_pins[i]->GetBoundingBox();
+            BOX2I pinRect = m_pins[i]->GetBoundingBox();
 
             pinsTop = std::min( pinsTop, pinRect.GetTop() );
             pinsBottom = std::max( pinsBottom, pinRect.GetBottom() );
@@ -522,7 +549,7 @@ int SCH_SHEET::GetMinHeight( bool aFromTop ) const
     else
         pinMinHeight = m_pos.y + m_size.y - pinsTop;
 
-    return std::max( pinMinHeight, Mils2iu( MIN_SHEET_HEIGHT ) );
+    return std::max( pinMinHeight, schIUScale.MilsToIU( MIN_SHEET_HEIGHT ) );
 }
 
 
@@ -552,7 +579,7 @@ void SCH_SHEET::CleanupSheet()
 }
 
 
-SCH_SHEET_PIN* SCH_SHEET::GetPin( const wxPoint& aPosition )
+SCH_SHEET_PIN* SCH_SHEET::GetPin( const VECTOR2I& aPosition )
 {
     for( SCH_SHEET_PIN* pin : m_pins )
     {
@@ -572,29 +599,29 @@ int SCH_SHEET::GetPenWidth() const
     if( Schematic() )
         return Schematic()->Settings().m_DefaultLineWidth;
 
-    return Mils2iu( DEFAULT_LINE_WIDTH_MILS );
+    return schIUScale.MilsToIU( DEFAULT_LINE_WIDTH_MILS );
 }
 
 
-void SCH_SHEET::AutoplaceFields( SCH_SCREEN* aScreen, bool aManual )
+void SCH_SHEET::AutoplaceFields( SCH_SCREEN* aScreen, bool /* aManual */ )
 {
-    wxSize textSize = m_fields[ SHEETNAME ].GetTextSize();
-    int    borderMargin = KiROUND( GetPenWidth() / 2.0 ) + 4;
-    int    margin = borderMargin + KiROUND( std::max( textSize.x, textSize.y ) * 0.5 );
+    VECTOR2I textSize = m_fields[SHEETNAME].GetTextSize();
+    int      borderMargin = KiROUND( GetPenWidth() / 2.0 ) + 4;
+    int      margin = borderMargin + KiROUND( std::max( textSize.x, textSize.y ) * 0.5 );
 
     if( IsVerticalOrientation() )
     {
-        m_fields[ SHEETNAME ].SetTextPos( m_pos + wxPoint( -margin, m_size.y ) );
-        m_fields[ SHEETNAME ].SetHorizJustify( GR_TEXT_HJUSTIFY_LEFT );
-        m_fields[ SHEETNAME ].SetVertJustify(GR_TEXT_VJUSTIFY_BOTTOM );
-        m_fields[ SHEETNAME ].SetTextAngle( TEXT_ANGLE_VERT );
+        m_fields[SHEETNAME].SetTextPos( m_pos + VECTOR2I( -margin, m_size.y ) );
+        m_fields[ SHEETNAME ].SetHorizJustify( GR_TEXT_H_ALIGN_LEFT );
+        m_fields[ SHEETNAME ].SetVertJustify( GR_TEXT_V_ALIGN_BOTTOM );
+        m_fields[ SHEETNAME ].SetTextAngle( ANGLE_VERTICAL );
     }
     else
     {
-        m_fields[ SHEETNAME ].SetTextPos( m_pos + wxPoint( 0, -margin ) );
-        m_fields[ SHEETNAME ].SetHorizJustify( GR_TEXT_HJUSTIFY_LEFT );
-        m_fields[ SHEETNAME ].SetVertJustify(GR_TEXT_VJUSTIFY_BOTTOM );
-        m_fields[ SHEETNAME ].SetTextAngle( TEXT_ANGLE_HORIZ );
+        m_fields[SHEETNAME].SetTextPos( m_pos + VECTOR2I( 0, -margin ) );
+        m_fields[ SHEETNAME ].SetHorizJustify( GR_TEXT_H_ALIGN_LEFT );
+        m_fields[ SHEETNAME ].SetVertJustify( GR_TEXT_V_ALIGN_BOTTOM );
+        m_fields[ SHEETNAME ].SetTextAngle( ANGLE_HORIZONTAL );
     }
 
     textSize = m_fields[ SHEETFILENAME ].GetTextSize();
@@ -602,17 +629,17 @@ void SCH_SHEET::AutoplaceFields( SCH_SCREEN* aScreen, bool aManual )
 
     if( IsVerticalOrientation() )
     {
-        m_fields[ SHEETFILENAME ].SetTextPos( m_pos + wxPoint( m_size.x + margin, m_size.y ) );
-        m_fields[ SHEETFILENAME ].SetHorizJustify( GR_TEXT_HJUSTIFY_LEFT );
-        m_fields[ SHEETFILENAME ].SetVertJustify(GR_TEXT_VJUSTIFY_TOP );
-        m_fields[ SHEETFILENAME ].SetTextAngle( TEXT_ANGLE_VERT );
+        m_fields[SHEETFILENAME].SetTextPos( m_pos + VECTOR2I( m_size.x + margin, m_size.y ) );
+        m_fields[ SHEETFILENAME ].SetHorizJustify( GR_TEXT_H_ALIGN_LEFT );
+        m_fields[ SHEETFILENAME ].SetVertJustify( GR_TEXT_V_ALIGN_TOP );
+        m_fields[ SHEETFILENAME ].SetTextAngle( ANGLE_VERTICAL );
     }
     else
     {
-        m_fields[ SHEETFILENAME ].SetTextPos( m_pos + wxPoint( 0, m_size.y + margin ) );
-        m_fields[ SHEETFILENAME ].SetHorizJustify( GR_TEXT_HJUSTIFY_LEFT );
-        m_fields[ SHEETFILENAME ].SetVertJustify(GR_TEXT_VJUSTIFY_TOP );
-        m_fields[ SHEETFILENAME ].SetTextAngle( TEXT_ANGLE_HORIZ );
+        m_fields[SHEETFILENAME].SetTextPos( m_pos + VECTOR2I( 0, m_size.y + margin ) );
+        m_fields[ SHEETFILENAME ].SetHorizJustify( GR_TEXT_H_ALIGN_LEFT );
+        m_fields[ SHEETFILENAME ].SetVertJustify( GR_TEXT_V_ALIGN_TOP );
+        m_fields[ SHEETFILENAME ].SetTextAngle( ANGLE_HORIZONTAL );
     }
 
     m_fieldsAutoplaced = FIELDS_AUTOPLACED_AUTO;
@@ -621,18 +648,21 @@ void SCH_SHEET::AutoplaceFields( SCH_SCREEN* aScreen, bool aManual )
 
 void SCH_SHEET::ViewGetLayers( int aLayers[], int& aCount ) const
 {
-    aCount      = 4;
-    aLayers[0]  = LAYER_HIERLABEL;
-    aLayers[1]  = LAYER_SHEET;
-    aLayers[2]  = LAYER_SHEET_BACKGROUND;
-    aLayers[3]  = LAYER_SELECTION_SHADOWS;
+    aCount     = 7;
+    aLayers[0] = LAYER_HIERLABEL;
+    aLayers[1] = LAYER_SHEETNAME;
+    aLayers[2] = LAYER_SHEETFILENAME;
+    aLayers[3] = LAYER_SHEETFIELDS;
+    aLayers[4] = LAYER_SHEET;
+    aLayers[5] = LAYER_SHEET_BACKGROUND;
+    aLayers[6] = LAYER_SELECTION_SHADOWS;
 }
 
 
-const EDA_RECT SCH_SHEET::GetBodyBoundingBox() const
+const BOX2I SCH_SHEET::GetBodyBoundingBox() const
 {
-    wxPoint  end;
-    EDA_RECT box( m_pos, m_size );
+    VECTOR2I end;
+    BOX2I    box( m_pos, m_size );
     int      lineWidth = GetPenWidth();
     int      textLength = 0;
 
@@ -650,20 +680,20 @@ const EDA_RECT SCH_SHEET::GetBodyBoundingBox() const
 }
 
 
-const EDA_RECT SCH_SHEET::GetBoundingBox() const
+const BOX2I SCH_SHEET::GetBoundingBox() const
 {
-    EDA_RECT box = GetBodyBoundingBox();
+    BOX2I bbox = GetBodyBoundingBox();
 
     for( const SCH_FIELD& field : m_fields )
-        box.Merge( field.GetBoundingBox() );
+        bbox.Merge( field.GetBoundingBox() );
 
-    return box;
+    return bbox;
 }
 
 
-wxPoint SCH_SHEET::GetRotationCenter() const
+VECTOR2I SCH_SHEET::GetRotationCenter() const
 {
-    EDA_RECT box( m_pos, m_size );
+    BOX2I box( m_pos, m_size );
     return box.GetCenter();
 }
 
@@ -735,14 +765,12 @@ bool SCH_SHEET::LocatePathOfScreen( SCH_SCREEN* aScreen, SCH_SHEET_PATH* aList )
         if( m_screen == aScreen )
             return true;
 
-        for( auto item : m_screen->Items().OfType( SCH_SHEET_T ) )
+        for( EDA_ITEM* item : m_screen->Items().OfType( SCH_SHEET_T ) )
         {
             SCH_SHEET* sheet = static_cast<SCH_SHEET*>( item );
 
             if( sheet->LocatePathOfScreen( aScreen, aList ) )
-            {
                 return true;
-            }
         }
 
         aList->pop_back();
@@ -775,14 +803,14 @@ void SCH_SHEET::GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_I
         SCH_SHEET_PATH path = schframe->GetCurrentSheet();
         path.push_back( this );
 
-        aList.emplace_back( _( "Hierarchical Path" ), path.PathHumanReadable( false ) );
+        aList.emplace_back( _( "Hierarchical Path" ), path.PathHumanReadable( false, true ) );
     }
 
     aList.emplace_back( _( "File Name" ), m_fields[ SHEETFILENAME ].GetText() );
 }
 
 
-void SCH_SHEET::Move( const wxPoint& aMoveVector )
+void SCH_SHEET::Move( const VECTOR2I& aMoveVector )
 {
     m_pos += aMoveVector;
 
@@ -794,12 +822,12 @@ void SCH_SHEET::Move( const wxPoint& aMoveVector )
 }
 
 
-void SCH_SHEET::Rotate( const wxPoint& aCenter )
+void SCH_SHEET::Rotate( const VECTOR2I& aCenter )
 {
-    wxPoint prev = m_pos;
+    VECTOR2I prev = m_pos;
 
-    RotatePoint( &m_pos, aCenter, 900 );
-    RotatePoint( &m_size.x, &m_size.y, 900 );
+    RotatePoint( m_pos, aCenter, ANGLE_90 );
+    RotatePoint( &m_size.x, &m_size.y, ANGLE_90 );
 
     if( m_size.x < 0 )
     {
@@ -827,7 +855,7 @@ void SCH_SHEET::Rotate( const wxPoint& aCenter )
         // Move the fields to the new position because the parent itself has moved.
         for( SCH_FIELD& field : m_fields )
         {
-            wxPoint pos = field.GetTextPos();
+            VECTOR2I pos = field.GetTextPos();
             pos.x -= prev.x - m_pos.x;
             pos.y -= prev.y - m_pos.y;
             field.SetTextPos( pos );
@@ -849,7 +877,7 @@ void SCH_SHEET::MirrorVertically( int aCenter )
 
     for( SCH_FIELD& field : m_fields )
     {
-        wxPoint pos = field.GetTextPos();
+        VECTOR2I pos = field.GetTextPos();
         pos.y -= dy;
         field.SetTextPos( pos );
     }
@@ -869,14 +897,14 @@ void SCH_SHEET::MirrorHorizontally( int aCenter )
 
     for( SCH_FIELD& field : m_fields )
     {
-        wxPoint pos = field.GetTextPos();
+        VECTOR2I pos = field.GetTextPos();
         pos.x -= dx;
         field.SetTextPos( pos );
     }
 }
 
 
-void SCH_SHEET::SetPosition( const wxPoint& aPosition )
+void SCH_SHEET::SetPosition( const VECTOR2I& aPosition )
 {
     // Remember the sheet and all pin sheet positions must be
     // modified. So use Move function to do that.
@@ -901,10 +929,8 @@ void SCH_SHEET::Resize( const wxSize& aSize )
 }
 
 
-bool SCH_SHEET::Matches( const wxFindReplaceData& aSearchData, void* aAuxData ) const
+bool SCH_SHEET::Matches( const EDA_SEARCH_DATA& aSearchData, void* aAuxData ) const
 {
-    wxLogTrace( traceFindItem, wxT( "  item " ) + GetSelectMenuText( EDA_UNITS::MILLIMETRES ) );
-
     // Sheets are searchable via the child field and pin item text.
     return false;
 }
@@ -922,6 +948,26 @@ void SCH_SHEET::renumberPins()
 }
 
 
+SCH_SHEET_PATH SCH_SHEET::findSelf() const
+{
+    wxCHECK_MSG( Schematic(), SCH_SHEET_PATH(), "Can't call findSelf without a schematic" );
+
+    SCH_SHEET_PATH sheetPath = Schematic()->CurrentSheet();
+
+    while( !sheetPath.empty() && sheetPath.Last() != this )
+        sheetPath.pop_back();
+
+    if( sheetPath.empty() )
+    {
+        // If we weren't in the hierarchy, then we must be a child of the current sheet.
+        sheetPath = Schematic()->CurrentSheet();
+        sheetPath.push_back( const_cast<SCH_SHEET*>( this ) );
+    }
+
+    return sheetPath;
+}
+
+
 void SCH_SHEET::GetEndPoints( std::vector <DANGLING_END_ITEM>& aItemList )
 {
     for( SCH_SHEET_PIN* sheetPin : m_pins )
@@ -946,9 +992,9 @@ bool SCH_SHEET::UpdateDanglingState( std::vector<DANGLING_END_ITEM>& aItemList,
 }
 
 
-std::vector<wxPoint> SCH_SHEET::GetConnectionPoints() const
+std::vector<VECTOR2I> SCH_SHEET::GetConnectionPoints() const
 {
-    std::vector<wxPoint> retval;
+    std::vector<VECTOR2I> retval;
 
     for( SCH_SHEET_PIN* sheetPin : m_pins )
         retval.push_back( sheetPin->GetPosition() );
@@ -957,41 +1003,40 @@ std::vector<wxPoint> SCH_SHEET::GetConnectionPoints() const
 }
 
 
-SEARCH_RESULT SCH_SHEET::Visit( INSPECTOR aInspector, void* testData, const KICAD_T aFilterTypes[] )
+INSPECT_RESULT SCH_SHEET::Visit( INSPECTOR aInspector, void* testData,
+                                 const std::vector<KICAD_T>& aScanTypes )
 {
-    KICAD_T stype;
-
-    for( const KICAD_T* p = aFilterTypes;  (stype = *p) != EOT;   ++p )
+    for( KICAD_T scanType : aScanTypes )
     {
         // If caller wants to inspect my type
-        if( stype == SCH_LOCATE_ANY_T || stype == Type() )
+        if( scanType == SCH_LOCATE_ANY_T || scanType == Type() )
         {
-            if( SEARCH_RESULT::QUIT == aInspector( this, nullptr ) )
-                return SEARCH_RESULT::QUIT;
+            if( INSPECT_RESULT::QUIT == aInspector( this, nullptr ) )
+                return INSPECT_RESULT::QUIT;
         }
 
-        if( stype == SCH_LOCATE_ANY_T || stype == SCH_FIELD_T )
+        if( scanType == SCH_LOCATE_ANY_T || scanType == SCH_FIELD_T )
         {
             // Test the sheet fields.
             for( SCH_FIELD& field : m_fields )
             {
-                if( SEARCH_RESULT::QUIT == aInspector( &field, this ) )
-                    return SEARCH_RESULT::QUIT;
+                if( INSPECT_RESULT::QUIT == aInspector( &field, this ) )
+                    return INSPECT_RESULT::QUIT;
             }
         }
 
-        if( stype == SCH_LOCATE_ANY_T || stype == SCH_SHEET_PIN_T )
+        if( scanType == SCH_LOCATE_ANY_T || scanType == SCH_SHEET_PIN_T )
         {
             // Test the sheet labels.
             for( SCH_SHEET_PIN* sheetPin : m_pins )
             {
-                if( SEARCH_RESULT::QUIT == aInspector( sheetPin, this ) )
-                    return SEARCH_RESULT::QUIT;
+                if( INSPECT_RESULT::QUIT == aInspector( sheetPin, this ) )
+                    return INSPECT_RESULT::QUIT;
             }
         }
     }
 
-    return SEARCH_RESULT::CONTINUE;
+    return INSPECT_RESULT::CONTINUE;
 }
 
 
@@ -1005,7 +1050,7 @@ void SCH_SHEET::RunOnChildren( const std::function<void( SCH_ITEM* )>& aFunction
 }
 
 
-wxString SCH_SHEET::GetSelectMenuText( EDA_UNITS aUnits ) const
+wxString SCH_SHEET::GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const
 {
     return wxString::Format( _( "Hierarchical Sheet %s" ),
                              m_fields[ SHEETNAME ].GetText() );
@@ -1018,9 +1063,9 @@ BITMAPS SCH_SHEET::GetMenuImage() const
 }
 
 
-bool SCH_SHEET::HitTest( const wxPoint& aPosition, int aAccuracy ) const
+bool SCH_SHEET::HitTest( const VECTOR2I& aPosition, int aAccuracy ) const
 {
-    EDA_RECT rect = GetBodyBoundingBox();
+    BOX2I rect = GetBodyBoundingBox();
 
     rect.Inflate( aAccuracy );
 
@@ -1028,9 +1073,9 @@ bool SCH_SHEET::HitTest( const wxPoint& aPosition, int aAccuracy ) const
 }
 
 
-bool SCH_SHEET::HitTest( const EDA_RECT& aRect, bool aContained, int aAccuracy ) const
+bool SCH_SHEET::HitTest( const BOX2I& aRect, bool aContained, int aAccuracy ) const
 {
-    EDA_RECT rect = aRect;
+    BOX2I rect = aRect;
 
     rect.Inflate( aAccuracy );
 
@@ -1041,10 +1086,11 @@ bool SCH_SHEET::HitTest( const EDA_RECT& aRect, bool aContained, int aAccuracy )
 }
 
 
-void SCH_SHEET::Plot( PLOTTER* aPlotter ) const
+void SCH_SHEET::Plot( PLOTTER* aPlotter, bool aBackground ) const
 {
-    wxString msg;
-    wxPoint  pos;
+    if( aBackground && !aPlotter->GetColorMode() )
+        return;
+
     auto*    settings = dynamic_cast<KIGFX::SCH_RENDER_SETTINGS*>( aPlotter->RenderSettings() );
     bool     override = settings ? settings->m_OverrideItemColors : false;
     COLOR4D  borderColor = GetBorderColor();
@@ -1056,34 +1102,47 @@ void SCH_SHEET::Plot( PLOTTER* aPlotter ) const
     if( override || backgroundColor == COLOR4D::UNSPECIFIED )
         backgroundColor = aPlotter->RenderSettings()->GetLayerColor( LAYER_SHEET_BACKGROUND );
 
-    // Do not fill shape in B&W mode, otherwise texts are unreadable
-    bool fill = aPlotter->GetColorMode();
-
-    if( fill )
+    if( aBackground )
     {
         aPlotter->SetColor( backgroundColor );
         aPlotter->Rect( m_pos, m_pos + m_size, FILL_T::FILLED_SHAPE, 1 );
     }
+    else
+    {
+        aPlotter->SetColor( borderColor );
 
-    aPlotter->SetColor( borderColor );
+        int penWidth = std::max( GetPenWidth(), aPlotter->RenderSettings()->GetMinPenWidth() );
+        aPlotter->Rect( m_pos, m_pos + m_size, FILL_T::NO_FILL, penWidth );
+    }
 
-    int penWidth = std::max( GetPenWidth(), aPlotter->RenderSettings()->GetMinPenWidth() );
-    aPlotter->Rect( m_pos, m_pos + m_size, FILL_T::NO_FILL, penWidth );
+    // Make the sheet object a clickable hyperlink (e.g. for PDF plotter)
+    std::vector<wxString> properties;
+
+    properties.emplace_back( EDA_TEXT::GotoPageHref( findSelf().GetPageNumber() ) );
+
+    for( const SCH_FIELD& field : GetFields() )
+    {
+        properties.emplace_back( wxString::Format( wxT( "!%s = %s" ),
+                                                   field.GetName(),
+                                                   field.GetShownText() ) );
+    }
+
+    aPlotter->HyperlinkMenu( GetBoundingBox(), properties );
 
     // Plot sheet pins
     for( SCH_SHEET_PIN* sheetPin : m_pins )
-        sheetPin->Plot( aPlotter );
+        sheetPin->Plot( aPlotter, aBackground );
 
     // Plot the fields
     for( const SCH_FIELD& field : m_fields )
-        field.Plot( aPlotter );
+        field.Plot( aPlotter, aBackground );
 }
 
 
-void SCH_SHEET::Print( const RENDER_SETTINGS* aSettings, const wxPoint& aOffset )
+void SCH_SHEET::Print( const RENDER_SETTINGS* aSettings, const VECTOR2I& aOffset )
 {
     wxDC*       DC = aSettings->GetPrintDC();
-    wxPoint     pos = m_pos + aOffset;
+    VECTOR2I    pos = m_pos + aOffset;
     int         lineWidth = std::max( GetPenWidth(), aSettings->GetDefaultPenWidth() );
     const auto* settings = dynamic_cast<const KIGFX::SCH_RENDER_SETTINGS*>( aSettings );
     bool        override = settings && settings->m_OverrideItemColors;
@@ -1100,12 +1159,9 @@ void SCH_SHEET::Print( const RENDER_SETTINGS* aSettings, const wxPoint& aOffset
         background = COLOR4D::UNSPECIFIED;
 
     if( background != COLOR4D::UNSPECIFIED )
-    {
-        GRFilledRect( nullptr, DC, pos.x, pos.y, pos.x + m_size.x, pos.y + m_size.y, 0,
-                      background, background );
-    }
+        GRFilledRect( DC, pos, pos + m_size, 0, background, background );
 
-    GRRect( nullptr, DC, pos.x, pos.y, pos.x + m_size.x, pos.y + m_size.y, lineWidth, border );
+    GRRect( DC, pos, pos + m_size, lineWidth, border );
 
     for( SCH_FIELD& field : m_fields )
         field.Print( aSettings, aOffset );
@@ -1162,24 +1218,25 @@ bool SCH_SHEET::operator <( const SCH_ITEM& aItem ) const
 }
 
 
-bool SCH_SHEET::AddInstance( const SCH_SHEET_PATH& aSheetPath )
+bool SCH_SHEET::addInstance( const SCH_SHEET_PATH& aSheetPath )
 {
     wxCHECK( aSheetPath.IsFullPath(), false );
+    wxCHECK( !aSheetPath.Last() || ( aSheetPath.Last()->m_Uuid != m_Uuid ), false );
 
     for( const SCH_SHEET_INSTANCE& instance : m_instances )
     {
         // if aSheetPath is found, nothing to do:
-        if( instance.m_Path == aSheetPath.PathWithoutRootUuid() )
+        if( instance.m_Path == aSheetPath.Path() )
             return false;
     }
 
     wxLogTrace( traceSchSheetPaths, wxT( "Adding instance `%s` to sheet `%s`." ),
-                aSheetPath.PathWithoutRootUuid().AsString(),
+                aSheetPath.Path().AsString(),
                 ( GetName().IsEmpty() ) ? wxT( "root" ) : GetName() );
 
     SCH_SHEET_INSTANCE instance;
 
-    instance.m_Path = aSheetPath.PathWithoutRootUuid();
+    instance.m_Path = aSheetPath.Path();
 
     // This entry does not exist: add it with an empty page number.
     m_instances.emplace_back( instance );
@@ -1187,12 +1244,65 @@ bool SCH_SHEET::AddInstance( const SCH_SHEET_PATH& aSheetPath )
 }
 
 
-wxString SCH_SHEET::GetPageNumber( const SCH_SHEET_PATH& aSheetPath ) const
+bool SCH_SHEET::getInstance( SCH_SHEET_INSTANCE& aInstance, const KIID_PATH& aSheetPath,
+                             bool aTestFromEnd ) const
+{
+    for( const SCH_SHEET_INSTANCE& instance : m_instances )
+    {
+        if( !aTestFromEnd )
+        {
+            if( instance.m_Path == aSheetPath )
+            {
+                aInstance = instance;
+                return true;
+            }
+        }
+        else if( instance.m_Path.EndsWith( aSheetPath ) )
+        {
+            aInstance = instance;
+            return true;
+        }
+    }
+
+    return false;
+}
+
+
+bool SCH_SHEET::HasRootInstance() const
+{
+    for( const SCH_SHEET_INSTANCE& instance : m_instances )
+    {
+        if( instance.m_Path.size() == 0 )
+            return true;
+    }
+
+    return false;
+}
+
+
+const SCH_SHEET_INSTANCE& SCH_SHEET::GetRootInstance() const
+{
+    for( const SCH_SHEET_INSTANCE& instance : m_instances )
+    {
+        if( instance.m_Path.size() == 0 )
+            return instance;
+    }
+
+    wxFAIL;
+
+    static SCH_SHEET_INSTANCE dummy;
+
+    return dummy;
+}
+
+
+wxString SCH_SHEET::getPageNumber( const SCH_SHEET_PATH& aSheetPath ) const
 {
     wxCHECK( aSheetPath.IsFullPath(), wxEmptyString );
+    wxCHECK( !aSheetPath.Last() || ( aSheetPath.Last()->m_Uuid != m_Uuid ), wxEmptyString );
 
     wxString pageNumber;
-    KIID_PATH path = aSheetPath.PathWithoutRootUuid();
+    KIID_PATH path = aSheetPath.Path();
 
     for( const SCH_SHEET_INSTANCE& instance : m_instances )
     {
@@ -1207,11 +1317,12 @@ wxString SCH_SHEET::GetPageNumber( const SCH_SHEET_PATH& aSheetPath ) const
 }
 
 
-void SCH_SHEET::SetPageNumber( const SCH_SHEET_PATH& aSheetPath, const wxString& aPageNumber )
+void SCH_SHEET::setPageNumber( const SCH_SHEET_PATH& aSheetPath, const wxString& aPageNumber )
 {
     wxCHECK( aSheetPath.IsFullPath(), /* void */ );
+    wxCHECK( !aSheetPath.Last() || ( aSheetPath.Last()->m_Uuid != m_Uuid ), /* void */ );
 
-    KIID_PATH path = aSheetPath.PathWithoutRootUuid();
+    KIID_PATH path = aSheetPath.Path();
 
     for( SCH_SHEET_INSTANCE& instance : m_instances )
     {
@@ -1251,6 +1362,9 @@ int SCH_SHEET::ComparePageNum( const wxString& aPageNumberA, const wxString& aPa
     // If not numeric, then sort as strings using natural sort
     int result = StrNumCmp( aPageNumberA, aPageNumberB );
 
+    // Divide by zero bad.
+    wxCHECK( result != 0, 0 );
+
     result = result / std::abs( result );
 
     return result;
