@@ -120,10 +120,10 @@ SIM_MODEL::INFO SIM_MODEL::TypeInfo( TYPE aType )
     case TYPE::PNP_VBIC:             return { DEVICE_T::PNP,    "VBIC",           "VBIC"                       };
     case TYPE::NPN_GUMMELPOON:       return { DEVICE_T::NPN,    "GUMMELPOON",     "Gummel-Poon"                };
     case TYPE::PNP_GUMMELPOON:       return { DEVICE_T::PNP,    "GUMMELPOON",     "Gummel-Poon"                };
-    //case TYPE::BJT_MEXTRAM:          return {};
+  //case TYPE::BJT_MEXTRAM:          return {};
     case TYPE::NPN_HICUM2:           return { DEVICE_T::NPN,    "HICUML2",        "HICUM level 2"              };
     case TYPE::PNP_HICUM2:           return { DEVICE_T::PNP,    "HICUML2",        "HICUM level 2"              };
-    //case TYPE::BJT_HICUM_L0:         return {};
+  //case TYPE::BJT_HICUM_L0:         return {};
 
     case TYPE::NJFET_SHICHMANHODGES: return { DEVICE_T::NJFET,  "SHICHMANHODGES", "Shichman-Hodges"            };
     case TYPE::PJFET_SHICHMANHODGES: return { DEVICE_T::PJFET,  "SHICHMANHODGES", "Shichman-Hodges"            };
@@ -161,18 +161,18 @@ SIM_MODEL::INFO SIM_MODEL::TypeInfo( TYPE aType )
     case TYPE::PMOS_B4SOI:           return { DEVICE_T::PMOS,   "B4SOI",          "BSIM4 SOI (B4SOI)"          };
     case TYPE::NMOS_BSIM4:           return { DEVICE_T::NMOS,   "BSIM4",          "BSIM4"                      };
     case TYPE::PMOS_BSIM4:           return { DEVICE_T::PMOS,   "BSIM4",          "BSIM4"                      };
-    //case TYPE::NMOS_EKV2_6:          return {};
-    //case TYPE::PMOS_EKV2_6:          return {};
-    //case TYPE::NMOS_PSP:             return {};
-    //case TYPE::PMOS_PSP:             return {};
+  //case TYPE::NMOS_EKV2_6:          return {};
+  //case TYPE::PMOS_EKV2_6:          return {};
+  //case TYPE::NMOS_PSP:             return {};
+  //case TYPE::PMOS_PSP:             return {};
     case TYPE::NMOS_B3SOIFD:         return { DEVICE_T::NMOS,   "B3SOIFD",        "B3SOIFD (BSIM3 FD-SOI)"     };
     case TYPE::PMOS_B3SOIFD:         return { DEVICE_T::PMOS,   "B3SOIFD",        "B3SOIFD (BSIM3 FD-SOI)"     };
     case TYPE::NMOS_B3SOIDD:         return { DEVICE_T::NMOS,   "B3SOIDD",        "B3SOIDD (BSIM3 SOI)"        };
     case TYPE::PMOS_B3SOIDD:         return { DEVICE_T::PMOS,   "B3SOIDD",        "B3SOIDD (BSIM3 SOI)"        };
     case TYPE::NMOS_B3SOIPD:         return { DEVICE_T::NMOS,   "B3SOIPD",        "B3SOIPD (BSIM3 PD-SOI)"     };
     case TYPE::PMOS_B3SOIPD:         return { DEVICE_T::PMOS,   "B3SOIPD",        "B3SOIPD (BSIM3 PD-SOI)"     };
-    //case TYPE::NMOS_STAG:            return {};
-    //case TYPE::PMOS_STAG:            return {};
+  //case TYPE::NMOS_STAG:            return {};
+  //case TYPE::PMOS_STAG:            return {};
     case TYPE::NMOS_HISIM2:          return { DEVICE_T::NMOS,   "HISIM2",         "HiSIM2"                     };
     case TYPE::PMOS_HISIM2:          return { DEVICE_T::PMOS,   "HISIM2",         "HiSIM2"                     };
     case TYPE::NMOS_HISIMHV1:        return { DEVICE_T::NMOS,   "HISIMHV1",       "HiSIM_HV1"                  };
@@ -184,8 +184,8 @@ SIM_MODEL::INFO SIM_MODEL::TypeInfo( TYPE aType )
     case TYPE::V_SIN:                return { DEVICE_T::V,      "SIN",            "Sine"                       };
     case TYPE::V_PULSE:              return { DEVICE_T::V,      "PULSE",          "Pulse"                      };
     case TYPE::V_EXP:                return { DEVICE_T::V,      "EXP",            "Exponential"                };
-    //case TYPE::V_SFAM:               return { DEVICE_TYPE::V,      "SFAM",           "Single-frequency AM"        };
-    //case TYPE::V_SFFM:               return { DEVICE_TYPE::V,      "SFFM",           "Single-frequency FM"        };
+  //case TYPE::V_SFAM:               return { DEVICE_TYPE::V,   "SFAM",           "Single-frequency AM"        };
+  //case TYPE::V_SFFM:               return { DEVICE_TYPE::V,   "SFFM",           "Single-frequency FM"        };
     case TYPE::V_PWL:                return { DEVICE_T::V,      "PWL",            "Piecewise linear"           };
     case TYPE::V_WHITENOISE:         return { DEVICE_T::V,      "WHITENOISE",     "White noise"                };
     case TYPE::V_PINKNOISE:          return { DEVICE_T::V,      "PINKNOISE",      "Pink noise (1/f)"           };
@@ -193,15 +193,15 @@ SIM_MODEL::INFO SIM_MODEL::TypeInfo( TYPE aType )
     case TYPE::V_RANDUNIFORM:        return { DEVICE_T::V,      "RANDUNIFORM",    "Random uniform"             };
     case TYPE::V_RANDNORMAL:         return { DEVICE_T::V,      "RANDNORMAL",     "Random normal"              };
     case TYPE::V_RANDEXP:            return { DEVICE_T::V,      "RANDEXP",        "Random exponential"         };
-    //case TYPE::V_RANDPOISSON:        return { DEVICE_TYPE::V,      "RANDPOISSON",    "Random Poisson"             };
+  //case TYPE::V_RANDPOISSON:        return { DEVICE_TYPE::V,   "RANDPOISSON",    "Random Poisson"             };
     case TYPE::V_BEHAVIORAL:         return { DEVICE_T::V,      "=",              "Behavioral"                 };
 
     case TYPE::I:                    return { DEVICE_T::I,      "DC",             "DC",                        };
     case TYPE::I_SIN:                return { DEVICE_T::I,      "SIN",            "Sine"                       };
     case TYPE::I_PULSE:              return { DEVICE_T::I,      "PULSE",          "Pulse"                      };
     case TYPE::I_EXP:                return { DEVICE_T::I,      "EXP",            "Exponential"                };
-    //case TYPE::I_SFAM:               return { DEVICE_TYPE::I,      "SFAM",           "Single-frequency AM"        };
-    //case TYPE::I_SFFM:               return { DEVICE_TYPE::I,      "SFFM",           "Single-frequency FM"        };
+  //case TYPE::I_SFAM:               return { DEVICE_TYPE::I,   "SFAM",           "Single-frequency AM"        };
+  //case TYPE::I_SFFM:               return { DEVICE_TYPE::I,   "SFFM",           "Single-frequency FM"        };
     case TYPE::I_PWL:                return { DEVICE_T::I,      "PWL",            "Piecewise linear"           };
     case TYPE::I_WHITENOISE:         return { DEVICE_T::I,      "WHITENOISE",     "White noise"                };
     case TYPE::I_PINKNOISE:          return { DEVICE_T::I,      "PINKNOISE",      "Pink noise (1/f)"           };
@@ -209,7 +209,7 @@ SIM_MODEL::INFO SIM_MODEL::TypeInfo( TYPE aType )
     case TYPE::I_RANDUNIFORM:        return { DEVICE_T::I,      "RANDUNIFORM",    "Random uniform"             };
     case TYPE::I_RANDNORMAL:         return { DEVICE_T::I,      "RANDNORMAL",     "Random normal"              };
     case TYPE::I_RANDEXP:            return { DEVICE_T::I,      "RANDEXP",        "Random exponential"         };
-    //case TYPE::I_RANDPOISSON:        return { DEVICE_TYPE::I,      "RANDPOISSON",    "Random Poisson"             };
+  //case TYPE::I_RANDPOISSON:        return { DEVICE_TYPE::I,   "RANDPOISSON",    "Random Poisson"             };
     case TYPE::I_BEHAVIORAL:         return { DEVICE_T::I,      "=",              "Behavioral"                 };
 
     case TYPE::SUBCKT:               return { DEVICE_T::SUBCKT, "",               "Subcircuit"                 };
@@ -258,13 +258,13 @@ SIM_MODEL::SPICE_INFO SIM_MODEL::SpiceInfo( TYPE aType )
     case TYPE::NPN_HICUM2:           return { "Q", "NPN",         "",        "8"   };
     case TYPE::PNP_HICUM2:           return { "Q", "PNP",         "",        "8"   };
 
-    case TYPE::NJFET_SHICHMANHODGES: return { "J", "NJF",         "",        "1"   };
-    case TYPE::PJFET_SHICHMANHODGES: return { "J", "PJF",         "",        "1"   };
+    case TYPE::NJFET_SHICHMANHODGES: return { "J", "NJF",         "",        "1",  true   };
+    case TYPE::PJFET_SHICHMANHODGES: return { "J", "PJF",         "",        "1",  true   };
     case TYPE::NJFET_PARKERSKELLERN: return { "J", "NJF",         "",        "2"   };
     case TYPE::PJFET_PARKERSKELLERN: return { "J", "PJF",         "",        "2"   };
 
-    case TYPE::NMES_STATZ:           return { "Z", "NMF",         "",        "1"   };
-    case TYPE::PMES_STATZ:           return { "Z", "PMF",         "",        "1"   };
+    case TYPE::NMES_STATZ:           return { "Z", "NMF",         "",        "1",  true   };
+    case TYPE::PMES_STATZ:           return { "Z", "PMF",         "",        "1",  true   };
     case TYPE::NMES_YTTERDAL:        return { "Z", "NMF",         "",        "2"   };
     case TYPE::PMES_YTTERDAL:        return { "Z", "PMF",         "",        "2"   };
     case TYPE::NMES_HFET1:           return { "Z", "NMF",         "",        "5"   };
@@ -274,8 +274,8 @@ SIM_MODEL::SPICE_INFO SIM_MODEL::SpiceInfo( TYPE aType )
 
     case TYPE::NMOS_VDMOS:           return { "M", "VDMOS NCHAN", };
     case TYPE::PMOS_VDMOS:           return { "M", "VDMOS PCHAN", };
-    case TYPE::NMOS_MOS1:            return { "M", "NMOS",        "",        "1"   };
-    case TYPE::PMOS_MOS1:            return { "M", "PMOS",        "",        "1"   };
+    case TYPE::NMOS_MOS1:            return { "M", "NMOS",        "",        "1",  true   };
+    case TYPE::PMOS_MOS1:            return { "M", "PMOS",        "",        "1",  true   };
     case TYPE::NMOS_MOS2:            return { "M", "NMOS",        "",        "2"   };
     case TYPE::PMOS_MOS2:            return { "M", "PMOS",        "",        "2"   };
     case TYPE::NMOS_MOS3:            return { "M", "NMOS",        "",        "3"   };
@@ -294,18 +294,18 @@ SIM_MODEL::SPICE_INFO SIM_MODEL::SpiceInfo( TYPE aType )
     case TYPE::PMOS_B4SOI:           return { "M", "PMOS",        "",        "10"  };
     case TYPE::NMOS_BSIM4:           return { "M", "NMOS",        "",        "14"  };
     case TYPE::PMOS_BSIM4:           return { "M", "PMOS",        "",        "14"  };
-    //case TYPE::NMOS_EKV2_6:          return {};
-    //case TYPE::PMOS_EKV2_6:          return {};
-    //case TYPE::NMOS_PSP:             return {};
-    //case TYPE::PMOS_PSP:             return {};
+  //case TYPE::NMOS_EKV2_6:          return {};
+  //case TYPE::PMOS_EKV2_6:          return {};
+  //case TYPE::NMOS_PSP:             return {};
+  //case TYPE::PMOS_PSP:             return {};
     case TYPE::NMOS_B3SOIFD:         return { "M", "NMOS",        "",        "55"  };
     case TYPE::PMOS_B3SOIFD:         return { "M", "PMOS",        "",        "55"  };
     case TYPE::NMOS_B3SOIDD:         return { "M", "NMOS",        "",        "56"  };
     case TYPE::PMOS_B3SOIDD:         return { "M", "PMOS",        "",        "56"  };
     case TYPE::NMOS_B3SOIPD:         return { "M", "NMOS",        "",        "57"  };
     case TYPE::PMOS_B3SOIPD:         return { "M", "PMOS",        "",        "57"  };
-    //case TYPE::NMOS_STAG:            return {};
-    //case TYPE::PMOS_STAG:            return {};
+  //case TYPE::NMOS_STAG:            return {};
+  //case TYPE::PMOS_STAG:            return {};
     case TYPE::NMOS_HISIM2:          return { "M", "NMOS",        "",        "68"  };
     case TYPE::PMOS_HISIM2:          return { "M", "PMOS",        "",        "68"  };
     case TYPE::NMOS_HISIMHV1:        return { "M", "NMOS",        "",        "73", false, false, "1.2.4" };
@@ -317,8 +317,8 @@ SIM_MODEL::SPICE_INFO SIM_MODEL::SpiceInfo( TYPE aType )
     case TYPE::V_SIN:                return { "V", "",            "SIN"      };
     case TYPE::V_PULSE:              return { "V", "",            "PULSE"    };
     case TYPE::V_EXP:                return { "V", "",            "EXP"      };
-    //case TYPE::V_SFAM:               return { "V", "",            "AM"       };
-    //case TYPE::V_SFFM:               return { "V", "",            "SFFM"     };
+  //case TYPE::V_SFAM:               return { "V", "",            "AM"       };
+  //case TYPE::V_SFFM:               return { "V", "",            "SFFM"     };
     case TYPE::V_PWL:                return { "V", "",            "PWL"      };
     case TYPE::V_WHITENOISE:         return { "V", "",            "TRNOISE"  };
     case TYPE::V_PINKNOISE:          return { "V", "",            "TRNOISE"  };
@@ -326,15 +326,15 @@ SIM_MODEL::SPICE_INFO SIM_MODEL::SpiceInfo( TYPE aType )
     case TYPE::V_RANDUNIFORM:        return { "V", "",            "TRRANDOM" };
     case TYPE::V_RANDNORMAL:         return { "V", "",            "TRRANDOM" };
     case TYPE::V_RANDEXP:            return { "V", "",            "TRRANDOM" };
-    //case TYPE::V_RANDPOISSON:        return { "V", "",            "TRRANDOM" };
+  //case TYPE::V_RANDPOISSON:        return { "V", "",            "TRRANDOM" };
     case TYPE::V_BEHAVIORAL:         return { "B"  };
 
     case TYPE::I:                    return { "I", "",            "DC"       };
     case TYPE::I_PULSE:              return { "I", "",            "PULSE"    };
     case TYPE::I_SIN:                return { "I", "",            "SIN"      };
     case TYPE::I_EXP:                return { "I", "",            "EXP"      };
-    //case TYPE::I_SFAM:               return { "V", "",            "AM"       };
-    //case TYPE::I_SFFM:               return { "V", "",            "SFFM"     };
+  //case TYPE::I_SFAM:               return { "V", "",            "AM"       };
+  //case TYPE::I_SFFM:               return { "V", "",            "SFFM"     };
     case TYPE::I_PWL:                return { "I", "",            "PWL"      };
     case TYPE::I_WHITENOISE:         return { "I", "",            "TRNOISE"  };
     case TYPE::I_PINKNOISE:          return { "I", "",            "TRNOISE"  };
@@ -342,7 +342,7 @@ SIM_MODEL::SPICE_INFO SIM_MODEL::SpiceInfo( TYPE aType )
     case TYPE::I_RANDUNIFORM:        return { "I", "",            "TRRANDOM" };
     case TYPE::I_RANDNORMAL:         return { "I", "",            "TRRANDOM" };
     case TYPE::I_RANDEXP:            return { "I", "",            "TRRANDOM" };
-    //case TYPE::I_RANDPOISSON:        return { "I", "",            "TRRANDOM" };
+  //case TYPE::I_RANDPOISSON:        return { "I", "",            "TRRANDOM" };
     case TYPE::I_BEHAVIORAL:         return { "B"  };
 
     case TYPE::SUBCKT:               return { "X"  };
@@ -620,7 +620,10 @@ std::string SIM_MODEL::GetFieldValue( const std::vector<T>* aFields, const wxStr
     for( const T& field : *aFields )
     {
         if( field.GetName() == aFieldName )
-            return aResolve ? field.GetShownText().ToStdString() : field.GetText().ToStdString();
+        {
+            return aResolve ? field.GetShownText( 0, false ).ToStdString()
+                            : field.GetText().ToStdString();
+        }
     }
 
     return "";
@@ -711,13 +714,13 @@ int SIM_MODEL::FindModelPinIndex( const std::string& aSymbolPinNumber )
 }
 
 
-void SIM_MODEL::AddParam( const PARAM::INFO& aInfo, bool aIsOtherVariant )
+void SIM_MODEL::AddParam( const PARAM::INFO& aInfo )
 {
-    m_params.emplace_back( aInfo, aIsOtherVariant );
+    m_params.emplace_back( aInfo );
 
     // Enums are initialized with their default values.
     if( aInfo.enumValues.size() >= 1 )
-        m_params.back().value->FromString( aInfo.defaultValue );
+        m_params.back().value = aInfo.defaultValue;
 }
 
 
@@ -790,7 +793,7 @@ void SIM_MODEL::SetPinSymbolPinNumber( const std::string& aPinName,
 
 const SIM_MODEL::PARAM& SIM_MODEL::GetParam( unsigned aParamIndex ) const
 {
-    if( m_baseModel && m_params.at( aParamIndex ).value->ToString() == "" )
+    if( m_baseModel && m_params.at( aParamIndex ).value == "" )
         return m_baseModel->GetParam( aParamIndex );
     else
         return m_params.at( aParamIndex );
@@ -847,17 +850,19 @@ const SIM_MODEL::PARAM& SIM_MODEL::GetBaseParam( unsigned aParamIndex ) const
 }
 
 
-void SIM_MODEL::SetParamValue( int aParamIndex, const SIM_VALUE& aValue )
+void SIM_MODEL::doSetParamValue( int aParamIndex, const std::string& aValue )
 {
-    *m_params.at( aParamIndex ).value = aValue;
+    m_params.at( aParamIndex ).value = aValue;
 }
 
 
 void SIM_MODEL::SetParamValue( int aParamIndex, const std::string& aValue,
                                SIM_VALUE::NOTATION aNotation )
 {
-    const SIM_VALUE& value = *GetParam( aParamIndex ).value;
-    SetParamValue( aParamIndex, *SIM_VALUE::Create( value.GetType(), aValue, aNotation ) );
+    // Note: also converts decimal separators to '.'
+    std::string value = SIM_VALUE::ConvertNotation( aValue, aNotation, SIM_VALUE::NOTATION::SI );
+
+    doSetParamValue( aParamIndex, value );
 }
 
 
@@ -878,42 +883,6 @@ void SIM_MODEL::SetParamValue( const std::string& aParamName, const std::string&
 }
 
 
-bool SIM_MODEL::HasOverrides() const
-{
-    for( const PARAM& param : m_params )
-    {
-        if( param.value->ToString() != "" )
-            return true;
-    }
-
-    return false;
-}
-
-
-bool SIM_MODEL::HasNonInstanceOverrides() const
-{
-    for( const PARAM& param : m_params )
-    {
-        if( !param.info.isInstanceParam && param.value->ToString() != "" )
-            return true;
-    }
-
-    return false;
-}
-
-
-bool SIM_MODEL::HasSpiceNonInstanceOverrides() const
-{
-    for( const PARAM& param : m_params )
-    {
-        if( !param.info.isSpiceInstanceParam && param.value->ToString() != "" )
-            return true;
-    }
-
-    return false;
-}
-
-
 std::unique_ptr<SIM_MODEL> SIM_MODEL::Create( TYPE aType )
 {
     switch( aType )
@@ -1034,12 +1003,21 @@ void SIM_MODEL::createPins( const std::vector<LIB_PIN*>& aSymbolPins )
 
     for( unsigned modelPinIndex = 0; modelPinIndex < pinNames.size(); ++modelPinIndex )
     {
+        wxString pinName = pinNames[ modelPinIndex ];
+        bool     optional = false;
+
+        if( pinName.StartsWith( '<' ) && pinName.EndsWith( '>' ) )
+        {
+            pinName = pinName.Mid( 1, pinName.Length() - 2 );
+            optional = true;
+        }
+
         if( modelPinIndex < aSymbolPins.size() )
         {
             AddPin( { pinNames.at( modelPinIndex ),
                       aSymbolPins[ modelPinIndex ]->GetNumber().ToStdString() } );
         }
-        else
+        else if( !optional )
         {
             AddPin( { pinNames.at( modelPinIndex ), "" } );
         }
@@ -1082,12 +1060,37 @@ void SIM_MODEL::doWriteFields( std::vector<T>& aFields ) const
 }
 
 
-bool SIM_MODEL::requiresSpiceModelLine() const
+bool SIM_MODEL::requiresSpiceModelLine( const SPICE_ITEM& aItem ) const
 {
-    for( const PARAM& param : GetParams() )
+    // Model must be written if there's no base model or the base model is an internal model
+    if( !m_baseModel || aItem.baseModelName == "" )
+        return true;
+
+    for( int ii = 0; ii < GetParamCount(); ++ii )
     {
-        if( !param.info.isSpiceInstanceParam )
-            return true;
+        const PARAM& param = m_params[ii];
+
+        // Instance parameters are written in item lines
+        if( param.info.isSpiceInstanceParam )
+            continue;
+
+        // Empty parameters are interpreted as default-value
+        if ( param.value == "" )
+            continue;
+
+        const SIM_MODEL* baseModel = dynamic_cast<const SIM_MODEL*>( m_baseModel );
+        std::string      baseValue = baseModel->m_params[ii].value;
+
+        if( param.value == baseValue )
+            continue;
+
+        // One more check for equivalence, mostly for early 7.0 files which wrote all parameters
+        // to the Sim.Params field in normalized format
+        if( param.value == SIM_VALUE::Normalize( SIM_VALUE::ToDouble( baseValue ) ) )
+            continue;
+
+        // Overrides must be written
+        return true;
     }
 
     return false;
@@ -1425,7 +1428,7 @@ void SIM_MODEL::MigrateSimModel( T_symbol& aSymbol, const PROJECT* aProject )
         // SPICE.  Here we remap them to 'r0' and 'r1'.
         if( T_field* deviceType = aSymbol.FindField( SIM_TYPE_FIELD ) )
         {
-            if( deviceType->GetShownText().Lower() == wxS( "pot" ) )
+            if( deviceType->GetShownText( 0, false ).Lower() == wxS( "pot" ) )
             {
                 if( T_field* pins = aSymbol.FindField( SIM_PINS_FIELD ) )
                 {
@@ -1446,6 +1449,8 @@ void SIM_MODEL::MigrateSimModel( T_symbol& aSymbol, const PROJECT* aProject )
         FIELD_INFO()
         {
             m_Attributes.m_Visible = false;
+            m_Attributes.m_Size = VECTOR2I( DEFAULT_SIZE_TEXT * schIUScale.IU_PER_MILS,
+                                            DEFAULT_SIZE_TEXT * schIUScale.IU_PER_MILS );
         };
 
         FIELD_INFO( const wxString& aText, T_field* aField ) :
@@ -1738,9 +1743,6 @@ void SIM_MODEL::MigrateSimModel( T_symbol& aSymbol, const PROJECT* aProject )
                             {
                                 model->SetParamValue( ii, tokenizer.GetNextToken().ToStdString(),
                                                       SIM_VALUE_GRAMMAR::NOTATION::SPICE );
-
-                                if( !model->GetParam( ii ).value->HasValue() )
-                                    THROW_IO_ERROR( "fall back to raw SPICE" );
                             }
 
                             spiceTypeInfo.m_Text = SIM_MODEL::TypeInfo( type ).fieldValue;
