@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2009-2015 Jean-Pierre Charras, jp.charras wanadoo.fr
  * Copyright (C) 2011 Wayne Stambaugh <stambaughw@gmail.com>
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -33,6 +33,7 @@
 
 
 #include <vector>
+#include <map>
 
 #include <wx/aui/aui.h>
 #include <layer_ids.h>
@@ -41,9 +42,10 @@
 #include <kiway_holder.h>
 #include <tool/tools_holder.h>
 #include <widgets/ui_common.h>
-#include <widgets/infobar.h>
+#include <widgets/wx_infobar.h>
 #include <undo_redo_container.h>
-#include <eda_units.h>
+#include <units_provider.h>
+#include <origin_transforms.h>
 
 // Option for main frames
 #define KICAD_DEFAULT_DRAWFRAME_STYLE wxDEFAULT_FRAME_STYLE | wxWANTS_CHARS
@@ -60,7 +62,6 @@ class wxChoice;
 class wxEvent;
 class wxFileName;
 class EDA_ITEM;
-class EDA_RECT;
 class EDA_DRAW_PANEL_GAL;
 class EDA_MSG_PANEL;
 class BASE_SCREEN;
@@ -100,7 +101,8 @@ wxDECLARE_EVENT( UNITS_CHANGED, wxCommandEvent );
  * calling it from the derived class's SaveSettings().  This class is not a #KIWAY_PLAYER
  * because #KICAD_MANAGER_FRAME is derived from it and that class is not a player.
  */
-class EDA_BASE_FRAME : public wxFrame, public TOOLS_HOLDER, public KIWAY_HOLDER
+class EDA_BASE_FRAME : public wxFrame, public TOOLS_HOLDER, public KIWAY_HOLDER,
+                       public UNITS_PROVIDER
 {
 public:
     /**
@@ -114,29 +116,23 @@ public:
 
     EDA_BASE_FRAME( wxWindow* aParent, FRAME_T aFrameType, const wxString& aTitle,
                     const wxPoint& aPos, const wxSize& aSize, long aStyle,
-                    const wxString& aFrameName, KIWAY* aKiway );
-
-    EDA_BASE_FRAME( FRAME_T aFrameType, KIWAY* aKiway );
+                    const wxString& aFrameName, KIWAY* aKiway, const EDA_IU_SCALE& aIuScale );
 
     ~EDA_BASE_FRAME();
 
-    /**
-     * Return the user units currently in use.
-     */
-    EDA_UNITS GetUserUnits() const
-    {
-        return m_userUnits;
-    }
-
-    void SetUserUnits( EDA_UNITS aUnits )
-    {
-        m_userUnits = aUnits;
-    }
-
     void ChangeUserUnits( EDA_UNITS aUnits );
 
     virtual void ToggleUserUnits() { }
 
+    /**
+     * Return a reference to the default ORIGIN_TRANSFORMS object
+     */
+    ORIGIN_TRANSFORMS& GetOriginTransforms() override
+    {
+        return m_originTransforms;
+    }
+
+
     SETTINGS_MANAGER* GetSettingsManager() const { return m_settingsManager; }
 
     virtual SEVERITY GetSeverity( int aErrorCode ) const { return RPT_SEVERITY_UNDEFINED; }
@@ -199,9 +195,7 @@ public:
 
     void OnMaximize( wxMaximizeEvent& aEvent );
 
-    void SetAutoSaveInterval( int aInterval );
-
-    int GetAutoSaveInterval() const { return m_autoSaveInterval; }
+    int GetAutoSaveInterval() const;
 
     bool IsType( FRAME_T aType ) const { return m_ident == aType; }
     FRAME_T GetFrameType() const { return m_ident; }
@@ -222,6 +216,12 @@ public:
      */
     void OnPreferences( wxCommandEvent& event );
 
+    /**
+     * Displays the preferences and settings of all opened editors paged dialog, starting with
+     * a particular page
+     */
+    void ShowPreferences( wxString aStartPage, wxString aStartParentPage );
+
     void PrintMsg( const wxString& text );
 
     void CreateInfoBar();
@@ -290,14 +290,6 @@ public:
      */
     virtual APP_SETTINGS_BASE* config() const;
 
-    /**
-     * Allow a frame to load its preference panels (if any) into the preferences dialog.
-     *
-     * @param aParent a paged dialog into which the preference panels should be installed.
-     */
-    virtual void InstallPreferences( PAGED_DIALOG* , PANEL_HOTKEYS_EDITOR* ) { }
-
-
     void LoadWindowState( const wxString& aFileName );
     /**
      * Load window settings from the given settings object.
@@ -443,7 +435,7 @@ public:
      *
      * Needed when the language or icons are changed
      */
-    virtual void ReCreateMenuBar();
+    void ReCreateMenuBar();
 
     /**
      * Adds the standard KiCad help menu to the menubar.
@@ -511,7 +503,9 @@ public:
      */
     virtual void ProjectChanged() {}
 
-    const wxString& GetAboutTitle() const { return m_aboutTitle; }
+    const wxString& GetAboutTitle() const { return wxGetTranslation( m_aboutTitle ); }
+
+    const wxString& GetUntranslatedAboutTitle() const { return m_aboutTitle; }
 
     /**
      * Get if the contents of the frame have been modified since the last save.
@@ -581,6 +575,12 @@ public:
 
     int GetMaxUndoItems() const { return m_undoRedoCountMax; }
 
+    /**
+     * Must be called after a model change in order to set the "modify" flag and do other
+     * frame-specific processing.
+     */
+    virtual void OnModify();
+
     bool NonUserClose( bool aForce )
     {
         m_isNonUserClose = true;
@@ -604,17 +604,28 @@ protected:
         return wxT( "_autosave-" );
     }
 
+    virtual void doReCreateMenuBar() {}
+
     /**
      * Handle the auto save timer event.
      */
     void onAutoSaveTimer( wxTimerEvent& aEvent );
 
+
+    /**
+     * Handle a window iconize event.
+     *
+     * @param aEvent is the data for the event.
+     */
+    virtual void handleIconizeEvent( wxIconizeEvent& aEvent ) {}
+    void         onIconize( wxIconizeEvent& aEvent );
+
     /**
      * Return the auto save status of the application.
      *
      * Override this function if your derived frame supports automatic file saving.
      */
-    virtual bool isAutoSaveRequired() const { return false; }
+    virtual bool isAutoSaveRequired() const { return m_autoSaveRequired; }
 
     /**
      * This should be overridden by the derived class to handle the auto save feature.
@@ -652,6 +663,21 @@ protected:
 
     void ensureWindowIsOnScreen();
 
+    /**
+     * Handles event fired when a file is dropped to the window.
+     * In this base class, stores the path of files accepted.
+     * Calls DoWithAcceptedFiles() to execute actions on files.
+     */
+    virtual void OnDropFiles( wxDropFilesEvent& aEvent );
+
+    /**
+     * Execute action on accepted dropped file.
+     * Called in OnDropFiles and should be populated with
+     * the action to execute in inherited classes.
+     */
+    virtual void            DoWithAcceptedFiles();
+    std::vector<wxFileName> m_AcceptedFiles;
+
     DECLARE_EVENT_TABLE()
 
 private:
@@ -715,19 +741,19 @@ private:
 
     FILE_HISTORY*   m_fileHistory;          // The frame's recently opened file list
 
-    bool            m_hasAutoSave;
-    bool            m_autoSaveState;
-    int             m_autoSaveInterval;     // The auto save interval time in seconds.
+    bool            m_supportsAutoSave;
+    bool            m_autoSavePending;
+    bool            m_autoSaveRequired;
     wxTimer*        m_autoSaveTimer;
 
-    int             m_undoRedoCountMax;     // undo/Redo command Max depth
+    int                 m_undoRedoCountMax; // undo/Redo command Max depth
 
     UNDO_REDO_CONTAINER m_undoList;         // Objects list for the undo command (old data)
     UNDO_REDO_CONTAINER m_redoList;         // Objects list for the redo command (old data)
 
-    wxString        m_mruPath;              // Most recently used path.
+    wxString            m_mruPath;          // Most recently used path.
 
-    EDA_UNITS       m_userUnits;
+    ORIGIN_TRANSFORMS   m_originTransforms; // Default display origin transforms object.
 
     ///< Map containing the UI update handlers registered with wx for each action.
     std::map<int, UIUpdateHandler> m_uiUpdateMap;
@@ -739,6 +765,10 @@ private:
     ///< Set by #NonUserClose() to indicate that the user did not request the current close.
     bool            m_isNonUserClose;
 
+    /**
+     * Associates files extensions with action to execute.
+     */
+    std::map<const wxString, TOOL_ACTION*> m_acceptedExts;
 };
 
 
