@@ -31,7 +31,6 @@
 #include <gal/gal_display_options.h>
 #include <gal/color4d.h>
 #include <class_draw_panel_gal.h>
-#include <origin_transforms.h>
 #include <kiid.h>
 #include "hotkeys_basic.h"
 
@@ -42,6 +41,7 @@ class COLOR_SETTINGS;
 class TOOL_MENU;
 class APP_SETTINGS_BASE;
 class wxFindReplaceData;
+class SEARCH_PANE;
 
 namespace KIGFX
 {
@@ -73,7 +73,7 @@ class EDA_DRAW_FRAME : public KIWAY_PLAYER
 public:
     EDA_DRAW_FRAME( KIWAY* aKiway, wxWindow* aParent, FRAME_T aFrameType, const wxString& aTitle,
                     const wxPoint& aPos, const wxSize& aSize, long aStyle,
-                    const wxString& aFrameName );
+                    const wxString& aFrameName, const EDA_IU_SCALE& aIuScale );
 
     ~EDA_DRAW_FRAME();
 
@@ -102,7 +102,7 @@ public:
      */
     bool IsScriptingConsoleVisible();
 
-    wxFindReplaceData& GetFindReplaceData() { return *m_findReplaceData; }
+    EDA_SEARCH_DATA& GetFindReplaceData() { return *m_findReplaceData; }
     wxArrayString& GetFindHistoryList() { return m_findStringHistoryList; }
 
     virtual void SetPageSettings( const PAGE_INFO& aPageSettings ) = 0;
@@ -136,8 +136,8 @@ public:
      * This is treated as a relative offset and snapping will occur at multiples of the grid
      * size relative to this point.
      */
-    virtual const wxPoint& GetGridOrigin() const = 0;
-    virtual void SetGridOrigin( const wxPoint& aPosition ) = 0;
+    virtual const VECTOR2I& GetGridOrigin() const = 0;
+    virtual void            SetGridOrigin( const VECTOR2I& aPosition ) = 0;
 
     /**
      * Return the nearest \a aGridSize location to \a aPosition.
@@ -145,7 +145,7 @@ public:
      * @param aPosition The position to check.
      * @return The nearest grid position.
      */
-    wxPoint GetNearestGridPosition( const wxPoint& aPosition ) const;
+    VECTOR2I GetNearestGridPosition( const VECTOR2I& aPosition ) const;
 
     /**
      * Return the nearest \a aGridSize / 2 location to \a aPosition.
@@ -156,14 +156,7 @@ public:
      * @param aPosition The position to check.
      * @return The nearest half-grid position.
      */
-    wxPoint GetNearestHalfGridPosition( const wxPoint& aPosition ) const;
-
-    /**
-     * Return a reference to the default ORIGIN_TRANSFORMS object
-     */
-    virtual ORIGIN_TRANSFORMS& GetOriginTransforms()
-    { return m_originTransforms; }
-
+    VECTOR2I GetNearestHalfGridPosition( const VECTOR2I& aPosition ) const;
 
     virtual const TITLE_BLOCK& GetTitleBlock() const = 0;
     virtual void SetTitleBlock( const TITLE_BLOCK& aTitleBlock ) = 0;
@@ -174,10 +167,7 @@ public:
     virtual void SetDrawBgColor( const COLOR4D& aColor) { m_drawBgColor= aColor ; }
 
     /// Returns a pointer to the active color theme settings
-    virtual COLOR_SETTINGS* GetColorSettings() const;
-
-    bool ShowPageLimits() const { return m_showPageLimits; }
-    void SetShowPageLimits( bool aShow ) { m_showPageLimits = aShow; }
+    virtual COLOR_SETTINGS* GetColorSettings( bool aForceRefresh = false ) const;
 
     /**
      * @param doOpen if true runs an Open Library browser, otherwise New Library
@@ -205,7 +195,6 @@ public:
 
     void EraseMsgBox();
 
-    void ReCreateMenuBar() override { }
     virtual void ReCreateHToolbar() = 0;
     virtual void ReCreateVToolbar() = 0;
     virtual void ReCreateOptToolbar() = 0;
@@ -247,21 +236,21 @@ public:
     void UpdateGridSelectBox();
 
     /**
-     * Update the checked item in the grid combobox.
+     * Update the checked item in the grid wxchoice.
      */
     void OnUpdateSelectGrid( wxUpdateUIEvent& aEvent );
 
+    /**
+     * Update the checked item in the zoom wxchoice.
+     */
+    void OnUpdateSelectZoom( wxUpdateUIEvent& aEvent );
+
     /**
      * Rebuild the grid combobox to respond to any changes in the GUI (units, user
      * grid changes, etc.)
      */
     void UpdateZoomSelectBox();
 
-    /**
-     * Update the checked item in the zoom combobox.
-     */
-    void OnUpdateSelectZoom( wxUpdateUIEvent& aEvent );
-
     /**
      * Return a human readable value for display in dialogs.
      */
@@ -302,7 +291,7 @@ public:
      *
      * @param aPos is the point to go to.
      */
-    void FocusOnLocation( const wxPoint& aPos );
+    void FocusOnLocation( const VECTOR2I& aPos );
 
     /**
      * Construct a "basic" menu for a tool, containing only items that apply to all tools
@@ -314,13 +303,14 @@ public:
      * Prints the drawing-sheet (frame and title block).
      *
      * @param aScreen screen to draw.
+     * @param aProperties Optional properties for text variable resolution.
      * @param aMils2Iu The mils to Iu conversion factor.
      * @param aFilename The filename to display in basic inscriptions.
      * @param aSheetLayer The layer displayed from PcbNew.
      */
     void PrintDrawingSheet( const RENDER_SETTINGS* aSettings, BASE_SCREEN* aScreen,
-                            double aMils2Iu, const wxString& aFilename,
-                            const wxString& aSheetLayer = wxEmptyString );
+                            const std::map<wxString, wxString>* aProperties, double aMils2Iu,
+                            const wxString& aFilename, const wxString& aSheetLayer = wxEmptyString );
 
     void DisplayToolMsg( const wxString& msg ) override;
 
@@ -455,6 +445,11 @@ public:
      */
     void RecreateToolbars();
 
+    /**
+     * Redraw the menus and what not in current language.
+     */
+    void ShowChangedLanguage() override;
+
     DECLARE_EVENT_TABLE()
 
 protected:
@@ -464,6 +459,8 @@ protected:
 
     void setupUnits( APP_SETTINGS_BASE* aCfg );
 
+    void doReCreateMenuBar() override { }
+
     std::vector<wxWindow*> findDialogs();
 
     /**
@@ -494,7 +491,6 @@ protected:
     ///< Prevents opening same file multiple times.
     std::unique_ptr<wxSingleInstanceChecker> m_file_checker;
 
-    bool               m_showPageLimits;    // True to display the page limits
     COLOR4D            m_gridColor;         // Grid color
     COLOR4D            m_drawBgColor;       // The background color of the draw canvas; BLACK for
                                             // Pcbnew, BLACK or WHITE for Eeschema
@@ -503,7 +499,6 @@ protected:
     bool               m_polarCoords;       // For those frames that support polar coordinates
 
     bool               m_showBorderAndTitleBlock;  // Show the drawing sheet (border & title block).
-    long               m_firstRunDialogSetting;    // Show first run dialog on startup
 
     wxChoice*          m_gridSelectBox;
     wxChoice*          m_zoomSelectBox;
@@ -513,7 +508,7 @@ protected:
     ACTION_TOOLBAR*    m_drawToolBar;       // Drawing tools (typically on right edge of window)
     ACTION_TOOLBAR*    m_optionsToolBar;    // Options (typically on left edge of window)
 
-    wxFindReplaceData* m_findReplaceData;
+    std::unique_ptr<EDA_SEARCH_DATA> m_findReplaceData;
     wxArrayString      m_findStringHistoryList;
     wxArrayString      m_replaceStringHistoryList;
 
@@ -521,19 +516,20 @@ protected:
     int                m_msgFrameHeight;
 
     COLOR_SETTINGS*    m_colorSettings;
+    SEARCH_PANE*       m_searchPane;
 
     ///< The current canvas type.
     EDA_DRAW_PANEL_GAL::GAL_TYPE    m_canvasType;
 
+    static bool m_openGLFailureOccured; ///< Has any failure occured when switching to OpenGL in
+                                        ///< any EDA_DRAW_FRAME?
+
 private:
     BASE_SCREEN*                m_currentScreen;      ///< current used SCREEN
     EDA_DRAW_PANEL_GAL*         m_canvas;
 
     ///< This the frame's interface to setting GAL display options.
     KIGFX::GAL_DISPLAY_OPTIONS  m_galDisplayOptions;
-
-    ///< Default display origin transforms object.
-    ORIGIN_TRANSFORMS           m_originTransforms;
 };
 
 #endif  // DRAW_FRAME_H_
