@@ -184,6 +184,11 @@ struct KIFACE
      */
     virtual void OnKifaceEnd() = 0;
 
+    /**
+     * Reloads global state.
+     */
+    virtual void Reset() = 0;
+
     /**
      * Create a wxWindow for the current project.
      *
