@@ -27,6 +27,7 @@
 #include <core/wx_stl_compat.h>
 #include <origin_transforms.h>
 #include <properties/eda_angle_variant.h>
+#include <properties/property_validator.h>
 
 #include <wx/any.h>
 #include <wx/string.h>
@@ -187,7 +188,8 @@ PROPERTY_BASE( const wxString& aName, PROPERTY_DISPLAY aDisplay = PT_DEFAULT,
         m_isInternal( false ),
         m_isDeprecated( false ),
         m_availFunc( [](INSPECTABLE*)->bool { return true; } ),
-        m_writeableFunc( [](INSPECTABLE*)->bool { return true; } )
+        m_writeableFunc( [](INSPECTABLE*)->bool { return true; } ),
+        m_validator( NullValidator )
     {
     }
 
@@ -195,10 +197,7 @@ PROPERTY_BASE( const wxString& aName, PROPERTY_DISPLAY aDisplay = PT_DEFAULT,
     {
     }
 
-    const wxString& Name() const
-    {
-        return m_name;
-    }
+    const wxString& Name() const { return m_name; }
 
     /**
      * Return a limited set of possible values (e.g. enum). Check with HasChoices() if a particular
@@ -238,9 +237,10 @@ PROPERTY_BASE( const wxString& aName, PROPERTY_DISPLAY aDisplay = PT_DEFAULT,
     /**
      * Set a callback function to determine whether an object provides this property.
      */
-    void SetAvailableFunc( std::function<bool(INSPECTABLE*)> aFunc )
+    PROPERTY_BASE& SetAvailableFunc( std::function<bool(INSPECTABLE*)> aFunc )
     {
         m_availFunc = aFunc;
+        return *this;
     }
 
     virtual bool Writeable( INSPECTABLE* aObject ) const
@@ -248,9 +248,10 @@ PROPERTY_BASE( const wxString& aName, PROPERTY_DISPLAY aDisplay = PT_DEFAULT,
         return m_writeableFunc( aObject );
     }
 
-    void SetWriteableFunc( std::function<bool(INSPECTABLE*)> aFunc )
+    PROPERTY_BASE& SetWriteableFunc( std::function<bool(INSPECTABLE*)> aFunc )
     {
         m_writeableFunc = aFunc;
+        return *this;
     }
 
     /**
@@ -268,21 +269,48 @@ PROPERTY_BASE( const wxString& aName, PROPERTY_DISPLAY aDisplay = PT_DEFAULT,
      */
     virtual size_t TypeHash() const = 0;
 
-    PROPERTY_DISPLAY Display() const
-    {
-        return m_display;
-    }
+    PROPERTY_DISPLAY Display() const { return m_display; }
+    PROPERTY_BASE& SetDisplay( PROPERTY_DISPLAY aDisplay ) { m_display = aDisplay; return *this; }
 
     ORIGIN_TRANSFORMS::COORD_TYPES_T CoordType() const { return m_coordType; }
+    PROPERTY_BASE& SetCoordType( ORIGIN_TRANSFORMS::COORD_TYPES_T aType )
+    {
+        m_coordType = aType;
+        return *this;
+    }
 
-    void SetIsInternal( bool aIsInternal = true ) { m_isInternal = aIsInternal; }
     bool IsInternal() const { return m_isInternal; }
+    PROPERTY_BASE& SetIsInternal( bool aIsInternal = true )
+    {
+        m_isInternal = aIsInternal;
+        return *this;
+    }
 
-    void SetIsDeprecated( bool aIsDeprecated = true ) { m_isDeprecated = aIsDeprecated; }
     bool IsDeprecated() const { return m_isDeprecated; }
+    PROPERTY_BASE& SetIsDeprecated( bool aIsDeprecated = true )
+    {
+        m_isDeprecated = aIsDeprecated;
+        return *this;
+    }
 
     wxString Group() const { return m_group; }
-    void SetGroup( const wxString& aGroup ) { m_group = aGroup; }
+    PROPERTY_BASE& SetGroup( const wxString& aGroup ) { m_group = aGroup; return *this; }
+
+    PROPERTY_BASE& SetValidator( PROPERTY_VALIDATOR_FN&& aValidator )
+    {
+        m_validator = aValidator;
+        return *this;
+    }
+
+    VALIDATOR_RESULT Validate( const wxAny&& aValue, EDA_ITEM* aItem )
+    {
+        return m_validator( std::move( aValue ), aItem );
+    }
+
+    static VALIDATOR_RESULT NullValidator( const wxAny&& aValue, EDA_ITEM* aItem )
+    {
+        return std::nullopt;
+    }
 
 protected:
     template<typename T>
@@ -328,9 +356,18 @@ private:
     virtual wxAny getter( const void* aObject ) const = 0;
 
 private:
-    const wxString         m_name;
-    const PROPERTY_DISPLAY m_display;
-    const ORIGIN_TRANSFORMS::COORD_TYPES_T m_coordType;
+    /**
+     * Permanent identifier for this property.  Property names are an API contract; changing them
+     * after release will impact the Custom DRC Rules system as well as the automatic API binding
+     * system.  Never rename properties; instead deprecate them and hide them from the GUI.
+     */
+    const wxString m_name;
+
+    /// The display style controls how properties are edited in the properties manager GUI
+    PROPERTY_DISPLAY m_display;
+
+    /// The coordinate type controls how distances are mapped to the user coordinate system
+    ORIGIN_TRANSFORMS::COORD_TYPES_T m_coordType;
 
     /// Internal properties are hidden from the GUI but not from the rules editor autocomplete
     bool m_isInternal;
@@ -345,6 +382,8 @@ private:
 
     std::function<bool(INSPECTABLE*)> m_writeableFunc;   ///< Eval to determine if prop is read-only
 
+    PROPERTY_VALIDATOR_FN m_validator;
+
     friend class INSPECTABLE;
 };
 
