@@ -26,37 +26,41 @@
 #ifndef __UNIT_BINDER_H_
 #define __UNIT_BINDER_H_
 
-#include <eda_units.h>
 #include <base_units.h>
-#include <origin_transforms.h>
+#include <units_provider.h>
 #include <libeval/numeric_evaluator.h>
 #include <wx/event.h>
 
+class EDA_BASE_FRAME;
 class EDA_DRAW_FRAME;
 class wxTextEntry;
 class wxSpinButton;
 class wxStaticText;
 
-
 class UNIT_BINDER : public wxEvtHandler
 {
 public:
 
     /**
-     * @param aParent is the parent EDA_DRAW_FRAME.
+     * @param aParent is the parent EDA_BASE_FRAME, used to fetch units and coordinate systems.
      * @param aLabel is the static text used to label the text input widget (note: the label
      *               text, trimmed of its colon, will also be used in error messages)
      * @param aValueCtrl is the control used to edit or display the given value (wxTextCtrl,
-     *               wxComboBox, wxStaticText, etc.).
-     * @param aUnitLabel is the units label displayed after the text input widget
-     * Can be nullptr.
+     *                   wxComboBox, wxStaticText, etc.).
+     * @param aUnitLabel (optional) is the units label displayed after the text input widget
      * @param aAllowEval indicates \a aTextInput's content should be eval'ed before storing
+     * @param aBindFocusEvent indicates the control should respond to DELAY_FOCUS from the
+     *                        parent frame
      */
     UNIT_BINDER( EDA_DRAW_FRAME* aParent,
                  wxStaticText* aLabel, wxWindow* aValueCtrl, wxStaticText* aUnitLabel,
-                 bool aAllowEval = true );
+                 bool aAllowEval = true, bool aBindFocusEvent = true );
 
-    ~UNIT_BINDER() override;
+    UNIT_BINDER( UNITS_PROVIDER* aUnitsProvider, wxWindow* aEventSource,
+                 wxStaticText* aLabel, wxWindow* aValueCtrl, wxStaticText* aUnitLabel,
+                 bool aAllowEval = true, bool aBindFocusEvent = true );
+
+    virtual ~UNIT_BINDER() override;
 
     /**
      * Normally not needed (as the UNIT_BINDER inherits from the parent frame), but can be
@@ -95,6 +99,8 @@ public:
      */
     virtual void SetDoubleValue( double aValue );
 
+    virtual void SetAngleValue( const EDA_ANGLE& aValue );
+
     /**
      * Set new value (in Internal Units) for the text field, taking care of units conversion
      * WITHOUT triggering the update routine.
@@ -111,6 +117,8 @@ public:
      */
     virtual void ChangeDoubleValue( double aValue );
 
+    virtual void ChangeAngleValue( const EDA_ANGLE& aValue );
+
     /**
      * Return the current value in Internal Units.
      */
@@ -124,6 +132,8 @@ public:
      */
     virtual double GetDoubleValue();
 
+    virtual EDA_ANGLE GetAngleValue();
+
     /**
      * Return true if the control holds the indeterminate value (for instance, if it
      * represents a multiple selection of differing values).
@@ -188,6 +198,8 @@ public:
     }
 
 protected:
+    void init( UNITS_PROVIDER* aProvider );
+    void onClick( wxMouseEvent& aEvent );
 
     void onSetFocus( wxFocusEvent& aEvent );
     void onKillFocus( wxFocusEvent& aEvent );
@@ -208,33 +220,52 @@ protected:
      */
     double setPrecision( double aValue, bool aValueUsesUserUnits );
 
-    EDA_DRAW_FRAME*   m_frame;
+protected:
+    wxWindow*           m_eventSource;
+    bool                m_bindFocusEvent;
 
     ///< The bound widgets
-    wxStaticText*     m_label;
-    wxWindow*         m_valueCtrl;
-    wxStaticText*     m_unitLabel;      ///< Can be nullptr
+    wxStaticText*       m_label;
+    wxWindow*           m_valueCtrl;
+    wxStaticText*       m_unitLabel;      ///< Can be nullptr
 
     ///< Currently used units.
-    EDA_UNITS         m_units;
-    bool              m_negativeZero;   ///< Indicates "-0" should be displayed for 0.
-    EDA_DATA_TYPE     m_dataType;
-    int               m_precision;      ///< 0 to 6
+    const EDA_IU_SCALE* m_iuScale;
+    EDA_UNITS           m_units;
+    bool                m_negativeZero;   ///< Indicates "-0" should be displayed for 0.
+    EDA_DATA_TYPE       m_dataType;
+    int                 m_precision;      ///< 0 to 6
 
-    wxString          m_errorMessage;
+    wxString            m_errorMessage;
 
-    NUMERIC_EVALUATOR m_eval;
-    bool              m_allowEval;
-    bool              m_needsEval;
+    NUMERIC_EVALUATOR   m_eval;
+    bool                m_allowEval;
+    bool                m_needsEval;
 
-    long              m_selStart;       ///< Selection start and end of the original text
-    long              m_selEnd;
+    long                m_selStart;       ///< Selection start and end of the original text
+    long                m_selEnd;
+
+    bool                m_unitsInValue;   ///< Units label should be included in value text
 
     /// A reference to an ORIGIN_TRANSFORMS object
-    ORIGIN_TRANSFORMS&               m_originTransforms;
+    ORIGIN_TRANSFORMS&  m_originTransforms;
 
     /// Type of coordinate for display origin transforms
     ORIGIN_TRANSFORMS::COORD_TYPES_T m_coordType;
 };
 
+
+/**
+ * Specialization for wxPropertyGrid, where we have no labels and units are displayed in the editor
+ */
+class PROPERTY_EDITOR_UNIT_BINDER : public UNIT_BINDER
+{
+public:
+    PROPERTY_EDITOR_UNIT_BINDER( EDA_DRAW_FRAME* aParent );
+
+    virtual ~PROPERTY_EDITOR_UNIT_BINDER();
+
+    void SetControl( wxWindow* aControl );
+};
+
 #endif /* __UNIT_BINDER_H_ */
