@@ -57,6 +57,8 @@ DIALOG_PCM::DIALOG_PCM( wxWindow* parent, std::shared_ptr<PLUGIN_CONTENT_MANAGER
         DIALOG_PCM_BASE( parent ),
         m_pcm( pcm )
 {
+    SetDoubleBuffered( true );
+
     m_defaultBitmap = KiBitmap( BITMAPS::icon_pcm );
 
     m_pcm->SetDialogWindow( this );
@@ -290,13 +292,12 @@ void DIALOG_PCM::OnInstallFromFileClicked( wxCommandEvent& event )
     PCM_TASK_MANAGER task_manager( m_pcm );
     task_manager.InstallFromFile( this, open_file_dialog.GetPath() );
 
+    m_changed_package_types.merge( task_manager.GetChangedPackageTypes() );
+
     setInstalledPackages();
 
     if( !m_selectedRepositoryId.IsEmpty() )
         setRepositoryData( m_selectedRepositoryId );
-
-    if( task_manager.ColorSettingsChanged() )
-        Pgm().GetSettingsManager().ReloadColorSettings();
 }
 
 
@@ -313,6 +314,8 @@ void DIALOG_PCM::OnRepositoryChoice( wxCommandEvent& event )
 
 void DIALOG_PCM::setRepositoryData( const wxString& aRepositoryId )
 {
+    m_dialogNotebook->Freeze();
+
     if( m_pcm->CacheRepository( aRepositoryId ) )
     {
         for( const auto& [ packageType, packagesView ] : m_repositoryContentPanels )
@@ -378,6 +381,8 @@ void DIALOG_PCM::setRepositoryData( const wxString& aRepositoryId )
         m_dialogNotebook->SetPageText( 0, wxString::Format( _( "Repository (%d)" ),
                                                             (int) packages.size() ) );
     }
+
+    m_dialogNotebook->Thaw();
 }
 
 
@@ -462,6 +467,8 @@ void DIALOG_PCM::OnApplyChangesClicked( wxCommandEvent& event )
 
     task_manager.RunQueue( this );
 
+    m_changed_package_types.merge( task_manager.GetChangedPackageTypes() );
+
     m_sdbSizer1OK->Enable();
     m_sdbSizer1Apply->Enable();
     m_sdbSizer1Cancel->Enable();
@@ -472,9 +479,6 @@ void DIALOG_PCM::OnApplyChangesClicked( wxCommandEvent& event )
 
     if( !m_selectedRepositoryId.IsEmpty() )
         setRepositoryData( m_selectedRepositoryId );
-
-    if( task_manager.ColorSettingsChanged() )
-        Pgm().GetSettingsManager().ReloadColorSettings();
 }
 
 
