@@ -2,7 +2,8 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2015-2019 CERN
- * Copyright (C) 2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2021-2022 KiCad Developers, see AUTHORS.txt for contributors.
+ *
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  * @author Alejandro García Montoro <alejandro.garciamontoro@gmail.com>
  *
@@ -42,6 +43,7 @@
 #include <vector>
 
 #include <clipper.hpp>                       // for Clipper, PolyNode, Clipp...
+#include <clipper2/clipper.h>
 #include <geometry/geometry_utils.h>
 #include <geometry/polygon_triangulation.h>
 #include <geometry/seg.h>                    // for SEG, OPT_VECTOR2I
@@ -52,9 +54,13 @@
 #include <math/util.h>                       // for KiROUND, rescale
 #include <math/vector2d.h>                   // for VECTOR2I, VECTOR2D, VECTOR2
 #include <md5_hash.h>
+#include <hash.h>
 #include <geometry/shape_segment.h>
 #include <geometry/shape_circle.h>
 
+// Do not keep this for release.  Only for testing clipper
+#include <advanced_config.h>
+
 #include <wx/log.h>
 
 
@@ -89,6 +95,8 @@ SHAPE_POLY_SET::SHAPE_POLY_SET( const SHAPE_POLY_SET& aOther ) :
 {
     if( aOther.IsTriangulationUpToDate() )
     {
+        m_triangulatedPolys.reserve( aOther.TriangulatedPolyCount() );
+
         for( unsigned i = 0; i < aOther.TriangulatedPolyCount(); i++ )
         {
             const TRIANGULATED_POLYGON* poly = aOther.TriangulatedPolygon( i );
@@ -107,6 +115,16 @@ SHAPE_POLY_SET::SHAPE_POLY_SET( const SHAPE_POLY_SET& aOther ) :
 }
 
 
+SHAPE_POLY_SET::SHAPE_POLY_SET( const SHAPE_POLY_SET& aOther, DROP_TRIANGULATION_FLAG ) :
+    SHAPE( aOther ),
+    m_polys( aOther.m_polys )
+{
+    m_triangulationValid = false;
+    m_hash = MD5_HASH();
+    m_triangulatedPolys.clear();
+}
+
+
 SHAPE_POLY_SET::~SHAPE_POLY_SET()
 {
 }
@@ -118,6 +136,12 @@ SHAPE* SHAPE_POLY_SET::Clone() const
 }
 
 
+SHAPE_POLY_SET SHAPE_POLY_SET::CloneDropTriangulation() const
+{
+    return SHAPE_POLY_SET( *this, DROP_TRIANGULATION_FLAG::SINGLETON );
+}
+
+
 bool SHAPE_POLY_SET::GetRelativeIndices( int aGlobalIdx,
                                          SHAPE_POLY_SET::VERTEX_INDEX* aRelativeIndices ) const
 {
@@ -249,7 +273,7 @@ int SHAPE_POLY_SET::Append( int x, int y, int aOutline, int aHole, bool aAllowDu
 }
 
 
-int SHAPE_POLY_SET::Append( SHAPE_ARC& aArc, int aOutline, int aHole )
+int SHAPE_POLY_SET::Append( SHAPE_ARC& aArc, int aOutline, int aHole, double aAccuracy )
 {
     assert( m_polys.size() );
 
@@ -266,7 +290,7 @@ int SHAPE_POLY_SET::Append( SHAPE_ARC& aArc, int aOutline, int aHole )
     assert( aOutline < (int) m_polys.size() );
     assert( idx < (int) m_polys[aOutline].size() );
 
-    m_polys[aOutline][idx].Append( aArc );
+    m_polys[aOutline][idx].Append( aArc, aAccuracy );
 
     return m_polys[aOutline][idx].PointCount();
 }
@@ -439,23 +463,46 @@ bool SHAPE_POLY_SET::GetNeighbourIndexes( int aGlobalIndex, int* aPrevious, int*
 
 bool SHAPE_POLY_SET::IsPolygonSelfIntersecting( int aPolygonIndex ) const
 {
-    CONST_SEGMENT_ITERATOR iterator = CIterateSegmentsWithHoles( aPolygonIndex );
-    CONST_SEGMENT_ITERATOR innerIterator;
+    std::vector<SEG> segments;
+    segments.reserve( FullPointCount() );
 
-    for( iterator = CIterateSegmentsWithHoles( aPolygonIndex ); iterator; iterator++ )
+    for( CONST_SEGMENT_ITERATOR it = CIterateSegmentsWithHoles( aPolygonIndex ); it; it++ )
+        segments.emplace_back( *it );
+
+    std::sort( segments.begin(), segments.end(), []( const SEG& a, const SEG& b )
+            {
+                int min_a_x = std::min( a.A.x, a.B.x );
+                int min_b_x = std::min( b.A.x, b.B.x );
+
+                return min_a_x < min_b_x || ( min_a_x == min_b_x && std::min( a.A.y, a.B.y ) < std::min( b.A.y, b.B.y ) );
+            } );
+
+    for( auto it = segments.begin(); it != segments.end(); ++it )
     {
-        SEG firstSegment = *iterator;
+        SEG& firstSegment = *it;
 
         // Iterate through all remaining segments.
-        innerIterator = iterator;
+        auto innerIterator = it;
+        int max_x = std::max( firstSegment.A.x, firstSegment.B.x );
+        int max_y = std::max( firstSegment.A.y, firstSegment.B.y );
 
         // Start in the next segment, we don't want to check collision between a segment and itself
-        for( innerIterator++; innerIterator; innerIterator++ )
+        for( innerIterator++; innerIterator != segments.end(); innerIterator++ )
         {
-            SEG secondSegment = *innerIterator;
+            SEG& secondSegment = *innerIterator;
+            int min_x = std::min( secondSegment.A.x, secondSegment.B.x );
+            int min_y = std::min( secondSegment.A.y, secondSegment.B.y );
+
+            // We are ordered in minimum point order, so checking the static max (first segment) against
+            // the ordered min will tell us if any of the following segments are withing the BBox
+            if( max_x < min_x || ( max_x == min_x && max_y < min_y ) )
+                break;
+
+            int index_diff = std::abs( firstSegment.Index() - secondSegment.Index() );
+            bool adjacent = ( index_diff == 1) || (index_diff == (segments.size() - 1) ); 
 
             // Check whether the two segments built collide, only when they are not adjacent.
-            if( !iterator.IsAdjacent( innerIterator ) && firstSegment.Collide( secondSegment, 0 ) )
+            if( !adjacent && firstSegment.Collide( secondSegment, 0 ) )
                 return true;
         }
     }
@@ -515,10 +562,10 @@ double SHAPE_POLY_SET::Area()
 
     for( int i = 0; i < OutlineCount(); i++ )
     {
-        area += Outline( i ).Area();
+        area += Outline( i ).Area( true );
 
         for( int j = 0; j < HoleCount( i ); j++ )
-            area -= Hole( i, j ).Area();
+            area -= Hole( i, j ).Area( true );
     }
 
     return area;
@@ -575,8 +622,8 @@ void SHAPE_POLY_SET::booleanOp( ClipperLib::ClipType aType, const SHAPE_POLY_SET
     if( ( aShape.OutlineCount() > 1 || aOtherShape.OutlineCount() > 0 )
         && ( aShape.ArcCount() > 0 || aOtherShape.ArcCount() > 0 ) )
     {
-        wxFAIL_MSG( "Boolean ops on curved polygons are not supported. You should call "
-                    "ClearArcs() before carrying out the boolean operation." );
+        wxFAIL_MSG( wxT( "Boolean ops on curved polygons are not supported. You should call "
+                         "ClearArcs() before carrying out the boolean operation." ) );
     }
 
     ClipperLib::Clipper c;
@@ -674,55 +721,187 @@ void SHAPE_POLY_SET::booleanOp( ClipperLib::ClipType aType, const SHAPE_POLY_SET
 }
 
 
+void SHAPE_POLY_SET::booleanOp( Clipper2Lib::ClipType aType, const SHAPE_POLY_SET& aOtherShape )
+{
+    booleanOp( aType, *this, aOtherShape );
+}
+
+
+void SHAPE_POLY_SET::booleanOp( Clipper2Lib::ClipType aType, const SHAPE_POLY_SET& aShape,
+                                const SHAPE_POLY_SET& aOtherShape )
+{
+    if( ( aShape.OutlineCount() > 1 || aOtherShape.OutlineCount() > 0 )
+        && ( aShape.ArcCount() > 0 || aOtherShape.ArcCount() > 0 ) )
+    {
+        wxFAIL_MSG( wxT( "Boolean ops on curved polygons are not supported. You should call "
+                         "ClearArcs() before carrying out the boolean operation." ) );
+    }
+
+    Clipper2Lib::Clipper64 c;
+
+    std::vector<CLIPPER_Z_VALUE> zValues;
+    std::vector<SHAPE_ARC> arcBuffer;
+    std::map<VECTOR2I, CLIPPER_Z_VALUE> newIntersectPoints;
+
+    Clipper2Lib::Paths64 paths;
+    Clipper2Lib::Paths64 clips;
+
+    for( const POLYGON& poly : aShape.m_polys )
+    {
+        for( size_t i = 0; i < poly.size(); i++ )
+        {
+            paths.push_back( poly[i].convertToClipper2( i == 0, zValues, arcBuffer ) );
+        }
+    }
+
+    for( const POLYGON& poly : aOtherShape.m_polys )
+    {
+        for( size_t i = 0; i < poly.size(); i++ )
+        {
+            clips.push_back( poly[i].convertToClipper2( i == 0, zValues, arcBuffer ) );
+        }
+    }
+
+    c.AddSubject( paths );
+    c.AddClip( clips );
+
+    Clipper2Lib::PolyTree64 solution;
+
+    Clipper2Lib::ZCallback64 callback =
+            [&]( const Clipper2Lib::Point64 & e1bot, const Clipper2Lib::Point64 & e1top,
+                 const Clipper2Lib::Point64 & e2bot, const Clipper2Lib::Point64 & e2top,
+                 Clipper2Lib::Point64 & pt )
+            {
+                auto arcIndex =
+                    [&]( const ssize_t& aZvalue, const ssize_t& aCompareVal = -1 ) -> ssize_t
+                    {
+                        ssize_t retval;
+
+                        retval = zValues.at( aZvalue ).m_SecondArcIdx;
+
+                        if( retval == -1 || ( aCompareVal > 0 && retval != aCompareVal ) )
+                            retval = zValues.at( aZvalue ).m_FirstArcIdx;
+
+                        return retval;
+                    };
+
+                auto arcSegment =
+                    [&]( const ssize_t& aBottomZ, const ssize_t aTopZ ) -> ssize_t
+                    {
+                        ssize_t retval = arcIndex( aBottomZ );
+
+                        if( retval != -1 )
+                        {
+                            if( retval != arcIndex( aTopZ, retval ) )
+                                retval = -1; // Not an arc segment as the two indices do not match
+                        }
+
+                        return retval;
+                    };
+
+                ssize_t e1ArcSegmentIndex = arcSegment( e1bot.z, e1top.z );
+                ssize_t e2ArcSegmentIndex = arcSegment( e2bot.z, e2top.z );
+
+                CLIPPER_Z_VALUE newZval;
+
+                if( e1ArcSegmentIndex != -1 )
+                {
+                    newZval.m_FirstArcIdx = e1ArcSegmentIndex;
+                    newZval.m_SecondArcIdx = e2ArcSegmentIndex;
+                }
+                else
+                {
+                    newZval.m_FirstArcIdx = e2ArcSegmentIndex;
+                    newZval.m_SecondArcIdx = -1;
+                }
+
+                size_t z_value_ptr = zValues.size();
+                zValues.push_back( newZval );
+
+                // Only worry about arc segments for later processing
+                if( newZval.m_FirstArcIdx != -1 )
+                    newIntersectPoints.insert( { VECTOR2I( pt.x, pt.y ), newZval } );
+
+                pt.z = z_value_ptr;
+                //@todo amend X,Y values to true intersection between arcs or arc and segment
+            };
+
+    c.SetZCallback( callback ); // register callback
+
+    c.Execute( aType, Clipper2Lib::FillRule::NonZero, solution );
+
+    importTree( solution, zValues, arcBuffer );
+    solution.Clear(); // Free used memory (not done in dtor)
+}
+
+
 void SHAPE_POLY_SET::BooleanAdd( const SHAPE_POLY_SET& b, POLYGON_MODE aFastMode )
 {
-    booleanOp( ClipperLib::ctUnion, b, aFastMode );
+    if( ADVANCED_CFG::GetCfg().m_UseClipper2 )
+        booleanOp( Clipper2Lib::ClipType::Union, b );
+    else
+        booleanOp( ClipperLib::ctUnion, b, aFastMode );
 }
 
 
 void SHAPE_POLY_SET::BooleanSubtract( const SHAPE_POLY_SET& b, POLYGON_MODE aFastMode )
 {
-    booleanOp( ClipperLib::ctDifference, b, aFastMode );
+    if( ADVANCED_CFG::GetCfg().m_UseClipper2 )
+        booleanOp( Clipper2Lib::ClipType::Difference, b );
+    else
+        booleanOp( ClipperLib::ctDifference, b, aFastMode );
 }
 
 
 void SHAPE_POLY_SET::BooleanIntersection( const SHAPE_POLY_SET& b, POLYGON_MODE aFastMode )
 {
-    booleanOp( ClipperLib::ctIntersection, b, aFastMode );
+    if( ADVANCED_CFG::GetCfg().m_UseClipper2 )
+        booleanOp( Clipper2Lib::ClipType::Intersection, b );
+    else
+        booleanOp( ClipperLib::ctIntersection, b, aFastMode );
 }
 
 
 void SHAPE_POLY_SET::BooleanAdd( const SHAPE_POLY_SET& a, const SHAPE_POLY_SET& b,
                                  POLYGON_MODE aFastMode )
 {
-    booleanOp( ClipperLib::ctUnion, a, b, aFastMode );
+    if( ADVANCED_CFG::GetCfg().m_UseClipper2 )
+        booleanOp( Clipper2Lib::ClipType::Union, a, b );
+    else
+        booleanOp( ClipperLib::ctUnion, a, b, aFastMode );
 }
 
 
 void SHAPE_POLY_SET::BooleanSubtract( const SHAPE_POLY_SET& a, const SHAPE_POLY_SET& b,
                                       POLYGON_MODE aFastMode )
 {
-    booleanOp( ClipperLib::ctDifference, a, b, aFastMode );
+    if( ADVANCED_CFG::GetCfg().m_UseClipper2 )
+        booleanOp( Clipper2Lib::ClipType::Difference, a, b );
+    else
+        booleanOp( ClipperLib::ctDifference, a, b, aFastMode );
 }
 
 
 void SHAPE_POLY_SET::BooleanIntersection( const SHAPE_POLY_SET& a, const SHAPE_POLY_SET& b,
                                           POLYGON_MODE aFastMode )
 {
-    booleanOp( ClipperLib::ctIntersection, a, b, aFastMode );
+    if( ADVANCED_CFG::GetCfg().m_UseClipper2 )
+        booleanOp( Clipper2Lib::ClipType::Intersection, a, b );
+    else
+        booleanOp( ClipperLib::ctIntersection, a, b, aFastMode );
 }
 
 
 void SHAPE_POLY_SET::InflateWithLinkedHoles( int aFactor, int aCircleSegmentsCount,
                                              POLYGON_MODE aFastMode )
 {
-    Simplify( aFastMode );
+    Unfracture( aFastMode );
     Inflate( aFactor, aCircleSegmentsCount );
     Fracture( aFastMode );
 }
 
 
-void SHAPE_POLY_SET::Inflate( int aAmount, int aCircleSegCount, CORNER_STRATEGY aCornerStrategy )
+void SHAPE_POLY_SET::inflate1( int aAmount, int aCircleSegCount, CORNER_STRATEGY aCornerStrategy )
 {
     using namespace ClipperLib;
     // A static table to avoid repetitive calculations of the coefficient
@@ -813,6 +992,110 @@ void SHAPE_POLY_SET::Inflate( int aAmount, int aCircleSegCount, CORNER_STRATEGY
 }
 
 
+void SHAPE_POLY_SET::inflate2( int aAmount, int aCircleSegCount, CORNER_STRATEGY aCornerStrategy )
+{
+    using namespace Clipper2Lib;
+    // A static table to avoid repetitive calculations of the coefficient
+    // 1.0 - cos( M_PI / aCircleSegCount )
+    // aCircleSegCount is most of time <= 64 and usually 8, 12, 16, 32
+    #define SEG_CNT_MAX 64
+    static double arc_tolerance_factor[SEG_CNT_MAX + 1];
+
+    ClipperOffset c;
+
+    // N.B. see the Clipper documentation for jtSquare/jtMiter/jtRound.  They are poorly named
+    // and are not what you'd think they are.
+    // http://www.angusj.com/delphi/clipper/documentation/Docs/Units/ClipperLib/Types/JoinType.htm
+    JoinType joinType = JoinType::Round;    // The way corners are offsetted
+    double   miterLimit = 2.0;      // Smaller value when using jtMiter for joinType
+    JoinType miterFallback = JoinType::Square;
+
+    switch( aCornerStrategy )
+    {
+    case ALLOW_ACUTE_CORNERS:
+        joinType = JoinType::Miter;
+        miterLimit = 10;        // Allows large spikes
+        break;
+
+    case CHAMFER_ACUTE_CORNERS: // Acute angles are chamfered
+        joinType = JoinType::Miter;
+        break;
+
+    case ROUND_ACUTE_CORNERS:   // Acute angles are rounded
+        joinType = JoinType::Miter;
+        break;
+
+    case CHAMFER_ALL_CORNERS:   // All angles are chamfered.
+        joinType = JoinType::Square;
+        break;
+
+    case ROUND_ALL_CORNERS:     // All angles are rounded.
+        joinType = JoinType::Round;
+        break;
+    }
+
+    std::vector<CLIPPER_Z_VALUE> zValues;
+    std::vector<SHAPE_ARC>       arcBuffer;
+
+    for( const POLYGON& poly : m_polys )
+    {
+        Paths64 paths;
+
+        for( size_t i = 0; i < poly.size(); i++ )
+            paths.push_back( poly[i].convertToClipper2( i == 0, zValues, arcBuffer ) );
+
+        c.AddPaths( paths, joinType, EndType::Polygon );
+    }
+
+    // Calculate the arc tolerance (arc error) from the seg count by circle. The seg count is
+    // nn = M_PI / acos(1.0 - c.ArcTolerance / abs(aAmount))
+    // http://www.angusj.com/delphi/clipper/documentation/Docs/Units/ClipperLib/Classes/ClipperOffset/Properties/ArcTolerance.htm
+
+    if( aCircleSegCount < 6 ) // avoid incorrect aCircleSegCount values
+        aCircleSegCount = 6;
+
+    double coeff;
+
+    if( aCircleSegCount > SEG_CNT_MAX || arc_tolerance_factor[aCircleSegCount] == 0 )
+    {
+        coeff = 1.0 - cos( M_PI / aCircleSegCount );
+
+        if( aCircleSegCount <= SEG_CNT_MAX )
+            arc_tolerance_factor[aCircleSegCount] = coeff;
+    }
+    else
+    {
+        coeff = arc_tolerance_factor[aCircleSegCount];
+    }
+
+    c.ArcTolerance( std::abs( aAmount ) * coeff );
+    c.MiterLimit( miterLimit );
+    c.MergeGroups( true );
+    Paths64 solution = c.Execute( aAmount );
+
+    // We get paths back but we need the tree to assign the holes to the correct
+    // outlines
+    Clipper64 c2;
+    PolyTree64 tree;
+    c2.PreserveCollinear = false;
+    c2.ReverseSolution = false;
+    c2.AddSubject( solution );
+    c2.Execute(ClipType::Union, FillRule::Positive, tree);
+
+    importTree( tree, zValues, arcBuffer );
+    tree.Clear();
+}
+
+
+void SHAPE_POLY_SET::Inflate( int aAmount, int aCircleSegCount, CORNER_STRATEGY aCornerStrategy )
+{
+    if( ADVANCED_CFG::GetCfg().m_UseClipper2 )
+        inflate2( aAmount, aCircleSegCount, aCornerStrategy );
+    else
+        inflate1( aAmount, aCircleSegCount, aCornerStrategy );
+}
+
+
 void SHAPE_POLY_SET::importTree( ClipperLib::PolyTree*               tree,
                                  const std::vector<CLIPPER_Z_VALUE>& aZValueBuffer,
                                  const std::vector<SHAPE_ARC>&       aArcBuffer )
@@ -837,6 +1120,69 @@ void SHAPE_POLY_SET::importTree( ClipperLib::PolyTree*               tree,
 }
 
 
+void SHAPE_POLY_SET::importPolyPath( Clipper2Lib::PolyPath64*            aPolyPath,
+                                     const std::vector<CLIPPER_Z_VALUE>& aZValueBuffer,
+                                     const std::vector<SHAPE_ARC>&       aArcBuffer )
+{
+    if( !aPolyPath->IsHole() )
+    {
+        POLYGON paths;
+        paths.reserve( aPolyPath->Count() + 1 );
+        paths.emplace_back( aPolyPath->Polygon(), aZValueBuffer, aArcBuffer );
+
+        for( Clipper2Lib::PolyPath64* child : *aPolyPath )
+        {
+            paths.emplace_back( child->Polygon(), aZValueBuffer, aArcBuffer );
+
+            for( Clipper2Lib::PolyPath64* grandchild : *child )
+                importPolyPath( grandchild, aZValueBuffer, aArcBuffer );
+        }
+
+        m_polys.push_back( paths );
+    }
+}
+
+
+void SHAPE_POLY_SET::importTree( Clipper2Lib::PolyTree64&            tree,
+                                 const std::vector<CLIPPER_Z_VALUE>& aZValueBuffer,
+                                 const std::vector<SHAPE_ARC>&       aArcBuffer )
+{
+    m_polys.clear();
+
+    for( Clipper2Lib::PolyPath64* n : tree )
+        importPolyPath( n, aZValueBuffer, aArcBuffer );
+}
+
+
+void SHAPE_POLY_SET::importPaths( Clipper2Lib::Paths64&              aPath,
+                                 const std::vector<CLIPPER_Z_VALUE>& aZValueBuffer,
+                                 const std::vector<SHAPE_ARC>&       aArcBuffer )
+{
+    m_polys.clear();
+    POLYGON path;
+
+    for( const Clipper2Lib::Path64& n : aPath )
+    {
+        if( Clipper2Lib::Area( n ) > 0 )
+        {
+            if( !path.empty() )
+                m_polys.emplace_back( path );
+
+            path.clear();
+        }
+        else
+        {
+            wxCHECK2_MSG( !path.empty(), continue, wxT( "Cannot add a hole before an outline" ) );
+        }
+
+        path.emplace_back( n, aZValueBuffer, aArcBuffer );
+    }
+
+    if( !path.empty() )
+        m_polys.emplace_back( path );
+}
+
+
 struct FractureEdge
 {
     FractureEdge( int y = 0 ) :
@@ -1029,7 +1375,7 @@ void SHAPE_POLY_SET::fractureSingle( POLYGON& paths )
         // If we can't handle the edge, the zone is broken (maybe)
         if( !num_processed )
         {
-            wxLogWarning( "Broken polygon, dropping path" );
+            wxLogWarning( wxT( "Broken polygon, dropping path" ) );
 
             for( FractureEdge* edge : edges )
                 delete edge;
@@ -1105,8 +1451,9 @@ void SHAPE_POLY_SET::unfractureSingle( SHAPE_POLY_SET::POLYGON& aPoly )
             std::size_t operator()(  const EDGE& aEdge ) const
             {
                 const SEG& a = aEdge.m_poly->CSegment( aEdge.m_index );
-
-                return (std::size_t) ( a.A.x + a.B.x + a.A.y + a.B.y );
+                std::size_t seed = 0xa82de1c0;
+                hash_combine( seed, a.A.x, a.B.x, a.A.y, a.B.y );
+                return seed;
             }
         };
     };
@@ -1190,6 +1537,7 @@ void SHAPE_POLY_SET::unfractureSingle( SHAPE_POLY_SET::POLYGON& aPoly )
     int outline = -1;
 
     POLYGON result;
+    double max_poly = 0.0;
 
     while( queue.size() )
     {
@@ -1214,10 +1562,13 @@ void SHAPE_POLY_SET::unfractureSingle( SHAPE_POLY_SET::POLYGON& aPoly )
 
         outl.SetClosed( true );
 
-        bool cw = outl.Area() > 0.0;
+        double area = std::fabs( outl.Area() );
 
-        if( cw )
+        if( area > max_poly )
+        {
             outline = n;
+            max_poly = area;
+        }
 
         result.push_back( outl );
         n++;
@@ -1258,7 +1609,10 @@ void SHAPE_POLY_SET::Simplify( POLYGON_MODE aFastMode )
 {
     SHAPE_POLY_SET empty;
 
-    booleanOp( ClipperLib::ctUnion, empty, aFastMode );
+    if( ADVANCED_CFG::GetCfg().m_UseClipper2 )
+        booleanOp( Clipper2Lib::ClipType::Union, empty );
+    else
+        booleanOp( ClipperLib::ctUnion, empty, aFastMode );
 }
 
 
@@ -1289,13 +1643,15 @@ int SHAPE_POLY_SET::NormalizeAreaOutlines()
         BooleanSubtract( holesBuffer, SHAPE_POLY_SET::PM_STRICTLY_SIMPLE );
     }
 
-    RemoveNullSegments();
+    // In degenerate cases, simplify might return no outlines
+    if( OutlineCount() > 0 )
+        RemoveNullSegments();
 
     return OutlineCount();
 }
 
 
-const std::string SHAPE_POLY_SET::Format() const
+const std::string SHAPE_POLY_SET::Format( bool aCplusPlus ) const
 {
     std::stringstream ss;
 
@@ -1517,7 +1873,7 @@ bool SHAPE_POLY_SET::Collide( const SHAPE* aShape, int aClearance, int* aActual,
         return false;
     }
 
-    const_cast<SHAPE_POLY_SET*>( this )->CacheTriangulation( true );
+    const_cast<SHAPE_POLY_SET*>( this )->CacheTriangulation( false );
 
     int      actual = INT_MAX;
     VECTOR2I location;
@@ -1589,6 +1945,7 @@ int SHAPE_POLY_SET::RemoveNullSegments()
     VECTOR2I    segmentStart, segmentEnd;
 
     VERTEX_INDEX indexStart;
+    std::vector<VERTEX_INDEX> indices_to_remove;
 
     while( iterator )
     {
@@ -1604,6 +1961,8 @@ int SHAPE_POLY_SET::RemoveNullSegments()
             // Advance
             iterator++;
 
+            // If we have rolled into the next contour, remember its position
+            // segmentStart and segmentEnd remain valid for comparison here
             if( iterator )
                 contourStart = *iterator;
         }
@@ -1612,22 +1971,25 @@ int SHAPE_POLY_SET::RemoveNullSegments()
             // Advance
             iterator++;
 
-            if( iterator )
-                segmentEnd = *iterator;
+            // If we have reached the end of the SHAPE_POLY_SET, something is broken here
+            wxCHECK_MSG( iterator, removed, wxT( "Invalid polygon.  Reached end without noticing.  Please report this error" ) );
+
+            segmentEnd = *iterator;
         }
 
         // Remove segment start if both points are equal
         if( segmentStart == segmentEnd )
         {
-            RemoveVertex( indexStart );
+            indices_to_remove.push_back( indexStart );
             removed++;
-
-            // Advance the iterator one position, as there is one vertex less.
-            if( iterator )
-                iterator++;
         }
     }
 
+    // Proceed in reverse direction to remove the vertices because they are stored as absolute indices in a vector
+    // Removing in reverse order preserves the remaining index values
+    for( auto it = indices_to_remove.rbegin(); it != indices_to_remove.rend(); ++it )
+        RemoveVertex( *it );
+
     return removed;
 }
 
@@ -1638,6 +2000,34 @@ void SHAPE_POLY_SET::DeletePolygon( int aIdx )
 }
 
 
+void SHAPE_POLY_SET::DeletePolygonAndTriangulationData( int aIdx, bool aUpdateHash )
+{
+    m_polys.erase( m_polys.begin() + aIdx );
+
+    if( m_triangulationValid )
+    {
+        for( int ii = m_triangulatedPolys.size() - 1; ii >= 0; --ii )
+        {
+            std::unique_ptr<TRIANGULATED_POLYGON>& triangleSet = m_triangulatedPolys[ii];
+
+            if( triangleSet->GetSourceOutlineIndex() == aIdx )
+                m_triangulatedPolys.erase( m_triangulatedPolys.begin() + ii );
+            else if( triangleSet->GetSourceOutlineIndex() > aIdx )
+                triangleSet->SetSourceOutlineIndex( triangleSet->GetSourceOutlineIndex() - 1 );
+        }
+
+        if( aUpdateHash )
+            m_hash = checksum();
+    }
+}
+
+
+void SHAPE_POLY_SET::UpdateTriangulationDataHash()
+{
+    m_hash = checksum();
+}
+
+
 void SHAPE_POLY_SET::Append( const SHAPE_POLY_SET& aSet )
 {
     m_polys.insert( m_polys.end(), aSet.m_polys.begin(), aSet.m_polys.end() );
@@ -1651,7 +2041,7 @@ void SHAPE_POLY_SET::Append( const VECTOR2I& aP, int aOutline, int aHole )
 
 
 bool SHAPE_POLY_SET::CollideVertex( const VECTOR2I& aPoint,
-                                    SHAPE_POLY_SET::VERTEX_INDEX& aClosestVertex,
+                                    SHAPE_POLY_SET::VERTEX_INDEX* aClosestVertex,
                                     int aClearance ) const
 {
     // Shows whether there was a collision
@@ -1659,10 +2049,8 @@ bool SHAPE_POLY_SET::CollideVertex( const VECTOR2I& aPoint,
 
     // Difference vector between each vertex and aPoint.
     VECTOR2D    delta;
-    double      distance, clearance;
-
-    // Convert clearance to double for precision when comparing distances
-    clearance = aClearance;
+    ecoord      distance_squared;
+    ecoord      clearance_squared = SEG::Square( aClearance );
 
     for( CONST_ITERATOR iterator = CIterateWithHoles(); iterator; iterator++ )
     {
@@ -1670,18 +2058,21 @@ bool SHAPE_POLY_SET::CollideVertex( const VECTOR2I& aPoint,
         delta = *iterator - aPoint;
 
         // Compute distance
-        distance = delta.EuclideanNorm();
+        distance_squared = delta.SquaredEuclideanNorm();
 
         // Check for collisions
-        if( distance <= clearance )
+        if( distance_squared <= clearance_squared )
         {
+            if( !aClosestVertex )
+                return true;
+
             collision = true;
 
-            // Update aClearance to look for closer vertices
-            clearance = distance;
+            // Update clearance to look for closer vertices
+            clearance_squared = distance_squared;
 
             // Store the indices that identify the vertex
-            aClosestVertex = iterator.GetIndex();
+            *aClosestVertex = iterator.GetIndex();
         }
     }
 
@@ -1690,27 +2081,31 @@ bool SHAPE_POLY_SET::CollideVertex( const VECTOR2I& aPoint,
 
 
 bool SHAPE_POLY_SET::CollideEdge( const VECTOR2I& aPoint,
-                                  SHAPE_POLY_SET::VERTEX_INDEX& aClosestVertex,
+                                  SHAPE_POLY_SET::VERTEX_INDEX* aClosestVertex,
                                   int aClearance ) const
 {
     // Shows whether there was a collision
-    bool collision = false;
+    bool   collision = false;
+    ecoord clearance_squared = SEG::Square( aClearance );
 
     for( CONST_SEGMENT_ITERATOR iterator = CIterateSegmentsWithHoles(); iterator; iterator++ )
     {
         const SEG currentSegment = *iterator;
-        int distance = currentSegment.Distance( aPoint );
+        ecoord    distance_squared = currentSegment.SquaredDistance( aPoint );
 
         // Check for collisions
-        if( distance <= aClearance )
+        if( distance_squared <= clearance_squared )
         {
+            if( !aClosestVertex )
+                return true;
+
             collision = true;
 
-            // Update aClearance to look for closer edges
-            aClearance = distance;
+            // Update clearance to look for closer edges
+            clearance_squared = distance_squared;
 
             // Store the indices that identify the vertex
-            aClosestVertex = iterator.GetIndex();
+            *aClosestVertex = iterator.GetIndex();
         }
     }
 
@@ -1838,7 +2233,7 @@ void SHAPE_POLY_SET::Mirror( bool aX, bool aY, const VECTOR2I& aRef )
 }
 
 
-void SHAPE_POLY_SET::Rotate( double aAngle, const VECTOR2I& aCenter )
+void SHAPE_POLY_SET::Rotate( const EDA_ANGLE& aAngle, const VECTOR2I& aCenter )
 {
     for( POLYGON& poly : m_polys )
     {
@@ -2155,11 +2550,11 @@ SHAPE_POLY_SET::POLYGON SHAPE_POLY_SET::chamferFilletPolygon( CORNER_MODE aMode,
                     argument = 1;
 
                 double arcAngle = acos( argument );
-                double arcAngleDegrees = arcAngle * 180.0 / M_PI;
-                int    segments = GetArcToSegmentCount( radius, aErrorMax, arcAngleDegrees );
+                int    segments = GetArcToSegmentCount( radius, aErrorMax,
+                                                        EDA_ANGLE( arcAngle, RADIANS_T ) );
 
-                double  deltaAngle  = arcAngle / segments;
-                double  startAngle  = atan2( -ys, xs );
+                double deltaAngle = arcAngle / segments;
+                double startAngle = atan2( -ys, xs );
 
                 // Flip arc for inner corners
                 if( xa * yb - ya * xb <= 0 )
@@ -2203,21 +2598,18 @@ SHAPE_POLY_SET &SHAPE_POLY_SET::operator=( const SHAPE_POLY_SET& aOther )
 {
     static_cast<SHAPE&>(*this) = aOther;
     m_polys = aOther.m_polys;
+
     m_triangulatedPolys.clear();
-    m_triangulationValid = false;
 
-    if( aOther.IsTriangulationUpToDate() )
+    for( unsigned i = 0; i < aOther.TriangulatedPolyCount(); i++ )
     {
-        for( unsigned i = 0; i < aOther.TriangulatedPolyCount(); i++ )
-        {
-            const TRIANGULATED_POLYGON* poly = aOther.TriangulatedPolygon( i );
-            m_triangulatedPolys.push_back( std::make_unique<TRIANGULATED_POLYGON>( *poly ) );
-        }
-
-        m_hash = aOther.GetHash();
-        m_triangulationValid = true;
+        const TRIANGULATED_POLYGON* poly = aOther.TriangulatedPolygon( i );
+        m_triangulatedPolys.push_back( std::make_unique<TRIANGULATED_POLYGON>( *poly ) );
     }
 
+    m_hash = aOther.m_hash;
+    m_triangulationValid = aOther.m_triangulationValid;
+
     return *this;
 }
 
@@ -2245,8 +2637,7 @@ bool SHAPE_POLY_SET::IsTriangulationUpToDate() const
 }
 
 
-static void partitionPolyIntoRegularCellGrid( const SHAPE_POLY_SET& aPoly, int aSize,
-                                              SHAPE_POLY_SET& aOut )
+static SHAPE_POLY_SET partitionPolyIntoRegularCellGrid( const SHAPE_POLY_SET& aPoly, int aSize )
 {
     BOX2I bb = aPoly.BBox();
 
@@ -2254,7 +2645,7 @@ static void partitionPolyIntoRegularCellGrid( const SHAPE_POLY_SET& aPoly, int a
     double h = bb.GetHeight();
 
     if( w == 0.0 || h == 0.0 )
-        return;
+        return aPoly;
 
     int n_cells_x, n_cells_y;
 
@@ -2305,17 +2696,17 @@ static void partitionPolyIntoRegularCellGrid( const SHAPE_POLY_SET& aPoly, int a
     ps1.Fracture( SHAPE_POLY_SET::PM_FAST );
     ps2.Fracture( SHAPE_POLY_SET::PM_FAST );
 
-    aOut = ps1;
-
     for( int i = 0; i < ps2.OutlineCount(); i++ )
-        aOut.AddOutline( ps2.COutline( i ) );
+        ps1.AddOutline( ps2.COutline( i ) );
 
-    if( !aOut.OutlineCount() )
-        aOut = aPoly;
+    if( ps1.OutlineCount() )
+        return ps1;
+    else
+        return aPoly;
 }
 
 
-void SHAPE_POLY_SET::CacheTriangulation( bool aPartition )
+void SHAPE_POLY_SET::CacheTriangulation( bool aPartition, bool aSimplify )
 {
     bool recalculate = !m_hash.IsValid();
     MD5_HASH hash;
@@ -2337,47 +2728,85 @@ void SHAPE_POLY_SET::CacheTriangulation( bool aPartition )
     if( !recalculate )
         return;
 
-    SHAPE_POLY_SET tmpSet;
+    auto triangulate =
+            []( SHAPE_POLY_SET& polySet, int forOutline,
+                std::vector<std::unique_ptr<TRIANGULATED_POLYGON>>& dest )
+            {
+                bool triangulationValid = false;
+                int pass = 0;
+
+                while( polySet.OutlineCount() > 0 )
+                {
+                    if( !dest.empty() && dest.back()->GetTriangleCount() == 0 )
+                        dest.erase( dest.end() - 1 );
+
+                    dest.push_back( std::make_unique<TRIANGULATED_POLYGON>( forOutline ) );
+                    PolygonTriangulation tess( *dest.back() );
+
+                    // If the tessellation fails, we re-fracture the polygon, which will
+                    // first simplify the system before fracturing and removing the holes
+                    // This may result in multiple, disjoint polygons.
+                    if( !tess.TesselatePolygon( polySet.Polygon( 0 ).front() ) )
+                    {
+                        ++pass;
+
+                        if( pass == 1 )
+                            polySet.Fracture( PM_FAST );
+                        else if( pass == 2 )
+                            polySet.Fracture( PM_STRICTLY_SIMPLE );
+                        else
+                            break;
+
+                        triangulationValid = false;
+                        continue;
+                    }
+
+                    polySet.DeletePolygon( 0 );
+                    triangulationValid = true;
+                }
+
+                return triangulationValid;
+            };
+
+    m_triangulatedPolys.clear();
+    m_triangulationValid = true;
 
     if( aPartition )
     {
-        // This partitions into regularly-sized grids (1cm in Pcbnew)
-        SHAPE_POLY_SET flattened( *this );
-        flattened.ClearArcs();
-        partitionPolyIntoRegularCellGrid( flattened, 1e7, tmpSet );
+        for( int ii = 0; ii < OutlineCount(); ++ii )
+        {
+            // This partitions into regularly-sized grids (1cm in Pcbnew)
+            SHAPE_POLY_SET flattened( Outline( ii ) );
+
+            for( int jj = 0; jj < HoleCount( ii ); ++jj )
+                flattened.AddHole( Hole( ii, jj ) );
+
+            flattened.ClearArcs();
+
+            if( flattened.HasHoles() || flattened.IsSelfIntersecting() )
+                flattened.Fracture( PM_FAST );
+            else if( aSimplify )
+                flattened.Simplify( PM_FAST );
+
+            SHAPE_POLY_SET partitions = partitionPolyIntoRegularCellGrid( flattened, 1e7 );
+
+            // This pushes the triangulation for all polys in partitions
+            // to be referenced to the ii-th polygon
+            m_triangulationValid &= triangulate( partitions, ii , m_triangulatedPolys );
+        }
     }
     else
     {
-        tmpSet = *this;
+        SHAPE_POLY_SET tmpSet( *this );
 
-        if( tmpSet.HasHoles() )
+        tmpSet.ClearArcs();
+
+        if( tmpSet.HasHoles() || tmpSet.IsSelfIntersecting() )
             tmpSet.Fracture( PM_FAST );
-    }
+        else if( aSimplify )
+            tmpSet.Simplify( PM_FAST );
 
-    m_triangulatedPolys.clear();
-    m_triangulationValid = false;
-
-    while( tmpSet.OutlineCount() > 0 )
-    {
-
-        if( !m_triangulatedPolys.empty() && m_triangulatedPolys.back()->GetTriangleCount() == 0 )
-            m_triangulatedPolys.erase( m_triangulatedPolys.end() - 1 );
-
-        m_triangulatedPolys.push_back( std::make_unique<TRIANGULATED_POLYGON>() );
-        PolygonTriangulation tess( *m_triangulatedPolys.back() );
-
-        // If the tessellation fails, we re-fracture the polygon, which will
-        // first simplify the system before fracturing and removing the holes
-        // This may result in multiple, disjoint polygons.
-        if( !tess.TesselatePolygon( tmpSet.Polygon( 0 ).front() ) )
-        {
-            tmpSet.Fracture( PM_FAST );
-            m_triangulationValid = false;
-            continue;
-        }
-
-        tmpSet.DeletePolygon( 0 );
-        m_triangulationValid = true;
+        m_triangulationValid = triangulate( tmpSet, -1, m_triangulatedPolys );
     }
 
     if( m_triangulationValid )
@@ -2463,7 +2892,7 @@ size_t SHAPE_POLY_SET::GetIndexableSubshapeCount() const
 }
 
 
-void SHAPE_POLY_SET:: GetIndexableSubshapes( std::vector<SHAPE*>& aSubshapes )
+void SHAPE_POLY_SET::GetIndexableSubshapes( std::vector<const SHAPE*>& aSubshapes ) const
 {
     aSubshapes.reserve( GetIndexableSubshapeCount() );
 
@@ -2496,6 +2925,7 @@ void SHAPE_POLY_SET::TRIANGULATED_POLYGON::AddTriangle( int a, int b, int c )
 
 SHAPE_POLY_SET::TRIANGULATED_POLYGON::TRIANGULATED_POLYGON( const TRIANGULATED_POLYGON& aOther )
 {
+    m_sourceOutline = aOther.m_sourceOutline;
     m_vertices = aOther.m_vertices;
     m_triangles = aOther.m_triangles;
 
@@ -2506,6 +2936,7 @@ SHAPE_POLY_SET::TRIANGULATED_POLYGON::TRIANGULATED_POLYGON( const TRIANGULATED_P
 
 SHAPE_POLY_SET::TRIANGULATED_POLYGON& SHAPE_POLY_SET::TRIANGULATED_POLYGON::operator=( const TRIANGULATED_POLYGON& aOther )
 {
+    m_sourceOutline = aOther.m_sourceOutline;
     m_vertices = aOther.m_vertices;
     m_triangles = aOther.m_triangles;
 
@@ -2516,7 +2947,8 @@ SHAPE_POLY_SET::TRIANGULATED_POLYGON& SHAPE_POLY_SET::TRIANGULATED_POLYGON::oper
 }
 
 
-SHAPE_POLY_SET::TRIANGULATED_POLYGON::TRIANGULATED_POLYGON()
+SHAPE_POLY_SET::TRIANGULATED_POLYGON::TRIANGULATED_POLYGON( int aSourceOutline ) :
+        m_sourceOutline( aSourceOutline )
 {
 }
 
@@ -2524,3 +2956,55 @@ SHAPE_POLY_SET::TRIANGULATED_POLYGON::TRIANGULATED_POLYGON()
 SHAPE_POLY_SET::TRIANGULATED_POLYGON::~TRIANGULATED_POLYGON()
 {
 }
+
+
+const SHAPE_POLY_SET
+SHAPE_POLY_SET::BuildPolysetFromOrientedPaths( const std::vector<SHAPE_LINE_CHAIN>& aPaths,
+                                               bool aReverseOrientation, bool aEvenOdd )
+{
+    ClipperLib::Clipper  clipper;
+    ClipperLib::PolyTree tree;
+
+    // fixme: do we need aReverseOrientation?
+
+    for( const SHAPE_LINE_CHAIN& path : aPaths )
+    {
+        ClipperLib::Path lc;
+
+        for( int i = 0; i < path.PointCount(); i++ )
+        {
+            lc.emplace_back( path.CPoint( i ).x, path.CPoint( i ).y );
+        }
+
+        clipper.AddPath( lc, ClipperLib::ptSubject, true );
+    }
+
+    clipper.StrictlySimple( true );
+    clipper.Execute( ClipperLib::ctUnion, tree,
+                     aEvenOdd ? ClipperLib::pftEvenOdd : ClipperLib::pftNonZero,
+                     ClipperLib::pftNonZero );
+    SHAPE_POLY_SET result;
+
+    for( ClipperLib::PolyNode* n = tree.GetFirst(); n; n = n->GetNext() )
+    {
+        if( !n->IsHole() )
+        {
+            int outl = result.NewOutline();
+
+            for( unsigned int i = 0; i < n->Contour.size(); i++ )
+                result.Outline( outl ).Append( n->Contour[i].X, n->Contour[i].Y );
+
+            for( unsigned int i = 0; i < n->Childs.size(); i++ )
+            {
+                int outh = result.NewHole( outl );
+                for( unsigned int j = 0; j < n->Childs[i]->Contour.size(); j++ )
+                {
+                    result.Hole( outl, outh )
+                            .Append( n->Childs[i]->Contour[j].X, n->Childs[i]->Contour[j].Y );
+                }
+            }
+        }
+    }
+
+    return result;
+}
