@@ -78,6 +78,10 @@ BOARD::BOARD() :
         m_designSettings( new BOARD_DESIGN_SETTINGS( nullptr, "board.design_settings" ) ),
         m_NetInfo( this )
 {
+    // A too small value do not allow connecting 2 shapes (i.e. segments) not exactly connected
+    // A too large value do not allow safely connecting 2 shapes like very short segments.
+    m_outlinesChainingEpsilon = pcbIUScale.mmToIU( DEFAULT_CHAINING_EPSILON_MM );
+
     // we have not loaded a board yet, assume latest until then.
     m_fileFormatVersionAtLoad = LEGACY_BOARD_FILE_VERSION;
 
@@ -316,6 +320,20 @@ std::vector<PCB_MARKER*> BOARD::ResolveDRCExclusions()
     {
         PCB_MARKER* marker = PCB_MARKER::Deserialize( exclusionData );
 
+        if( !marker )
+            continue;
+
+        // Check to see if items still exist
+        for( const KIID& guid : marker->GetRCItem()->GetIDs() )
+        {
+            if( GetItem( guid ) == DELETED_BOARD_ITEM::GetInstance() )
+            {
+                delete marker;
+                marker = nullptr;
+                break;
+            }
+        }
+
         if( marker )
         {
             marker->SetExcluded( true );
@@ -1954,7 +1972,8 @@ ZONE* BOARD::AddArea( PICKED_ITEMS_LIST* aNewZonesList, int aNetcode, PCB_LAYER_
 bool BOARD::GetBoardPolygonOutlines( SHAPE_POLY_SET& aOutlines,
                                      OUTLINE_ERROR_HANDLER* aErrorHandler )
 {
-    int chainingEpsilon = pcbIUScale.mmToIU( 0.02 ); // max dist from one endPt to next startPt
+    // max dist from one endPt to next startPt: use the current value
+    int chainingEpsilon = GetOutlinesChainingEpsilon();
 
     bool success = BuildBoardPolygonOutlines( this, aOutlines, GetDesignSettings().m_MaxError,
                                               chainingEpsilon, aErrorHandler );
@@ -2330,3 +2349,21 @@ void BOARD::ConvertBrdLayerToPolygonalContours( PCB_LAYER_ID aLayer,
 }
 
 
+const std::set<BOARD_ITEM*> BOARD::GetItemSet()
+{
+    std::set<BOARD_ITEM*> items;
+
+#define INSERT_ITEMS( collection )                                                                 \
+    for( BOARD_ITEM * item : collection )                                                          \
+        items.insert( item );
+
+    INSERT_ITEMS( m_tracks )
+    INSERT_ITEMS( m_footprints )
+    INSERT_ITEMS( m_drawings )
+    INSERT_ITEMS( m_zones )
+    INSERT_ITEMS( m_markers )
+
+#undef INSERT_ITEMS
+
+    return items;
+}
