@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2019 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 2019-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2019-2023 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,6 +29,7 @@
 #include <dialogs/dialog_color_picker.h>
 #include <widgets/paged_dialog.h>
 #include <widgets/layer_box_selector.h>
+#include <widgets/wx_panel.h>
 #include <wx/log.h>
 #include <wx/rawbmp.h>
 #include <math/util.h>      // for KiROUND
@@ -48,7 +49,7 @@
 
 #include <locale_io.h>
 #include <eda_list_dialog.h>
-#include <string_utils.h>               // for Double2Str()
+#include <string_utils.h>               // for UIDouble2Str()
 
 
 // Some wx widget ID to know what widget has fired a event:
@@ -73,17 +74,6 @@ static wxColor pasteColor( 200, 200, 200 );
 static void drawBitmap( wxBitmap& aBitmap, wxColor aColor );
 
 
-wxString getColourAsHexString( const wxColour aColour )
-{
-    // NB: wxWidgets 3.0's color.GetAsString( wxC2S_HTML_SYNTAX ) pukes on alpha
-    return wxString::Format( wxT("#%02X%02X%02X%02X" ),
-                             aColour.Red(),
-                             aColour.Green(),
-                             aColour.Blue(),
-                             aColour.Alpha() );
-}
-
-
 PANEL_SETUP_BOARD_STACKUP::PANEL_SETUP_BOARD_STACKUP( PAGED_DIALOG* aParent, PCB_EDIT_FRAME* aFrame,
                                                       PANEL_SETUP_LAYERS* aPanelLayers ):
         PANEL_SETUP_BOARD_STACKUP_BASE( aParent->GetTreebook() ),
@@ -96,7 +86,8 @@ PANEL_SETUP_BOARD_STACKUP::PANEL_SETUP_BOARD_STACKUP( PAGED_DIALOG* aParent, PCB
     m_panelLayers = aPanelLayers;
     m_board = m_frame->GetBoard();
     m_brdSettings = &m_board->GetDesignSettings();
-    m_units = aFrame->GetUserUnits();
+
+    m_panel1->SetBorders( false, false, true, true );
 
     m_panelLayers->SetPhysicalStackupPanel( this );
 
@@ -104,17 +95,17 @@ PANEL_SETUP_BOARD_STACKUP::PANEL_SETUP_BOARD_STACKUP( PAGED_DIALOG* aParent, PCB
 
     // Calculates a good size for color swatches (icons) in this dialog
     wxClientDC dc( this );
-    m_colorSwatchesSize = dc.GetTextExtent( "XX" );
-    m_colorIconsSize = dc.GetTextExtent( "XXXX" );
+    m_colorSwatchesSize = dc.GetTextExtent( wxT( "XX" ) );
+    m_colorIconsSize = dc.GetTextExtent( wxT( "XXXX" ) );
 
     // Calculates a good size for wxTextCtrl to enter Epsilon R and Loss tan
     // ("0.0000000" + margins)
-    m_numericFieldsSize = dc.GetTextExtent( "X.XXXXXXX" );
+    m_numericFieldsSize = dc.GetTextExtent( wxT( "X.XXXXXXX" ) );
     m_numericFieldsSize.y = -1;     // Use default for the vertical size
 
     // Calculates a minimal size for wxTextCtrl to enter a dim with units
     // ("000.0000000 mils" + margins)
-    m_numericTextCtrlSize = dc.GetTextExtent( "XXX.XXXXXXX mils" );
+    m_numericTextCtrlSize = dc.GetTextExtent( wxT( "XXX.XXXXXXX mils" ) );
     m_numericTextCtrlSize.y = -1;     // Use default for the vertical size
 
     // The grid column containing the lock checkbox is kept to a minimal
@@ -142,8 +133,10 @@ PANEL_SETUP_BOARD_STACKUP::~PANEL_SETUP_BOARD_STACKUP()
 
 void PANEL_SETUP_BOARD_STACKUP::onCopperLayersSelCount( wxCommandEvent& event )
 {
+    int oldBoardWidth = static_cast<int>( m_frame->ValueFromString( m_tcCTValue->GetValue() ) );
     updateCopperLayerCount();
     showOnlyActiveLayers();
+    setDefaultLayerWidths( oldBoardWidth );
     computeBoardThickness();
     Layout();
 }
@@ -175,9 +168,8 @@ void PANEL_SETUP_BOARD_STACKUP::onAdjustDielectricThickness( wxCommandEvent& eve
         }
 
         wxTextCtrl* textCtrl = static_cast<wxTextCtrl*>( ui_item.m_ThicknessCtrl );
-        wxString txt = textCtrl->GetValue();
 
-        int item_thickness = ValueFromString( m_frame->GetUserUnits(), txt );
+        int item_thickness = m_frame->ValueFromString( textCtrl->GetValue() );
         min_thickness += item_thickness;
     }
 
@@ -186,12 +178,12 @@ void PANEL_SETUP_BOARD_STACKUP::onAdjustDielectricThickness( wxCommandEvent& eve
     if( min_thickness == 0 )
     {
         title.Printf( _( "Enter board thickness in %s:" ),
-                      GetAbbreviatedUnitsLabel( m_frame->GetUserUnits() ).Trim( false ) );
+                      EDA_UNIT_UTILS::GetText( m_frame->GetUserUnits() ).Trim( false ) );
     }
     else
     {
         title.Printf( _( "Enter expected board thickness (min value %s):" ),
-                      StringFromValue( m_frame->GetUserUnits(), min_thickness, true ) );
+                      m_frame->StringFromValue( min_thickness, true ) );
     }
 
     wxTextEntryDialog dlg( this, title, _( "Adjust Unlocked Dielectric Layers" ) );
@@ -199,14 +191,12 @@ void PANEL_SETUP_BOARD_STACKUP::onAdjustDielectricThickness( wxCommandEvent& eve
     if( dlg.ShowModal() != wxID_OK )
         return;
 
-    wxString result = dlg.GetValue();
+    int iu_thickness = m_frame->ValueFromString( dlg.GetValue() );
 
-    int iu_thickness = ValueFromString( m_frame->GetUserUnits(), result );
-
-    if( iu_thickness <= min_thickness )
+    if( iu_thickness < min_thickness )
     {
         wxMessageBox( wxString::Format( _("Value too small (min value %s)." ),
-                      StringFromValue( m_frame->GetUserUnits(), min_thickness, true ) ) );
+                                        m_frame->StringFromValue( min_thickness, true ) ) );
         return;
     }
 
@@ -214,14 +204,7 @@ void PANEL_SETUP_BOARD_STACKUP::onAdjustDielectricThickness( wxCommandEvent& eve
 
     if( items_candidate.size() )
     {
-        int thickness_layer = ( iu_thickness - min_thickness ) / items_candidate.size();
-        wxString txt = StringFromValue( m_frame->GetUserUnits(), thickness_layer, true );
-
-        for( BOARD_STACKUP_ROW_UI_ITEM* ui_item : items_candidate )
-        {
-            wxTextCtrl* textCtrl = static_cast<wxTextCtrl*>( ui_item->m_ThicknessCtrl );
-            textCtrl->ChangeValue( txt );
-        }
+        setDefaultLayerWidths( iu_thickness );
     }
     else
     {
@@ -307,7 +290,8 @@ void PANEL_SETUP_BOARD_STACKUP::onAddDielectricLayer( wxCommandEvent& event )
         }
     }
 
-    EDA_LIST_DIALOG dlg( m_parentDialog, _( "Add Dielectric Layer" ), headers, d_list );
+    EDA_LIST_DIALOG dlg( m_parentDialog, _( "Add Dielectric Layer" ), headers, d_list,
+                         wxEmptyString, false /* do not sort the list: it is **expected** in stack order */);
     dlg.SetListLabel( _( "Select layer to add:" ) );
     dlg.HideFilter();
 
@@ -363,7 +347,8 @@ void PANEL_SETUP_BOARD_STACKUP::onRemoveDielectricLayer( wxCommandEvent& event )
         }
     }
 
-    EDA_LIST_DIALOG dlg( m_parentDialog, _( "Remove Dielectric Layer" ), headers, d_list );
+    EDA_LIST_DIALOG dlg( m_parentDialog, _( "Remove Dielectric Layer" ), headers, d_list,
+                         wxEmptyString, false /* do not sort the list: it is **expected** in stack order */ );
     dlg.SetListLabel( _( "Select layer to remove:" ) );
     dlg.HideFilter();
 
@@ -408,7 +393,7 @@ void PANEL_SETUP_BOARD_STACKUP::onExportToClipboard( wxCommandEvent& event )
         return;
 
     // Build a ASCII representation of stackup and copy it in the clipboard
-    wxString report = BuildStackupReport( m_stackup, m_units );
+    wxString report = BuildStackupReport( m_stackup, m_frame->GetUserUnits() );
 
     wxLogNull doNotLog; // disable logging of failed clipboard actions
 
@@ -432,14 +417,120 @@ wxColor PANEL_SETUP_BOARD_STACKUP::GetSelectedColor( int aRow ) const
 
     int idx = choice ? choice->GetSelection() : 0;
 
-    if( idx != GetColorUserDefinedListIdx() ) // a standard color is selected
-        return GetColorStandardList()[idx].GetColor( item->GetType() );
+    if( IsCustomColorIdx( item->GetType(), idx ) )
+        return m_rowUiItemsList[aRow].m_UserColor.ToColour();
     else
-        return m_rowUiItemsList[aRow].m_UserColor;
+        return GetStandardColor( item->GetType(), idx ).ToColour();
 }
 
 
-void PANEL_SETUP_BOARD_STACKUP::computeBoardThickness()
+void PANEL_SETUP_BOARD_STACKUP::setDefaultLayerWidths( int targetThickness )
+{
+    // This function tries to set the PCB thickness to the parameter and uses a fixed prepreg thickness
+    // of 0.1 mm. The core thickness is calculated accordingly as long as it also stays above 0.1mm.
+    // If the core thickness would be smaller than the default pregreg thickness given here,
+    // both are reduced towards zero to arrive at the correct PCB width
+    const int prePregDefaultThickness = pcbIUScale.mmToIU( 0.1 );
+
+    int copperLayerCount = GetCopperLayerCount();
+
+    // This code is for a symmetrical PCB stackup with even copper layer count
+    // If asymmetric stackups were to be implemented, the following layer count calculations
+    // for dielectric/core layers might need adjustments.
+    wxASSERT( copperLayerCount % 2 == 0 );
+
+    int  dielectricLayerCount = copperLayerCount - 1;
+    int  coreLayerCount = copperLayerCount / 2 - 1;
+
+    wxASSERT( dielectricLayerCount > 0 );
+
+    bool currentLayerIsCore = false;
+
+    // start with prepreg layer on the outside, except when creating two-layer-board
+    if( copperLayerCount == 2 )
+    {
+        coreLayerCount = 1;
+        currentLayerIsCore = true;
+    }
+
+    wxASSERT( coreLayerCount > 0 );
+
+    int prePregLayerCount = dielectricLayerCount - coreLayerCount;
+
+    int totalWidthOfFixedItems = 0;
+
+    for( BOARD_STACKUP_ROW_UI_ITEM& ui_item : m_rowUiItemsList )
+    {
+        BOARD_STACKUP_ITEM* item = ui_item.m_Item;
+
+        if( !item->IsThicknessEditable() || !ui_item.m_isEnabled )
+            continue;
+
+        wxCheckBox* cbLock = dynamic_cast<wxCheckBox*>( ui_item.m_ThicknessLockCtrl );
+        wxChoice*   layerType = dynamic_cast<wxChoice*>( ui_item.m_LayerTypeCtrl );
+
+        if( ( item->GetType() == BS_ITEM_TYPE_DIELECTRIC && !layerType ) || item->GetType() == BS_ITEM_TYPE_SOLDERMASK
+            || item->GetType() == BS_ITEM_TYPE_COPPER || ( cbLock && cbLock->GetValue() ) )
+        {
+            // secondary dielectric layers, mask and copper layers and locked layers will be counted as fixed width
+            wxTextCtrl* textCtrl = static_cast<wxTextCtrl*>( ui_item.m_ThicknessCtrl );
+            int         item_thickness = m_frame->ValueFromString( textCtrl->GetValue() );
+
+            totalWidthOfFixedItems += item_thickness;
+        }
+    }
+
+    // Width that hasn't been allocated by fixed items
+    int remainingWidth =
+            targetThickness - totalWidthOfFixedItems - ( prePregDefaultThickness * prePregLayerCount );
+
+    int prePregThickness = prePregDefaultThickness;
+    int coreThickness = remainingWidth / coreLayerCount;
+
+    if( coreThickness < prePregThickness )
+    {
+        // There's not enough room for prepreg and core layers of at least 0.1 mm, so adjust both down
+        remainingWidth = targetThickness - totalWidthOfFixedItems;
+        prePregThickness = coreThickness = std::max( 0, remainingWidth / dielectricLayerCount );
+    }
+
+    for( BOARD_STACKUP_ROW_UI_ITEM& ui_item : m_rowUiItemsList )
+    {
+        BOARD_STACKUP_ITEM* item = ui_item.m_Item;
+
+        if( item->GetType() != BS_ITEM_TYPE_DIELECTRIC || !ui_item.m_isEnabled )
+            continue;
+
+        wxChoice* layerType = dynamic_cast<wxChoice*>( ui_item.m_LayerTypeCtrl );
+
+        if( !layerType )
+        {
+            // ignore secondary dielectric layers
+            continue;
+        }
+
+        wxCheckBox* cbLock = dynamic_cast<wxCheckBox*>( ui_item.m_ThicknessLockCtrl );
+
+        if( cbLock && cbLock->GetValue() )
+        {
+            currentLayerIsCore = !currentLayerIsCore;
+
+            // Don't override width of locked layer
+            continue;
+        }
+
+        int layerThickness = currentLayerIsCore ? coreThickness : prePregThickness;
+
+        wxTextCtrl* textCtrl = static_cast<wxTextCtrl*>( ui_item.m_ThicknessCtrl );
+        layerType->SetSelection( currentLayerIsCore ? 0 : 1 );
+        textCtrl->SetValue( m_frame->StringFromValue( layerThickness ) );
+
+        currentLayerIsCore = !currentLayerIsCore;
+    }
+}
+
+
+int PANEL_SETUP_BOARD_STACKUP::computeBoardThickness()
 {
     int thickness = 0;
 
@@ -451,17 +542,18 @@ void PANEL_SETUP_BOARD_STACKUP::computeBoardThickness()
             continue;
 
         wxTextCtrl* textCtrl = static_cast<wxTextCtrl*>( ui_item.m_ThicknessCtrl );
-        wxString txt = textCtrl->GetValue();
+        int         item_thickness = m_frame->ValueFromString( textCtrl->GetValue() );
 
-        int item_thickness = ValueFromString( m_frame->GetUserUnits(), txt );
         thickness += item_thickness;
     }
 
-    wxString thicknessStr = StringFromValue( m_units, thickness, true );
+    wxString thicknessStr = m_frame->StringFromValue( thickness, true );
 
     // The text in the event will translate to the value for the text control
     // and is only updated if it changed
     m_tcCTValue->ChangeValue( thicknessStr );
+
+    return thickness;
 }
 
 
@@ -488,7 +580,6 @@ void PANEL_SETUP_BOARD_STACKUP::updateCopperLayerCount()
 void PANEL_SETUP_BOARD_STACKUP::synchronizeWithBoard( bool aFullSync )
 {
     const BOARD_STACKUP&   brd_stackup = m_brdSettings->GetStackupDescriptor();
-    const FAB_LAYER_COLOR* color_list = GetColorStandardList();
 
     if( aFullSync )
     {
@@ -527,8 +618,7 @@ void PANEL_SETUP_BOARD_STACKUP::synchronizeWithBoard( bool aFullSync )
             wxTextCtrl* textCtrl = dynamic_cast<wxTextCtrl*>( ui_row_item.m_ThicknessCtrl );
 
             if( textCtrl )
-                textCtrl->ChangeValue( StringFromValue( m_units,
-                                        item->GetThickness( sub_item ), true ) );
+                textCtrl->ChangeValue( m_frame->StringFromValue( item->GetThickness( sub_item ), true ) );
 
             if( item->GetType() == BS_ITEM_TYPE_DIELECTRIC )
             {
@@ -542,19 +632,22 @@ void PANEL_SETUP_BOARD_STACKUP::synchronizeWithBoard( bool aFullSync )
         if( item->IsColorEditable() )
         {
             auto bm_combo = dynamic_cast<wxBitmapComboBox*>( ui_row_item.m_ColorCtrl );
+            int  selected = 0;  // The "not specified" item
 
-            if( item->GetColor().StartsWith( "#" ) )  // User defined color
+            if( item->GetColor( sub_item ).StartsWith( wxT( "#" ) ) )  // User defined color
             {
-                ui_row_item.m_UserColor = COLOR4D( item->GetColor() ).ToColour();
+                COLOR4D custom_color( item->GetColor( sub_item ) );
+
+                ui_row_item.m_UserColor = custom_color;
+
+                selected = GetColorUserDefinedListIdx( item->GetType() );
 
                 if( bm_combo )      // Update user color shown in the wxBitmapComboBox
                 {
-                    bm_combo->SetString( GetColorUserDefinedListIdx(), item->GetColor() );
+                    bm_combo->SetString( selected, item->GetColor( sub_item ) );
                     wxBitmap layerbmp( m_colorSwatchesSize.x, m_colorSwatchesSize.y );
-                    LAYER_SELECTOR::DrawColorSwatch( layerbmp, COLOR4D(),
-                                                     wxColour( item->GetColor() ) );
-                    bm_combo->SetItemBitmap( GetColorUserDefinedListIdx(), layerbmp );
-                    bm_combo->SetSelection( GetColorUserDefinedListIdx() );
+                    LAYER_SELECTOR::DrawColorSwatch( layerbmp, COLOR4D(), custom_color );
+                    bm_combo->SetItemBitmap( selected, layerbmp );
                 }
             }
             else
@@ -563,22 +656,24 @@ void PANEL_SETUP_BOARD_STACKUP::synchronizeWithBoard( bool aFullSync )
                 {
                     // Note: don't use bm_combo->FindString() because the combo strings are
                     // translated.
-                    for( int ii = 0; ii < GetColorStandardListCount(); ii++ )
+                    for( size_t ii = 0; ii < GetStandardColors( item->GetType() ).size(); ii++ )
                     {
-                        if( color_list[ii].GetName() == item->GetColor() )
+                        if( GetStandardColorName( item->GetType(), ii ) == item->GetColor( sub_item ) )
                         {
-                            bm_combo->SetSelection( ii );
+                            selected = ii;
                             break;
                         }
                     }
                 }
             }
 
+            if( bm_combo )
+                bm_combo->SetSelection( selected );
         }
 
         if( item->HasEpsilonRValue() )
         {
-            wxString txt = Double2Str( item->GetEpsilonR( sub_item ) );
+            wxString txt = UIDouble2Str( item->GetEpsilonR( sub_item ) );
             wxTextCtrl* textCtrl = dynamic_cast<wxTextCtrl*>( ui_row_item.m_EpsilonCtrl );
 
             if( textCtrl )
@@ -587,7 +682,7 @@ void PANEL_SETUP_BOARD_STACKUP::synchronizeWithBoard( bool aFullSync )
 
         if( item->HasLossTangentValue() )
         {
-            wxString txt = Double2Str( item->GetLossTangent( sub_item ) );
+            wxString txt = UIDouble2Str( item->GetLossTangent( sub_item ) );
             wxTextCtrl* textCtrl = dynamic_cast<wxTextCtrl*>( ui_row_item.m_LossTgCtrl );
 
             if( textCtrl )
@@ -604,7 +699,6 @@ void PANEL_SETUP_BOARD_STACKUP::synchronizeWithBoard( bool aFullSync )
 
 void PANEL_SETUP_BOARD_STACKUP::showOnlyActiveLayers()
 {
-
     // Now enable/disable stackup items, according to the m_enabledLayers config
     // Calculate copper layer count from m_enabledLayers, and *do not use* brd_stackup
     // for that, because it is not necessary up to date
@@ -696,8 +790,6 @@ BOARD_STACKUP_ROW_UI_ITEM PANEL_SETUP_BOARD_STACKUP::createRowData( int aRow,
     BOARD_STACKUP_ITEM* item = aStackupItem;
     int row = aRow;
 
-    const FAB_LAYER_COLOR* color_list = GetColorStandardList();
-
     // Add color swatch icon. The color will be updated later,
     // when all widgets are initialized
     wxStaticBitmap* bitmap = new wxStaticBitmap( m_scGridWin, wxID_ANY, wxNullBitmap );
@@ -712,7 +804,8 @@ BOARD_STACKUP_ROW_UI_ITEM PANEL_SETUP_BOARD_STACKUP::createRowData( int aRow,
 
         if( item->GetSublayersCount() > 1 )
         {
-            lname <<  "  (" << aSublayerIdx+1 << "/" << item->GetSublayersCount() << ")";
+            lname <<  wxT( "  (" ) << aSublayerIdx+1 << wxT( "/" )
+                                   << item->GetSublayersCount() << wxT( ")" );
         }
 
         wxStaticText* st_text = new wxStaticText( m_scGridWin, wxID_ANY, lname );
@@ -769,7 +862,7 @@ BOARD_STACKUP_ROW_UI_ITEM PANEL_SETUP_BOARD_STACKUP::createRowData( int aRow,
     {
         wxTextCtrl* textCtrl = new wxTextCtrl( m_scGridWin, ID_ITEM_THICKNESS+row );
         textCtrl->SetMinSize( m_numericTextCtrlSize );
-        textCtrl->ChangeValue( StringFromValue( m_units, item->GetThickness( aSublayerIdx ), true ) );
+        textCtrl->ChangeValue( m_frame->StringFromValue( item->GetThickness( aSublayerIdx ), true ) );
         m_fgGridSizer->Add( textCtrl, 0, wxLEFT|wxRIGHT|wxALIGN_CENTER_VERTICAL, 2 );
         m_controlItemsList.push_back( textCtrl );
         textCtrl->Connect( wxEVT_COMMAND_TEXT_UPDATED,
@@ -798,34 +891,37 @@ BOARD_STACKUP_ROW_UI_ITEM PANEL_SETUP_BOARD_STACKUP::createRowData( int aRow,
 
     if( item->IsColorEditable() )
     {
-        if( item->GetColor().StartsWith( "#" ) )  // User defined color
+        if( item->GetColor( aSublayerIdx ).StartsWith( wxT( "#" ) ) )  // User defined color
         {
-            ui_row_item.m_UserColor = COLOR4D( item->GetColor() ).ToColour();
+            ui_row_item.m_UserColor = COLOR4D( item->GetColor( aSublayerIdx ) ).ToColour();
         }
         else
             ui_row_item.m_UserColor = GetDefaultUserColor( item->GetType() );
 
         wxBitmapComboBox* bm_combo = createColorBox( item, row );
+        int               selected = 0;     // The "not specified" item
+
         m_fgGridSizer->Add( bm_combo, 1, wxLEFT|wxRIGHT|wxALIGN_CENTER_VERTICAL|wxEXPAND, 2 );
 
-        if( item->GetColor().StartsWith( "#" ) )
+        if( item->GetColor( aSublayerIdx ).StartsWith( wxT( "#" ) ) )
         {
-            bm_combo->SetString( GetColorUserDefinedListIdx(), item->GetColor() );
-            bm_combo->SetSelection( GetColorUserDefinedListIdx() );
+            selected = GetColorUserDefinedListIdx( item->GetType() );
+            bm_combo->SetString( selected, item->GetColor( aSublayerIdx ) );
         }
         else
         {
             // Note: don't use bm_combo->FindString() because the combo strings are translated.
-            for( int ii = 0; ii < GetColorStandardListCount(); ii++ )
+            for( size_t ii = 0; ii < GetStandardColors( item->GetType() ).size(); ii++ )
             {
-                if( color_list[ii].GetName() == item->GetColor() )
+                if( GetStandardColorName( item->GetType(), ii ) == item->GetColor( aSublayerIdx ) )
                 {
-                    bm_combo->SetSelection( ii );
+                    selected = ii;
                     break;
                 }
             }
         }
 
+        bm_combo->SetSelection( selected );
         ui_row_item.m_ColorCtrl = bm_combo;
     }
     else
@@ -835,7 +931,7 @@ BOARD_STACKUP_ROW_UI_ITEM PANEL_SETUP_BOARD_STACKUP::createRowData( int aRow,
 
     if( item->HasEpsilonRValue() )
     {
-        wxString txt = Double2Str( item->GetEpsilonR( aSublayerIdx ) );
+        wxString txt = UIDouble2Str( item->GetEpsilonR( aSublayerIdx ) );
         wxTextCtrl* textCtrl = new wxTextCtrl( m_scGridWin, wxID_ANY, wxEmptyString,
                                                wxDefaultPosition, m_numericFieldsSize );
         textCtrl->ChangeValue( txt );
@@ -849,7 +945,7 @@ BOARD_STACKUP_ROW_UI_ITEM PANEL_SETUP_BOARD_STACKUP::createRowData( int aRow,
 
     if( item->HasLossTangentValue() )
     {
-        wxString txt = Double2Str( item->GetLossTangent( aSublayerIdx ) );;
+        wxString txt = UIDouble2Str( item->GetLossTangent( aSublayerIdx ) );;
         wxTextCtrl* textCtrl = new wxTextCtrl( m_scGridWin, wxID_ANY, wxEmptyString,
                                                wxDefaultPosition, m_numericFieldsSize );
         textCtrl->ChangeValue( txt );
@@ -865,7 +961,7 @@ BOARD_STACKUP_ROW_UI_ITEM PANEL_SETUP_BOARD_STACKUP::createRowData( int aRow,
 }
 
 
-void PANEL_SETUP_BOARD_STACKUP::rebuildLayerStackPanel()
+void PANEL_SETUP_BOARD_STACKUP::rebuildLayerStackPanel( bool aRelinkItems )
 {
     wxWindowUpdateLocker locker( m_scGridWin );
     m_scGridWin->Hide();
@@ -878,7 +974,7 @@ void PANEL_SETUP_BOARD_STACKUP::rebuildLayerStackPanel()
     m_controlItemsList.clear();
 
     // Delete widgets (handled by the wxPanel parent)
-    for( BOARD_STACKUP_ROW_UI_ITEM ui_item: m_rowUiItemsList )
+    for( BOARD_STACKUP_ROW_UI_ITEM& ui_item: m_rowUiItemsList )
     {
         // This remove and delete the current ui_item.m_MaterialCtrl sizer
         ui_item.m_MaterialCtrl->SetSizer( nullptr );
@@ -923,7 +1019,7 @@ void PANEL_SETUP_BOARD_STACKUP::rebuildLayerStackPanel()
 
 
     // Now, rebuild the widget list from the new m_stackup items:
-    buildLayerStackPanel( false );
+    buildLayerStackPanel( false, aRelinkItems );
 
     // Now enable/disable stackup items, according to the m_enabledLayers config
     showOnlyActiveLayers();
@@ -933,17 +1029,22 @@ void PANEL_SETUP_BOARD_STACKUP::rebuildLayerStackPanel()
 }
 
 
-void PANEL_SETUP_BOARD_STACKUP::buildLayerStackPanel( bool aCreatedInitialStackup )
+void PANEL_SETUP_BOARD_STACKUP::buildLayerStackPanel( bool aCreateInitialStackup,
+                                                      bool aRelinkStackup )
 {
     // Build a full stackup for the dialog, with a active copper layer count
     // = current board layer count to calculate a reasonable default stackup:
-    if( aCreatedInitialStackup )
+    if( aCreateInitialStackup || aRelinkStackup )
     {
-        // Creates a full BOARD_STACKUP with 32 copper layers.
-        // extra layers will be hidden later.
-        // but if the number of layer is changed in the dialog, the corresponding
-        // widgets will be available with their previous values.
-        m_stackup.BuildDefaultStackupList( nullptr, m_brdSettings->GetCopperLayerCount() );
+        if( aCreateInitialStackup )
+        {
+            // Creates a full BOARD_STACKUP with 32 copper layers.
+            // extra layers will be hidden later.
+            // but if the number of layer is changed in the dialog, the corresponding
+            // widgets will be available with their previous values.
+            m_stackup.BuildDefaultStackupList( nullptr, m_brdSettings->GetCopperLayerCount() );
+        }
+
         const BOARD_STACKUP& brd_stackup = m_brdSettings->GetStackupDescriptor();
 
         // Now initialize all stackup items to the board values, when exist
@@ -996,7 +1097,6 @@ void PANEL_SETUP_BOARD_STACKUP::buildLayerStackPanel( bool aCreatedInitialStacku
 // Transfer current UI settings to m_stackup but not to the board
 bool PANEL_SETUP_BOARD_STACKUP::transferDataFromUIToStackup()
 {
-    wxString txt;
     wxString error_msg;
     bool success = true;
     double value;
@@ -1024,7 +1124,7 @@ bool PANEL_SETUP_BOARD_STACKUP::transferDataFromUIToStackup()
         if( item->HasEpsilonRValue() )
         {
             wxTextCtrl* textCtrl = static_cast<wxTextCtrl*>( ui_item.m_EpsilonCtrl );
-            txt = textCtrl->GetValue();
+            wxString    txt = textCtrl->GetValue();
 
             if( txt.ToDouble( &value ) && value >= 0.0 )
                 item->SetEpsilonR( value, sub_item );
@@ -1041,7 +1141,7 @@ bool PANEL_SETUP_BOARD_STACKUP::transferDataFromUIToStackup()
         if( item->HasLossTangentValue() )
         {
             wxTextCtrl* textCtrl = static_cast<wxTextCtrl*>( ui_item.m_LossTgCtrl );
-            txt = textCtrl->GetValue();
+            wxString    txt = textCtrl->GetValue();
 
             if( txt.ToDouble( &value ) && value >= 0.0 )
                 item->SetLossTangent( value, sub_item );
@@ -1052,7 +1152,7 @@ bool PANEL_SETUP_BOARD_STACKUP::transferDataFromUIToStackup()
                 success = false;
 
                 if( !error_msg.IsEmpty() )
-                    error_msg << "\n";
+                    error_msg << wxT( "\n" );
 
                 error_msg << _( "Incorrect value for Loss tg (Loss tg must be positive or null "
                                 "if not used)" );
@@ -1089,9 +1189,8 @@ bool PANEL_SETUP_BOARD_STACKUP::transferDataFromUIToStackup()
         if( item->IsThicknessEditable() )
         {
             wxTextCtrl* textCtrl = static_cast<wxTextCtrl*>( ui_item.m_ThicknessCtrl );
-            txt = textCtrl->GetValue();
+            int         new_thickness = m_frame->ValueFromString( textCtrl->GetValue() );
 
-            int new_thickness = ValueFromString( m_frame->GetUserUnits(), txt );
             item->SetThickness( new_thickness, sub_item );
 
             if( new_thickness < 0 )
@@ -1099,7 +1198,7 @@ bool PANEL_SETUP_BOARD_STACKUP::transferDataFromUIToStackup()
                 success = false;
 
                 if( !error_msg.IsEmpty() )
-                    error_msg << "\n";
+                    error_msg << wxT( "\n" );
 
                 error_msg << _( "A layer thickness is < 0. Fix it" );
             }
@@ -1107,31 +1206,23 @@ bool PANEL_SETUP_BOARD_STACKUP::transferDataFromUIToStackup()
             if( item->GetType() == BS_ITEM_TYPE_DIELECTRIC )
             {
                 // Dielectric thickness layer can have a locked thickness:
-                wxCheckBox* cb_box = static_cast<wxCheckBox*>
-                                        ( ui_item.m_ThicknessLockCtrl );
+                wxCheckBox* cb_box = static_cast<wxCheckBox*>( ui_item.m_ThicknessLockCtrl );
                 item->SetThicknessLocked( cb_box && cb_box->GetValue(), sub_item );
             }
         }
 
-        if( sub_item == 0 && item->IsColorEditable() )
+        if( item->IsColorEditable() )
         {
-            const FAB_LAYER_COLOR* color_list = GetColorStandardList();
-
             wxBitmapComboBox* choice = dynamic_cast<wxBitmapComboBox*>( ui_item.m_ColorCtrl );
 
             if( choice )
             {
                 int idx = choice->GetSelection();
 
-                if( idx == GetColorUserDefinedListIdx() )
-                {
-                    // NB: wxWidgets 3.0's color.GetAsString( wxC2S_HTML_SYNTAX ) pukes on alpha
-                    item->SetColor( getColourAsHexString( ui_item.m_UserColor ) );
-                }
+                if( IsCustomColorIdx( item->GetType(), idx ) )
+                    item->SetColor( ui_item.m_UserColor.ToHexString(), sub_item );
                 else
-                {
-                    item->SetColor( color_list[idx].GetName() );
-                }
+                    item->SetColor( GetStandardColorName( item->GetType(), idx ), sub_item );
             }
         }
 
@@ -1211,13 +1302,15 @@ bool PANEL_SETUP_BOARD_STACKUP::TransferDataFromWindow()
 void PANEL_SETUP_BOARD_STACKUP::ImportSettingsFrom( BOARD* aBoard )
 {
     BOARD* savedBrd = m_board;
+    m_board = aBoard;
+
     BOARD_DESIGN_SETTINGS* savedSettings = m_brdSettings;
     m_brdSettings = &aBoard->GetDesignSettings();
 
-    m_enabledLayers = m_panelLayers->GetUILayerMask() & BOARD_STACKUP::StackupAllowedBrdLayers();
-    synchronizeWithBoard( true );
+    m_enabledLayers = m_board->GetEnabledLayers() & BOARD_STACKUP::StackupAllowedBrdLayers();
 
-    rebuildLayerStackPanel();
+    rebuildLayerStackPanel( true );
+    synchronizeWithBoard( true );
     computeBoardThickness();
 
     m_brdSettings = savedSettings;
@@ -1227,6 +1320,10 @@ void PANEL_SETUP_BOARD_STACKUP::ImportSettingsFrom( BOARD* aBoard )
 
 void PANEL_SETUP_BOARD_STACKUP::OnLayersOptionsChanged( LSET aNewLayerSet )
 {
+    // Can be called spuriously from events before the layers page is even created
+    if( !m_panelLayers->IsInitialized() )
+        return;
+
     // First, verify the list of layers currently in stackup:
     // if it does not mach the list of layers set in PANEL_SETUP_LAYERS
     // rebuild the panel
@@ -1249,12 +1346,12 @@ void PANEL_SETUP_BOARD_STACKUP::OnLayersOptionsChanged( LSET aNewLayerSet )
 
 void PANEL_SETUP_BOARD_STACKUP::onColorSelected( wxCommandEvent& event )
 {
-    int idx = event.GetSelection();
-    int item_id = event.GetId();
+    int                 idx = event.GetSelection();
+    int                 item_id = event.GetId();
+    int                 row = item_id - ID_ITEM_COLOR;
+    BOARD_STACKUP_ITEM* item = m_rowUiItemsList[row].m_Item;
 
-    int row = item_id - ID_ITEM_COLOR;
-
-    if( idx == GetColorStandardListCount() - 1 )   // Set user color is the last option in list
+    if( IsCustomColorIdx( item->GetType(), idx ) )   // user color is the last option in list
     {
         DIALOG_COLOR_PICKER dlg( this, m_rowUiItemsList[row].m_UserColor, true, nullptr,
                                  GetDefaultUserColor( m_rowUiItemsList[row].m_Item->GetType() ) );
@@ -1268,16 +1365,15 @@ void PANEL_SETUP_BOARD_STACKUP::onColorSelected( wxCommandEvent& event )
         if( dlg.ShowModal() == wxID_OK )
         {
             wxBitmapComboBox* combo = static_cast<wxBitmapComboBox*>( FindWindowById( item_id ) );
-            wxColour          color = dlg.GetColor().ToColour();
+            COLOR4D           color = dlg.GetColor();
 
             m_rowUiItemsList[row].m_UserColor = color;
 
-            // NB: wxWidgets 3.0's color.GetAsString( wxC2S_HTML_SYNTAX ) pukes on alpha
-            combo->SetString( idx, getColourAsHexString( color ) );
+            combo->SetString( idx, color.ToHexString() );
 
             wxBitmap layerbmp( m_colorSwatchesSize.x, m_colorSwatchesSize.y );
-            LAYER_SELECTOR::DrawColorSwatch( layerbmp, COLOR4D( 0, 0, 0, 0 ), COLOR4D( color ) );
-            combo->SetItemBitmap( combo->GetCount()-1, layerbmp );
+            LAYER_SELECTOR::DrawColorSwatch( layerbmp, COLOR4D( 0, 0, 0, 0 ), color );
+            combo->SetItemBitmap( combo->GetCount() - 1, layerbmp );
 
             combo->SetSelection( idx );
         }
@@ -1356,10 +1452,43 @@ void PANEL_SETUP_BOARD_STACKUP::onMaterialChange( wxCommandEvent& event )
     item->SetEpsilonR( substrate.m_EpsilonR, sub_item );
     item->SetLossTangent( substrate.m_LossTangent, sub_item );
 
-    wxTextCtrl* textCtrl;
-    textCtrl = static_cast<wxTextCtrl*>( m_rowUiItemsList[row].m_MaterialCtrl );
+    wxTextCtrl* textCtrl = static_cast<wxTextCtrl*>( m_rowUiItemsList[row].m_MaterialCtrl );
     textCtrl->ChangeValue( item->GetMaterial( sub_item ) );
 
+    if( item->GetType() == BS_ITEM_TYPE_DIELECTRIC
+            && !item->GetColor( sub_item ).StartsWith( "#" ) /* User defined color */ )
+    {
+        if( substrate.m_Name.IsSameAs( "PTFE" )
+              || substrate.m_Name.IsSameAs( "Teflon" ) )
+        {
+            item->SetColor( "PTFE natural", sub_item );
+        }
+        else if( substrate.m_Name.IsSameAs( "Polyimide" )
+              || substrate.m_Name.IsSameAs( "Kapton" ) )
+        {
+            item->SetColor( "Polyimide", sub_item );
+        }
+        else if( substrate.m_Name.IsSameAs( "Al" ) )
+        {
+            item->SetColor( "Aluminum", sub_item );
+        }
+        else
+        {
+            item->SetColor( "FR4 natural", sub_item );
+        }
+    }
+
+    wxBitmapComboBox* picker = static_cast<wxBitmapComboBox*>( m_rowUiItemsList[row].m_ColorCtrl );
+
+    for( size_t ii = 0; ii < GetStandardColors( item->GetType() ).size(); ii++ )
+    {
+        if( GetStandardColorName( item->GetType(), ii ) == item->GetColor( sub_item ) )
+        {
+            picker->SetSelection( ii );
+            break;
+        }
+    }
+
     // some layers have a material choice but not EpsilonR ctrl
     if( item->HasEpsilonRValue() )
     {
@@ -1388,7 +1517,7 @@ void PANEL_SETUP_BOARD_STACKUP::onThicknessChange( wxCommandEvent& event )
     BOARD_STACKUP_ITEM* item = GetStackupItem( row );
     int idx = GetSublayerId( row );
 
-    item->SetThickness( ValueFromString( m_frame->GetUserUnits(), value ), idx );
+    item->SetThickness( m_frame->ValueFromString( value ), idx );
 
     computeBoardThickness();
 }
@@ -1426,11 +1555,11 @@ wxColor PANEL_SETUP_BOARD_STACKUP::getColorIconItem( int aRow )
 
     default:
     case BS_ITEM_TYPE_UNDEFINED:
-        wxFAIL_MSG( "PANEL_SETUP_BOARD_STACKUP::getColorIconItem: unrecognized item type" );
+        wxFAIL_MSG( wxT( "PANEL_SETUP_BOARD_STACKUP::getColorIconItem: unrecognized item type" ) );
         break;
     }
 
-    wxASSERT_MSG( color.IsOk(), "Invalid color in PCB stackup" );
+    wxASSERT_MSG( color.IsOk(), wxT( "Invalid color in PCB stackup" ) );
 
     return color;
 }
@@ -1468,32 +1597,29 @@ wxBitmapComboBox* PANEL_SETUP_BOARD_STACKUP::createColorBox( BOARD_STACKUP_ITEM*
                                                     wxDefaultSize, 0, nullptr, wxCB_READONLY );
 
     // Fills the combo box with choice list + bitmaps
-    const FAB_LAYER_COLOR*  color_list = GetColorStandardList();
     BOARD_STACKUP_ITEM_TYPE itemType = aStackupItem ? aStackupItem->GetType()
                                                     : BS_ITEM_TYPE_SILKSCREEN;
 
-    for( int ii = 0; ii < GetColorStandardListCount(); ii++ )
+    for( size_t ii = 0; ii < GetStandardColors( itemType ).size(); ii++ )
     {
-        wxColor  curr_color;
         wxString label;
+        COLOR4D  curr_color;
 
         // Defined colors have a name, the user color uses HTML notation ( i.e. #FF000080)
-        if( ii == GetColorUserDefinedListIdx()
-                && aStackupItem && aStackupItem->GetColor().StartsWith( "#" ) )
+        if( IsCustomColorIdx( itemType, ii )
+                && aStackupItem && aStackupItem->GetColor().StartsWith( wxT( "#" ) ) )
         {
-            curr_color = wxColour( COLOR4D( aStackupItem->GetColor() ).ToColour() );
-
-            // NB: wxWidgets 3.0's color.GetAsString( wxC2S_HTML_SYNTAX ) pukes on alpha
-            label = getColourAsHexString( curr_color );
+            label = aStackupItem->GetColor();
+            curr_color = COLOR4D( label );
         }
         else
         {
-            curr_color = color_list[ii].GetColor( itemType );
-            label = _( color_list[ii].GetName() );
+            label = wxGetTranslation( GetStandardColorName( itemType, ii ) );
+            curr_color = GetStandardColor( itemType, ii );
         }
 
         wxBitmap layerbmp( m_colorSwatchesSize.x, m_colorSwatchesSize.y );
-        LAYER_SELECTOR::DrawColorSwatch( layerbmp, COLOR4D( 0, 0, 0, 0 ), COLOR4D( curr_color ) );
+        LAYER_SELECTOR::DrawColorSwatch( layerbmp, COLOR4D( 0, 0, 0, 0 ), curr_color );
 
         combo->Append( label, layerbmp );
     }
