@@ -989,7 +989,7 @@ bool PANEL_FP_LIB_TABLE::TransferDataFromWindow()
             m_global->Clear();
             m_global->m_rows.transfer( m_global->m_rows.end(), global_model()->m_rows.begin(),
                                      global_model()->m_rows.end(), global_model()->m_rows );
-            m_global->reindex();
+            m_global->reindex( true );
         }
 
         if( project_model() && *project_model() != *m_project )
@@ -999,7 +999,7 @@ bool PANEL_FP_LIB_TABLE::TransferDataFromWindow()
             m_project->Clear();
             m_project->m_rows.transfer( m_project->m_rows.end(), project_model()->m_rows.begin(),
                                       project_model()->m_rows.end(), project_model()->m_rows );
-            m_project->reindex();
+            m_project->reindex( true );
         }
 
         return true;
@@ -1130,19 +1130,8 @@ void InvokePcbLibTableEditor( KIWAY* aKiway, wxWindow* aCaller )
         }
     }
 
-    auto editor = (FOOTPRINT_EDIT_FRAME*) aKiway->Player( FRAME_FOOTPRINT_EDITOR, false );
-
-    if( editor )
-    {
-        editor->SyncLibraryTree( true );
-        editor->RefreshLibraryTree();
-    }
-
-    auto viewer = (FOOTPRINT_VIEWER_FRAME*) aKiway->Player( FRAME_FOOTPRINT_VIEWER, false );
-
-    if( viewer )
-        viewer->ReCreateLibraryList();
-
     std::string payload = "";
+    aKiway->ExpressMail( FRAME_FOOTPRINT_EDITOR, MAIL_RELOAD_LIB, payload );
+    aKiway->ExpressMail( FRAME_FOOTPRINT_VIEWER, MAIL_RELOAD_LIB, payload );
     aKiway->ExpressMail( FRAME_CVPCB, MAIL_RELOAD_LIB, payload );
 }
