@@ -881,15 +881,6 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
                     REPORT( wxString::Format( _( "Checking %s clearance: %s." ),
                                               EscapeHTML( c->constraint.GetName() ),
                                               MessageTextFromValue( c->constraint.m_Value.Min() ) ) )
-
-                    if( c->constraint.GetValue().Min() < 0 )
-                    {
-                        // JEY TODO: translate for 8.0....
-                        REPORT( implicit ? wxS( "Negative clearance specified; constraint ignored." )
-                                         : wxS( "Negative clearance specified; rule ignored." ) )
-                        return true;
-                    }
-
                     break;
 
                 case DIFF_PAIR_MAX_UNCOUPLED_CONSTRAINT:
@@ -1226,7 +1217,24 @@ DRC_CONSTRAINT DRC_ENGINE::EvalRules( DRC_CONSTRAINT_T aConstraintType, const BO
                         }
                     }
 
-                    constraint = c->constraint;
+                    if( c->constraint.m_Value.HasMin() )
+                        constraint.m_Value.SetMin( c->constraint.m_Value.Min() );
+
+                    if( c->constraint.m_Value.HasOpt() )
+                        constraint.m_Value.SetOpt( c->constraint.m_Value.Opt() );
+
+                    if( c->constraint.m_Value.HasMax() )
+                        constraint .m_Value.SetMax( c->constraint.m_Value.Max() );
+
+                    // While the expectation would be to OR the disallow flags, we've already
+                    // masked them down to aItem's type -- so we're really only looking for a
+                    // boolean here.
+                    constraint.m_DisallowFlags = c->constraint.m_DisallowFlags;
+
+                    constraint.m_ZoneConnection = c->constraint.m_ZoneConnection;
+
+                    constraint.SetParentRule( c->constraint.GetParentRule() );
+
                     return true;
                 }
                 else
