@@ -252,10 +252,16 @@ private:
 
             std::sort( queue.begin(), queue.end(), []( const Vertex* a, const Vertex* b )
             {
-                return a->z < b->z || ( a->z == b->z
-                            && ( ( a->x < b->x )
-                              || ( a->y < b->y )
-                              || ( a->i < b->i ) ) );
+                if( a->z != b->z )
+                    return a->z < b->z;
+
+                if( a->x != b->x )
+                    return a->x < b->x;
+
+                if( a->y != b->y )
+                    return a->y < b->y;
+
+                return a->i < b->i;
             } );
 
             Vertex* prev_elem = nullptr;
@@ -330,12 +336,13 @@ private:
         // in the wrong direction from the fracture and should
         // fall through to the next point
         if( same_point( aPt, nz )
-                && !( same_point( nz->next, nz->next->prevZ ) || same_point( nz->next, nz->next->nextZ ) ) )
+                && aPt->y == aPt->next->y )
         {
             return nz->next;
         }
-        else if( same_point( aPt, pz )
-                && !( same_point( pz->next, pz->next->prevZ ) || same_point( pz->next, pz->next->nextZ ) ) )
+
+        if( same_point( aPt, pz )
+                && aPt->y == aPt->next->y )
         {
             return pz->next;
         }
@@ -356,12 +363,13 @@ private:
         // in the wrong direction from the fracture and should
         // fall through to the next point
         if( same_point( aPt, nz )
-                && !( same_point( nz->prev, nz->prev->nextZ ) || same_point( nz->prev, nz->prev->prevZ ) ) )
+                && aPt->y == aPt->prev->y)
         {
             return nz->prev;
         }
-        else if( same_point( aPt, pz )
-                && !( same_point( pz->prev, pz->prev->nextZ ) || same_point( pz->prev, pz->prev->prevZ ) ) )
+
+        if( same_point( aPt, pz )
+                && aPt->y == aPt->prev->y )
         {
             return pz->prev;
         }
@@ -417,12 +425,7 @@ private:
             if( diff_x > m_max_error || diff_y > m_max_error || p == aB )
                 p0 = p;
 
-            if( same_point( p, p->nextZ ) )
-                p = p->nextZ->next;
-            else if( same_point( p, p->prevZ ) )
-                p = p->prevZ->next;
-            else
-                p = p->next;
+            p = getNextOutlineVertex( p );
 
             ++checked;
         }
@@ -457,12 +460,7 @@ private:
             if( diff_x > m_max_error || diff_y > m_max_error || p == aB )
                 p0 = p;
 
-            if( same_point( p, p->nextZ ) )
-                p = p->nextZ->prev;
-            else if( same_point( p, p->prevZ ) )
-                p = p->prevZ->prev;
-            else
-                p = p->prev;
+            p = getPrevOutlineVertex( p );
 
             ++checked;
         }
@@ -517,7 +515,12 @@ private:
         // z-order range for the current point ± limit bounding box
         const int32_t     maxZ = zOrder( aPt->x + m_limit, aPt->y + m_limit );
         const int32_t     minZ = zOrder( aPt->x - m_limit, aPt->y - m_limit );
-        const SEG::ecoord limit2 = SEG::Square( m_limit );
+
+        // Subtract 1 to account for rounding inaccuracies in SquaredEuclideanNorm()
+        // below.  We would usually test for rounding in the final value but since we
+        // are working in squared integers here, we allow the 1nm slop rather than
+        // force a separate calculation
+        const SEG::ecoord limit2 = SEG::Square( m_limit - 1 );
 
         // first look for points in increasing z-order
         Vertex* p = aPt->nextZ;
@@ -531,7 +534,7 @@ private:
             SEG::ecoord dist2 = diff.SquaredEuclideanNorm();
 
             if( delta_i > 1 && dist2 < limit2 && dist2 < min_dist && dist2 > 0.0
-                    && locallyInside( p, aPt ) && isSubstantial( p, aPt ) )
+                    && locallyInside( p, aPt ) && isSubstantial( p, aPt ) && isSubstantial( aPt, p ) )
             {
                 min_dist = dist2;
                 retval = p;
@@ -549,7 +552,7 @@ private:
             SEG::ecoord dist2 = diff.SquaredEuclideanNorm();
 
             if( delta_i > 1 && dist2 < limit2 && dist2 < min_dist && dist2 > 0.0
-                    && locallyInside( p, aPt ) && isSubstantial( p, aPt ) )
+                    && locallyInside( p, aPt ) && isSubstantial( p, aPt ) && isSubstantial( aPt, p ) )
             {
                 min_dist = dist2;
                 retval = p;
