@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2015 Jean-Pierre Charras, jp.charras at wanadoo.fr
- * Copyright (C) 1992-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 1992-2022 KiCad Developers, see AUTHORS.txt for contributors.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -30,7 +30,6 @@
 #include <board_item_container.h>
 #include <board_item.h>
 #include <collectors.h>
-#include <convert_to_biu.h>
 #include <layer_ids.h> // ALL_LAYERS definition.
 #include <lib_id.h>
 #include <list>
@@ -40,6 +39,7 @@
 #include <pcb_item_containers.h>
 #include <fp_text.h>
 #include <functional>
+#include <math/vector3.h>
 
 class LINE_READER;
 class EDA_3D_CANVAS;
@@ -65,12 +65,14 @@ enum INCLUDE_NPTH_T
  */
 enum FOOTPRINT_ATTR_T
 {
-    FP_THROUGH_HOLE           = 0x0001,
-    FP_SMD                    = 0x0002,
-    FP_EXCLUDE_FROM_POS_FILES = 0x0004,
-    FP_EXCLUDE_FROM_BOM       = 0x0008,
-    FP_BOARD_ONLY             = 0x0010,   // Footprint has no corresponding symbol
-    FP_JUST_ADDED             = 0x0020    // Footprint just added by netlist update
+    FP_THROUGH_HOLE             = 0x0001,
+    FP_SMD                      = 0x0002,
+    FP_EXCLUDE_FROM_POS_FILES   = 0x0004,
+    FP_EXCLUDE_FROM_BOM         = 0x0008,
+    FP_BOARD_ONLY               = 0x0010,   // Footprint has no corresponding symbol
+    FP_JUST_ADDED               = 0x0020,   // Footprint just added by netlist update
+    FP_ALLOW_SOLDERMASK_BRIDGES = 0x0040,
+    FP_ALLOW_MISSING_COURTYARD  = 0x0080
 };
 
 class FP_3DMODEL
@@ -86,11 +88,6 @@ public:
     {
     }
 
-    struct VECTOR3D
-    {
-        double x, y, z;
-    };
-
     VECTOR3D m_Scale;       ///< 3D model scaling factor (dimensionless)
     VECTOR3D m_Rotation;    ///< 3D model rotation (degrees)
     VECTOR3D m_Offset;      ///< 3D model offset (mm)
@@ -120,8 +117,12 @@ public:
         return aItem && aItem->Type() == PCB_FOOTPRINT_T;
     }
 
+    LSET GetPrivateLayers() const { return m_privateLayers; }
+    void SetPrivateLayers( LSET aLayers ) { m_privateLayers = aLayers; }
+
     ///< @copydoc BOARD_ITEM_CONTAINER::Add()
-    void Add( BOARD_ITEM* aItem, ADD_MODE aMode = ADD_MODE::INSERT ) override;
+    void Add( BOARD_ITEM* aItem, ADD_MODE aMode = ADD_MODE::INSERT,
+              bool aSkipConnectivity = false ) override;
 
     ///< @copydoc BOARD_ITEM_CONTAINER::Remove()
     void Remove( BOARD_ITEM* aItem, REMOVE_MODE aMode = REMOVE_MODE::NORMAL ) override;
@@ -136,7 +137,7 @@ public:
     void ClearAllNets();
 
     /**
-     * Old footprints do not alway have a valid UUID (some can be set to null uuid)
+     * Old footprints do not always have a valid UUID (some can be set to null uuid)
      * However null UUIDs, having a special meaning in editor, create issues when
      * editing a footprint
      * So all null uuids a re replaced by a valid uuid
@@ -153,7 +154,7 @@ public:
      *
      * @return The rectangle containing the pads for the normalized footprint.
      */
-    EDA_RECT GetFpPadsLocalBbox() const;
+    BOX2I GetFpPadsLocalBbox() const;
 
     /**
      * Return a bounding polygon for the shapes and pads in the footprint.
@@ -163,8 +164,8 @@ public:
     SHAPE_POLY_SET GetBoundingHull() const;
 
     // Virtual function
-    const EDA_RECT GetBoundingBox() const override;
-    const EDA_RECT GetBoundingBox( bool aIncludeText, bool aIncludeInvisibleText ) const;
+    const BOX2I GetBoundingBox() const override;
+    const BOX2I GetBoundingBox( bool aIncludeText, bool aIncludeInvisibleText ) const;
 
     PADS& Pads()             { return m_pads; }
     const PADS& Pads() const { return m_pads; }
@@ -180,25 +181,44 @@ public:
 
     bool HasThroughHolePads() const;
 
-    std::list<FP_3DMODEL>& Models()             { return m_3D_Drawings; }
-    const std::list<FP_3DMODEL>& Models() const { return m_3D_Drawings; }
+    std::vector<FP_3DMODEL>& Models()             { return m_3D_Drawings; }
+    const std::vector<FP_3DMODEL>& Models() const { return m_3D_Drawings; }
 
-    void SetPosition( const wxPoint& aPos ) override;
-    wxPoint GetPosition() const override { return m_pos; }
+    void     SetPosition( const VECTOR2I& aPos ) override;
+    VECTOR2I GetPosition() const override { return m_pos; }
 
-    void SetOrientation( double aNewAngle );
-    void SetOrientationDegrees( double aOrientation ) { SetOrientation( aOrientation * 10.0 ); }
-    double GetOrientation() const { return m_orient; }
-    double GetOrientationDegrees() const { return m_orient / 10.0; }
-    double GetOrientationRadians() const { return m_orient * M_PI / 1800; }
+    void SetOrientation( const EDA_ANGLE& aNewAngle );
+    EDA_ANGLE GetOrientation() const { return m_orient; }
+
+    /**
+     * Used as Layer property setter -- performs a flip if necessary to set the footprint layer
+     * @param aLayer is the target layer (F_Cu or B_Cu)
+     */
+    void SetLayerAndFlip( PCB_LAYER_ID aLayer );
+
+    // to make property magic work
+    PCB_LAYER_ID GetLayer() const override { return BOARD_ITEM::GetLayer(); }
+
+    // For property system:
+    void SetOrientationDegrees( double aOrientation )
+    {
+        SetOrientation( EDA_ANGLE( aOrientation, DEGREES_T ) );
+    }
+    double GetOrientationDegrees() const
+    {
+        return m_orient.AsDegrees();
+    }
 
     const LIB_ID& GetFPID() const { return m_fpid; }
     void SetFPID( const LIB_ID& aFPID ) { m_fpid = aFPID; }
 
-    const wxString& GetDescription() const { return m_doc; }
+    wxString GetFPIDAsString() const { return m_fpid.Format(); }
+    void SetFPIDAsString( const wxString& aFPID ) { m_fpid.Parse( aFPID ); }
+
+    wxString GetDescription() const { return m_doc; }
     void SetDescription( const wxString& aDoc ) { m_doc = aDoc; }
 
-    const wxString& GetKeywords() const { return m_keywords; }
+    wxString GetKeywords() const { return m_keywords; }
     void SetKeywords( const wxString& aKeywords ) { m_keywords = aKeywords; }
 
     const KIID_PATH& GetPath() const { return m_path; }
@@ -227,12 +247,6 @@ public:
     void SetZoneConnection( ZONE_CONNECTION aType ) { m_zoneConnection = aType; }
     ZONE_CONNECTION GetZoneConnection() const { return m_zoneConnection; }
 
-    void SetThermalWidth( int aWidth ) { m_thermalWidth = aWidth; }
-    int GetThermalWidth() const { return m_thermalWidth; }
-
-    void SetThermalGap( int aGap ) { m_thermalGap = aGap; }
-    int GetThermalGap() const { return m_thermalGap; }
-
     int GetAttributes() const { return m_attributes; }
     void SetAttributes( int aAttributes ) { m_attributes = aAttributes; }
 
@@ -240,12 +254,44 @@ public:
     void IncrementFlag() { m_arflag += 1; }
     int GetFlag() const { return m_arflag; }
 
-    // A bit of a hack until net ties are supported as first class citizens
     bool IsNetTie() const
     {
-        return GetKeywords().StartsWith( wxT( "net tie" ) );
+        for( const wxString& group : m_netTiePadGroups )
+        {
+            if( !group.IsEmpty() )
+                return true;
+        }
+
+        return false;
     }
 
+    /**
+     * @return a list of pad groups, each of which is allowed to short nets within their group.
+     *         A pad group is a comma-separated list of pad numbers.
+     */
+    const std::vector<wxString>& GetNetTiePadGroups() const { return m_netTiePadGroups; }
+
+    void ClearNetTiePadGroups()
+    {
+        m_netTiePadGroups.clear();
+    }
+
+    void AddNetTiePadGroup( const wxString& aGroup )
+    {
+        m_netTiePadGroups.emplace_back( aGroup );
+    }
+
+    /**
+     * @return a map from pad numbers to net-tie group indices.  If a pad is not a member of
+     *         a net-tie group its index will be -1.
+     */
+    std::map<wxString, int> MapPadNumbersToNetTieGroups() const;
+
+    /**
+     * @return a list of pads that appear in \a aPad's net-tie pad group.
+     */
+    std::vector<PAD*> GetNetTiePads( PAD* aPad ) const;
+
     /**
      * Returns the most likely attribute based on pads
      * Either FP_THROUGH_HOLE/FP_SMD/OTHER(0)
@@ -253,11 +299,11 @@ public:
      */
     int GetLikelyAttribute() const;
 
-    void Move( const wxPoint& aMoveVector ) override;
+    void Move( const VECTOR2I& aMoveVector ) override;
 
-    void Rotate( const wxPoint& aRotCentre, double aAngle ) override;
+    void Rotate( const VECTOR2I& aRotCentre, const EDA_ANGLE& aAngle ) override;
 
-    void Flip( const wxPoint& aCentre, bool aFlipLeftRight ) override;
+    void Flip( const VECTOR2I& aCentre, bool aFlipLeftRight ) override;
 
     /**
      * Move the reference point of the footprint.
@@ -270,7 +316,7 @@ public:
      * (a move footprint does not change these local coordinates,
      * but changes the footprint position)
      */
-    void MoveAnchorPosition( const wxPoint& aMoveVector );
+    void MoveAnchorPosition( const VECTOR2I& aMoveVector );
 
     /**
      * @return true if the footprint is flipped, i.e. on the back side of the board
@@ -309,6 +355,11 @@ public:
             m_fpStatus &= ~FP_is_LOCKED;
     }
 
+    /**
+     * @return true if the footprint is flagged with conflicting with some item
+     */
+    bool IsConflicting() const;
+
     bool IsPlaced() const { return m_fpStatus & FP_is_PLACED;  }
     void SetIsPlaced( bool isPlaced )
     {
@@ -329,54 +380,63 @@ public:
 
     bool LegacyPadsLocked() const { return m_fpStatus & FP_PADS_are_LOCKED;  }
 
-    /*
-    void SetPadsLocked( bool aPadsLocked )
-    {
-        if( aPadsLocked )
-            m_fpStatus |= FP_PADS_are_LOCKED;
-        else
-            m_fpStatus &= ~FP_PADS_are_LOCKED;
-    }
-    */
-
-    void SetLastEditTime( timestamp_t aTime ) { m_lastEditTime = aTime; }
-    timestamp_t GetLastEditTime() const { return m_lastEditTime; }
-
     /**
      * Test if footprint attributes for type (SMD/Through hole/Other) match the expected
      * type based on the pads in the footprint.
-     * Footprints with plated through-hole pads should usually be marked through hole even if they also
-     * have SMD because they might not be auto-placed.  Exceptions to this might be shielded connectors
-     * Otherwise, footprints with SMD pads should be marked SMD
+     * Footprints with plated through-hole pads should usually be marked through hole even if they
+     * also have SMD because they might not be auto-placed.  Exceptions to this might be shielded
+     * connectors.  Otherwise, footprints with SMD pads should be marked SMD.
      * Footprints with no connecting pads should be marked "Other"
      *
      * @param aErrorHandler callback to handle the error messages generated
      */
-    void CheckFootprintAttributes( const std::function<void( const wxString& msg )>* aErrorHandler );
+    void CheckFootprintAttributes( const std::function<void( const wxString& )>& aErrorHandler );
 
     /**
-     * Test if footprint attributes for type (SMD/Through hole/Other) match the expected
-     * type based on the pads in the footprint.
-     * Footprints with plated through-hole pads should usually be marked through hole even if they also
-     * have SMD because they might not be auto-placed.  Exceptions to this might be shielded connectors
-     * Otherwise, footprints with SMD pads should be marked SMD
-     * Footprints with no connecting pads should be marked "Other"
+     * Run non-board-specific DRC checks on footprint's pads.  These are the checks supported by
+     * both the PCB DRC and the Footprint Editor Footprint Checker.
      *
      * @param aErrorHandler callback to handle the error messages generated
      */
-    void CheckFootprintTHPadNoHoles( const std::function
-                                     <void( const wxString& msg, const wxPoint& position )>*
-                                     aErrorHandler );
+    void CheckPads( const std::function<void( const PAD*, int, const wxString& )>& aErrorHandler );
+
+    /**
+     * Check for overlapping, different-numbered, non-net-tie pads.
+     *
+     * @param aErrorHandler callback to handle the error messages generated
+     */
+    void CheckShortingPads( const std::function<void( const PAD*,
+                                                      const PAD*,
+                                                      const VECTOR2I& )>& aErrorHandler );
+
+    /**
+     * Check for un-allowed shorting of pads in net-tie footprints.  If two pads are shorted,
+     * they must both appear in one of the allowed-shorting lists.
+     *
+     * @param aErrorHandler callback to handle the error messages generated
+     */
+    void CheckNetTies( const std::function<void( const BOARD_ITEM* aItem,
+                                                 const BOARD_ITEM* bItem,
+                                                 const BOARD_ITEM* cItem,
+                                                 const VECTOR2I& )>& aErrorHandler );
+
+    /**
+     * Sanity check net-tie pad groups.  Pads cannot be listed more than once, and pad numbers
+     * must correspond to a pad.
+     *
+     * @param aErrorHandler callback to handle the error messages generated
+     */
+    void CheckNetTiePadGroups( const std::function<void( const wxString& )>& aErrorHandler );
 
     /**
      * Generate pads shapes on layer \a aLayer as polygons and adds these polygons to
-     * \a aCornerBuffer.
+     * \a aBuffer.
      *
      * Useful to generate a polygonal representation of a footprint in 3D view and plot functions,
      * when a full polygonal approach is needed.
      *
      * @param aLayer is the layer to consider, or #UNDEFINED_LAYER to consider all layers.
-     * @param aCornerBuffer i the buffer to store polygons.
+     * @param aBuffer i the buffer to store polygons.
      * @param aClearance is an additional size to add to pad shapes.
      * @param aMaxError is the maximum deviation from true for arcs.
      * @param aSkipNPTHPadsWihNoCopper if true, do not add a NPTH pad shape, if the shape has
@@ -388,43 +448,39 @@ public:
      * @param aSkipPlatedPads is used on 3D-Viewer to extract plated and non-plated pads.
      * @param aSkipNonPlatedPads is used on 3D-Viewer to extract plated and plated pads.
      */
-    void TransformPadsWithClearanceToPolygon( SHAPE_POLY_SET& aCornerBuffer,
-                                              PCB_LAYER_ID aLayer, int aClearance,
-                                              int aMaxError, ERROR_LOC aErrorLoc,
-                                              bool aSkipNPTHPadsWihNoCopper = false,
-                                              bool aSkipPlatedPads = false,
-                                              bool aSkipNonPlatedPads = false ) const;
+    void TransformPadsToPolySet( SHAPE_POLY_SET& aBuffer, PCB_LAYER_ID aLayer, int aClearance,
+                                 int aMaxError, ERROR_LOC aErrorLoc,
+                                 bool aSkipNPTHPadsWihNoCopper = false,
+                                 bool aSkipPlatedPads = false,
+                                 bool aSkipNonPlatedPads = false ) const;
 
     /**
      * Generate shapes of graphic items (outlines) on layer \a aLayer as polygons and adds these
-     * polygons to \a aCornerBuffer.
+     * polygons to \a aBuffer.
      *
      * Useful to generate a polygonal representation of a footprint in 3D view and plot functions,
      * when a full polygonal approach is needed.
      *
      * @param aLayer is the layer to consider, or #UNDEFINED_LAYER to consider all.
-     * @param aCornerBuffer is the buffer to store polygons.
+     * @param aBuffer is the buffer to store polygons.
      * @param aClearance is a value to inflate shapes.
      * @param aError is the maximum error between true arc and polygon approximation.
      * @param aIncludeText set to true to transform text shapes.
      * @param aIncludeShapes set to true to transform footprint shapes.
      */
-    void TransformFPShapesWithClearanceToPolygon( SHAPE_POLY_SET& aCornerBuffer,
-                                                  PCB_LAYER_ID aLayer, int aClearance,
-                                                  int aError, ERROR_LOC aErrorLoc,
-                                                  bool aIncludeText = true,
-                                                  bool aIncludeShapes = true ) const;
+    void TransformFPShapesToPolySet( SHAPE_POLY_SET& aBuffer, PCB_LAYER_ID aLayer, int aClearance,
+                                     int aError, ERROR_LOC aErrorLoc,
+                                     bool aIncludeText = true,
+                                     bool aIncludeShapes = true,
+                                     bool aIncludePrivateItems = false ) const;
 
     /**
-     * This function is the same as TransformGraphicShapesWithClearanceToPolygonSet
-     * but only generate text.
+     * This function is the same as TransformFPShapesToPolySet but only generates text.
      */
-    void TransformFPTextWithClearanceToPolygonSet( SHAPE_POLY_SET& aCornerBuffer,
-                                                   PCB_LAYER_ID aLayer, int aClearance,
-                                                   int aError, ERROR_LOC aErrorLoc ) const
+    void TransformFPTextToPolySet( SHAPE_POLY_SET& aBuffer, PCB_LAYER_ID aLayer, int aClearance,
+                                   int aError, ERROR_LOC aErrorLoc ) const
     {
-        TransformFPShapesWithClearanceToPolygon( aCornerBuffer, aLayer, aClearance, aError,
-                                                 aErrorLoc, true, false );
+        TransformFPShapesToPolySet( aBuffer, aLayer, aClearance, aError, aErrorLoc, true, false );
     }
 
     /**
@@ -442,7 +498,7 @@ public:
     ///< @copydoc EDA_ITEM::GetMsgPanelInfo
     void GetMsgPanelInfo( EDA_DRAW_FRAME* aFrame, std::vector<MSG_PANEL_ITEM>& aList ) override;
 
-    bool HitTest( const wxPoint& aPosition, int aAccuracy = 0 ) const override;
+    bool HitTest( const VECTOR2I& aPosition, int aAccuracy = 0 ) const override;
 
     /**
      * Test if a point is inside the bounding polygon of the footprint.
@@ -453,9 +509,9 @@ public:
      * @param aPosition is the point to test
      * @return true if aPosition is inside the bounding polygon
      */
-    bool HitTestAccurate( const wxPoint& aPosition, int aAccuracy = 0 ) const;
+    bool HitTestAccurate( const VECTOR2I& aPosition, int aAccuracy = 0 ) const;
 
-    bool HitTest( const EDA_RECT& aRect, bool aContained, int aAccuracy = 0 ) const override;
+    bool HitTest( const BOX2I& aRect, bool aContained, int aAccuracy = 0 ) const override;
 
     /**
      * @return reference designator text.
@@ -474,6 +530,12 @@ public:
         m_reference->SetText( aReference );
     }
 
+    // Property system doesn't like const references
+    wxString GetReferenceAsString() const
+    {
+        return GetReference();
+    }
+
     /**
      * Bump the current reference by \a aDelta.
      */
@@ -495,6 +557,12 @@ public:
         m_value->SetText( aValue );
     }
 
+    // Property system doesn't like const references
+    wxString GetValueAsString() const
+    {
+        return GetValue();
+    }
+
     /// read/write accessors:
     FP_TEXT& Value()           { return *m_value; }
     FP_TEXT& Reference()       { return *m_reference; }
@@ -503,14 +571,53 @@ public:
     FP_TEXT& Value() const     { return *m_value; }
     FP_TEXT& Reference() const { return *m_reference; }
 
-    const std::map<wxString, wxString>& GetProperties() const { return m_properties; }
+    const std::map<wxString, wxString>& GetProperties() const        { return m_properties; }
     void SetProperties( const std::map<wxString, wxString>& aProps ) { m_properties = aProps; }
-    const wxString& GetProperty( const wxString& aKey) { return m_properties[ aKey ]; }
+    const wxString& GetProperty( const wxString& aKey)               { return m_properties[ aKey ]; }
     bool HasProperty( const wxString& aKey)
     {
         return m_properties.find( aKey ) != m_properties.end();
     }
-    void SetProperty( const wxString& aKey, const wxString& aVal ) { m_properties[ aKey ] = aVal; }
+    void SetProperty( const wxString& aKey, const wxString& aVal )   { m_properties[ aKey ] = aVal; }
+
+    bool IsBoardOnly() const { return m_attributes & FP_BOARD_ONLY; }
+    void SetBoardOnly( bool aIsBoardOnly = true )
+    {
+        if( aIsBoardOnly )
+            m_attributes |= FP_BOARD_ONLY;
+        else
+            m_attributes &= ~FP_BOARD_ONLY;
+    }
+
+    bool IsExcludedFromPosFiles() const { return m_attributes & FP_EXCLUDE_FROM_POS_FILES; }
+    void SetExcludedFromPosFiles( bool aExclude = true )
+    {
+        if( aExclude )
+            m_attributes |= FP_EXCLUDE_FROM_POS_FILES;
+        else
+            m_attributes &= ~FP_EXCLUDE_FROM_POS_FILES;
+    }
+
+    bool IsExcludedFromBOM() const { return m_attributes & FP_EXCLUDE_FROM_BOM; }
+    void SetExcludedFromBOM( bool aExclude = true )
+    {
+        if( aExclude )
+            m_attributes |= FP_EXCLUDE_FROM_BOM;
+        else
+            m_attributes &= ~FP_EXCLUDE_FROM_BOM;
+    }
+
+    bool AllowMissingCourtyard() const { return m_attributes & FP_ALLOW_MISSING_COURTYARD; }
+    void SetAllowMissingCourtyard( bool aAllow = true )
+    {
+        if( aAllow )
+            m_attributes |= FP_ALLOW_MISSING_COURTYARD;
+        else
+            m_attributes &= ~FP_ALLOW_MISSING_COURTYARD;
+    }
+
+    void SetFileFormatVersionAtLoad( int aVersion ) { m_fileFormatVersionAtLoad = aVersion; }
+    int GetFileFormatVersionAtLoad() const { return m_fileFormatVersionAtLoad; }
 
     /**
      * Return a #PAD with a matching number.
@@ -526,13 +633,11 @@ public:
     /**
      * Get a pad at \a aPosition on \a aLayerMask in the footprint.
      *
-     * @param aPosition A wxPoint object containing the position to hit test.
+     * @param aPosition A VECTOR2I object containing the position to hit test.
      * @param aLayerMask A layer or layers to mask the hit test.
      * @return A pointer to a #PAD object if found otherwise NULL.
      */
-    PAD* GetPad( const wxPoint& aPosition, LSET aLayerMask = LSET::AllLayersMask() );
-
-    PAD* GetTopLeftPad();
+    PAD* GetPad( const VECTOR2I& aPosition, LSET aLayerMask = LSET::AllLayersMask() );
 
     /**
      * Return the number of pads.
@@ -575,12 +680,6 @@ public:
     KIID GetLink() const { return m_link; }
     void SetLink( const KIID& aLink ) { m_link = aLink; }
 
-    int GetPlacementCost180() const { return m_rot180Cost; }
-    void SetPlacementCost180( int aCost )   { m_rot180Cost = aCost; }
-
-    int GetPlacementCost90() const { return m_rot90Cost; }
-    void SetPlacementCost90( int aCost )    { m_rot90Cost = aCost; }
-
     BOARD_ITEM* Duplicate() const override;
 
     /**
@@ -597,14 +696,15 @@ public:
      */
     void Add3DModel( FP_3DMODEL* a3DModel );
 
-    SEARCH_RESULT Visit( INSPECTOR inspector, void* testData, const KICAD_T scanTypes[] ) override;
+    INSPECT_RESULT Visit( INSPECTOR inspector, void* testData,
+                          const std::vector<KICAD_T>& aScanTypes ) override;
 
     wxString GetClass() const override
     {
         return wxT( "FOOTPRINT" );
     }
 
-    wxString GetSelectMenuText( EDA_UNITS aUnits ) const override;
+    wxString GetItemDescription( UNITS_PROVIDER* aUnitsProvider ) const override;
 
     BITMAPS GetMenuImage() const override;
 
@@ -619,15 +719,6 @@ public:
      */
     void RunOnChildren( const std::function<void (BOARD_ITEM*)>& aFunction ) const;
 
-    /**
-     * Return a set of all layers that this footprint has drawings on similar to ViewGetLayers().
-     *
-     * @param aLayers is an array to store layer ids.
-     * @param aCount is the number of layers stored in the array.
-     * @param aIncludePads controls whether to also include pad layers.
-     */
-    void GetAllDrawingLayers( int aLayers[], int& aCount, bool aIncludePads = true ) const;
-
     virtual void ViewGetLayers( int aLayers[], int& aCount ) const override;
 
     double ViewGetLOD( int aLayer, KIGFX::VIEW* aView ) const override;
@@ -654,6 +745,11 @@ public:
      */
     static const wxChar* StringLibNameInvalidChars( bool aUserReadable );
 
+    /**
+     * Return true if a board footprint differs from the library version.
+     */
+    bool FootprintNeedsUpdate( const FOOTPRINT* aLibFootprint );
+
     /**
      * Take ownership of caller's heap allocated aInitialComments block.
      *
@@ -692,13 +788,7 @@ public:
      *
      * @return the courtyard polygon.
      */
-    const SHAPE_POLY_SET& GetPolyCourtyard( PCB_LAYER_ID aLayer ) const
-    {
-        if( IsBackLayer( aLayer ) )
-            return m_poly_courtyard_back;
-        else
-            return m_poly_courtyard_front;
-    }
+    const SHAPE_POLY_SET& GetCourtyard( PCB_LAYER_ID aLayer ) const;
 
     /**
      * Build complex polygons of the courtyard areas from graphic items on the courtyard layers.
@@ -706,15 +796,19 @@ public:
      * @note Set the #MALFORMED_F_COURTYARD and #MALFORMED_B_COURTYARD status flags if the given
      *       courtyard layer does not contain a (single) closed shape.
      */
-    void BuildPolyCourtyards( OUTLINE_ERROR_HANDLER* aErrorHandler = nullptr );
+    void BuildCourtyardCaches( OUTLINE_ERROR_HANDLER* aErrorHandler = nullptr );
 
-    virtual std::shared_ptr<SHAPE> GetEffectiveShape( PCB_LAYER_ID aLayer = UNDEFINED_LAYER ) const override;
+    // @copydoc BOARD_ITEM::GetEffectiveShape
+    std::shared_ptr<SHAPE> GetEffectiveShape( PCB_LAYER_ID aLayer = UNDEFINED_LAYER,
+                                              FLASHING aFlash = FLASHING::DEFAULT ) const override;
 
-    virtual void SwapData( BOARD_ITEM* aImage ) override;
+#if defined(DEBUG)
+    virtual void Show( int nestLevel, std::ostream& os ) const override { ShowDummy( os ); }
+#endif
 
     struct cmp_drawings
     {
-        bool operator()( const BOARD_ITEM* aFirst, const BOARD_ITEM* aSecond ) const;
+        bool operator()( const BOARD_ITEM* itemA, const BOARD_ITEM* itemB ) const;
     };
 
     struct cmp_pads
@@ -722,10 +816,13 @@ public:
         bool operator()( const PAD* aFirst, const PAD* aSecond ) const;
     };
 
+    struct cmp_zones
+    {
+        bool operator()( const FP_ZONE* aFirst, const FP_ZONE* aSecond ) const;
+    };
 
-#if defined(DEBUG)
-    virtual void Show( int nestLevel, std::ostream& os ) const override { ShowDummy( os ); }
-#endif
+protected:
+    virtual void swapData( BOARD_ITEM* aImage ) override;
 
 private:
     DRAWINGS        m_drawings;          // BOARD_ITEMs for drawings on the board, owned by pointer.
@@ -733,13 +830,14 @@ private:
     FP_ZONES        m_fp_zones;          // FP_ZONE items, owned by pointer
     FP_GROUPS       m_fp_groups;         // PCB_GROUP items, owned by pointer
 
-    double          m_orient;            // Orientation in tenths of a degree, 900=90.0 degrees.
-    wxPoint         m_pos;               // Position of footprint on the board in internal units.
+    EDA_ANGLE       m_orient;            // Orientation
+    VECTOR2I        m_pos;               // Position of footprint on the board in internal units.
     FP_TEXT*        m_reference;         // Component reference designator value (U34, R18..)
     FP_TEXT*        m_value;             // Component value (74LS00, 22K..)
     LIB_ID          m_fpid;              // The #LIB_ID of the FOOTPRINT.
-    int             m_attributes;        // Flag bits ( see FOOTPRINT_ATTR_T )
+    int             m_attributes;        // Flag bits (see FOOTPRINT_ATTR_T)
     int             m_fpStatus;          // For autoplace: flags (LOCKED, FIELDS_AUTOPLACED)
+    int             m_fileFormatVersionAtLoad;
 
     // Bounding box caching strategy:
     // While we attempt to notice the low-hanging fruit operations and update the bounding boxes
@@ -751,18 +849,20 @@ private:
     // that any edit that could affect the bounding boxes (including edits to the footprint
     // children) marked the bounding boxes dirty.  It would definitely be faster -- but also more
     // fragile.
-    mutable EDA_RECT       m_cachedBoundingBox;
+    mutable BOX2I          m_cachedBoundingBox;
     mutable int            m_boundingBoxCacheTimeStamp;
-    mutable EDA_RECT       m_cachedVisibleBBox;
+    mutable BOX2I          m_cachedVisibleBBox;
     mutable int            m_visibleBBoxCacheTimeStamp;
-    mutable EDA_RECT       m_cachedTextExcludedBBox;
+    mutable BOX2I          m_cachedTextExcludedBBox;
     mutable int            m_textExcludedBBoxCacheTimeStamp;
     mutable SHAPE_POLY_SET m_cachedHull;
     mutable int            m_hullCacheTimeStamp;
 
+    // A list of pad groups, each of which is allowed to short nets within their group.
+    // A pad group is a comma-separated list of pad numbers.
+    std::vector<wxString> m_netTiePadGroups;
+
     ZONE_CONNECTION m_zoneConnection;
-    int             m_thermalWidth;
-    int             m_thermalGap;
     int             m_localClearance;
     int             m_localSolderMaskMargin;       // Solder mask margin
     int             m_localSolderPasteMargin;      // Solder paste margin absolute value
@@ -774,16 +874,16 @@ private:
     timestamp_t     m_lastEditTime;
     int             m_arflag;            // Use to trace ratsnest and auto routing.
     KIID            m_link;              // Temporary logical link used during editing
-    int             m_rot90Cost;         // Horizontal automatic placement cost ( 0..10 ).
-    int             m_rot180Cost;        // Vertical automatic placement cost ( 0..10 ).
+    LSET            m_privateLayers;     // Layers visible only in the footprint editor
 
-    std::list<FP_3DMODEL>         m_3D_Drawings;       // Linked list of 3D models.
+    std::vector<FP_3DMODEL>       m_3D_Drawings;       // 3D models.
     std::map<wxString, wxString>  m_properties;
     wxArrayString*                m_initial_comments;  // s-expression comments in the footprint,
                                                        // lazily allocated only if needed for speed
 
-    SHAPE_POLY_SET  m_poly_courtyard_front;  // Note that a footprint can have both front and back
-    SHAPE_POLY_SET  m_poly_courtyard_back;   // courtyards populated.
+    SHAPE_POLY_SET  m_courtyard_cache_front;  // Note that a footprint can have both front and back
+    SHAPE_POLY_SET  m_courtyard_cache_back;   // courtyards populated.
+    mutable int     m_courtyard_cache_timestamp;
 };
 
 #endif     // FOOTPRINT_H
