@@ -402,7 +402,7 @@ void BRDITEMS_PLOTTER::PlotFootprintTextItem( const FP_TEXT* aText, const COLOR4
     m_plotter->SetColor( color );
 
     // calculate some text parameters :
-    VECTOR2I      size = aText->GetTextSize();
+    //VECTOR2I      size = aText->GetTextSize();
     VECTOR2I      pos = aText->GetTextPos();
     int           thickness = aText->GetEffectiveTextPenWidth();
     KIFONT::FONT* font = aText->GetFont();
@@ -415,14 +415,15 @@ void BRDITEMS_PLOTTER::PlotFootprintTextItem( const FP_TEXT* aText, const COLOR4
                                       aText->IsBold(), aText->IsItalic() );
     }
 
-    if( aText->IsMirrored() )
-        size.x = -size.x;  // Text is mirrored
-
     // Non bold texts thickness is clamped at 1/6 char size by the low level draw function.
     // but in Pcbnew we do not manage bold texts and thickness up to 1/4 char size
     // (like bold text) and we manage the thickness.
     // So we set bold flag to true
-    bool allow_bold = true;
+    TEXT_ATTRIBUTES attrs = aText->GetAttributes();
+    attrs.m_StrokeWidth = thickness;
+    attrs.m_Angle = aText->GetDrawRotation();
+    attrs.m_Bold = true;
+    attrs.m_Multiline = false;
 
     GBR_METADATA gbr_metadata;
 
@@ -435,9 +436,35 @@ void BRDITEMS_PLOTTER::PlotFootprintTextItem( const FP_TEXT* aText, const COLOR4
 
     m_plotter->SetCurrentLineWidth( thickness );
 
-    m_plotter->Text( pos, aColor, aText->GetShownText(), aText->GetDrawRotation(), size,
-                     aText->GetHorizJustify(), aText->GetVertJustify(), thickness,
-                     aText->IsItalic(), allow_bold, false, font, &gbr_metadata );
+    if( aText->IsKnockout() )
+    {
+        KIGFX::GAL_DISPLAY_OPTIONS empty_opts;
+        SHAPE_POLY_SET             knockouts;
+
+        CALLBACK_GAL callback_gal( empty_opts,
+                // Polygon callback
+                [&]( const SHAPE_LINE_CHAIN& aPoly )
+                {
+                    knockouts.AddOutline( aPoly );
+                } );
+
+        callback_gal.SetIsFill( font->IsOutline() );
+        callback_gal.SetIsStroke( font->IsStroke() );
+        font->Draw( &callback_gal, aText->GetShownText(), aText->GetDrawPos(), attrs );
+
+        SHAPE_POLY_SET finalPoly;
+        int            margin = attrs.m_StrokeWidth * 1.5
+                                    + GetKnockoutTextMargin( attrs.m_Size, attrs.m_StrokeWidth );
+
+        aText->TransformBoundingBoxToPolygon( &finalPoly, margin );
+        finalPoly.BooleanSubtract( knockouts, SHAPE_POLY_SET::PM_FAST );
+        finalPoly.Fracture( SHAPE_POLY_SET::PM_FAST );
+
+        for( int ii = 0; ii < finalPoly.OutlineCount(); ++ii )
+            m_plotter->PlotPoly( finalPoly.Outline( ii ), FILL_T::FILLED_SHAPE, 0, &gbr_metadata );
+    }
+    else
+        m_plotter->PlotText( pos, aColor, aText->GetShownText(), attrs, font, &gbr_metadata );
 }
 
 
@@ -746,10 +773,20 @@ void BRDITEMS_PLOTTER::PlotFootprintShape( const FP_SHAPE* aShape )
 
                     for( int jj = 0; jj < tmpPoly.OutlineCount(); ++jj )
                     {
-                        SHAPE_LINE_CHAIN &poly = tmpPoly.Outline( jj );
-                        m_plotter->PlotPoly( poly, aShape->IsFilled() ? FILL_T::FILLED_SHAPE
-                                                                      : FILL_T::NO_FILL,
-                                             thickness, &gbr_metadata );
+                        SHAPE_LINE_CHAIN& poly = tmpPoly.Outline( jj );
+                        FILL_T fill_mode = aShape->IsFilled() ? FILL_T::FILLED_SHAPE
+                                                              : FILL_T::NO_FILL;
+                        // Plot the current filled area
+                        // (as region for Gerber plotter to manage attributes)
+                        if( m_plotter->GetPlotterType() == PLOT_FORMAT::GERBER )
+                        {
+                            static_cast<GERBER_PLOTTER*>( m_plotter )->
+                                            PlotPolyAsRegion( poly, fill_mode,
+                                                              thickness, &gbr_metadata );
+                        }
+                        else
+                            m_plotter->PlotPoly( poly, fill_mode,
+                                                 thickness, &gbr_metadata );
                     }
                 }
             }
@@ -800,7 +837,6 @@ void BRDITEMS_PLOTTER::PlotPcbText( const EDA_TEXT* aText, PCB_LAYER_ID aLayer,
     }
 
     wxString        shownText( aText->GetShownText() );
-    TEXT_ATTRIBUTES attrs = aText->GetAttributes();
 
     if( shownText.IsEmpty() )
         return;
@@ -816,13 +852,16 @@ void BRDITEMS_PLOTTER::PlotPcbText( const EDA_TEXT* aText, PCB_LAYER_ID aLayer,
     COLOR4D color = getColor( aLayer );
     m_plotter->SetColor( color );
 
-    VECTOR2I size = aText->GetTextSize();
+    //VECTOR2I size = aText->GetTextSize();
     VECTOR2I pos = aText->GetTextPos();
 
+    TEXT_ATTRIBUTES attrs = aText->GetAttributes();
     attrs.m_StrokeWidth = aText->GetEffectiveTextPenWidth();
+    attrs.m_Angle = aText->GetDrawRotation();
+    attrs.m_Multiline = false;
 
-    if( aText->IsMirrored() )
-        size.x = -size.x;
+    //if( aText->IsMirrored() )
+    //    size.x = -size.x;
 
     m_plotter->SetCurrentLineWidth( attrs.m_StrokeWidth );
 
@@ -865,16 +904,12 @@ void BRDITEMS_PLOTTER::PlotPcbText( const EDA_TEXT* aText, PCB_LAYER_ID aLayer,
         for( unsigned ii = 0; ii < strings_list.Count(); ii++ )
         {
             wxString& txt =  strings_list.Item( ii );
-            m_plotter->Text( positions[ii], color, txt, aText->GetDrawRotation(), size,
-                             attrs.m_Halign, attrs.m_Valign, attrs.m_StrokeWidth, attrs.m_Italic,
-                             attrs.m_Bold, false, font, &gbr_metadata );
+            m_plotter->PlotText( positions[ii], color, txt, attrs, font, &gbr_metadata );
         }
     }
     else
     {
-        m_plotter->Text( pos, color, shownText, aText->GetDrawRotation(), size, attrs.m_Halign,
-                         attrs.m_Valign, attrs.m_StrokeWidth, attrs.m_Italic, attrs.m_Bold, false,
-                         font, &gbr_metadata );
+        m_plotter->PlotText( pos, color, shownText, attrs, font, &gbr_metadata );
     }
 }
 
@@ -1043,7 +1078,15 @@ void BRDITEMS_PLOTTER::PlotPcbShape( const PCB_SHAPE* aShape )
                         // Ensure the polygon is closed:
                         poly.SetClosed( true );
 
-                        m_plotter->PlotPoly( poly, fill, thickness, &gbr_metadata );
+                        // Plot the current filled area
+                        // (as region for Gerber plotter to manage attributes)
+                        if( m_plotter->GetPlotterType() == PLOT_FORMAT::GERBER )
+                        {
+                            static_cast<GERBER_PLOTTER*>( m_plotter )->
+                                        PlotPolyAsRegion( poly, fill, thickness, &gbr_metadata );
+                        }
+                        else
+                            m_plotter->PlotPoly( poly, fill, thickness, &gbr_metadata );
                     }
                 }
             }
@@ -1054,22 +1097,32 @@ void BRDITEMS_PLOTTER::PlotPcbShape( const PCB_SHAPE* aShape )
         {
             std::vector<VECTOR2I> pts = aShape->GetRectCorners();
 
-            if( sketch || thickness > 0 )
+            if( sketch )
             {
                 m_plotter->ThickSegment( pts[0], pts[1], thickness, GetPlotMode(), &gbr_metadata );
                 m_plotter->ThickSegment( pts[1], pts[2], thickness, GetPlotMode(), &gbr_metadata );
                 m_plotter->ThickSegment( pts[2], pts[3], thickness, GetPlotMode(), &gbr_metadata );
                 m_plotter->ThickSegment( pts[3], pts[0], thickness, GetPlotMode(), &gbr_metadata );
             }
-
-            if( !sketch && aShape->IsFilled() )
+            else
             {
                 SHAPE_LINE_CHAIN poly;
 
                 for( const VECTOR2I& pt : pts )
                     poly.Append( pt );
 
-                m_plotter->PlotPoly( poly, FILL_T::FILLED_SHAPE, -1, &gbr_metadata );
+                poly.Append( pts[0] );  // Close polygon.
+
+                FILL_T fill_mode = aShape->IsFilled() ? FILL_T::FILLED_SHAPE
+                                                      : FILL_T::NO_FILL;
+
+                if( m_plotter->GetPlotterType() == PLOT_FORMAT::GERBER )
+                {
+                    static_cast<GERBER_PLOTTER*>( m_plotter )->
+                                PlotPolyAsRegion( poly, fill_mode, thickness, &gbr_metadata );
+                }
+                else
+                    m_plotter->PlotPoly( poly, fill_mode, thickness, &gbr_metadata );
             }
 
             break;
