@@ -235,6 +235,17 @@ bool PNS_PCBNEW_RULE_RESOLVER::IsNetTieExclusion( const PNS::ITEM* aItem,
     std::shared_ptr<DRC_ENGINE> drcEngine = m_board->GetDesignSettings().m_DRCEngine;
     BOARD_ITEM*                 collidingItem = aCollidingItem->Parent();
 
+    FOOTPRINT* collidingFp = static_cast<FOOTPRINT*>( collidingItem->GetParentFootprint() );
+    FOOTPRINT* itemFp      = aItem->Parent()
+                                ? static_cast<FOOTPRINT*>( aItem->Parent()->GetParentFootprint() )
+                                : nullptr;
+
+    if( collidingFp && itemFp && ( collidingFp == itemFp ) && itemFp->IsNetTie() )
+    {
+        // Two items colliding from the same net tie footprint are not checked
+        return true;
+    }
+
     if( drcEngine && collidingItem )
     {
         return drcEngine->IsNetTieExclusion( aItem->Net(), ToLAYER_ID( aItem->Layer() ),
@@ -725,7 +736,13 @@ bool PNS_KICAD_IFACE_BASE::ImportSizes( PNS::SIZES_SETTINGS& aSizes, PNS::ITEM*
     aSizes.SetDiffPairViaGapSameAsTraceGap( false );
 
     int      holeToHoleMin = bds.m_HoleToHoleMin;
-    PNS::VIA dummyVia;
+    PNS::VIA dummyVia, coupledVia;
+
+    if( aStartItem )
+    {
+        dummyVia.SetNet( aStartItem->Net() );
+        coupledVia.SetNet( m_ruleResolver->DpCoupledNet( aStartItem->Net() ) );
+    }
 
     if( m_ruleResolver->QueryConstraint( PNS::CONSTRAINT_TYPE::CT_HOLE_TO_HOLE, &dummyVia,
                                          &dummyVia, UNDEFINED_LAYER, &constraint ) )
@@ -735,6 +752,14 @@ bool PNS_KICAD_IFACE_BASE::ImportSizes( PNS::SIZES_SETTINGS& aSizes, PNS::ITEM*
 
     aSizes.SetHoleToHole( holeToHoleMin );
 
+    if( m_ruleResolver->QueryConstraint( PNS::CONSTRAINT_TYPE::CT_HOLE_TO_HOLE, &dummyVia,
+                                         &coupledVia, UNDEFINED_LAYER, &constraint ) )
+    {
+        holeToHoleMin = constraint.m_Value.Min();
+    }
+
+    aSizes.SetDiffPairHoleToHole( holeToHoleMin );
+
     return true;
 }
 
