@@ -2,7 +2,7 @@
  * This program source code file is part of KiCad, a free EDA CAD application.
  *
  * Copyright (C) 2014 CERN
- * Copyright (C) 2018-2021 KiCad Developers, see AUTHORS.txt for contributors.
+ * Copyright (C) 2018-2022 KiCad Developers, see AUTHORS.txt for contributors.
  * @author Tomasz Wlostowski <tomasz.wlostowski@cern.ch>
  *
  * This program is free software; you can redistribute it and/or
@@ -24,7 +24,6 @@
  */
 
 #include <functional>
-#include <board.h>
 #include <pcb_dimension.h>
 #include <fp_shape.h>
 #include <footprint.h>
@@ -79,41 +78,52 @@ PCB_GRID_HELPER::PCB_GRID_HELPER( TOOL_MANAGER* aToolMgr, MAGNETIC_SETTINGS* aMa
 
 VECTOR2I PCB_GRID_HELPER::AlignToSegment( const VECTOR2I& aPoint, const SEG& aSeg )
 {
-    OPT_VECTOR2I pts[6];
+    const int c_gridSnapEpsilon_sq = 4;
 
-    const int c_gridSnapEpsilon = 2;
+    VECTOR2I aligned = Align( aPoint );
 
     if( !m_enableSnap )
-        return aPoint;
+        return aligned;
 
-    VECTOR2I nearest = Align( aPoint );
+    std::vector<VECTOR2I> points;
 
-    SEG pos_slope( nearest + VECTOR2I( -1, 1 ), nearest + VECTOR2I( 1, -1 ) );
-    SEG neg_slope( nearest + VECTOR2I( -1, -1 ), nearest + VECTOR2I( 1, 1 ) );
-    int max_i = 2;
+    const SEG testSegments[] = { SEG( aligned, aligned + VECTOR2( 1, 0 ) ),
+                                 SEG( aligned, aligned + VECTOR2( 0, 1 ) ),
+                                 SEG( aligned, aligned + VECTOR2( 1, 1 ) ),
+                                 SEG( aligned, aligned + VECTOR2( 1, -1 ) ) };
 
-    pts[0] = aSeg.A;
-    pts[1] = aSeg.B;
-
-    if( !aSeg.ApproxParallel( pos_slope ) )
-        pts[max_i++] = aSeg.IntersectLines( pos_slope );
-
-    if( !aSeg.ApproxParallel( neg_slope ) )
-        pts[max_i++] = aSeg.IntersectLines( neg_slope );
-
-    int min_d = std::numeric_limits<int>::max();
-
-    for( int i = 0; i < max_i; i++ )
+    for( const SEG& seg : testSegments )
     {
-        if( pts[i] && aSeg.Distance( *pts[i] ) <= c_gridSnapEpsilon )
-        {
-            int d = (*pts[i] - aPoint).EuclideanNorm();
+        OPT_VECTOR2I vec = aSeg.IntersectLines( seg );
 
-            if( d < min_d )
-            {
-                min_d = d;
-                nearest = *pts[i];
-            }
+        if( vec && aSeg.SquaredDistance( *vec ) <= c_gridSnapEpsilon_sq )
+            points.push_back( *vec );
+    }
+
+    VECTOR2I    nearest = aligned;
+    SEG::ecoord min_d_sq = VECTOR2I::ECOORD_MAX;
+
+    // Snap by distance between pointer and endpoints
+    for( const VECTOR2I& pt : { aSeg.A, aSeg.B } )
+    {
+        SEG::ecoord d_sq = ( pt - aPoint ).SquaredEuclideanNorm();
+
+        if( d_sq < min_d_sq )
+        {
+            min_d_sq = d_sq;
+            nearest = pt;
+        }
+    }
+
+    // Snap by distance between aligned cursor and intersections
+    for( const VECTOR2I& pt : points )
+    {
+        SEG::ecoord d_sq = ( pt - aligned ).SquaredEuclideanNorm();
+
+        if( d_sq < min_d_sq )
+        {
+            min_d_sq = d_sq;
+            nearest = pt;
         }
     }
 
@@ -123,28 +133,43 @@ VECTOR2I PCB_GRID_HELPER::AlignToSegment( const VECTOR2I& aPoint, const SEG& aSe
 
 VECTOR2I PCB_GRID_HELPER::AlignToArc( const VECTOR2I& aPoint, const SHAPE_ARC& aArc )
 {
+    VECTOR2I aligned = Align( aPoint );
+
     if( !m_enableSnap )
-        return aPoint;
+        return aligned;
 
-    const VECTOR2D gridOffset( GetOrigin() );
-    const VECTOR2D gridSize( GetGrid() );
+    std::vector<VECTOR2I> points;
 
-    VECTOR2I nearest( KiROUND( ( aPoint.x - gridOffset.x ) / gridSize.x ) * gridSize.x + gridOffset.x,
-                      KiROUND( ( aPoint.y - gridOffset.y ) / gridSize.y ) * gridSize.y + gridOffset.y );
+    aArc.IntersectLine( SEG( aligned, aligned + VECTOR2( 1, 0 ) ), &points );
+    aArc.IntersectLine( SEG( aligned, aligned + VECTOR2( 0, 1 ) ), &points );
+    aArc.IntersectLine( SEG( aligned, aligned + VECTOR2( 1, 1 ) ), &points );
+    aArc.IntersectLine( SEG( aligned, aligned + VECTOR2( 1, -1 ) ), &points );
 
-    int min_d = std::numeric_limits<int>::max();
+    VECTOR2I    nearest = aligned;
+    SEG::ecoord min_d_sq = VECTOR2I::ECOORD_MAX;
 
-    for( auto pt : { aArc.GetP0(), aArc.GetP1() } )
+    // Snap by distance between pointer and endpoints
+    for( const VECTOR2I& pt : { aArc.GetP0(), aArc.GetP1() } )
     {
-        int d = ( pt - aPoint ).EuclideanNorm();
+        SEG::ecoord d_sq = ( pt - aPoint ).SquaredEuclideanNorm();
 
-        if( d < min_d )
+        if( d_sq < min_d_sq )
         {
-            min_d = d;
+            min_d_sq = d_sq;
+            nearest = pt;
+        }
+    }
+
+    // Snap by distance between aligned cursor and intersections
+    for( const VECTOR2I& pt : points )
+    {
+        SEG::ecoord d_sq = ( pt - aligned ).SquaredEuclideanNorm();
+
+        if( d_sq < min_d_sq )
+        {
+            min_d_sq = d_sq;
             nearest = pt;
         }
-        else
-            break;
     }
 
     return nearest;
@@ -168,7 +193,7 @@ VECTOR2I PCB_GRID_HELPER::AlignToNearestPad( const VECTOR2I& aMousePos, PADS& aP
         if( ( ORIGIN & a.flags ) != ORIGIN )
             continue;
 
-        if( !item->HitTest( wxPoint( aMousePos.x, aMousePos.y ) ) )
+        if( !item->HitTest( aMousePos ) )
             continue;
 
         double dist = a.Distance( aMousePos );
@@ -185,12 +210,13 @@ VECTOR2I PCB_GRID_HELPER::AlignToNearestPad( const VECTOR2I& aMousePos, PADS& aP
 
 
 VECTOR2I PCB_GRID_HELPER::BestDragOrigin( const VECTOR2I &aMousePos,
-                                          std::vector<BOARD_ITEM*>& aItems )
+                                          std::vector<BOARD_ITEM*>& aItems,
+                                          const SELECTION_FILTER_OPTIONS* aSelectionFilter )
 {
     clearAnchors();
 
     for( BOARD_ITEM* item : aItems )
-        computeAnchors( item, aMousePos, true );
+        computeAnchors( item, aMousePos, true, aSelectionFilter );
 
     double worldScale = m_toolMgr->GetView()->GetGAL()->GetWorldScale();
     double lineSnapMinCornerDistance = 50.0 / worldScale;
@@ -259,12 +285,15 @@ VECTOR2I PCB_GRID_HELPER::BestSnapAnchor( const VECTOR2I& aOrigin, const LSET& a
     // points that are visible can always be snapped to.
     // see https://gitlab.com/kicad/code/kicad/-/issues/5638
     // see https://gitlab.com/kicad/code/kicad/-/issues/7125
-    double snapScale = snapSize / m_toolMgr->GetView()->GetGAL()->GetWorldScale();
-    int    snapRange = std::min( KiROUND( snapScale ), GetGrid().x );
-    int    snapDist  = snapRange;
+    // see https://gitlab.com/kicad/code/kicad/-/issues/12303
+    int snapScale = KiROUND( snapSize / m_toolMgr->GetView()->GetGAL()->GetWorldScale() );
+    int snapRange = m_enableGrid ? std::min( snapScale, GetVisibleGrid().x ) : snapScale;
+    int snapDist = snapRange;
 
-    BOX2I bb( VECTOR2I( aOrigin.x - snapRange / 2, aOrigin.y - snapRange / 2 ),
-              VECTOR2I( snapRange, snapRange ) );
+    //Respect limits of coordinates representation
+    BOX2I bb;
+    bb.SetOrigin( GetClampedCoords<double, int>( VECTOR2D( aOrigin ) - snapRange / 2.0 ) );
+    bb.SetEnd( GetClampedCoords<double, int>( VECTOR2D( aOrigin ) + snapRange / 2.0 ) );
 
     clearAnchors();
 
@@ -350,10 +379,10 @@ std::set<BOARD_ITEM*> PCB_GRID_HELPER::queryVisible( const BOX2I& aArea,
     std::set<BOARD_ITEM*> items;
     std::vector<KIGFX::VIEW::LAYER_ITEM_PAIR> selectedItems;
 
-    KIGFX::VIEW*                  view = m_toolMgr->GetView();
-    RENDER_SETTINGS*              settings = view->GetPainter()->GetSettings();
-    const std::set<unsigned int>& activeLayers = settings->GetHighContrastLayers();
-    bool                          isHighContrast = settings->GetHighContrast();
+    KIGFX::VIEW*         view = m_toolMgr->GetView();
+    RENDER_SETTINGS*     settings = view->GetPainter()->GetSettings();
+    const std::set<int>& activeLayers = settings->GetHighContrastLayers();
+    bool                 isHighContrast = settings->GetHighContrast();
 
     view->Query( aArea, selectedItems );
 
@@ -384,12 +413,13 @@ std::set<BOARD_ITEM*> PCB_GRID_HELPER::queryVisible( const BOX2I& aArea,
 }
 
 
-void PCB_GRID_HELPER::computeAnchors( BOARD_ITEM* aItem, const VECTOR2I& aRefPos, bool aFrom )
+void PCB_GRID_HELPER::computeAnchors( BOARD_ITEM* aItem, const VECTOR2I& aRefPos, bool aFrom,
+                                      const SELECTION_FILTER_OPTIONS* aSelectionFilter )
 {
-    KIGFX::VIEW*                  view = m_toolMgr->GetView();
-    RENDER_SETTINGS*              settings = view->GetPainter()->GetSettings();
-    const std::set<unsigned int>& activeLayers = settings->GetHighContrastLayers();
-    bool                          isHighContrast = settings->GetHighContrast();
+    KIGFX::VIEW*         view = m_toolMgr->GetView();
+    RENDER_SETTINGS*     settings = view->GetPainter()->GetSettings();
+    const std::set<int>& activeLayers = settings->GetHighContrastLayers();
+    bool                 isHighContrast = settings->GetHighContrast();
 
     auto handlePadShape =
             [&]( PAD* aPad )
@@ -400,109 +430,177 @@ void PCB_GRID_HELPER::computeAnchors( BOARD_ITEM* aItem, const VECTOR2I& aRefPos
                 if( aFrom )
                     return;
 
-                const std::shared_ptr<SHAPE> eshape = aPad->GetEffectiveShape();
-
-                wxASSERT( eshape->Type() == SH_COMPOUND );
-                const std::vector<SHAPE*> shapes =
-                        static_cast<const SHAPE_COMPOUND*>( eshape.get() )->Shapes();
-
-                for( const SHAPE* shape : shapes )
+                switch( aPad->GetShape() )
                 {
-                    switch( shape->Type() )
-                    {
-                    case SH_RECT:
-                    {
-                        const SHAPE_RECT* rect    = static_cast<const SHAPE_RECT*>( shape );
-                        SHAPE_LINE_CHAIN  outline = rect->Outline();
+                case PAD_SHAPE::CIRCLE:
+                {
+                    int      r      = aPad->GetSizeX() / 2;
+                    VECTOR2I center = aPad->ShapePos();
 
-                        for( int i = 0; i < outline.SegmentCount(); i++ )
+                    addAnchor( center + VECTOR2I( -r, 0 ), OUTLINE | SNAPPABLE, aPad );
+                    addAnchor( center + VECTOR2I( r, 0 ), OUTLINE | SNAPPABLE, aPad );
+                    addAnchor( center + VECTOR2I( 0, -r ), OUTLINE | SNAPPABLE, aPad );
+                    addAnchor( center + VECTOR2I( 0, r ), OUTLINE | SNAPPABLE, aPad );
+                    break;
+                }
+
+                case PAD_SHAPE::OVAL:
+                {
+                    VECTOR2I pos = aPad->ShapePos();
+                    VECTOR2I half_size = aPad->GetSize() / 2;
+                    int      half_width = std::min( half_size.x, half_size.y );
+                    VECTOR2I half_len( half_size.x - half_width, half_size.y - half_width );
+
+                    RotatePoint( half_len, aPad->GetOrientation() );
+
+                    VECTOR2I a( pos - half_len );
+                    VECTOR2I b( pos + half_len );
+                    VECTOR2I normal = b - a;
+                    normal.Resize( half_width );
+                    RotatePoint( normal, ANGLE_90 );
+
+                    addAnchor( a + normal, OUTLINE | SNAPPABLE, aPad );
+                    addAnchor( a - normal, OUTLINE | SNAPPABLE, aPad );
+                    addAnchor( b + normal, OUTLINE | SNAPPABLE, aPad );
+                    addAnchor( b - normal, OUTLINE | SNAPPABLE, aPad );
+                    addAnchor( pos + normal, OUTLINE | SNAPPABLE, aPad );
+                    addAnchor( pos - normal, OUTLINE | SNAPPABLE, aPad );
+
+                    RotatePoint( normal, -ANGLE_90 );
+
+                    addAnchor( a - normal, OUTLINE | SNAPPABLE, aPad );
+                    addAnchor( b + normal, OUTLINE | SNAPPABLE, aPad );
+                    break;
+                }
+
+                case PAD_SHAPE::RECT:
+                case PAD_SHAPE::TRAPEZOID:
+                case PAD_SHAPE::ROUNDRECT:
+                case PAD_SHAPE::CHAMFERED_RECT:
+                {
+                    VECTOR2I half_size( aPad->GetSize() / 2 );
+                    VECTOR2I trap_delta( 0, 0 );
+
+                    if( aPad->GetShape() == PAD_SHAPE::TRAPEZOID )
+                        trap_delta = aPad->GetDelta() / 2;
+
+                    SHAPE_LINE_CHAIN corners;
+
+                    corners.Append( -half_size.x - trap_delta.y,  half_size.y + trap_delta.x );
+                    corners.Append(  half_size.x + trap_delta.y,  half_size.y - trap_delta.x );
+                    corners.Append(  half_size.x - trap_delta.y, -half_size.y + trap_delta.x );
+                    corners.Append( -half_size.x + trap_delta.y, -half_size.y - trap_delta.x );
+                    corners.SetClosed( true );
+
+                    corners.Rotate( aPad->GetOrientation() );
+                    corners.Move( aPad->ShapePos() );
+
+                    for( size_t ii = 0; ii < corners.GetSegmentCount(); ++ii )
+                    {
+                        const SEG& seg = corners.GetSegment( ii );
+                        addAnchor( seg.A, OUTLINE | SNAPPABLE, aPad );
+                        addAnchor( seg.Center(), OUTLINE | SNAPPABLE, aPad );
+
+                        if( ii == corners.GetSegmentCount() - 1 )
+                            addAnchor( seg.B, OUTLINE | SNAPPABLE, aPad );
+                    }
+
+                    break;
+                }
+
+                default:
+                {
+                    const std::shared_ptr<SHAPE_POLY_SET>& outline = aPad->GetEffectivePolygon();
+
+                    if( !outline->IsEmpty() )
+                    {
+                        for( const VECTOR2I& pt : outline->Outline( 0 ).CPoints() )
+                            addAnchor( pt, OUTLINE | SNAPPABLE, aPad );
+                    }
+
+                    break;
+                }
+                }
+            };
+
+    auto handleShape =
+            [&]( PCB_SHAPE* shape )
+            {
+                VECTOR2I   start = shape->GetStart();
+                VECTOR2I   end = shape->GetEnd();
+
+                switch( shape->GetShape() )
+                {
+                    case SHAPE_T::CIRCLE:
+                    {
+                        int r = ( start - end ).EuclideanNorm();
+
+                        addAnchor( start, ORIGIN | SNAPPABLE, shape );
+                        addAnchor( start + VECTOR2I( -r, 0 ), OUTLINE | SNAPPABLE, shape );
+                        addAnchor( start + VECTOR2I( r, 0 ), OUTLINE | SNAPPABLE, shape );
+                        addAnchor( start + VECTOR2I( 0, -r ), OUTLINE | SNAPPABLE, shape );
+                        addAnchor( start + VECTOR2I( 0, r ), OUTLINE | SNAPPABLE, shape );
+                        break;
+                    }
+
+                    case SHAPE_T::ARC:
+                        addAnchor( shape->GetStart(), CORNER | SNAPPABLE, shape );
+                        addAnchor( shape->GetEnd(), CORNER | SNAPPABLE, shape );
+                        addAnchor( shape->GetArcMid(), CORNER | SNAPPABLE, shape );
+                        addAnchor( shape->GetCenter(), ORIGIN | SNAPPABLE, shape );
+                        break;
+
+                    case SHAPE_T::RECT:
+                    {
+                        VECTOR2I point2( end.x, start.y );
+                        VECTOR2I point3( start.x, end.y );
+                        SEG first( start, point2 );
+                        SEG second( point2, end );
+                        SEG third( end, point3 );
+                        SEG fourth( point3, start );
+
+                        addAnchor( first.A,         CORNER | SNAPPABLE, shape );
+                        addAnchor( first.Center(),  CORNER | SNAPPABLE, shape );
+                        addAnchor( second.A,        CORNER | SNAPPABLE, shape );
+                        addAnchor( second.Center(), CORNER | SNAPPABLE, shape );
+                        addAnchor( third.A,         CORNER | SNAPPABLE, shape );
+                        addAnchor( third.Center(),  CORNER | SNAPPABLE, shape );
+                        addAnchor( fourth.A,        CORNER | SNAPPABLE, shape );
+                        addAnchor( fourth.Center(), CORNER | SNAPPABLE, shape );
+                        break;
+                    }
+
+                    case SHAPE_T::SEGMENT:
+                        addAnchor( start, CORNER | SNAPPABLE, shape );
+                        addAnchor( end, CORNER | SNAPPABLE, shape );
+                        addAnchor( shape->GetCenter(), CORNER | SNAPPABLE, shape );
+                        break;
+
+                    case SHAPE_T::POLY:
+                    {
+                        SHAPE_LINE_CHAIN lc;
+                        lc.SetClosed( true );
+                        std::vector<VECTOR2I> poly;
+                        shape->DupPolyPointsList( poly );
+
+                        for( const VECTOR2I& p : poly )
                         {
-                            const SEG& seg = outline.CSegment( i );
-                            addAnchor( seg.A,         OUTLINE | SNAPPABLE, aPad );
-                            addAnchor( seg.Center(),  OUTLINE | SNAPPABLE, aPad );
+                            addAnchor( p, CORNER | SNAPPABLE, shape );
+                            lc.Append( p );
                         }
 
+                        addAnchor( lc.NearestPoint( aRefPos ), OUTLINE, aItem );
                         break;
                     }
 
-                    case SH_SEGMENT:
-                    {
-                        const SHAPE_SEGMENT* segment = static_cast<const SHAPE_SEGMENT*>( shape );
+                    case SHAPE_T::BEZIER:
+                        addAnchor( start, CORNER | SNAPPABLE, shape );
+                        addAnchor( end, CORNER | SNAPPABLE, shape );
+                        KI_FALLTHROUGH;
 
-                        int offset = segment->GetWidth() / 2;
-                        SEG seg    = segment->GetSeg();
-                        VECTOR2I normal = ( seg.B - seg.A ).Resize( offset ).Rotate( -M_PI_2 );
-
-                        /*
-                         * TODO: This creates more snap points than necessary for rounded rect pads
-                         * because they are built up of overlapping segments.  We could fix this if
-                         * desired by testing these to see if they are "inside" the pad.
-                         */
-
-                        addAnchor( seg.A + normal, OUTLINE | SNAPPABLE, aPad );
-                        addAnchor( seg.A - normal, OUTLINE | SNAPPABLE, aPad );
-                        addAnchor( seg.B + normal, OUTLINE | SNAPPABLE, aPad );
-                        addAnchor( seg.B - normal, OUTLINE | SNAPPABLE, aPad );
-                        addAnchor( seg.Center() + normal, OUTLINE | SNAPPABLE, aPad );
-                        addAnchor( seg.Center() - normal, OUTLINE | SNAPPABLE, aPad );
-
-                        normal = normal.Rotate( M_PI_2 );
-
-                        addAnchor( seg.A - normal, OUTLINE | SNAPPABLE, aPad );
-                        addAnchor( seg.B + normal, OUTLINE | SNAPPABLE, aPad );
-                        break;
-                    }
-
-                    case SH_CIRCLE:
-                    {
-                        const SHAPE_CIRCLE* circle = static_cast<const SHAPE_CIRCLE*>( shape );
-
-                        int      r     = circle->GetRadius();
-                        VECTOR2I start = circle->GetCenter();
-
-                        addAnchor( start + VECTOR2I( -r, 0 ), OUTLINE | SNAPPABLE, aPad );
-                        addAnchor( start + VECTOR2I( r, 0 ), OUTLINE | SNAPPABLE, aPad );
-                        addAnchor( start + VECTOR2I( 0, -r ), OUTLINE | SNAPPABLE, aPad );
-                        addAnchor( start + VECTOR2I( 0, r ), OUTLINE | SNAPPABLE, aPad );
-                        break;
-                    }
-
-                    case SH_ARC:
-                    {
-                        const SHAPE_ARC* arc = static_cast<const SHAPE_ARC*>( shape );
-
-                        addAnchor( arc->GetP0(), OUTLINE | SNAPPABLE, aPad );
-                        addAnchor( arc->GetP1(), OUTLINE | SNAPPABLE, aPad );
-                        addAnchor( arc->GetArcMid(), OUTLINE | SNAPPABLE, aPad );
-                        break;
-                    }
-
-                    case SH_SIMPLE:
-                    {
-                        const SHAPE_SIMPLE* poly = static_cast<const SHAPE_SIMPLE*>( shape );
-
-                        for( size_t i = 0; i < poly->GetSegmentCount(); i++ )
-                        {
-                            const SEG& seg = poly->GetSegment( i );
-
-                            addAnchor( seg.A, OUTLINE | SNAPPABLE, aPad );
-                            addAnchor( seg.Center(), OUTLINE | SNAPPABLE, aPad );
-
-                            if( i == poly->GetSegmentCount() - 1 )
-                                addAnchor( seg.B, OUTLINE | SNAPPABLE, aPad );
-                        }
-
-                        break;
-                    }
-
-                    case SH_POLY_SET:
-                    case SH_LINE_CHAIN:
-                    case SH_COMPOUND:
-                    case SH_POLY_SET_TRIANGLE:
-                    case SH_NULL:
                     default:
+                        addAnchor( shape->GetPosition(), ORIGIN | SNAPPABLE, shape );
                         break;
-                    }
                 }
             };
 
@@ -514,18 +612,43 @@ void PCB_GRID_HELPER::computeAnchors( BOARD_ITEM* aItem, const VECTOR2I& aRefPos
 
             for( PAD* pad : footprint->Pads() )
             {
-                // Getting pads from the footprint requires re-checking that the pad is shown
-                if( ( aFrom || m_magneticSettings->pads == MAGNETIC_OPTIONS::CAPTURE_ALWAYS )
-                    && pad->GetBoundingBox().Contains( wxPoint( aRefPos.x, aRefPos.y ) )
-                    && view->IsVisible( pad )
-                    && ( !isHighContrast || activeLayers.count( pad->GetLayer() ) )
-                    && pad->ViewGetLOD( pad->GetLayer(), view ) < view->GetScale() )
+                if( aFrom )
                 {
-                    handlePadShape( pad );
-                    break;
+                    if( aSelectionFilter && !aSelectionFilter->pads )
+                        continue;
+                }
+                else
+                {
+                    if( m_magneticSettings->pads != MAGNETIC_OPTIONS::CAPTURE_ALWAYS )
+                        continue;
+                }
+
+                if( !view->IsVisible( pad ) || !pad->GetBoundingBox().Contains( aRefPos ) )
+                    continue;
+
+                // Getting pads from a footprint requires re-checking that the pads are shown
+                bool onActiveLayer = !isHighContrast;
+                bool isLODVisible = false;
+
+                for( PCB_LAYER_ID layer : pad->GetLayerSet().Seq() )
+                {
+                    if( !onActiveLayer && activeLayers.count( layer ) )
+                        onActiveLayer = true;
+
+                    if( !isLODVisible && pad->ViewGetLOD( layer, view ) < view->GetScale() )
+                        isLODVisible = true;
+
+                    if( onActiveLayer && isLODVisible )
+                    {
+                        handlePadShape( pad );
+                        break;
+                    }
                 }
             }
 
+            if( aFrom && aSelectionFilter && !aSelectionFilter->footprints )
+                break;
+
             // if the cursor is not over a pad, then drag the footprint by its origin
             VECTOR2I position = footprint->GetPosition();
             addAnchor( position, ORIGIN | SNAPPABLE, footprint );
@@ -541,113 +664,72 @@ void PCB_GRID_HELPER::computeAnchors( BOARD_ITEM* aItem, const VECTOR2I& aRefPos
         }
 
         case PCB_PAD_T:
-        {
-            if( aFrom || m_magneticSettings->pads == MAGNETIC_OPTIONS::CAPTURE_ALWAYS )
+            if( aFrom )
             {
-                PAD* pad = static_cast<PAD*>( aItem );
-                handlePadShape( pad );
+                if( aSelectionFilter && !aSelectionFilter->pads )
+                    break;
+            }
+            else
+            {
+                if( m_magneticSettings->pads != MAGNETIC_OPTIONS::CAPTURE_ALWAYS )
+                    break;
             }
 
+            handlePadShape( static_cast<PAD*>( aItem ) );
+
+            break;
+
+        case PCB_FP_TEXTBOX_T:
+        case PCB_TEXTBOX_T:
+            if( aFrom )
+            {
+                if( aSelectionFilter && !aSelectionFilter->text )
+                    break;
+            }
+            else
+            {
+                if( !m_magneticSettings->graphics )
+                    break;
+            }
+
+            handleShape( static_cast<PCB_SHAPE*>( aItem ) );
             break;
-        }
 
         case PCB_FP_SHAPE_T:
         case PCB_SHAPE_T:
-        {
-            if( !m_magneticSettings->graphics )
-                break;
-
-            PCB_SHAPE* shape = static_cast<PCB_SHAPE*>( aItem );
-            VECTOR2I   start = shape->GetStart();
-            VECTOR2I   end = shape->GetEnd();
-
-            switch( shape->GetShape() )
+            if( aFrom )
             {
-                case SHAPE_T::CIRCLE:
-                {
-                    int r = ( start - end ).EuclideanNorm();
-
-                    addAnchor( start, ORIGIN | SNAPPABLE, shape );
-                    addAnchor( start + VECTOR2I( -r, 0 ), OUTLINE | SNAPPABLE, shape );
-                    addAnchor( start + VECTOR2I( r, 0 ), OUTLINE | SNAPPABLE, shape );
-                    addAnchor( start + VECTOR2I( 0, -r ), OUTLINE | SNAPPABLE, shape );
-                    addAnchor( start + VECTOR2I( 0, r ), OUTLINE | SNAPPABLE, shape );
-                    break;
-                }
-
-                case SHAPE_T::ARC:
-                    addAnchor( shape->GetStart(), CORNER | SNAPPABLE, shape );
-                    addAnchor( shape->GetEnd(), CORNER | SNAPPABLE, shape );
-                    addAnchor( shape->GetArcMid(), CORNER | SNAPPABLE, shape );
-                    addAnchor( shape->GetCenter(), ORIGIN | SNAPPABLE, shape );
-                    break;
-
-                case SHAPE_T::RECT:
-                {
-                    VECTOR2I point2( end.x, start.y );
-                    VECTOR2I point3( start.x, end.y );
-                    SEG first( start, point2 );
-                    SEG second( point2, end );
-                    SEG third( end, point3 );
-                    SEG fourth( point3, start );
-
-                    addAnchor( first.A,         CORNER | SNAPPABLE, shape );
-                    addAnchor( first.Center(),  CORNER | SNAPPABLE, shape );
-                    addAnchor( second.A,        CORNER | SNAPPABLE, shape );
-                    addAnchor( second.Center(), CORNER | SNAPPABLE, shape );
-                    addAnchor( third.A,         CORNER | SNAPPABLE, shape );
-                    addAnchor( third.Center(),  CORNER | SNAPPABLE, shape );
-                    addAnchor( fourth.A,        CORNER | SNAPPABLE, shape );
-                    addAnchor( fourth.Center(), CORNER | SNAPPABLE, shape );
-                    break;
-                }
-
-                case SHAPE_T::SEGMENT:
-                    addAnchor( start, CORNER | SNAPPABLE, shape );
-                    addAnchor( end, CORNER | SNAPPABLE, shape );
-                    addAnchor( shape->GetCenter(), CORNER | SNAPPABLE, shape );
-                    break;
-
-                case SHAPE_T::POLY:
-                {
-                    SHAPE_LINE_CHAIN lc;
-                    lc.SetClosed( true );
-                    std::vector<wxPoint> poly;
-                    shape->DupPolyPointsList( poly );
-
-                    for( const wxPoint& p : poly )
-                    {
-                        addAnchor( p, CORNER | SNAPPABLE, shape );
-                        lc.Append( p );
-                    }
-
-                    addAnchor( lc.NearestPoint( aRefPos ), OUTLINE, aItem );
-                    break;
-                }
-
-                case SHAPE_T::BEZIER:
-                    addAnchor( start, CORNER | SNAPPABLE, shape );
-                    addAnchor( end, CORNER | SNAPPABLE, shape );
-                    KI_FALLTHROUGH;
-
-                default:
-                    addAnchor( shape->GetPosition(), ORIGIN | SNAPPABLE, shape );
+                if( aSelectionFilter && !aSelectionFilter->graphics )
                     break;
             }
+            else
+            {
+                if( !m_magneticSettings->graphics )
+                    break;
+            }
+
+            handleShape( static_cast<PCB_SHAPE*>( aItem ) );
             break;
-        }
 
         case PCB_TRACE_T:
         case PCB_ARC_T:
         {
-            if( aFrom || m_magneticSettings->tracks == MAGNETIC_OPTIONS::CAPTURE_ALWAYS )
+            if( aFrom )
             {
-                PCB_TRACK* track = static_cast<PCB_TRACK*>( aItem );
-
-                addAnchor( track->GetStart(), CORNER | SNAPPABLE, track );
-                addAnchor( track->GetEnd(), CORNER | SNAPPABLE, track );
-                addAnchor( track->GetCenter(), ORIGIN, track);
+                if( aSelectionFilter && !aSelectionFilter->tracks )
+                    break;
             }
+            else
+            {
+                if( m_magneticSettings->tracks != MAGNETIC_OPTIONS::CAPTURE_ALWAYS )
+                    break;
+            }
+
+            PCB_TRACK* track = static_cast<PCB_TRACK*>( aItem );
+
+            addAnchor( track->GetStart(), CORNER | SNAPPABLE, track );
+            addAnchor( track->GetEnd(), CORNER | SNAPPABLE, track );
+            addAnchor( track->GetCenter(), ORIGIN, track);
 
             break;
         }
@@ -658,15 +740,26 @@ void PCB_GRID_HELPER::computeAnchors( BOARD_ITEM* aItem, const VECTOR2I& aRefPos
             break;
 
         case PCB_VIA_T:
-        {
-            if( aFrom || m_magneticSettings->tracks == MAGNETIC_OPTIONS::CAPTURE_ALWAYS )
-                addAnchor( aItem->GetPosition(), ORIGIN | CORNER | SNAPPABLE, aItem );
+            if( aFrom )
+            {
+                if( aSelectionFilter && !aSelectionFilter->vias )
+                    break;
+            }
+            else
+            {
+                if( m_magneticSettings->tracks != MAGNETIC_OPTIONS::CAPTURE_ALWAYS )
+                    break;
+            }
+
+            addAnchor( aItem->GetPosition(), ORIGIN | CORNER | SNAPPABLE, aItem );
 
             break;
-        }
 
         case PCB_ZONE_T:
         {
+            if( aFrom && aSelectionFilter && !aSelectionFilter->zones )
+                break;
+
             const SHAPE_POLY_SET* outline = static_cast<const ZONE*>( aItem )->Outline();
 
             SHAPE_LINE_CHAIN lc;
@@ -674,7 +767,28 @@ void PCB_GRID_HELPER::computeAnchors( BOARD_ITEM* aItem, const VECTOR2I& aRefPos
 
             for( auto iter = outline->CIterateWithHoles(); iter; iter++ )
             {
-                addAnchor( *iter, CORNER, aItem );
+                addAnchor( *iter, CORNER | SNAPPABLE, aItem );
+                lc.Append( *iter );
+            }
+
+            addAnchor( lc.NearestPoint( aRefPos ), OUTLINE, aItem );
+
+            break;
+        }
+
+        case PCB_FP_ZONE_T:
+        {
+            if( aFrom && aSelectionFilter && !aSelectionFilter->zones )
+                break;
+
+            const SHAPE_POLY_SET* outline = static_cast<const FP_ZONE*>( aItem )->Outline();
+
+            SHAPE_LINE_CHAIN lc;
+            lc.SetClosed( true );
+
+            for( auto iter = outline->CIterateWithHoles(); iter; iter++ )
+            {
+                addAnchor( *iter, CORNER | SNAPPABLE, aItem );
                 lc.Append( *iter );
             }
 
@@ -685,7 +799,12 @@ void PCB_GRID_HELPER::computeAnchors( BOARD_ITEM* aItem, const VECTOR2I& aRefPos
 
         case PCB_DIM_ALIGNED_T:
         case PCB_DIM_ORTHOGONAL_T:
+        case PCB_FP_DIM_ALIGNED_T:
+        case PCB_FP_DIM_ORTHOGONAL_T:
         {
+            if( aFrom && aSelectionFilter && !aSelectionFilter->dimensions )
+                break;
+
             const PCB_DIM_ALIGNED* dim = static_cast<const PCB_DIM_ALIGNED*>( aItem );
             addAnchor( dim->GetCrossbarStart(), CORNER | SNAPPABLE, aItem );
             addAnchor( dim->GetCrossbarEnd(), CORNER | SNAPPABLE, aItem );
@@ -695,7 +814,11 @@ void PCB_GRID_HELPER::computeAnchors( BOARD_ITEM* aItem, const VECTOR2I& aRefPos
         }
 
         case PCB_DIM_CENTER_T:
+        case PCB_FP_DIM_CENTER_T:
         {
+            if( aFrom && aSelectionFilter && !aSelectionFilter->dimensions )
+                break;
+
             const PCB_DIM_CENTER* dim = static_cast<const PCB_DIM_CENTER*>( aItem );
             addAnchor( dim->GetStart(), CORNER | SNAPPABLE, aItem );
             addAnchor( dim->GetEnd(), CORNER | SNAPPABLE, aItem );
@@ -705,15 +828,33 @@ void PCB_GRID_HELPER::computeAnchors( BOARD_ITEM* aItem, const VECTOR2I& aRefPos
 
             for( int i = 0; i < 2; i++ )
             {
-                radial = radial.Rotate( DEG2RAD( 90 ) );
+                RotatePoint( radial, -ANGLE_90 );
                 addAnchor( start + radial, CORNER | SNAPPABLE, aItem );
             }
 
             break;
         }
 
-        case PCB_DIM_LEADER_T:
+        case PCB_DIM_RADIAL_T:
+        case PCB_FP_DIM_RADIAL_T:
         {
+            if( aFrom && aSelectionFilter && !aSelectionFilter->dimensions )
+                break;
+
+            const PCB_DIM_RADIAL* radialDim = static_cast<const PCB_DIM_RADIAL*>( aItem );
+            addAnchor( radialDim->GetStart(), CORNER | SNAPPABLE, aItem );
+            addAnchor( radialDim->GetEnd(), CORNER | SNAPPABLE, aItem );
+            addAnchor( radialDim->GetKnee(), CORNER | SNAPPABLE, aItem );
+            addAnchor( radialDim->Text().GetPosition(), CORNER | SNAPPABLE, aItem );
+            break;
+        }
+
+        case PCB_DIM_LEADER_T:
+        case PCB_FP_DIM_LEADER_T:
+        {
+            if( aFrom && aSelectionFilter && !aSelectionFilter->dimensions )
+                break;
+
             const PCB_DIM_LEADER* leader = static_cast<const PCB_DIM_LEADER*>( aItem );
             addAnchor( leader->GetStart(), CORNER | SNAPPABLE, aItem );
             addAnchor( leader->GetEnd(), CORNER | SNAPPABLE, aItem );
@@ -723,6 +864,9 @@ void PCB_GRID_HELPER::computeAnchors( BOARD_ITEM* aItem, const VECTOR2I& aRefPos
 
         case PCB_FP_TEXT_T:
         case PCB_TEXT_T:
+            if( aFrom && aSelectionFilter && !aSelectionFilter->text )
+                break;
+
             addAnchor( aItem->GetPosition(), ORIGIN, aItem );
             break;
 
