@@ -34,6 +34,7 @@
 #include <zone.h>
 #include <string_utils.h>
 #include <math_for_graphics.h>
+#include <properties/property_validators.h>
 #include <settings/color_settings.h>
 #include <settings/settings_manager.h>
 #include <trigo.h>
@@ -904,7 +905,7 @@ void ZONE::HatchBorder()
     int     layer = GetFirstLayer();
     int     slope_flag = (layer & 1) ? 1 : -1;  // 1 or -1
     double  slope = 0.707106 * slope_flag;      // 45 degrees slope
-    int     max_a, min_a;
+    int64_t max_a, min_a;
 
     if( slope_flag == 1 )
     {
@@ -928,7 +929,7 @@ void ZONE::HatchBorder()
     std::vector<VECTOR2I> pointbuffer;
     pointbuffer.reserve( 256 );
 
-    for( int a = min_a; a < max_a; a += spacing )
+    for( int64_t a = min_a; a < max_a; a += spacing )
     {
         pointbuffer.clear();
 
@@ -1162,6 +1163,9 @@ bool ZONE::BuildSmoothedPoly( SHAPE_POLY_SET& aSmoothedPoly, PCB_LAYER_ID aLayer
         BOX2I          sameNetBoundingBox = sameNetZone->GetBoundingBox();
         SHAPE_POLY_SET sameNetPoly = sameNetZone->Outline()->CloneDropTriangulation();
 
+        SHAPE_POLY_SET diffNetPoly;
+        SHAPE_POLY_SET sumPoly = Outline()->CloneDropTriangulation();
+
         sameNetPoly.ClearArcs();
 
         // Of course there's always a wrinkle.  The same-net intersecting zone *might* get knocked
@@ -1171,11 +1175,19 @@ bool ZONE::BuildSmoothedPoly( SHAPE_POLY_SET& aSmoothedPoly, PCB_LAYER_ID aLayer
             if( otherNetZone->HigherPriority( sameNetZone )
                     && otherNetZone->GetBoundingBox().Intersects( sameNetBoundingBox ) )
             {
-                sameNetPoly.BooleanSubtract( *otherNetZone->Outline(), SHAPE_POLY_SET::PM_FAST );
+                diffNetPoly.BooleanAdd( *otherNetZone->Outline(), SHAPE_POLY_SET::PM_FAST );
             }
         }
 
-        aSmoothedPoly.BooleanAdd( sameNetPoly, SHAPE_POLY_SET::PM_FAST );
+        // Second wrinkle.  After unioning the higher priority, different net zones together,
+        // we need to check to see if they completely enclose our zone.  If they do, then
+        // we need to treat the enclosed zone as isolated, not connected to the outer zone
+        // #13915
+        if( diffNetPoly.OutlineCount() )
+            sumPoly.BooleanSubtract( diffNetPoly, SHAPE_POLY_SET::PM_FAST );
+
+        if( sumPoly.OutlineCount() )
+            aSmoothedPoly.BooleanAdd( sameNetPoly, SHAPE_POLY_SET::PM_FAST );
     }
 
     if( aBoardOutline )
@@ -1431,55 +1443,74 @@ static struct ZONE_DESC
                     return false;
                 };
 
-        auto layer = new PROPERTY_ENUM<ZONE, PCB_LAYER_ID>( _HKI( "Layer" ),
-                    &ZONE::SetLayer, &ZONE::GetLayer );
-        layer->SetIsInternal( true );
-        propMgr.ReplaceProperty( TYPE_HASH( BOARD_CONNECTED_ITEM ), _HKI( "Layer" ), layer );
+        // Layer property is internal because it only holds a single layer and zones actually use
+        // a layer set
+        propMgr.ReplaceProperty( TYPE_HASH( BOARD_CONNECTED_ITEM ), _HKI( "Layer" ),
+                                 new PROPERTY_ENUM<ZONE, PCB_LAYER_ID>( _HKI( "Layer" ),
+                                                                        &ZONE::SetLayer,
+                                                                        &ZONE::GetLayer ) )
+                .SetIsInternal();
 
         propMgr.OverrideAvailability( TYPE_HASH( ZONE ), TYPE_HASH( BOARD_CONNECTED_ITEM ),
                                       _HKI( "Net" ), isCopperZone );
         propMgr.OverrideAvailability( TYPE_HASH( ZONE ), TYPE_HASH( BOARD_CONNECTED_ITEM ),
                                       _HKI( "Net Class" ), isCopperZone );
 
-        auto priority = new PROPERTY<ZONE, unsigned>( _HKI( "Priority" ),
-                    &ZONE::SetAssignedPriority, &ZONE::GetAssignedPriority );
-        priority->SetAvailableFunc( isCopperZone );
-        propMgr.AddProperty( priority );
+        propMgr.AddProperty( new PROPERTY<ZONE, unsigned>( _HKI( "Priority" ),
+                                                           &ZONE::SetAssignedPriority,
+                                                           &ZONE::GetAssignedPriority ) )
+                .SetAvailableFunc( isCopperZone );
 
         propMgr.AddProperty( new PROPERTY<ZONE, wxString>( _HKI( "Name" ),
                     &ZONE::SetZoneName, &ZONE::GetZoneName ) );
 
         const wxString groupFill = _HKI( "Fill Style" );
 
-        auto fillMode = new PROPERTY_ENUM<ZONE, ZONE_FILL_MODE>( _HKI( "Fill Mode" ),
-                    &ZONE::SetFillMode, &ZONE::GetFillMode );
-        // Fill mode can't be exposed to the UI until validation is moved to the ZONE class.
-        // see https://gitlab.com/kicad/code/kicad/-/issues/13811
-        fillMode->SetIsInternal();
-        propMgr.AddProperty( fillMode, groupFill );
+        propMgr.AddProperty( new PROPERTY_ENUM<ZONE, ZONE_FILL_MODE>( _HKI( "Fill Mode" ),
+                                                                      &ZONE::SetFillMode,
+                                                                      &ZONE::GetFillMode ),
+                             groupFill );
 
-        auto hatchOrientation = new PROPERTY<ZONE, EDA_ANGLE>( _HKI( "Orientation" ),
-                    &ZONE::SetHatchOrientation, &ZONE::GetHatchOrientation,
-                    PROPERTY_DISPLAY::PT_DEGREE );
-        hatchOrientation->SetWriteableFunc( isHatchedFill );
-        hatchOrientation->SetIsInternal();
-        propMgr.AddProperty( hatchOrientation, groupFill );
+        propMgr.AddProperty( new PROPERTY<ZONE, EDA_ANGLE>( _HKI( "Orientation" ),
+                                                            &ZONE::SetHatchOrientation,
+                                                            &ZONE::GetHatchOrientation,
+                                                            PROPERTY_DISPLAY::PT_DEGREE ),
+                             groupFill )
+                .SetWriteableFunc( isHatchedFill );
 
-        //TODO: Switch to translated
-        auto hatchWidth = new PROPERTY<ZONE, int>( wxT( "Hatch Width" ),
-                    &ZONE::SetHatchThickness, &ZONE::GetHatchThickness,
-                    PROPERTY_DISPLAY::PT_SIZE );
-        hatchWidth->SetWriteableFunc( isHatchedFill );
-        hatchWidth->SetIsInternal();
-        propMgr.AddProperty( hatchWidth, groupFill );
+        // TODO: Switch to translated
+        auto atLeastMinWidthValidator =
+                []( const wxAny&& aValue, EDA_ITEM* aZone ) -> VALIDATOR_RESULT
+                {
+                    int   val = aValue.As<int>();
+                    ZONE* zone = dynamic_cast<ZONE*>( aZone );
+                    wxCHECK( zone, std::nullopt );
 
-        //TODO: Switch to translated
-        auto hatchGap = new PROPERTY<ZONE, int>( wxT( "Hatch Gap" ),
-                    &ZONE::SetHatchGap, &ZONE::GetHatchGap,
-                    PROPERTY_DISPLAY::PT_SIZE );
-        hatchGap->SetWriteableFunc( isHatchedFill );
-        hatchGap->SetIsInternal();
-        propMgr.AddProperty( hatchGap, groupFill );
+                    if( val < zone->GetMinThickness() )
+                    {
+                        return std::make_unique<VALIDATION_ERROR_MSG>(
+                                wxT( "Cannot be less than zone minimum width" ) );
+                    }
+
+                    return std::nullopt;
+                };
+
+        // TODO: Switch to translated
+        propMgr.AddProperty( new PROPERTY<ZONE, int>( wxT( "Hatch Width" ),
+                                                      &ZONE::SetHatchThickness,
+                                                      &ZONE::GetHatchThickness,
+                                                      PROPERTY_DISPLAY::PT_SIZE ),
+                             groupFill )
+                .SetWriteableFunc( isHatchedFill )
+                .SetValidator( atLeastMinWidthValidator );
+
+        // TODO: Switch to translated
+        propMgr.AddProperty( new PROPERTY<ZONE, int>( wxT( "Hatch Gap" ), &ZONE::SetHatchGap,
+                                                      &ZONE::GetHatchGap,
+                                                      PROPERTY_DISPLAY::PT_SIZE ),
+                             groupFill )
+                .SetWriteableFunc( isHatchedFill )
+                .SetValidator( atLeastMinWidthValidator );
 
         // TODO: Smoothing effort needs to change to enum (in dialog too)
         // TODO: Smoothing amount (double)
@@ -1491,11 +1522,16 @@ static struct ZONE_DESC
                     &ZONE::SetLocalClearance, &ZONE::GetLocalClearance,
                     PROPERTY_DISPLAY::PT_SIZE );
         clearanceOverride->SetAvailableFunc( isCopperZone );
+        constexpr int maxClearance = pcbIUScale.mmToIU( ZONE_CLEARANCE_MAX_VALUE_MM );
+        clearanceOverride->SetValidator( PROPERTY_VALIDATORS::RangeIntValidator<0, maxClearance> );
 
         auto minWidth = new PROPERTY<ZONE, int>( _HKI( "Minimum Width" ),
                     &ZONE::SetMinThickness, &ZONE::GetMinThickness,
                     PROPERTY_DISPLAY::PT_SIZE );
         minWidth->SetAvailableFunc( isCopperZone );
+        constexpr int minMinWidth = pcbIUScale.mmToIU( ZONE_THICKNESS_MIN_VALUE_MM );
+        clearanceOverride->SetValidator( PROPERTY_VALIDATORS::RangeIntValidator<minMinWidth,
+                                                                                INT_MAX> );
 
         auto padConnections = new PROPERTY_ENUM<ZONE, ZONE_CONNECTION>( _HKI( "Pad Connections" ),
                     &ZONE::SetPadConnection, &ZONE::GetPadConnection );
@@ -1505,11 +1541,13 @@ static struct ZONE_DESC
                     &ZONE::SetThermalReliefGap, &ZONE::GetThermalReliefGap,
                     PROPERTY_DISPLAY::PT_SIZE );
         thermalGap->SetAvailableFunc( isCopperZone );
+        thermalGap->SetValidator( PROPERTY_VALIDATORS::PositiveIntValidator );
 
         auto thermalSpokeWidth = new PROPERTY<ZONE, int>( _HKI( "Thermal Relief Spoke Width" ),
                     &ZONE::SetThermalReliefSpokeWidth, &ZONE::GetThermalReliefSpokeWidth,
                     PROPERTY_DISPLAY::PT_SIZE );
         thermalSpokeWidth->SetAvailableFunc( isCopperZone );
+        thermalSpokeWidth->SetValidator( atLeastMinWidthValidator );
 
         propMgr.AddProperty( clearanceOverride, groupOverrides );
         propMgr.AddProperty( minWidth, groupOverrides );
