@@ -217,7 +217,7 @@ bool ZONE_FILLER::Fill( std::vector<ZONE*>& aZones, bool aCheck, wxWindow* aPare
 
         return false;
     };
-    
+
     // Determine state of conditional via flashing
     for( PCB_TRACK* track : m_board->Tracks() )
     {
@@ -1263,6 +1263,10 @@ void ZONE_FILLER::buildCopperItemClearances( const ZONE* aZone, PCB_LAYER_ID aLa
         if( checkForCancel( m_progressReporter ) )
             return;
 
+        // Negative clearance permits zones to short
+        if( evalRulesForItems( CLEARANCE_CONSTRAINT, aZone, otherZone, aLayer ) < 0 )
+            continue;
+
         if( otherZone->GetIsRuleArea() )
         {
             if( otherZone->GetDoNotAllowCopperPour() && !aZone->IsTeardropArea() )
@@ -1524,7 +1528,7 @@ bool ZONE_FILLER::fillCopperZone( const ZONE* aZone, PCB_LAYER_ID aLayer, PCB_LA
      */
 
     if( half_min_width - epsilon > epsilon )
-        aFillPolys.Deflate( half_min_width - epsilon, numSegs, cornerStrategy );
+        aFillPolys.Deflate( half_min_width - epsilon, numSegs, fastCornerStrategy );
 
     // Min-thickness is the web thickness.  On the other hand, a blob min-thickness by
     // min-thickness is not useful.  Since there's no obvious definition of web vs. blob, we
@@ -1809,7 +1813,7 @@ void ZONE_FILLER::buildThermalSpokes( const ZONE* aZone, PCB_LAYER_ID aLayer,
 
         // If the spokes are at a cardinal angle then we can generate them from a bounding box
         // without trig.
-        if( pad->GetThermalSpokeAngle().IsCardinal() )
+        if( ( pad->GetOrientation() + pad->GetThermalSpokeAngle() ).IsCardinal() )
         {
             BOX2I spokesBox = pad->GetBoundingBox();
             spokesBox.Inflate( thermalReliefGap + epsilon );
